/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.servlets.admin.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.sctserver.servlets.explore.*;
import fr.exemole.sctserver.request.simple.*;
import fr.exemole.sctserver.request.uri.*;
import fr.exemole.sctserver.request.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import net.mapeadores.scrutari.db.*;
import fr.exemole.sctserver.*;
import fr.exemole.sctserver.request.xml.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.servlets.global.*;

/**
 *
 * @author Vincent Calame
 */
public class ScrutariServlet extends HttpServlet {
    
    public void doGet(HttpServletRequest requete,HttpServletResponse reponse) throws IOException,ServletException {
        process(requete, reponse);
    }
    
    public void doPost(HttpServletRequest requete,HttpServletResponse reponse) throws IOException,ServletException {
        process(requete, reponse);
    }
    
    private void process(HttpServletRequest request,HttpServletResponse response) throws IOException,ServletException {
        String pathInfo = request.getPathInfo();
        if (pathInfo == null) {
            pathInfo = "/";
        }
        int idx = pathInfo.indexOf('/',2);
        if (idx != -1) {
            String name = pathInfo.substring(1,idx);
            String sctserverPathInfo = pathInfo.substring(idx+1);
            processSctServerRequest(name, sctserverPathInfo, request, response);
        } else {
            processGlobalRequest(pathInfo, request, response);
        }
    }
    
    private void processGlobalRequest(String path, HttpServletRequest request,HttpServletResponse response) throws IOException {
        SctServerContext sctServerContext = SctServerContext.getSctServerContext(getServletContext());
        StringBuffer requestURLString = request.getRequestURL();
        if (path.length() == 1) {
            if (requestURLString.charAt(requestURLString.length() - 1) != '/') {
                response.sendRedirect(requestURLString.append('/').toString());
                return;
            }
        }
        ResponsePrinter responsePrinter = null;
        if (path.endsWith(".css")) {
            responsePrinter = SctDocStreamPrinter.getCssPrinter(request,path);
        } else if (path.endsWith(".png")) {
             responsePrinter = SctDocStreamPrinter.getPngPrinter(request,path);
        }
        else {
            GlobalRequest globalRequest = GlobalRequest.init(path);
            if (globalRequest == null) {
                if (sctServerContext.getSctBundle(path) != null) {
                    response.sendRedirect(requestURLString.append('/').toString());
                    return;
                }
            } else {
                globalRequest.run(getServletContext(), sctServerContext);
                responsePrinter = globalRequest.getResponsePrinter();
            }
        }
        if (responsePrinter != null) {
            responsePrinter.printResponse(response);
        } else {
            printNotFound(request, response);
        }
    }
    
    private void processSctServerRequest(String name, String sctserverPathInfo, HttpServletRequest request, HttpServletResponse response) throws IOException {
        SctServerContext sctServerContext = SctServerContext.getSctServerContext(getServletContext());
        SctBundle sctBundle = sctServerContext.getSctBundle(name);
        if (sctBundle == null) {
            printNotFound(request, response);
            return;
        }
        if (sctserverPathInfo.length() == 0) {
            response.sendError(HttpServletResponse.SC_FORBIDDEN);
            return;
        }
        char carac = sctserverPathInfo.charAt(0);
        if ((carac >= 'A') && (carac <= 'Z')) {
            if ((sctserverPathInfo.equals("Simple")) || (sctserverPathInfo.equals("RchfPlain"))) {
                printSimple(request, response, sctBundle);
            } else if ((sctserverPathInfo.equals("Uri")) || (sctserverPathInfo.equals("RchfUri"))) {
                printURI(request, response, sctBundle);
            } else if ((sctserverPathInfo.equals("Xml")) || (sctserverPathInfo.equals("RchfXml"))) {
                printXML(request, response, sctBundle);
            } else {
                printNotFound(request, response);
                return;
            }
        } else {
            int idx = sctserverPathInfo.indexOf('/');
            if (idx == -1) {
                printNotFound(request, response);
                return;
            }
            String servletPath = sctserverPathInfo.substring(0,idx);
            sctserverPathInfo = sctserverPathInfo.substring(idx);
            ResponsePrinter responsePrinter = null;
            if (servletPath.equals("admin")) {
                AdminRequest adminRequest = AdminRequest.init(sctBundle,sctserverPathInfo, request);
                if (adminRequest != null) responsePrinter = adminRequest.getResponsePrinter();
            } else if (servletPath.equals("explore")) {
                ExploreRequest exploreRequest = ExploreRequest.init(sctBundle, sctserverPathInfo, request);
                if (exploreRequest != null) responsePrinter = exploreRequest.getResponsePrinter();
            } else if (servletPath.equals("css")) {
                responsePrinter = SctDocStreamPrinter.getCssPrinter(request,sctserverPathInfo);
            } else if (servletPath.equals("dtd")) {
                responsePrinter = SctDocStreamPrinter.getDtdPrinter(request, sctserverPathInfo);
            } else if (servletPath.equals("js")) {
                responsePrinter = SctDocStreamPrinter.getJsPrinter(request, sctserverPathInfo);
            } else if (servletPath.equals("logs")) {
                responsePrinter = SctDocStreamPrinter.getLogsPrinter(request, sctserverPathInfo, sctBundle.getSctServer().getLogAccess());
            }
            if (responsePrinter != null) {
                responsePrinter.printResponse(response);
            } else {
                printNotFound(request, response);
                return;
            }
            
        }
    }
    
    private void printSimple(HttpServletRequest requete, HttpServletResponse reponse, SctBundle sctBundle) throws IOException {
        reponse.setHeader("Cache-Control","no-store");
        reponse.setContentType("text/plain;charset=UTF-8");
        SimpleRequestMap requestMap = new SimpleRequestMap(requete);
        try {
            SimpleRequest plainRequest = SimpleRequestFactory.getSimpleRequest(sctBundle.getSctServer(),requestMap);
            plainRequest.printResponse(reponse.getWriter());
        } catch(SimpleParameterErrorException ppee) {
            PrintWriter pw = reponse.getWriter();
            pw.print("!-");
            pw.print(String.valueOf(ppee.getType()));
            String parameterName = ppee.getParameterName();
            if (parameterName != null) {
                pw.print('\n');
                pw.print("parameterName: ");
                pw.print(parameterName);
            }
            String parameterValue = ppee.getParameterValue();
            if (parameterValue != null) {
                pw.print('\n');
                pw.print("parameterValue: ");
                pw.print(parameterValue);
            }
            pw.close();
            
        }
    }
    
    
    private void printURI(HttpServletRequest requete, HttpServletResponse reponse, SctBundle sctBundle) throws IOException {
        reponse.setHeader("Cache-Control","no-store");
        String param_uri = requete.getParameter("uri");
        if (param_uri != null) {
            URIRequest uriRequest = new URIRequest(param_uri,sctBundle.getSctServer().getScrutariDB());
            uriRequest.printResponse(reponse);
            return;
        }
    }
    
    private void printXML(HttpServletRequest requete, HttpServletResponse reponse, SctBundle sctBundle) throws IOException {
        reponse.setHeader("Cache-Control","no-store");
        UTF8Bridge bridge = new UTF8Bridge(requete.getCharacterEncoding());
        RequestInfo requestInfo = newRequestInfo(requete);
        try {
            SctServer sctServer = sctServer = sctBundle.getSctServer();
            String param_cmd = requete.getParameter("cmd");
            String param_xml = bridge.toUTF8(requete.getParameter("xml"));
            String param_searchid = requete.getParameter("searchid");
            if (param_cmd == null) throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.MISSING_CMD_PARAMETER);
            ScrutariDB generalDatabase = sctServer.getScrutariDB();
            if (param_cmd.equals("metadata")) {
                MetadataRequest  metadataRequest = new MetadataRequest(generalDatabase);
                metadataRequest.printResponse(reponse);
            } else if (param_cmd.equals("search")) {
                SearchRequest searchRequest = new SearchRequest(param_xml,sctServer, requestInfo);
                searchRequest.printResponse(reponse);
            }
            else if (param_cmd.equals("fiches")) {
                FichesRequest fichesRequest = new FichesRequest(param_searchid,sctServer);
                fichesRequest.printResponse(reponse);
            }
            else if (param_cmd.equals("motscles")) {
                MotsclesRequest motsclesRequest = new MotsclesRequest(param_searchid,sctServer);
                motsclesRequest.printResponse(reponse);
                
            } else {
                throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.WRONG_CMD_PARAMETER,param_cmd);
            }
        } catch(XmlRequestException xre) {
            xre.printErrorResponse(reponse);
            Throwable cause = xre.getCause();
            if (cause != null) cause.printStackTrace();
        }
    }
    
    private void printNotFound(HttpServletRequest request,HttpServletResponse response) throws IOException {
        response.sendError(HttpServletResponse.SC_NOT_FOUND,request.getPathInfo());
    }
    
    /* à transférer */
    public final static String LOG_PARAMETER = "log";
    public final static String SITE_PARAMETER = "site";
    public final static String REFERRER_PARAMETER = "referrer";
    
     public static RequestInfo newRequestInfo(HttpServletRequest requete) {
        String remoteAddress = requete.getRemoteAddr();
        String param_log = requete.getParameter(LOG_PARAMETER);
        boolean withLog = true;
        if ((param_log != null) && (param_log.equals("0"))) withLog = false;
        String site = requete.getParameter(SITE_PARAMETER);
        if (site != null) {
            if (!StringUtils.isTechnicalName(site, true)) {
                site = null;
            }
        }
        String referrer = requete.getParameter(REFERRER_PARAMETER);
        return new RequestInfo(remoteAddress,withLog, site, referrer);
    }
}
