/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets;

import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.util.io.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.sctserver.storage.*;
import fr.exemole.sctserver.request.xml.*;

/**
 *
 * @author Vincent Calame
 */
public class SctDocStreamPrinter extends DocStreamPrinter {
    public final static short CSS_STREAM = 1;
    public final static short JS_STREAM = 2;
    public final static short DTD_STREAM = 3;
    public final static short LOGS_STREAM = 4;
    public final static short PNG_STREAM = 5;
    private short type;
    private LogAccess logAccess;
    private String fileName;
    
    private SctDocStreamPrinter(short type, HttpServletRequest request, String name,LogAccess logAccess) {
        super(request);
        this.logAccess = logAccess;
        this.type = type;
        this.fileName = name.substring(1);
    }
    
    public static SctDocStreamPrinter getCssPrinter(HttpServletRequest request, String name) {
        return new SctDocStreamPrinter(CSS_STREAM,request, name, null);
    }
    
    public static SctDocStreamPrinter getDtdPrinter(HttpServletRequest request, String name) {
        return new SctDocStreamPrinter(DTD_STREAM,request, name, null);
    }
    
    public static SctDocStreamPrinter getJsPrinter(HttpServletRequest request, String name) {
        return new SctDocStreamPrinter(JS_STREAM,request, name, null);
    }
    
    public static SctDocStreamPrinter getLogsPrinter(HttpServletRequest request, String name,LogAccess logAccess) {
        return new SctDocStreamPrinter(LOGS_STREAM,request, name, logAccess);
    }
    
    public static SctDocStreamPrinter getPngPrinter(HttpServletRequest request, String name) {
        return new SctDocStreamPrinter(PNG_STREAM,request, name, null);
    }
     
     public DocStream getDocStream(String servletPath, String pathInfo) {
        switch(type) {
            case CSS_STREAM :
                return getCssDocStream();
            case JS_STREAM :
                return getJsDocStream();
            case DTD_STREAM :
                return getDtdDocStream();
            case LOGS_STREAM :
                return getLogsDocStream();
            case PNG_STREAM :
                return getPngDocStream();
            default : 
                return null;
        }
     }
     
     private DocStream getCssDocStream() {
        InputStream inputStream = getClass().getResourceAsStream("resources/theme/css/" + fileName);
        if (inputStream == null) return null;
        return new DefaultDocStream(inputStream,"text/css","UTF-8");
    }
    
     
     private DocStream getDtdDocStream() {
        InputStream inputStream = XmlRequestUtils.getDtdInputStream(fileName);
        if (inputStream == null) return null;
        return new DefaultDocStream(inputStream,"application/xml-dtd","UTF-8");
    }
     
     private DocStream getJsDocStream() {
        InputStream inputStream = getClass().getResourceAsStream("resources/js/" + fileName);
        if (inputStream == null) return null;
        return new DefaultDocStream(inputStream,"text/javascript","UTF-8");
    }
     
     private DocStream getLogsDocStream() {
             InputStream inputStream = logAccess.getLogInputStream(fileName);
             if (inputStream == null) return null;
             return new DefaultDocStream(inputStream,"text/plain","UTF-8");
    }

    private DocStream getPngDocStream() {
        InputStream inputStream = getClass().getResourceAsStream("resources/theme/png/" + fileName);
        if (inputStream == null) return null;
        return new DefaultDocStream(inputStream,"image/png");
    }
}
