/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin;

import javax.servlet.http.*;
import fr.exemole.sctserver.servlets.*;
import java.util.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.logging.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.sctserver.servlets.selection.*;
import fr.exemole.sctserver.servlets.admin.htmlpages.*;
import fr.exemole.sctserver.servlets.admin.xmldata.*;
import fr.exemole.sctserver.servlets.admin.contentstreams.*;
import net.mapeadores.util.servlets.specialresponses.*;

/**
 *
 * @author Vincent Calame
 */
public class AdminRequest {
    private SctBundle sctBundle;
    private AdminResponseParameters adminResponseParameters;
    private String path;
    private boolean notFound = false;
    
    private AdminRequest(SctBundle sctBundle,String pathInfo, HttpServletRequest request) {
        this.sctBundle = sctBundle;
        path = request.getPathInfo();;
        adminResponseParameters = new AdminResponseParameters(sctBundle,Locale.FRENCH);
        if (pathInfo.equals("/")) {
            initAndDoCommand(new CommandRequestMap(request));
        } else if (pathInfo.equals("/Test")) {
            initAndTestCommand(new CommandRequestMap(request));
        } else if (pathInfo.equals("/index.html")) {
            initPage(AdminResponseParameters.INDEX_PAGE);
        } else if (pathInfo.equals("/stats.html")) {
            initPage(AdminResponseParameters.STATS_PAGE);
        } else if (pathInfo.equals("/fiches.html")) {
            initPage(AdminResponseParameters.FICHES_PAGE);
        } else if (pathInfo.equals("/lexies.html")) {
            initPage(AdminResponseParameters.LEXIES_PAGE);
        } else if (pathInfo.equals("/uri.txt")) {
            adminResponseParameters.setContentStreamType(AdminResponseParameters.URI_STREAM);
        } else if (pathInfo.equals("/journaux.html")) {
            initPage(AdminResponseParameters.JOURNAUX_PAGE);
        } else if (pathInfo.startsWith("/motscles")) {
            Parameters parameters = new Parameters();
            if (!parameters.init(pathInfo.substring(9), false)) {
                setNotFound();
                return;
            }
            adminResponseParameters.setSelectionUnit(parameters.getSelectionUnit());
            String extension = parameters.getExtension();
            if (extension.equals("html")) {
                adminResponseParameters.setHtmlPageType(AdminResponseParameters.MOTSCLES_PAGE);
            } else if (extension.equals("ods")) {
                adminResponseParameters.setContentStreamType(AdminResponseParameters.MOTSCLES_STREAM);
            } else {
                setNotFound();
            }
        } else if (pathInfo.startsWith("/lexies_")) {
            Parameters parameters = new Parameters();
            if (!parameters.init(pathInfo.substring(8), true)) {
                setNotFound();
                return;
            }
            adminResponseParameters.setSelectionUnit(parameters.getSelectionUnit());
            adminResponseParameters.setLangInteger(parameters.getLangInteger());
            String extension = parameters.getExtension();
            if (extension.equals("html")) {
                adminResponseParameters.setHtmlPageType(AdminResponseParameters.LEXIES_PAGE);
            } else if (extension.equals("ods")) {
                adminResponseParameters.setContentStreamType(AdminResponseParameters.LEXIES_STREAM);
            } else {
                setNotFound();
            }
        } else if (pathInfo.startsWith("/journal_")) {
            JournalParameters journalParameters = new JournalParameters();
            if (!journalParameters.init(pathInfo.substring(9))) {
                setNotFound();
                return;
            }
            String extension = journalParameters.getExtension();
            if (extension.equals("html")) {
                adminResponseParameters.setHtmlPageType(AdminResponseParameters.JOURNAL_PAGE);
                adminResponseParameters.setDateInteger(journalParameters.getDateInteger());
            } else {
                setNotFound();
            }
        } else {
            setNotFound();
        }
    }
    
    public static AdminRequest init(SctBundle sctBundle,String pathInfo, HttpServletRequest request) {
        AdminRequest adminRequest = new AdminRequest(sctBundle, pathInfo, request);
        if (adminRequest.notFound) {
            return null;
        } else {
            return adminRequest;
        }
    }
    
    public AdminResponseParameters getResponseParameters() {
        return adminResponseParameters;
    }
    
    public ResponsePrinter getResponsePrinter() {
         switch(adminResponseParameters.getCategory()) {
            case ResponseParameters.HTMLPAGE_CATEGORY :
                return AdminHtmlPage.getPage(adminResponseParameters);
            case ResponseParameters.XMLDATA_CATEGORY :
                return AdminXmlData.getXmlData(adminResponseParameters);
             case ResponseParameters.CONTENTSTREAM_CATEGORY :
                 return AdminContentStream.getInstance(adminResponseParameters);
             default :
                throw new SwitchException("category = " + adminResponseParameters.getCategory());
        }
    }
    
    private void initAndDoCommand(CommandRequestMap requestMap) {
        String cmd = requestMap.getCommandParameter();
        if (cmd == null) {
            initPage(AdminResponseParameters.INDEX_PAGE);
        } else {
            Command command = Command.getCommandInstance(cmd);
            if (command == null) {
                adminResponseParameters.setHtmlPageType(AdminResponseParameters.ERROR_PAGE);
                adminResponseParameters.setCommandMessage(CommandMessage.error("error_unknowncommand",cmd));
            } else {
                command.setAdminResponseParameters(adminResponseParameters);
                command.doCommand(requestMap);
            }
        }
    }
    
    private void initAndTestCommand(CommandRequestMap requestMap) {
        adminResponseParameters.setXmlDataType(adminResponseParameters.TEST_XMLDATA);
        String cmd = requestMap.getCommandParameter();
        if (cmd != null) {
            Command command = Command.getCommandInstance(cmd);
            if (command != null) {
                command.setAdminResponseParameters(adminResponseParameters);
                command.testCommand(requestMap);
            }
        }
    }
    
    private void initPage(short type) {
        adminResponseParameters.setHtmlPageType(type);
    }
    
    public static String pageToString(short pageType) {
        switch(pageType) {
            case AdminResponseParameters.INDEX_PAGE :
                return "index";
            case AdminResponseParameters.STATS_PAGE :
                return "stats";
            case AdminResponseParameters.FICHES_PAGE :
                return "fiches";
            case AdminResponseParameters.MOTSCLES_PAGE :
                return "motscles";
            case AdminResponseParameters.LEXIES_PAGE :
                return "lexies";
            case AdminResponseParameters.JOURNAUX_PAGE :
                return "journaux";
            default :
                return null;
        }
    }
    
    private void setNotFound() {
        notFound = true;
    }
    
    private class JournalParameters {
        private String extension;
        private int dateInteger;
        
        JournalParameters() {
            
        }
        
        boolean init(String pathInfo) {
            int idx = pathInfo.lastIndexOf(".");
            if (idx == -1) return false;
            extension = pathInfo.substring(idx+1);
            pathInfo = pathInfo.substring(0, idx);
            try {
                dateInteger = IntDate.parse(pathInfo);
            } catch(IntDateException ide) {
                return false;
            }
            return true;
        }
        
        public String getExtension() {
            return extension;
        }
        
        public int getDateInteger() {
            return dateInteger;
        }
    }
    
    private class Parameters {
        private String extension;
        private SelectionUnit selectionUnit;
        private int langInteger;
        
        Parameters() {
        }
        
        boolean init(String pathInfo, boolean testLangue) {
            int idx = pathInfo.lastIndexOf(".");
            if (idx == -1) return false;
            extension = pathInfo.substring(idx+1);
            pathInfo = pathInfo.substring(0, idx);
            int idx2 = pathInfo.lastIndexOf("-");
            if (idx2 != -1) {
                String suname = pathInfo.substring(idx2+1);
                selectionUnit = sctBundle.getSelectionManager().getSelectionUnit(suname);
                if (getSelectionUnit() == null) return false;
                pathInfo = pathInfo.substring(0, idx2);
            }
            if (testLangue) {
                try {
                    langInteger = LangInteger.parse(pathInfo);
                } catch(LangIntegerException lie) {
                    return false;
                }
            }
            return true;
        }
        
        public String getExtension() {
            return extension;
        }
        
        public SelectionUnit getSelectionUnit() {
            return selectionUnit;
        }
        
        public int getLangInteger() {
            return langInteger;
        }
    }
}
