/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin;

import net.mapeadores.util.servlets.*;
import net.mapeadores.util.logging.*;
import fr.exemole.sctserver.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class Command {
    private boolean maySetType = true;
    private AdminResponseParameters adminResponseParameters;
    
    public Command() {
    }
    
    public void setAdminResponseParameters(AdminResponseParameters adminResponseParameters) {
        this.adminResponseParameters = adminResponseParameters;
        if (adminResponseParameters.isCategorySet()) maySetType = false;
    }
    
    public void setHtmlPageType(short pageType) {
        if (maySetType) adminResponseParameters.setHtmlPageType(pageType);
    }
    
    public void setCommandMessage(CommandMessage commandMessage) {
        adminResponseParameters.setCommandMessage(commandMessage);
    }
    
    public SctServer getSctServer() {
        return adminResponseParameters.getSctServer();
    }
    
    public abstract void doCommand(RequestMap requestMap);
    
    public abstract boolean testCommand(RequestMap requestMap);

    
    public static Command getCommandInstance(String commandName) {
        if (commandName.equals(RestartCommand.getCommandString())) {
            return new RestartCommand();
        }
        else if (commandName.equals(ConfUpdateCommand.getCommandString())) {
            return new ConfUpdateCommand();
        }
        else {
            return null;
        }
    }
    
    
    
    public void setMissingParameterError(String paramName) {
        setHtmlPageType(AdminResponseParameters.ERROR_PAGE);
        adminResponseParameters.setCommandMessage(CommandMessage.error("error_missingparameter",paramName));
    }
    
    public void setWrongParameterError(String paramName, Object paramValue) {
        setHtmlPageType(AdminResponseParameters.ERROR_PAGE);
        Object[] val = new Object[2];
        val[0] = paramName;
        val[1] = paramValue;
        adminResponseParameters.setCommandMessage(CommandMessage.error("error_wrongparameter",val));
    }
    
}
