/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.contentstreams;

import net.mapeadores.opendocument.io.*;
import net.mapeadores.util.xml.*;
import fr.exemole.sctserver.servlets.admin.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.stats.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.servlets.selection.*;
import net.mapeadores.scrutari.uri.*;

/**
 *
 * @author Vincent Calame
 */
public class LexiesOdsStream extends AdminContentStream {
    private int langInteger;
    private SelectionUnit selectionUnit;
    
    public LexiesOdsStream(AdminResponseParameters adminParameters) {
        super(adminParameters);
        langInteger = adminParameters.getLangInteger();
        selectionUnit = adminParameters.getSelectionUnit();
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        LexieOdsWriter lexieOdsWriter = new LexieOdsWriter();
        lexieOdsWriter.export(outputStream);
        outputStream.close();
        
    }
    
    
    private class LexieOdsWriter {
        private OdZip odZip;
        
        LexieOdsWriter() {
            odZip = new OdZip();
            odZip.setOdtype(OdUtils.OD_SPREADSHEET);
            odZip.setContentZipEntrySource(new ContentZipEntrySource());
        }
        
        public void export(OutputStream outputStream) throws IOException {
            odZip.zip(outputStream);
        }
        
        private class ContentZipEntrySource implements ZipEntrySource {
            
            public void writeEntry(OutputStream outputStream) throws IOException {
                BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
                XMLDestination xmlDestination = XMLDestinationFactory.newInstance(writer);
                LexieXMLWriter xmlWriter = new LexieXMLWriter();
                xmlWriter.setXmlDestination(xmlDestination);
                xmlWriter.setIndentLength(0);
                xmlWriter.init();
                xmlWriter.start();
                xmlWriter.printTable();
                xmlWriter.end();
                writer.flush();
            }
            
            private class LexieXMLWriter extends OdContentWriter {
                ThesaurusData[] thesaurusDataArray;
                String[] baseLibelle;
                String[] thesaurusLibelle;
                int[] codethesaurusArray;
                int[] codecorpusArray;
                String[] name;
                int thesaurusCount;
                
                CountStats countStats;
                DataManager dataManager;
                LexieManager2 lexieManager;
                Map<String, LexieBundle> bundleMap = new TreeMap<String, LexieBundle>();

                
                LexieXMLWriter() {
                    
                }
                
                void init() {
                    dataManager = sctServer.getScrutariDB().getDataManager();
                    lexieManager = sctServer.getScrutariDB().getLexieManager();
                    countStats = sctServer.getScrutariDB().getCountStats();
                    int thesaurusMaxCount = dataManager.getThesaurusCount();
                    codecorpusArray = dataManager.getCodecorpusArray();
                    thesaurusDataArray = new ThesaurusData[thesaurusMaxCount];
                    name = new String[thesaurusMaxCount];
                    thesaurusLibelle = new String[thesaurusMaxCount];
                    baseLibelle = new String[thesaurusMaxCount];
                    codethesaurusArray = new int[thesaurusMaxCount];
                    int p = 0;
                    for(int i = 0; i < thesaurusMaxCount; i++) {
                        ThesaurusData thData = dataManager.getThesaurusDataByIndex(i);
                        ThesaurusURI thesaurusURI = thData.getThesaurusURI();
                        if ((selectionUnit == null) || (selectionUnit.contains(thesaurusURI))) {
                            thesaurusDataArray[p] = thData;
                            codethesaurusArray[p] = thData.getCodeThesaurus();
                            name[p] = thesaurusURI.getThesaurusName();
                            thesaurusLibelle[p] = LibelleUtils.getLibelleString(thData, workingLangInteger,"?");
                            BaseData baseData = dataManager.getBaseDataByCode(thData.getCodeBase());
                            baseLibelle[p] = LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT,workingLangInteger,"?");
                            p++;
                        }
                    }
                    thesaurusCount = p;
                    initBundleMap();
                    
                }
                
                private void initBundleMap() {
                    List list = sctServer.getScrutariDB().getLexieManager().getLexieQuery().getScrutariLexieUnitList(langInteger);
                    for(Iterator it = list.iterator(); it.hasNext();) {
                        ScrutariLexieUnit lexieUnit = (ScrutariLexieUnit) it.next();
                        if (lexieUnit.hasMotcleAndLexieLink()) {
                            initLexieBundle(lexieUnit);
                        }   
                    }
                }
                
                private void initLexieBundle(ScrutariLexieUnit lexieUnit) {
                    String canonicalLexie = lexieUnit.getCanonicalLexie();
                    LexieBundle lexieBundle = new LexieBundle(canonicalLexie);
                    boolean avec = false;
                    long lexieId = lexieUnit.getLexieId();
                    for(int i = 0; i < thesaurusCount; i++) {
                        int codethesaurus = codethesaurusArray[i];
                        BestPosition[] array = lexieManager.getLexieQuery().getBestPositionArray(lexieId,codethesaurus);
                        if (array != null) {
                            for(int j = 0; j < array.length; j++) {
                                int codemotcle = array[j].getCode();
                                MotcleData motcleData =dataManager.getMotcleDataByCode(codemotcle);
                                MotcleCountStats motcleCountStats = countStats.getMotcleCountStats(codemotcle);
                                int count = motcleCountStats.getFicheCount();
                                Libelle libelle = motcleData.getLibelleByLang(langInteger);
                                lexieBundle.add(i,libelle.getLibelleString(), count);
                                avec = true;
                            }
                        }
                    }
                    if (avec) {
                        bundleMap.put(lexieUnit.getCollatedLexie(), lexieBundle);
                    }
                }
                
                
                private void start() throws IOException{
                    appendXMLDeclaration();
                    openDocumentContent();
                /*openAutomaticStyles();
                 
                openStyle("co1","table-column",null);
                closeStyle();
                 
                startDateStyleOpenTag("N84", "data-style", Boolean.TRUE);
                endOpenTag();
                addISONumber();
                closeDateStyle();
                 
                startStyleOpenTag("ce3","table-cell","Default");
                addAttribute("style:data-style-name","N84");
                closeEmptyTag();
                 
                closeAutomaticStyles();*/
                    openBody();
                    openSpreadsheet();
                }
                
                private void printTable() throws IOException {
                    openTable("lexies_" + LangInteger.toString(langInteger));
                    for(LexieBundle lexieBundle : bundleMap.values()) {
                        printLexieBundle(lexieBundle);
                    }
                    closeTable();
                }
                
                private void printLexieBundle(LexieBundle lexieBundle) throws IOException {
                    String canonicalLexie = lexieBundle.getCanonicalLexie();
                    boolean premier = true;
                    for(MotcleBundle motcleBundle : lexieBundle.getMotcleBundleList()) {
                        printMotcleBundle(canonicalLexie, premier,motcleBundle);
                        premier = false;
                    }
                }
                
                
                private void  printMotcleBundle(String canonicalLexie, boolean premier, MotcleBundle motcleBundle) throws IOException {
                    int thesaurusIndex = motcleBundle.getThesaurusIndex();
                    openTableRow();
                    if (premier) {
                        addStringTableCell(canonicalLexie);
                    }
                    else {
                        addEmptyTableCell();
                    }
                    addStringTableCell(canonicalLexie);
                    addStringTableCell(motcleBundle.getLibelleString());
                    //addStringTableCell(name[thesaurusIndex]);
                    addStringTableCell(thesaurusLibelle[thesaurusIndex]);
                    addStringTableCell(baseLibelle[thesaurusIndex]);
                    
                    addIntegerTableCell(motcleBundle.getCount());
                    closeTableRow();
                }
                
                private void end() throws IOException {
                    closeSpreadsheet();
                    closeBody();
                    closeDocumentContent();
                }
            }
        }
    }
    
    private static class LexieBundle {
        private String canonicalLexie;
        private List<MotcleBundle> motcleBundleList = new ArrayList<MotcleBundle>();
        
        LexieBundle(String canonicalLexie) {
            this.canonicalLexie = canonicalLexie;
        }
        
        public void add(int thesaurusIndex, String libelleString, int count) {
            MotcleBundle motcleBundle = new MotcleBundle();
            motcleBundle.setThesaurusIndex(thesaurusIndex);
            motcleBundle.setLibelleString(libelleString);
            motcleBundle.setCount(count);
            motcleBundleList.add(motcleBundle);
        }

        public List<MotcleBundle> getMotcleBundleList() {
            return motcleBundleList;
        }

        public String getCanonicalLexie() {
            return canonicalLexie;
        }
        
        
    }
    
    private static class MotcleBundle {
            private int count;
            private int thesaurusIndex;
            private String libelleString;
            
            MotcleBundle() {
                
            }

            public int getCount() {
                return count;
            }

            public void setCount(int count) {
                this.count = count;
            }

            public int getThesaurusIndex() {
                return thesaurusIndex;
            }

            public void setThesaurusIndex(int thesaurusIndex) {
                this.thesaurusIndex = thesaurusIndex;
            }

            public String getLibelleString() {
                return libelleString;
            }

            public void setLibelleString(String libelleString) {
                this.libelleString = libelleString;
            }
        }
    
    
    
    
}
