/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.contentstreams;

import net.mapeadores.opendocument.io.*;
import net.mapeadores.util.xml.*;
import fr.exemole.sctserver.servlets.admin.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.servlets.selection.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.scrutari.db.stats.*;

/**
 *
 * @author Vincent Calame
 */
public class MotsclesOdsStream extends AdminContentStream {
    
    public MotsclesOdsStream(AdminResponseParameters adminParameters) {
        super(adminParameters);
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        response.setContentType(OdUtils.getMimeType(OdUtils.OD_SPREADSHEET));
        OutputStream outputStream = response.getOutputStream();
        MotsclesOdsWriter lexieOdsWriter = new MotsclesOdsWriter();
        lexieOdsWriter.export(outputStream);
        outputStream.close();
        
    }
    
     private class MotsclesOdsWriter {
        private OdZip odZip;
        
        MotsclesOdsWriter() {
            odZip = new OdZip();
            odZip.setOdtype(OdUtils.OD_SPREADSHEET);
            odZip.setContentZipEntrySource(new ContentZipEntrySource());
        }
        
        public void export(OutputStream outputStream) throws IOException {
            odZip.zip(outputStream);
        }
        
        private class ContentZipEntrySource implements ZipEntrySource {
            String currentBaseLibelle;
            String currentThesaurusLibelle;
            
            
            public void writeEntry(OutputStream outputStream) throws IOException {
                BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, "UTF-8"));
                XMLDestination xmlDestination = XMLDestinationFactory.newInstance(writer);
                MotsclesXMLWriter xmlWriter = new MotsclesXMLWriter();
                xmlWriter.setXmlDestination(xmlDestination);
                xmlWriter.setIndentLength(0);
                xmlWriter.init();
                xmlWriter.start();
                xmlWriter.printMotscles();
                xmlWriter.end();
                writer.flush();
            }
            
            private class MotsclesXMLWriter extends OdContentWriter {
                ScrutariDB scrutariDB;
                int[] motcleLangArray;
                DataManager dataManager;
                SelectionUnit selectionUnit;
                CountStats countStats;
                
                
                void init() {
                    scrutariDB = adminParameters.getSctServer().getScrutariDB();
                    dataManager = scrutariDB.getDataManager();
                    motcleLangArray = scrutariDB.getLangStats().getAllMotcleLangArray();
                    selectionUnit = adminParameters.getSelectionUnit();
                    countStats = scrutariDB.getCountStats();
                }
                
                private void start() throws IOException {
                    appendXMLDeclaration();
                    openDocumentContent();
                    openBody();
                    openSpreadsheet();
                    openTable("motscles");
                    printLibRow();
  
                }
                
                private void printLibRow() throws IOException {
                    openTableRow();
                    
                    int length = motcleLangArray.length;
                    for(int i = 0; i < length;i++) {
                        addStringTableCell(LangInteger.toString(motcleLangArray[i]));
                    }
                    addStringTableCell("occurence");
                    addStringTableCell("thesaurus");
                    addStringTableCell("base");
                    closeTableRow();
                }
                
                private void printMotscles() throws IOException {
                    int baseCount = dataManager.getBaseCount();
                    for(int i = 0; i< baseCount; i++) {
                        BaseData baseData = dataManager.getBaseDataByIndex(i);
                        printBaseData(baseData);
                    }
                }
                
                private void printBaseData(BaseData baseData) throws IOException {
                    currentBaseLibelle = LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_SHORT,workingLangInteger,"?");
                    int thesaurusCount = baseData.getThesaurusCount();
                    for(int i = 0; i < thesaurusCount; i++) {
                        ThesaurusData thesaurusData = dataManager.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
                        printThesaurusData(thesaurusData);
                    }
                }
                
                private void printThesaurusData(ThesaurusData thesaurusData) throws IOException {
                    if ((selectionUnit != null) && (!selectionUnit.contains(thesaurusData.getThesaurusURI()))) return;
                    currentThesaurusLibelle = LibelleUtils.getLibelleString(thesaurusData, workingLangInteger,"?");
                    int motcleCount = thesaurusData.getMotcleCount();
                    for(int i = 0; i < motcleCount; i++) {
                        printMotcleData(dataManager.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
                    }
                }
                
                private void printMotcleData(MotcleData motcleData) throws IOException {
                    openTableRow();
                    int length = motcleLangArray.length;
                    for(int i = 0; i < length;i++) {
                        Libelle libelle = motcleData.getLibelleByLang(motcleLangArray[i]);
                        if (libelle != null) addStringTableCell(libelle.getLibelleString());
                        else addEmptyTableCell();
                    }
                    MotcleCountStats motcleCountStats = countStats.getMotcleCountStats(motcleData.getCodeMotcle());
                    addIntegerTableCell(motcleCountStats.getFicheCount());
                    addStringTableCell(currentThesaurusLibelle);
                    addStringTableCell(currentBaseLibelle);
                    closeTableRow();
                }
                
                private void end() throws IOException {
                    closeTable();
                    closeSpreadsheet();
                    closeBody();
                    closeDocumentContent();
                }
                
            }
        }
        
        
     }
    
}
