/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.servlets.admin.*;
import net.mapeadores.util.html.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.localisation.*;
import fr.exemole.sctserver.servlets.resources.theme.*;
import net.mapeadores.util.logging.*;
import fr.exemole.sctserver.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.sctserver.servlets.selection.*;
import fr.exemole.sctserver.servlets.admin.htmlpages.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class AdminHtmlPage extends HTMLPage implements SctCssCatalog, ResponsePrinter {
    private final static HTMLAttributes divAttr = new HTMLAttributes();
    private final static HTMLAttributes spanAttr = new HTMLAttributes();
    private final static HTMLAttributes aAttr = new HTMLAttributes();
    static {
        divAttr.setClasses(TOOL_DIVCLASS);
        spanAttr.setClasses(CURRENT_SPANCLASS);
        aAttr.setClasses(PAGE_ACLASS);
    }
    protected AdminResponseParameters adminParameters;
    protected SctServer sctServer;
    protected SelectionManager selectionManager;
    protected int workingLangInteger;
    
    public AdminHtmlPage(AdminResponseParameters adminParameters) {
        this.adminParameters = adminParameters;
        Locale locale = adminParameters.getLocale();
        workingLangInteger = LangInteger.fromLocale(locale);
        setResourceBundle(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctadmin", locale));
        ResourceLocalisation resourceLocalisation = new BundleLocalisation(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctfb", locale));
        setCommandKeyLocalisation(resourceLocalisation);
        sctServer = adminParameters.getSctServer();
        selectionManager = adminParameters.getSelectionManager();
        addCssUrl("../css/admin.css");
        if (containsForm()) {
            HTMLAttributes formAttributes = new HTMLAttributes();
            formAttributes.setAction(".");
            formAttributes.setMethod("get");
            formAttributes.setOnSubmit("return testForm(this);");
            getCommandBoxParameters().setFormAttributes(formAttributes);
            String[] urlArray = {"../js/testform.js"};
            setJsUrlArray(urlArray);
        }
    }
    
    public abstract boolean containsForm();

    
    public static AdminHtmlPage getPage(AdminResponseParameters adminParameters) {
        switch(adminParameters.getHtmlPageType()) {
            case AdminResponseParameters.INDEX_PAGE :
                return new IndexPage(adminParameters);
            case AdminResponseParameters.ERROR_PAGE :
                return new ErrorPage(adminParameters);
            case AdminResponseParameters.STATS_PAGE :
                return new StatsPage(adminParameters);
            case AdminResponseParameters.FICHES_PAGE :
                return new FichesPage(adminParameters);
            case AdminResponseParameters.MOTSCLES_PAGE :
                return new MotsclesPage(adminParameters);
            case AdminResponseParameters.LEXIES_PAGE :
                return new LexiesPage(adminParameters);
            case AdminResponseParameters.JOURNAUX_PAGE :
                return new JournauxPage(adminParameters);
            case AdminResponseParameters.JOURNAL_PAGE :
                return new JournalPage(adminParameters);
            default :
                return null;
        }
    }
    
    protected void initResponse(HttpServletResponse response) throws IOException{
        response.setContentType("text/html;charset=UTF-8");
        initPrinter(response.getWriter());
        
    }
    
    protected void printTools() {
        startDiv(divAttr);
        printToolLink(AdminResponseParameters.INDEX_PAGE);
        printToolSeparator();
        printToolLink(AdminResponseParameters.STATS_PAGE);
        printToolSeparator();
        printToolLink(AdminResponseParameters.FICHES_PAGE);
        printToolSeparator();
        printToolLink(AdminResponseParameters.MOTSCLES_PAGE);
        printToolSeparator();
        printToolLink(AdminResponseParameters.LEXIES_PAGE);
        printToolSeparator();
        printToolLink(AdminResponseParameters.JOURNAUX_PAGE);
        endDiv();

    }
    
    protected void printMessage() {
        CommandMessage commandMessage = adminParameters.getCommandMessage();
        if (commandMessage != null) printMessage(commandMessage);
    }

    
    protected void start() {
        start(workingLangInteger);
    }
    
    private void printToolSeparator() {
        append(" — ");
    }
    
    private void printToolLink(short page) {
        String key = AdminRequest.pageToString(page);
        short currentPage =adminParameters.getHtmlPageType();
        if (currentPage == page) {
            startSpan(spanAttr);
        }
        else {
            HTMLAttributes aCopy = aAttr.copy();
            aCopy.setHref(key + ".html");
            startA(aCopy);
        }
        localize("page_" + key);
        if (currentPage == page) {
            endSpan();
        }
        else {
            endA();
        }
    }
    
    public SctServer getSctServer() {
        return sctServer;
    }
    
}
