/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.servlets.admin.*;
import fr.exemole.sctserver.admin.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.langinteger.*;

/**
 *
 * @author Vincent Calame
 */
public class FichesPage extends AdminHtmlPage {
    HTMLAttributes baseAttributes = new HTMLAttributes();
    HTMLAttributes corpusAttributes = new HTMLAttributes();
    HTMLAttributes ficheAttributes = new HTMLAttributes();
    HTMLAttributes ulAttributes = new HTMLAttributes();
    {
        baseAttributes.setClasses("d_SCT_lst_base");
        corpusAttributes.setClasses("l_SCT_lst_corpus");
        ficheAttributes.setClasses("l_SCT_lst_fiche");
        ulAttributes.setStyle("display: none;");
        ulAttributes.setClasses("u_SCT_lst_u");
    }
    ScrutariDB scrutariDB;
    DataManager dataManager;
    int langUi = LangInteger.FRENCH;
    static String[] jsUrl = {"../js/affichage.js"};
    
    public FichesPage(AdminResponseParameters adminParameters) {
        super(adminParameters);
        scrutariDB = adminParameters.getSctServer().getScrutariDB();
        dataManager = scrutariDB.getDataManager();
        addCssUrl("../css/listes.css");
        setJsUrlArray(jsUrl);
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        
        int baseCount = dataManager.getBaseCount();
        for(int i = 0; i< baseCount; i++) {
            BaseData baseData = dataManager.getBaseDataByIndex(i);
            printBaseData(baseData);
        }
        end();
    }
    
    public boolean containsForm() {
        return false;
    }
    
    
    private void printBaseData(BaseData baseData) {
        int codebase = baseData.getCodeBase();
        int corpusCount = baseData.getCorpusCount();
        String onClick = "affiche('ul_" + codebase + "');";
        baseAttributes.setOnClick(onClick);
        startDiv(baseAttributes);
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG,langUi,"?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(corpusCount));
        append("]");
        endDiv();
        
        ulAttributes.setId("ul_" + codebase);
        startUl(ulAttributes);
        for(int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataManager.getCorpusDataByCode(baseData.getCodeCorpus(i));
            printCorpusData(corpusData);
        }
        endUl();
    }
    
    private int printCorpusData(CorpusData corpusData) {
        int codecorpus = corpusData.getCodeCorpus();
        int ficheCount = corpusData.getFicheCount();
        String onClick = "affiche('ul_" + codecorpus + "');";
        corpusAttributes.setOnClick(onClick);
        startLi(corpusAttributes);
        escape(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER,langUi,"?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(ficheCount));
        append("]");
        endCode();
        endLi();
        
        ulAttributes.setId("ul_" + codecorpus);
        ulAttributes.setOnClick(onClick);
        startUl(ulAttributes);
        for(int i = 0; i < ficheCount; i++) {
            FicheData ficheData = dataManager.getFicheDataByCode(corpusData.getCodeFiche(i));
            printFicheData(ficheData);
        }
        endUl();
        return ficheCount;
    }
    
    private void printFicheData(FicheData ficheData) {
        startLi(ficheAttributes);
        escape(ficheData.getTitre());
        endLi();
    }
}
