/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.servlets.admin.*;
import fr.exemole.sctserver.conf.*;
import fr.exemole.sctserver.admin.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;
import fr.exemole.sctserver.*;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.*;

/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends AdminHtmlPage implements ConfConstants {
    private OptionsManager optionsManager;
    private ConfState confState;
    
    public IndexPage(AdminResponseParameters adminParameters) {
        super(adminParameters);
        confState = sctServer.getConfState();
        optionsManager = sctServer.getOptionsManager();
    }
    
    public boolean containsForm() {
        return true;
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        
        if (sctServer.hasUpdateProcessLog()) {
            startP();
            startSpan(ERRORMESSAGE_SPANCLASS);
            localize("info_updateprocesslog");
            space();
            startA("../logs/updateprocesslog.txt");
            append("updateprocesslog.txt");
            endA();
            endSpan();
            endP();
        }
        
        printRestartForm();
        
        printConfState();
        
        end();
    }
    
    private void printConfState() {
        appendH2("title_conf_files");
        short state = confState.getGlobalState();
        boolean printConfDetails = false;
        switch(state) {
            case ConfState.GLOBAL_CONFIO_ERROR :
                startP();
                startSpan(ERRORMESSAGE_SPANCLASS);
                localize("error_global_confio");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_UNDEFINED :
                startP();
                startSpan(ERRORMESSAGE_SPANCLASS);
                localize("error_global_confnull");
                append(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_OK :
                startUl();
                printConfFileLi(SOURCES_NAME);
                printConfFileLi(CORPUS_CLASSES_NAME);
                printConfFileLi(SEARCHLOG_FILTER_NAME);
                endUl();
                printConfDetails = true;
                break;
        }
        startCommandBox(ConfUpdateCommand.getCommandString(),ConfUpdateCommand.getCommandKey());
        endCommandBox("submit_update");
        if (printConfDetails) {
            appendH2("title_conf_details");
            printSourcesDetails();
            printCorpusClassesDetails();
            printSearchLogFilterDetails();
        }
    }
    
    private void printConfFileLi(String name) {
        startLi();
        printFileInfo(name, true);
        endLi();
    }
    
    private void printSourcesDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(SOURCES_NAME);
        startH3(h3Attributes);
        localize("title_conf_sources");
        append(" / ");
        printFileInfo(SOURCES_NAME, false);
        endH3();
        printLog(SOURCES_NAME);
        SourcesPrinter.printSourceInfoList(this);
        
    }
    
    private void printCorpusClassesDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(CORPUS_CLASSES_NAME);
        startH3(h3Attributes);
        localize("title_conf_corpusclasses");
        append(" / ");
        printFileInfo(CORPUS_CLASSES_NAME, false);
        endH3();
        printLog(CORPUS_CLASSES_NAME);
        CorpusClassesPrinter.printCorpusClasses(this);
        
    }
    
    private void printSearchLogFilterDetails() {
        HTMLAttributes h3Attributes = new HTMLAttributes();
        h3Attributes.setId(SEARCHLOG_FILTER_NAME);
        startH3(h3Attributes);
        localize("title_conf_searchlogfilter");
        append(" / ");
        printFileInfo(SEARCHLOG_FILTER_NAME, false);
        endH3();
        printLog(SEARCHLOG_FILTER_NAME);
        SearchLogFilterPrinter.printSearchLogFilter(this);
        
    }
    
    private void printRestartForm() throws IOException {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setName(RestartCommand.FORCE_PARAMNAME);
        htmlAttributes.setValue("1");
        startCommandBox(RestartCommand.getCommandString(), RestartCommand.getCommandKey());
        startP();
        addCheck(htmlAttributes, "lib_restart_force", null);
        endP();
        endCommandBox("submit_restart");
    }
    
    
    
    
    
        /*if (sctServer.hasOptionsProcessLog()) {
            startP();
            startSpan(ERRORMESSAGE_SPANCLASS);
            localize("info_optionsprocesslog");
            space();
            startA("../logs/optionsprocesslog.txt");
            append("optionsprocesslog.txt");
            endA();
            endSpan();
            endP();
        }*/
    
    
    private void printLog(String name) {
        short fileState = confState.getFileState(name);
        switch(fileState) {
            case FILE_STATE_UNKNOWN :
            case FILE_MISSING :
            case FILE_OK :
                return;
        }
        startP();
        localize("title_log");
        endP();
        HTMLAttributes preAttributes = new HTMLAttributes();
        preAttributes.addClass(LOG_PRECLASS);
        startPre(preAttributes);
        escape(sctServer.getLog(name), true);
        endPre();
    }
            
    
    private void printFileInfo(String name, boolean withLink) {
        String[] array = getKeys(confState.getFileState(name));
        HTMLAttributes spanAttributes = new HTMLAttributes();
        spanAttributes.addClass(array[1]);
        if (withLink) startA("#" + name);
        append(name);
        append(".xml");
        if (withLink) endA();
        append(" (");
        startSpan(spanAttributes);
        localize("file_" + array[0]);
        endSpan();
        append(")");
    }
    
    private String[] getKeys(short fileState) {
        String spanClass = null;
        String key = null;
        switch(fileState) {
            case FILE_STATE_UNKNOWN :
                key = "state_unknown";
                spanClass = NEUTRAL_SPANCLASS;
                break;
            case FILE_MISSING :
                key = "missing";
                spanClass = NEUTRAL_SPANCLASS;
                break;
            case FILE_XML_MALFORMED :
                key = "xml_malformed";
                spanClass = ERROR_SPANCLASS;
                break;
            case FILE_XML_ERROR :
                key = "xml_error";
                spanClass = ERROR_SPANCLASS;
                break;
            case FILE_XML_WARNING :
                key = "xml_warning";
                spanClass = WARNING_SPANCLASS;
                break;
            case FILE_OK :
                key = "ok";
                spanClass = OK_SPANCLASS;
                break;
            default :
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
        String[] array = new String[2];
        array[0] = key;
        array[1] = spanClass;
        return array;
    }
    
}
