/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.servlets.admin.*;
import fr.exemole.sctserver.admin.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import fr.exemole.sctserver.*;


/**
 *
 * @author Vincent Calame
 */
public class JournalPage extends AdminHtmlPage {
    private int dateInteger;
    private HTMLAttributes tableAttributes;
    
    public JournalPage(AdminResponseParameters adminParameters) {
        super(adminParameters);
        dateInteger = adminParameters.getDateInteger();
        tableAttributes = new HTMLAttributes();
        tableAttributes.addClass(TABLE_STATS);
    }
    
     public void printResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        startH1();
        append(IntDate.toString(dateInteger));
        endH1();
        SearchLog.DayLog[] dayLogArray = sctServer.getSearchLog().getDayLogArray(dateInteger);
        for(int i = 0; i < dayLogArray.length; i++) {
            printDayLog(dayLogArray[i]);
        }
        end();
    }
     
     private void printDayLog(SearchLog.DayLog dayLog) {
         SearchLog.DayStats dayStats = dayLog.getDayStats();
         startH2();
         append(String.valueOf(IntDate.getJour(dayLog.getDateInteger())));
         space();
         startSmall();
         append("(");
         appendInt(dayStats.getFicheCount());
         append(" / ");
         appendInt(dayStats.getMotcleCount());
         append(")");
         endSmall();
         endH2();
         startTable(tableAttributes);
         SearchLog.QueryLog[] queryLogArray = dayLog.getQueryLogArray();
         for(int i = 0; i < queryLogArray.length; i++) {
            printQueryLog(queryLogArray[i]);
        }
         endTable();
     }
     
     private void printQueryLog(SearchLog.QueryLog queryLog) {
         startRow();
         startCell();
         append(queryLog.getRemoteAddress());
         endCell();
         startCell();
         escape(queryLog.getSearchSequence());
         endCell();
         startCell();
         append(queryLog.getSite());
         endCell();
         startCell();
         append(queryLog.getReferrer());
         endCell();
         startCell();
         appendInt(queryLog.getFicheCount());
         endCell();
         startCell();
         appendInt(queryLog.getFicheMaximum());
         endCell();
         startCell();
         appendInt(queryLog.getCompleteMotcleCount());
         endCell();
         startCell();
         appendInt(queryLog.getAllMotcleCount());
         endCell();
         endRow();
     }
     
     private void appendInt(int val) {
         if (val < 0) return;
         append(String.valueOf(val));
     }
    
     public boolean containsForm() {
        return false;
    }
    
}
