/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.servlets.admin.*;
import fr.exemole.sctserver.admin.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.servlets.selection.*;

/**
 *
 * @author Vincent Calame
 */
public class LexiesPage extends AdminHtmlPage {
    private int langInteger;
    private int[] codethesaurusArray;
    
    public LexiesPage(AdminResponseParameters adminParameters) {
        super(adminParameters);
        langInteger = adminParameters.getLangInteger();
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        
        if (langInteger == 0) {
            printIndexResponse(response);
        }
        else {
            printLangResponse(response);
        }
        
        end();
    }
    
    public boolean containsForm() {
        return false;
    }
    
    public void printIndexResponse(HttpServletResponse response) throws IOException {
        int[] langArray = sctServer.getScrutariDB().getLangStats().getAllMotcleLangArray();
        int length = langArray.length;
        Set set = selectionManager.getNameSet();
        for (int i = 0; i < length; i++) {
            String langString = LangInteger.toString(langArray[i]);
            startP();
            startA("lexies_" + langString + ".html");
            append(langString);
            endA();
            append(" (");
            append("ods : ");
            startA("lexies_" + langString + ".ods");
            append("_all");
            endA();
            for(Iterator it = set.iterator(); it.hasNext();) {
                String name = (String) it.next();
                printLien(langString, name);
            }
            append(")");
            endP();
        }
    }
    
    private void printLien(String langString, String name) {
        append(" - ");
        startA("lexies_" + langString + "-" + name + ".ods");
        append(name);
        endA();
    }
    
    public void printLangResponse(HttpServletResponse response) throws IOException {
        codethesaurusArray = sctServer.getScrutariDB().getDataManager().getCodethesaurusArray();
        List list = sctServer.getScrutariDB().getLexieManager().getLexieQuery().getScrutariLexieUnitList(langInteger);
        startUl();
        for(Iterator it = list.iterator(); it.hasNext();) {
            ScrutariLexieUnit lexieUnit = (ScrutariLexieUnit) it.next();
            if (lexieUnit.hasMotcleAndLexieLink()) {
                printLexieUnit(lexieUnit);
            }
        }
        endUl();
    }
    
    private void printLexieUnit(ScrutariLexieUnit lexieUnit) throws IOException {
        DataManager dataManager = sctServer.getScrutariDB().getDataManager();
        LexieManager2 lexieManager = sctServer.getScrutariDB().getLexieManager();
        startLi();
        startP();
        escape(lexieUnit.getCanonicalLexie());
        endP();
        startUl();
        long lexieId = lexieUnit.getLexieId();
        int length = codethesaurusArray.length;
        for(int i = 0; i < length; i++) {
            int codethesaurus = codethesaurusArray[i];
            BestPosition[] array = lexieManager.getLexieQuery().getBestPositionArray(lexieId,codethesaurus);
            if (array != null) {
                for(int j = 0; j < array.length; j++) {
                    int codemotcle = array[j].getCode();
                    MotcleData motcleData =dataManager.getMotcleDataByCode(codemotcle);
                    startLi();
                    Libelle libelle = motcleData.getLibelleByLang(langInteger);
                    escape(libelle.getLibelleString());
                    endLi();
                }
            }
        }
        endUl();
        endLi();
    }
    
}
