/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.servlets.admin.*;
import fr.exemole.sctserver.admin.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class MotsclesPage extends AdminHtmlPage {
    HTMLAttributes theadAttributes = new HTMLAttributes();
    HTMLAttributes baseTBodyAttributes = new HTMLAttributes();
    HTMLAttributes baseCellAttributes = new HTMLAttributes();
    HTMLAttributes thesaurusTBodyAttributes = new HTMLAttributes();
    HTMLAttributes thesaurusCellAttributes = new HTMLAttributes();
    HTMLAttributes motcleTBodyAttributes = new HTMLAttributes();

    {
       /* baseAttributes.setClasses("d_SCT_lst_base");
        thesaurusAttributes.setClasses("l_SCT_lst_corpus");
        ficheAttributes.setClasses("l_SCT_lst_fiche");
        
        ulAttributes.setClasses("u_SCT_lst_u");*/
        thesaurusCellAttributes.setClasses("c_SCT_lst_thesaurus");
        motcleTBodyAttributes.setStyle("display: none;");
    }
    ScrutariDB scrutariDB;
    DataManager dataManager;
    static String[] jsUrl = {"../js/affichage.js"};
    int[] motcleLangArray;
    
    public MotsclesPage(AdminResponseParameters adminParameters) {
        super(adminParameters);
        scrutariDB = adminParameters.getSctServer().getScrutariDB();
        dataManager = scrutariDB.getDataManager();
        addCssUrl("../css/listes.css");
        setJsUrlArray(jsUrl);
        motcleLangArray = scrutariDB.getLangStats().getAllMotcleLangArray();
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        
        startP();
        append("ods : ");
        startA("motscles.ods");
        append("_all");
        endA();
        for(Iterator it = selectionManager.getNameSet().iterator(); it.hasNext();) {
            String name = (String) it.next();
            printLien(name);
        }
        endP();
        
        startTable();
        startTHead(theadAttributes);
        startRow();
        startCell();
        append("&nbsp;");
        endCell();
        startCell();
        append("&nbsp;");
        endCell();
     
        int length = motcleLangArray.length;
        for(int i = 0; i < length;i++) {
            startCell();
            append(LangInteger.toString(motcleLangArray[i]));
            endCell();
        }
        endRow();
        endTHead();
       
        baseCellAttributes.setColSpan(2 + length);
        thesaurusCellAttributes.setColSpan(1 + length);
        
        int baseCount = dataManager.getBaseCount();
        for(int i = 0; i< baseCount; i++) {
            BaseData baseData = dataManager.getBaseDataByIndex(i);
            printBaseData(baseData);
        }
        endTable();
        end();
    }
    
    public boolean containsForm() {
        return false;
    }
    
    private void printLien(String name) {
        append(" - ");
        startA("motscles-" + name + ".ods");
        append(name);
        endA();
    }
    
    private void printBaseData(BaseData baseData) {
        int codebase = baseData.getCodeBase();
        int thesaurusCount = baseData.getThesaurusCount();
        /*String onClick = "affiche('ul_" + codebase + "');";
        baseAttributes.setOnClick(onClick);*/
        startTBody(baseTBodyAttributes);
        startRow();
        startCell(baseCellAttributes);
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG,workingLangInteger,"?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(thesaurusCount));
        append("]");
        endCode();
        endCell();
        endRow();
        endTBody();
        
        for(int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataManager.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
            printThesaurusData(thesaurusData);
        }

    }
    
    private void printThesaurusData(ThesaurusData thesaurusData) {
        int codethesaurus = thesaurusData.getCodeThesaurus();
        int motcleCount = thesaurusData.getMotcleCount();
        String onClick = "afficheRowGroup('tbody_" + codethesaurus + "');";
        thesaurusCellAttributes.setOnClick(onClick);
        startTBody(thesaurusTBodyAttributes);
        startRow();
        appendEmptyCell();
        startCell(thesaurusCellAttributes);
        escape(LibelleUtils.getLibelleString(thesaurusData, workingLangInteger,"?"));
        append(" ");
        startCode();
        append("[");
        append(String.valueOf(motcleCount));
        append("]");
        endCode();
        endCell();
        
        endRow();
        endTBody();
        
        motcleTBodyAttributes.setId("tbody_" + codethesaurus);
        startTBody(motcleTBodyAttributes);
        for(int i = 0; i < motcleCount; i++) {
            printMotcleData(dataManager.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
        }
        endTBody();
        
    }
    
    private void printMotcleData(MotcleData motcleData) {
        startRow();
        appendEmptyCell();
        appendEmptyCell();
        int length = motcleLangArray.length;
        for(int i = 0; i < length;i++) {
            startCell();
            Libelle libelle = motcleData.getLibelleByLang(motcleLangArray[i]);
            if (libelle != null) escape(libelle.getLibelleString());
            endCell();
        }
        endRow();
    }
    
}
