/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.servlets.admin.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.util.langinteger.*;

/**
 *
 * @author Vincent Calame
 */
public class StatsPage extends AdminHtmlPage {
    ScrutariDB scrutariDB;
    DataManager dataManager;
    int langUi = LangInteger.FRENCH;
    
    public StatsPage(AdminResponseParameters adminParameters) {
        super(adminParameters);
        scrutariDB = adminParameters.getSctServer().getScrutariDB();
        dataManager = scrutariDB.getDataManager();
        adminParameters.getSelectionManager().update();
        System.gc();
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        
        CountStats countStats = scrutariDB.getCountStats();
        
        appendH2("title_stats");
        startTable();
        printRow("lib_stats_corpus",countStats.getCorpusCount());
        printRow("lib_stats_thesaurus",countStats.getThesaurusCount());
        printRow("lib_stats_fiches",countStats.getFicheCount());
        printRow("lib_stats_motscles",countStats.getMotcleCount());
        printRow("lib_stats_lexie",countStats.getLexieCount());
        printRow("lib_stats_indexation",countStats.getIndexationCount());
        endTable();
        
        appendH2("title_memory");
        Runtime run = Runtime.getRuntime();
        
        startTable();
        printRow("lib_memory_free",(int) run.freeMemory()/(1024*1024));
        printRow("lib_memory_total",(int) run.totalMemory()/(1024*1024));
        printRow("lib_memory_max",(int) run.maxMemory()/(1024*1024));
        endTable();
        
        printCorpusURIList();
        
        end();
    }
    
    public boolean containsForm() {
        return false;
    }
    
    private void printRow(String key, int count) {
        startRow();
        printLibCells(key, false);
        startCell();
        append(String.valueOf(count));
        endCell();
        endRow();
    }
    
    private void printCorpusURIList() {
        appendH2("title_corpus_list");
        int baseCount = dataManager.getBaseCount();
        startUl();
        for(int i = 0; i< baseCount; i++) {
            BaseData baseData = dataManager.getBaseDataByIndex(i);
            printBaseData(baseData);
        }
        endUl();
    }
    
    private void printBaseData(BaseData baseData) {
        int codebase = baseData.getCodeBase();
        int corpusCount = baseData.getCorpusCount();
        startLi();
        startP();
        escape(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG,langUi,"?"));
        endP();
        startUl();
        for(int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataManager.getCorpusDataByCode(baseData.getCodeCorpus(i));
            printCorpusData(corpusData);
        }
        endUl();
        endLi();
    }
    
    private void printCorpusData(CorpusData corpusData) {
        startLi();
        startP();
        escape(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER,langUi,"?"));
        append(" ");
        startCode();
        append("[");
        append(corpusData.getCorpusURI().toString());
        append("]");
        endCode();
        endP();
        endLi();
    }
    
}
