/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.servlets.admin.htmlpages.*;
import java.util.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.html.*;
import net.mapeadores.scrutari.db.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.servlets.resources.theme.*;

/**
 *
 * @author Vincent Calame
 */
public class CorpusClassesPrinter implements SctCssCatalog {
    
    private CorpusClassesPrinter() {
    }
    
    public static void printCorpusClasses(AdminHtmlPage page) {
        OptionsManager optionsManager = page.getSctServer().getOptionsManager();
        int count = optionsManager.getCorpusClasseCount();
        if (count == 0) {
            page.startP();
            page.startEm();
            page.localize("info_corpusclasses_none");
            page.endEm();
            page.endP();
        }
        else {
            page.startUl();
            for(int i = 0; i < count; i++) {
                CorpusClasse corpusClasse = optionsManager.getCorpusClasse(i);
                printCorpusClasseLi(page, corpusClasse);
            }
            page.endUl();
        }
        
    }
    
    private static void printCorpusClasseLi(AdminHtmlPage page, CorpusClasse corpusClasse) {
        HTMLAttributes errorAttributes = new HTMLAttributes();
        errorAttributes.addClass(ERROR_SPANCLASS);
        
        DataManager dataManager = page.getSctServer().getScrutariDB().getDataManager();
        String name = corpusClasse.getName();
        if (name == null) name="_default";
        page.startLi();
        page.startP();
        page.startCode();
        page.append("[");
        page.append(String.valueOf(corpusClasse.getClasseValue()));
        page.append(" / ");
        page.append(name);
        page.append("]");
        page.endCode();
        page.append(" - ");
        boolean premier = true;
        for (Iterator it = corpusClasse.getLibelleList().iterator(); it.hasNext();) {
            if (premier) premier = false;
            else page.append(" / ");
            Libelle libelle = (Libelle) it.next();
            page.escape(libelle.getLibelleString());
        }
        page.endP();
        int uriCount = corpusClasse.getCorpusURICount();
        page.startUl();
        if (uriCount == 0) {
            page.startLi();
            page.startP();
            page.startEm();
            if (name.equals("_default")) {
                page.localize("info_corpusuri_default");
            }
            else {
                page.localize("info_corpusuri_none");
            }
            page.endEm();
            page.endP();
            page.endLi();
        }
        else {
            for(int i = 0; i < uriCount; i++) {
                CorpusURI corpusURI = corpusClasse.getCorpusURI(i);
                int codecorpus = dataManager.getScrutariURICoder().getCode(corpusURI);
                boolean isHere = (codecorpus > 0);
                page.startLi();
                page.startP();
                if (!isHere) page.startSpan(errorAttributes);
                page.startCode();
                page.append("[");
                page.append(corpusURI.toString());
                page.append("]");
                page.endCode();
                if (!isHere) page.endSpan();
                page.endP();
                page.endLi();
            }
        }
        page.endUl();
        page.endLi();
    }
    
}
