/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.servlets.admin.htmlpages.*;
import java.util.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.html.*;
import net.mapeadores.scrutari.db.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.servlets.resources.theme.*;
import net.mapeadores.util.conditions.*;


/**
 *
 * @author Vincent Calame
 */
public class SearchLogFilterPrinter implements SctCssCatalog {
    
    private SearchLogFilterPrinter() {
    }
    
     public static void printSearchLogFilter(AdminHtmlPage page) {
         OptionsManager optionsManager = page.getSctServer().getOptionsManager();
         int count = optionsManager.getSearchLogDenyCount();
         if (count == 0) {
            page.startP();
            page.startEm();
            page.localize("info_searchlogfilter_none");
            page.endEm();
            page.endP();
        }
         else {
             HTMLAttributes tableAttributes = new HTMLAttributes();
             tableAttributes.addClass(TABLE_SEARCHLOGFILTER);
             page.startTable(tableAttributes);
             page.startRow();
             page.startCell();
             page.append("IP");
             page.endCell();
             page.startCell();
             page.append("SITE");
             page.endCell();
             page.startCell();
             page.append("REF");
             page.endCell();
             page.endRow();
             for(int i = 0; i < count; i++) {
                 printSearchLogDenyRow(page, optionsManager.getSearchLogDeny(i));
             }
             page.endTable();
         }
     }
     
     private static void printSearchLogDenyRow(AdminHtmlPage page, SearchLogDeny searchLogDeny) {
         page.startRow();
         printContentCondition(page, searchLogDeny.getContentCondition(searchLogDeny.IP_DENY));
         printContentCondition(page, searchLogDeny.getContentCondition(SearchLogDeny.SITE_DENY));
         printContentCondition(page, searchLogDeny.getContentCondition(SearchLogDeny.REF_DENY));
         page.endRow();
     }
     
     private static void printContentCondition(AdminHtmlPage page, ContentCondition contentCondition) {
         if (contentCondition == null) {
             page.appendEmptyCell();
         }
         else {
             page.startCell();
             page.escape(Conditions.contentConditionToString(contentCondition));
             page.endCell();
         }
     }
    
}
