/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.servlets.admin.htmlpages.*;
import fr.exemole.sctserver.servlets.resources.theme.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.util.html.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;

/**
 *
 * @author Vincent Calame
 */
public class SourcesPrinter implements SctCssCatalog {
    
    private SourcesPrinter() {
    }
    
    public static void printSourceInfoList(AdminHtmlPage page) {
        SourceInfoManager scrutariInfoManager = page.getSctServer().getScrutariInfoManager();
        int count = scrutariInfoManager.getSourceCount();
        if (count == 0) {
            HTMLAttributes infoAttributes = new HTMLAttributes();
            infoAttributes.setClasses(INFO_SPANCLASS);
            page.startP();
            page.startSpan(infoAttributes);
            page.localize("info_list_none");
            page.endSpan();
            page.endP();
            return;
        }
        page.startTable();
        for(int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = scrutariInfoManager.getSourceInfo(i);
            page.startRow();
            page.startCell();
            page.append(scrutariInfo.getUrl().toString());
            page.endCell();
            
            short state = scrutariInfo.getState();
            page.startCell();
            page.startSpan(getSpanAttributes(state));
            page.localize(getLocKey(state));
            page.endSpan();
            page.endCell();
            page.startCell();
            page.append(IntDate.toISOString(scrutariInfo.getLastUpdate()));
            page.endCell();
            page.endRow();
        }
        page.endTable();
    }
    
    private static HTMLAttributes  getSpanAttributes(short state) {
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        switch(state) {
            case ScrutariInfo.OK_STATE :
                htmlAttributes.setClasses(HTMLPage.DONEMESSAGE_SPANCLASS);
                break;
            case ScrutariInfo.UNKNOWN_STATE :
            case  ScrutariInfo.INFO_NOCONNECTION_STATE :
            case ScrutariInfo.INFO_BROKENURL_STATE :
            case ScrutariInfo.INFO_DOWNLOADERROR_STATE :
            case ScrutariInfo.INFO_NOTXML_STATE :
            case ScrutariInfo.INFO_WRONGXML_STATE :
            case ScrutariInfo.DATA_NOCONNECTION_STATE :
            case ScrutariInfo.DATA_BROKENURL_STATE :
            case ScrutariInfo.DATA_DOWNLOADERROR_STATE :
            case ScrutariInfo.DATA_NOTXML_STATE :
                htmlAttributes.setClasses(HTMLPage.ERRORMESSAGE_SPANCLASS);
                break;
            default :
                throw new SwitchException("state = " + state);
        }
        return htmlAttributes;
    }
    
    private static String getLocKey(short state) {
        switch(state) {
            case ScrutariInfo.OK_STATE :
                return "state_ok";
            case ScrutariInfo.UNKNOWN_STATE :
                return "state_unknown";
            case  ScrutariInfo.INFO_NOCONNECTION_STATE :
                return "state_info_noconnection";
            case ScrutariInfo.INFO_BROKENURL_STATE :
                return "state_info_brokenurl";
            case ScrutariInfo.INFO_DOWNLOADERROR_STATE :
                return "state_info_downloaderror";
            case ScrutariInfo.INFO_NOTXML_STATE :
                return "state_info_notxml";
            case ScrutariInfo.INFO_WRONGXML_STATE :
                return "state_info_wrongxml";
            case ScrutariInfo.DATA_NOCONNECTION_STATE :
                return "state_data_noconnection";
            case ScrutariInfo.DATA_BROKENURL_STATE :
                return "state_data_brokenurl";
            case ScrutariInfo.DATA_DOWNLOADERROR_STATE :
                return "state_data_downloaderror";
            case ScrutariInfo.DATA_NOTXML_STATE :
                return "state_data_notxml";
            default :
                throw new SwitchException("state = " + state);
        }
    }
    
}
