/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.admin.xmldata;

import fr.exemole.sctserver.servlets.admin.*;
import net.mapeadores.util.xml.*;
import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.servlets.*;
import net.mapeadores.util.localisation.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class AdminXmlData implements ResponsePrinter {
    protected AdminResponseParameters adminParameters;
    private ResourceLocalisation messageLocalisation;

    
    public AdminXmlData(AdminResponseParameters adminParameters) {
        this.adminParameters = adminParameters;
        Locale locale = adminParameters.getLocale();
        messageLocalisation = new BundleLocalisation(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctadmin", locale));
    }
    
    public static AdminXmlData getXmlData(AdminResponseParameters adminParameters) {
        switch(adminParameters.getXmlDataType()) {
            case AdminResponseParameters.TEST_XMLDATA :
                return new TestData(adminParameters);
                default :
                return null;
        }
    }
    
    protected XMLDestination initResponse(HttpServletResponse response) throws IOException{
        response.setContentType("text/xml;charset=UTF-8");
        return XMLDestinationFactory.newInstance(response.getWriter());
    }
    
    public String localize(ResourceItem resourceItem) {
        return messageLocalisation.toString(resourceItem);
    }
    
    public String localize(String resourceKey, Object value) {
        return messageLocalisation.toString(resourceKey,value);
    }
    
}
