/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.explore.xmldata;

import fr.exemole.sctserver.servlets.explore.*;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.sctserver.*;
import net.mapeadores.util.xml.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.scrutari.db.*;
import java.net.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.primitives.*;

/**
 *
 * @author Vincent Calame
 */
public class BaseListXmlData extends ExploreXmlData {
    
    public BaseListXmlData(ExploreResponseParameters exploreParameters) {
        super(exploreParameters);
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        BaseListWriter baseListWriter = new BaseListWriter(xmlWriter);
        baseListWriter.appendBaseList(sctServer);
        closeResponse();
     }
    
    
    static class BaseListWriter extends UtilXMLWriterPart {
        
        BaseListWriter(XMLWriter xmlWriter) {
            super(xmlWriter);
        }
        
        public void appendBaseList(SctServer sctServer) throws IOException {
            SourceInfoManager infoManager = sctServer.getScrutariInfoManager();
            DataManager dataManager = sctServer.getScrutariDB().getDataManager();
            ScrutariURICoder coder = dataManager.getScrutariURICoder();
            openTag("base-list");
            int count = infoManager.getSourceCount();
            for(int i = 0; i < count; i++) {
                ScrutariInfo scrutariInfo = infoManager.getSourceInfo(i);
                BaseURI baseURI = scrutariInfo.getBaseURI();
                if (baseURI != null) {
                    int code = coder.getCode(baseURI);
                    if (code > 0) {
                        BaseData baseData = dataManager.getBaseDataByCode(code);
                        startOpenTag("base");
                        addAttribute("uri", baseURI.toString());
                        endOpenTag();
                        addSimpleElement("info-url", getUrlString(scrutariInfo.getUrl()));
                        addSimpleElement("last-update", IntDate.toISOString(scrutariInfo.getLastUpdate()));
                        addSimpleElement("scrutaridata-url", getUrlString(scrutariInfo.getScrutariDataUrl()));
                        addSimpleElement("authority-uuid", baseURI.getAuthorityUUID());
                        addSimpleElement("base-name", baseURI.getBaseName());
                        
                        int langCount = baseData.getLangDataCount();
                        openTag("intitule-short");
                        for(int j = 0; j < langCount; j++) {
                            BaseData.BaseLangData baseLangData = baseData.getLangDataByIndex(j);
                            int langInteger = baseLangData.getLangInteger();
                            String lib = baseLangData.getLibelleString(BaseData.INTITULE_SHORT);
                            if (lib != null) {
                               addLibElement(langInteger, lib);
                            }
                        }
                        closeTag("intitule-short");
                        openTag("intitule-long");
                        for(int j = 0; j < langCount; j++) {
                            BaseData.BaseLangData baseLangData = baseData.getLangDataByIndex(j);
                            int langInteger = baseLangData.getLangInteger();
                            String lib = baseLangData.getLibelleString(BaseData.INTITULE_LONG);
                            if (lib != null) {
                               addLibElement(langInteger, lib);
                            }
                        }
                        closeTag("intitule-long");
                        closeTag("base");
                    }
                }
            }
                      
            closeTag("base-list");
        }
    }
    
    private static String getUrlString(URL url) {
        if (url == null) return null;
        return url.toString();
    }
    
}
