/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.explore.xmldata;

import fr.exemole.sctserver.servlets.explore.*;
import net.mapeadores.util.html.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.localisation.*;
import fr.exemole.sctserver.servlets.resources.theme.*;
import net.mapeadores.util.logging.*;
import fr.exemole.sctserver.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.sctserver.servlets.selection.*;
import net.mapeadores.util.xml.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class ExploreXmlData implements ResponsePrinter {
    protected ExploreResponseParameters exploreParameters;
    protected SctServer sctServer;
    protected XMLWriter xmlWriter;
    private PrintWriter pw;
    
    public ExploreXmlData(ExploreResponseParameters exploreParameters) {
        this.exploreParameters = exploreParameters;
        sctServer = exploreParameters.getSctServer();
    }
    
    public static ExploreXmlData getInstance(ExploreResponseParameters exploreParameters) {
        switch(exploreParameters.getXmlDataType()) {
            case ExploreResponseParameters.BASE_LIST_XMLDATA :
                return new BaseListXmlData(exploreParameters);
            case ExploreResponseParameters.LOG_LIST_XMLDATA :
                return new LogListXmlData(exploreParameters);
            case ExploreResponseParameters.MONTH_LOG_XMLDATA :
                return new MonthLogXmlData(exploreParameters);
            default :
                throw new SwitchException("xmlDatatype = " + exploreParameters.getXmlDataType());
        }
    }
    
    protected void initResponse(HttpServletResponse response) throws IOException{
        response.setContentType("text/xml;charset=UTF-8");
        pw = response.getWriter();
        XMLDestination xmlDestination = XMLDestinationFactory.newInstance(pw);
        DefaultXMLWriter defaultXmlWriter = new DefaultXMLWriter();
        defaultXmlWriter.setXmlDestination(xmlDestination);
        defaultXmlWriter.setIndentLength(0);
        defaultXmlWriter.appendXMLDeclaration();
        xmlWriter = defaultXmlWriter;
    }
    
    protected void closeResponse() {
        pw.close();
    }
    
}
