/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.explore.xmldata;

import fr.exemole.sctserver.servlets.explore.*;
import net.mapeadores.util.html.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import net.mapeadores.util.localisation.*;
import fr.exemole.sctserver.servlets.resources.theme.*;
import net.mapeadores.util.logging.*;
import fr.exemole.sctserver.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.sctserver.servlets.selection.*;
import net.mapeadores.util.xml.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.scrutari.db.*;
import java.net.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.primitives.*;

/**
 *
 * @author Vincent Calame
 */
public class LogXmlWriter extends UtilXMLWriterPart {
    private SctServer sctServer;
    
    public LogXmlWriter(XMLWriter xmlWriter, SctServer sctServer) {
        super(xmlWriter);
        this.sctServer = sctServer;
    }
    
    public void appendLogList() throws IOException {
        openTag("log-list");
        int[] moisArray = sctServer.getSearchLog().getMonthIntegerArray();
        for(int i = 0; i < moisArray.length; i++) {
            int mois = moisArray[i];
            startLog(mois);
            closeEmptyTag();
        }
        closeTag("log-list");
    }
    
    private void startLog(int mois) throws IOException{
        startOpenTag("log");
        int annee = IntDate.getAnnee(mois);
        addAttribute("y", String.valueOf(annee));
        int m = IntDate.getMois(mois);
        String s = String.valueOf(m);
        if (m < 10) s = "0" + s;
        addAttribute("m",s);
        addAttribute("c", IntDate.toString(mois));
    }
    
    public void appendLog(int mois) throws IOException {
        startLog(mois);
        endOpenTag();
        SearchLog.DayLog[] dayLogArray = sctServer.getSearchLog().getDayLogArray(mois);
        for(int i = 0; i < dayLogArray.length; i++) {
            appendDayLog(dayLogArray[i]);
        }
        closeTag("log");
    }
    
    
    public void appendDayLog(SearchLog.DayLog dayLog) throws IOException {
        IpMap ipMap = new IpMap();
        int jour = IntDate.getJour(dayLog.getDateInteger());
        String n = String.valueOf(jour);
        String s = (jour < 10)?"0"+n:n;
        SearchLog.QueryLog[] queryLogArray = dayLog.getQueryLogArray();
        for(int i = 0; i < queryLogArray.length; i++) {
            ipMap.addQueryLog(queryLogArray[i]);
        }
        startOpenTag("day");
        addAttribute("n", n);
        addAttribute("s", s);
        SearchLog.DayStats dayStats = dayLog.getDayStats();
        addIntAttribute("fiche",dayStats.getFicheCount());
        addIntAttribute("motcle",dayStats.getMotcleCount());
        addIntAttribute("corpus",dayStats.getCorpusCount());
        addIntAttribute("thesaurus",dayStats.getThesaurusCount());
        addIntAttribute("base",dayStats.getBaseCount());
        addIntAttribute("lexie",dayStats.getLexieCount());
        addIntAttribute("indexation",dayStats.getIndexationCount());
        endOpenTag();
        ipMap.append();
        closeTag("day");
    }
    
    private void appendQueryLog(SearchLog.QueryLog queryLog) throws IOException {
        startOpenTag("q");
        addAttribute("seq", queryLog.getSearchSequence());
        addAttribute("site", queryLog.getSite());
        addAttribute("ref", queryLog.getReferrer());
        addIntAttribute("fc", queryLog.getFicheCount());
        addIntAttribute("fm", queryLog.getFicheMaximum());
        addIntAttribute("cmc", queryLog.getCompleteMotcleCount());
        addIntAttribute("amc", queryLog.getAllMotcleCount());
        closeEmptyTag();
    }
    
    private void addIntAttribute(String name, int val) throws IOException{
        if (val < 0) return;
        addAttribute(name, val);
    }
    
    private class IpMap {
        Map<String, List<SearchLog.QueryLog>> map = new LinkedHashMap<String, List<SearchLog.QueryLog>>();
        
        void addQueryLog(SearchLog.QueryLog queryLog) {
            String ip = queryLog.getRemoteAddress();
            List<SearchLog.QueryLog> list = map.get(ip);
            if (list == null) {
                list = new ArrayList<SearchLog.QueryLog>();
                map.put(ip, list);
            }
            list.add(queryLog);
        }
        
        void append() throws IOException {
            for(List<SearchLog.QueryLog> list : map.values()) {
                openTag("ip");
                for(SearchLog.QueryLog queryLog : list) {
                    appendQueryLog(queryLog);
                }
                closeTag("ip");
            }
        }

    }
    
}
