/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.global.htmlpages;

import fr.exemole.sctserver.servlets.explore.*;
import net.mapeadores.util.html.*;
import javax.servlet.http.*;
import java.io.*;
import fr.exemole.sctserver.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.servlets.*;
import fr.exemole.sctserver.servlets.global.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class GlobalHtmlPage extends HTMLPage implements ResponsePrinter {
    protected GlobalResponseParameters globalResponseParameters;
    protected int workingLangInteger = LangInteger.FRENCH;
    
    public GlobalHtmlPage(GlobalResponseParameters globalResponseParameters) {
        this.globalResponseParameters = globalResponseParameters;
        Locale locale = globalResponseParameters.getLocale();
        setResourceBundle(ResourceBundle.getBundle("fr.exemole.sctserver.servlets.resources.l10n.sctadmin", locale));
    }
    
    public static GlobalHtmlPage getPage(GlobalResponseParameters globalParameters) {
        switch(globalParameters.getHtmlPageType()) {
            case GlobalResponseParameters.INDEX_PAGE :
                return new IndexPage(globalParameters);
            case GlobalResponseParameters.LIST_PAGE :
                return new ListPage(globalParameters);
            default :
                return null;
        }
    }
    
    protected void initResponse(HttpServletResponse response) throws IOException{
        response.setContentType("text/html;charset=UTF-8");
        initPrinter(response.getWriter());
        
    }
    
    public void start() {
        setIconPng("icon.png");
        addCssUrl("global.css");
        setTitle("Scrutari");
        start(workingLangInteger);
        HTMLAttributes divAttributes = new HTMLAttributes();
        divAttributes.setId("BLOC_GLOBAL");
        startDiv(divAttributes);
        divAttributes.setId("BLOC_LOGO");
        startDiv(divAttributes);
        HTMLAttributes htmlAttributes = new HTMLAttributes();
        htmlAttributes.setSrc("logo.png");
        appendImg(htmlAttributes);
        endDiv();
        divAttributes.setId("BLOC_CORPS");
        startDiv(divAttributes);
    }
    
    public void end() {
        endDiv();
        endDiv();
        super.end();
    }
    
}
