/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.servlets.selection;

import java.io.*;
import java.util.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class SelectionManager {
    private File rootDirectory;
    private Map<String,SelectionUnit> selectionUnitMap = new LinkedHashMap<String,SelectionUnit>();
    
    public SelectionManager(File rootDirectory) {
        this.rootDirectory = rootDirectory;
        update();
    }
    
    public void clear() {
        selectionUnitMap.clear();
    }
    
    public void update() {
        clear();
        if ((!rootDirectory.exists()) || (!rootDirectory.isDirectory())) {
            return;
        }
        File[] fichiers = rootDirectory.listFiles();
        for(int i = 0; i < fichiers.length; i++) {
            File f = fichiers[i];
            if (f.isDirectory()) continue;
            String name = f.getName();
            if (!name.endsWith(".txt")) continue;
            String sub = name.substring(0, name.length() -4);
            if (sub.length() == 0) continue;
            if (!StringUtils.isTechnicalName(sub, true)) continue;
            try {
                InputStreamReader inputStreamReader = new InputStreamReader(new FileInputStream(f),"UTF-8");
                SelectionUnit selectionUnit = SelectionUnit.read(sub, inputStreamReader);
                addSelectionUnit(selectionUnit);
                inputStreamReader.close();
            } catch(IOException ioe) {
                
            }
        }
    }
    
    public int getCount() {
        return selectionUnitMap.size();
    }
    
    public Set getNameSet() {
        return selectionUnitMap.keySet();
    }
    
    public SelectionUnit getSelectionUnit(String name) {
        return selectionUnitMap.get(name);
    }
    
    private void addSelectionUnit(SelectionUnit selectionUnit) {
        selectionUnitMap.put(selectionUnit.getName(),selectionUnit);
    }
    
}
