/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.db.stats.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.scrutari.data.*;

/**
 *
 * @author Vincent Calame
 */
public class CountStats {
    private int ficheCount;
    private int motcleCount;
    private int thesaurusCount;
    private int corpusCount;
    private int baseCount;
    private int lexieCount;
    private int indexationCount;
    private LinkedIntHashMap countStatsMap = new LinkedIntHashMap();
    
    public CountStats() {
        
    }
    
    public int getCorpusCount() {
        return corpusCount;
    }
    
    public int getThesaurusCount() {
        return thesaurusCount;
    }
    
    public int getMotcleCount() {
        return motcleCount;
    }
    
    public int getFicheCount() {
        return ficheCount;
    }
    
    public int getBaseCount() {
        return baseCount;
    }
    
    public int getLexieCount() {
        return lexieCount;
    }
    
    public int getIndexationCount() {
        return indexationCount;
    }
    
    public BaseCountStats getBaseCountStats(int codebase) {
        return (BaseCountStats) countStatsMap.get(codebase);
    }
    
    public ThesaurusCountStats getThesaurusCountStats(int codethesaurus) {
        return (ThesaurusCountStats) countStatsMap.get(codethesaurus);
    }
    
    public CorpusCountStats getCorpusCountStats(int codecorpus) {
        return (CorpusCountStats) countStatsMap.get(codecorpus);
    }
    
    public MotcleCountStats getMotcleCountStats(int codemotcle) {
        return (MotcleCountStats) countStatsMap.get(codemotcle);
    }
    
    public void update(ScrutariDB scrutariDB) {
        DataManager dataManager = scrutariDB.getDataManager();
        corpusCount = dataManager.getCorpusCount();
        thesaurusCount = dataManager.getThesaurusCount();
        motcleCount = DataUtils.getMotcleCountByBase(dataManager,null);
        ficheCount = DataUtils.getFicheCountByBase(dataManager,null,null);
        baseCount = dataManager.getBaseCount();
        lexieCount = scrutariDB.getLexieManager().getLexieCount();
        indexationCount = dataManager.getIndexationCount();
        DataQuery dataQuery = dataManager.getDataQuery();
        DataQuery.IndexationInfoMap infoMap = dataQuery.getIndexationInfoMap();
        for(int i = 0; i < baseCount; i++) {
             BaseData baseData = dataManager.getBaseDataByIndex(i);
             updateBase(dataManager,infoMap, baseData);
        }
    }
    
    private void updateBase(DataManager dataManager,DataQuery.IndexationInfoMap infoMap, BaseData baseData) {
        BaseCountStats baseCountStats = new BaseCountStats();
        int corpusCount = baseData.getCorpusCount();
        int thesaurusCount = baseData.getThesaurusCount();
        int ficheCount = 0;
        int motcleCount = 0;
        int indexationCount=0;
        baseCountStats.setCorpusCount(corpusCount);
        baseCountStats.setThesaurusCount(thesaurusCount);
        for (int i = 0; i < corpusCount;i++) {
            int codecorpus = baseData.getCodeCorpus(i);
            CorpusData corpusData = dataManager.getCorpusDataByCode(codecorpus);
            CorpusCountStats corpusCountStats = updateCorpus(dataManager,infoMap.getIndexationInfo(codecorpus),corpusData);
            ficheCount = ficheCount + corpusCountStats.getFicheCount();
            indexationCount = indexationCount + corpusCountStats.getIndexationCount();
        }
        for (int i = 0; i < thesaurusCount;i++) {
            int codethesaurus = baseData.getCodeThesaurus(i);
            ThesaurusData thesaurusData = dataManager.getThesaurusDataByCode(codethesaurus);
            ThesaurusCountStats thesaurusCountInfo =  updateThesaurus(dataManager, infoMap,thesaurusData);
            motcleCount = motcleCount + thesaurusCountInfo.getMotcleCount();
        }
        baseCountStats.setFicheCount(ficheCount);
        baseCountStats.setMotcleCount(motcleCount);
        baseCountStats.setIndexationCount(indexationCount);
        countStatsMap.put(baseData.getCodeBase(),baseCountStats);
    }
    
    private ThesaurusCountStats updateThesaurus(DataManager dataManager, DataQuery.IndexationInfoMap infoMap,ThesaurusData thesaurusData) {
        ThesaurusCountStats thesaurusCountStats = new ThesaurusCountStats();
        int codethesaurus = thesaurusData.getCodeThesaurus();
        int motcleCount = thesaurusData.getMotcleCount();
        thesaurusCountStats.setMotcleCount(motcleCount);
        DataQuery.IndexationInfo info = infoMap.getIndexationInfo(codethesaurus);
        thesaurusCountStats.setIndexationCount(info.getIndexationCount());
        thesaurusCountStats.setFicheCount(info.getIndexedCount());
        for(int i =0; i< motcleCount; i++) {
            int codemotcle = thesaurusData.getCodeMotcle(i);
            MotcleCountStats motcleCountStats = new MotcleCountStats();
            DataQuery.IndexationInfo infoMotcle = infoMap.getIndexationInfo(codemotcle);
            motcleCountStats.setFicheCount(infoMotcle.getIndexationCount());
            countStatsMap.put(codemotcle, motcleCountStats);
        }
        countStatsMap.put(codethesaurus, thesaurusCountStats);
        return thesaurusCountStats;
    }
    
    
    private CorpusCountStats updateCorpus(DataManager dataManager, DataQuery.IndexationInfo info,CorpusData corpusData) {
        CorpusCountStats corpusCountStats = new CorpusCountStats();
        int codecorpus = corpusData.getCodeCorpus();
        corpusCountStats.setMaxnumber(corpusData.getComplementMaxNumber());
        corpusCountStats.setFicheCount(corpusData.getFicheCount());
        corpusCountStats.setIndexationCount(info.getIndexationCount());
        corpusCountStats.setMotcleCount(info.getIndexedCount());
        countStatsMap.put(codecorpus,corpusCountStats);
        return corpusCountStats;
    }
    
}
