/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import com.sleepycat.je.*;
import com.sleepycat.bind.tuple.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.primitives.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class DataQuery {
    private Database uriDB;
    private Database indexationDB;
    
    DataQuery(Database uriDB,Database indexationDB) {
        this.uriDB = uriDB;
        this.indexationDB = indexationDB;
    }
    
    public IndexationInfoMap getIndexationInfoMap() {
        return new IndexationInfoMap(indexationDB);
    }
    
    /*public IndexationInfo getFicheInIndexationCount(int codethesaurus) {
        HashSet codeSet = new HashSet();
        int count = 0;
        try {
            Cursor cursor = indexationDB.openCursor(null, null);
            DatabaseEntry foundKey = new DatabaseEntry();
            DatabaseEntry foundData = new DatabaseEntry();
            while (cursor.getNext(foundKey, foundData, null) == OperationStatus.SUCCESS) {
                IndexationData indexationData = IndexationData.fromDatabaseEntry(foundData);
                if (indexationData.getCodeThesaurus() == codethesaurus) {
                    codeSet.add(new Integer(indexationData.getCodeFiche()));
                    count++;
                }
            }
            cursor.close();
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
        IndexationInfo indexationInfo = new IndexationInfo(count,codeSet.size());
        return indexationInfo;
    }
    
    public IndexationInfo getMotcleInIndexationCount(int codecorpus) {
        HashSet codeSet = new HashSet();
        int count = 0;
        try {
            Cursor cursor = indexationDB.openCursor(null, null);
            DatabaseEntry foundKey = new DatabaseEntry();
            DatabaseEntry foundData = new DatabaseEntry();
            while (cursor.getNext(foundKey, foundData, null) == OperationStatus.SUCCESS) {
                IndexationData indexationData = IndexationData.fromDatabaseEntry(foundData);
                if (indexationData.getCodeCorpus() == codecorpus) {
                    codeSet.add(new Integer());
                    count++;
                }
            }
            cursor.close();
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
        IndexationInfo indexationInfo = new IndexationInfo(count,codeSet.size());
        return indexationInfo;
    }*/
    
    
   
    
    public static class IndexationInfo {
        private int indexationCount = 0;
        private HashSet codeSet; 
        
        IndexationInfo() {
            
        }

        public int getIndexationCount() {
            return indexationCount;
        }

        public int getIndexedCount() {
            if (codeSet == null) return 0;
            return codeSet.size();
        }
        
        private void incrementCount() {
            indexationCount++;
        }
        
        private void addIndexedCode(int code) {
            if (codeSet == null) codeSet = new HashSet();
            codeSet.add(new Integer(code));
        }
    }
    
    public static class IndexationInfoMap {
        LinkedIntHashMap map = new LinkedIntHashMap();
        
        IndexationInfoMap(Database indexationDB) {
            init(indexationDB);
        }
        
        public IndexationInfo getIndexationInfo(int code) {
            IndexationInfo indexationInfo = (IndexationInfo) map.get(code);
            if (indexationInfo == null) {
                return new IndexationInfo();
            }
            return indexationInfo;
        }
        
        private void init(Database indexationDB) {
            try {
                Cursor cursor = indexationDB.openCursor(null, null);
                DatabaseEntry foundKey = new DatabaseEntry();
                DatabaseEntry foundData = new DatabaseEntry();
                while (cursor.getNext(foundKey, foundData, null) == OperationStatus.SUCCESS) {
                    IndexationData indexationData = IndexationData.fromDatabaseEntry(foundData);
                    int codemotcle = indexationData.getCodeMotcle();
                    int codefiche =  indexationData.getCodeFiche();
                    IndexationInfo corpusInfo = getOrCreate(indexationData.getCodeCorpus());
                    IndexationInfo thesaurusInfo = getOrCreate(indexationData.getCodeThesaurus());
                    IndexationInfo motcleInfo = getOrCreate(codemotcle);
                    IndexationInfo ficheInfo = getOrCreate(codemotcle);
                    motcleInfo.incrementCount();
                    ficheInfo.incrementCount();
                    corpusInfo.incrementCount();
                    thesaurusInfo.incrementCount();
                    corpusInfo.addIndexedCode(codemotcle);
                    thesaurusInfo.addIndexedCode(codefiche);
                }
                cursor.close();
            } catch(DatabaseException de) {
                throw new NestedLibraryException(de);
            }
        }
        
        private IndexationInfo getOrCreate(int code) {
            IndexationInfo indexationInfo = (IndexationInfo) map.get(code);
            if (indexationInfo == null) {
                indexationInfo = new IndexationInfo();
                map.put(code, indexationInfo);
            }
            return indexationInfo;
        }
    }
    
}
