/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.util.collections.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class LangStats {
    private int[] allLangArray;
    private int[] allFicheLangArray;
    private int[] allMotcleLangArray;
    private int[] contentLangArray;
    private LinkedIntHashMap langArrayMap;
    
    public LangStats() {
    }
    
    public int[] getAllLangArray() {
        return allLangArray;
    }
    
    public int[] getAllFicheLangArray() {
        return allFicheLangArray;
    }

    public int[] getAllMotcleLangArray() {
        return allMotcleLangArray;
    }
    
    public int[] getContentLangArray() {
        return contentLangArray;
    }
    
    void update(ScrutariDB scrutariDB) {
        LangStatsBuilder builder = new LangStatsBuilder(this,scrutariDB);
        builder.run();
    }
    
    static class LangStatsBuilder {
        LangStats langStats;
        DataManager dataManager;
        LangueBundle allLang = new LangueBundle();
        LangueBundle allFicheLang = new LangueBundle();
        LangueBundle allMotcleLang = new LangueBundle();
        LinkedIntHashMap langArrayMap = new LinkedIntHashMap();
        
        LangStatsBuilder(LangStats langStats, ScrutariDB scrutariDB) {
            this.langStats = langStats;
            this.dataManager = scrutariDB.getDataManager();
        }
        
        void run() {
            checkBase();
            checkCorpus();
            checkThesaurus();
            end();
        }
        
        void end() {
            langStats.allLangArray = allLang.toArray();
            langStats.langArrayMap = langArrayMap;
            langStats.allFicheLangArray = allFicheLang.toArray();
            langStats.allMotcleLangArray = allMotcleLang.toArray();
            allFicheLang.add(allMotcleLang);
            langStats.contentLangArray = allFicheLang.toArray();
        }
        
        void checkBase() {
            int count = dataManager.getBaseCount();
            for (int i = 0; i < count;i++) {
                BaseData baseData = dataManager.getBaseDataByIndex(i);
                int langCount = baseData.getLangDataCount();
                for (int j = 0; j < langCount;j++) {
                    BaseData.BaseLangData langData = baseData.getLangDataByIndex(j);
                    allLang.put(langData.getLangInteger());
                }
            }
        }
        
        void checkCorpus() {
            int count = dataManager.getCorpusCount();
                for (int i = 0; i < count;i++) {
                CorpusData corpusData = dataManager.getCorpusDataByIndex(i);
                int langCount = corpusData.getLangDataCount();
                for (int j = 0; j < langCount;j++) {
                    CorpusData.CorpusLangData langData = corpusData.getLangDataByIndex(j);
                    allLang.put(langData.getLangInteger());
                }
                LangueBundle ficheLang = new LangueBundle();
                int ficheCount = corpusData.getFicheCount();
                for(int j = 0; j < ficheCount; j++) {
                    FicheData ficheData = dataManager.getFicheDataByCode(corpusData.getCodeFiche(j));
                    int li = ficheData.getLangInteger();
                    allLang.put(li);
                    ficheLang.put(li);
                    allFicheLang.put(li);
                }
                langArrayMap.put(corpusData.getCodeCorpus(),ficheLang.toArray());
            }
            
        }
        
        void checkThesaurus() {
            int count = dataManager.getThesaurusCount();
            for (int i = 0; i < count;i++) {
                ThesaurusData thesaurusData = dataManager.getThesaurusDataByIndex(i);
                
                int langCount = thesaurusData.getThesaurusLangDataCount();
                for (int j = 0; j < langCount;j++) {
                    ThesaurusData.ThesaurusLangData langData = thesaurusData.getThesaurusLangDataByIndex(j);
                    allLang.put(langData.getLangInteger());
                }
                LangueBundle motcleLang = new LangueBundle();
                int motcleCount = thesaurusData.getMotcleCount();
                for(int j = 0; j < motcleCount; j++) {
                    MotcleData motcleData = dataManager.getMotcleDataByCode(thesaurusData.getCodeMotcle(j));
                    int libelleCount = motcleData.getLibelleCount();
                    for(int k = 0; k < libelleCount; k++) {
                        Libelle libelle = motcleData.getLibelleByIndex(k);
                        int li = libelle.getLangInteger();
                        allLang.put(li);
                        motcleLang.put(li);
                        allMotcleLang.put(li);
                    }
                }
                langArrayMap.put(thesaurusData.getCodeThesaurus(),motcleLang.toArray());
            }
        }
        
    }
    
    static private class LangueBundle {
        LinkedIntHashMap intHashMap = new LinkedIntHashMap();
        
        LangueBundle() {
            
        }
        
        void add(LangueBundle langBundle) {
             LinkedIntHashMap other = langBundle.intHashMap;
             int size = other.size();
             for(int i = 0; i< size; i++) {
                 put(other.getKeyAt(i));
             }
        }
        
        void put(int langInteger) {
            intHashMap.put(langInteger,Boolean.TRUE);
        }
        
        int[] toArray() {
            return intHashMap.getKeyArray();
        }
        
    }

    
    
}
