/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieManager {
    private LinkedIntHashMap mapOfMap = new LinkedIntHashMap();
    
    public LexieManager() {
    }

    public int getLangCount() {
        return mapOfMap.size();
    }
    
    public int getLang(int index) {
        return mapOfMap.getKeyAt(index);
    }
    
    public CollatedKeyMap getCollatedLexieMapByIndex(int index) {
        return (CollatedKeyMap) mapOfMap.getValueAt(index);
    }
    
    /**
     * Retourne pour une langue donnée une table avec comme clés, les lexies sous forme « collatée »
     * et comme valeur des instances de ScrutariLexieUnit.
     */
    public CollatedKeyMap getCollatedLexieMapByLang(int langInteger) {
        return (CollatedKeyMap) mapOfMap.get(langInteger);
    }
    
    private SortedCollatedKeyMap getOrCreateCollatedStringMap(int langInt) {
        SortedCollatedKeyMap collatedStringMap = (SortedCollatedKeyMap) mapOfMap.get(langInt);
        if (collatedStringMap == null) {
            collatedStringMap = new SortedCollatedKeyMap(langInt);
            mapOfMap.put(langInt,collatedStringMap);
        }
        return collatedStringMap;
    }
    
    
    /*private class LexieManagerEditor implements Editor {
        SortedCollatedKeyMap currentCollatedStringMap;
        int currentCodeFiche,currentCodeCorpus,currentCodeMotcle,currentCodeThesaurus, currentLangInt;
        int currentFieldNumber = 0;
        boolean onFiche;
        HashMap tempMap = new HashMap();
        LexieFilter currentWordFilter;
        
        
        public void initFicheData(FicheData ficheData) {
            onFiche = true;
            tempMap.clear();
            currentLangInt = ficheData.getLangInteger();
            currentWordFilter = LocalisationLexieFilters.getLexieFilter(currentLangInt);
            currentCollatedStringMap = getOrCreateCollatedStringMap(currentLangInt);
            currentCodeFiche = ficheData.getCodeFiche();
            currentCodeCorpus = ficheData.getCodeCorpus();
            currentFieldNumber = FicheAndLexieLink.TITRE_FIELDNUMBER;
            tokenize(ficheData.getTitre());
            currentFieldNumber = FicheAndLexieLink.SOUSTITRE_FIELDNUMBER;
            tokenize(ficheData.getSoustitre());
            for (int i=1; i <= ficheData.getComplementMaxNumber();i++) {
                String complement = ficheData.getComplementByNumber(i);
                currentFieldNumber = i;
                tokenize(complement);
            }
        }
        
        public void initMotcleData(MotcleData motcleData) {
            onFiche = false;
            currentCodeMotcle = motcleData.getCodeMotcle();
            currentCodeThesaurus = motcleData.getCodeThesaurus();
            for (int i = 0; i < motcleData.getLibelleCount();i++) {
                Libelle libelle = motcleData.getLibelleByIndex(i);
                tempMap.clear();
                currentLangInt = libelle.getLangInteger();
                currentWordFilter = LocalisationLexieFilters.getLexieFilter(currentLangInt);
                currentCollatedStringMap = getOrCreateCollatedStringMap(currentLangInt);
                tokenize(libelle.getLibelleString());
            }
        }
        
        
        private void tokenize(String chaine) {
            if (chaine == null) return;
            InternalLexieParserListener listener = new InternalLexieParserListener(chaine);
            listener.parse();
        }
        
        private Object createLexie(String s,int position,int wholeLength, Object previous) {
            if (onFiche) return createFicheAndLexieLink(s,position,wholeLength, (FicheAndLexieLink.InfichePosition) previous);
            else return createMotcleAndLexieLink(s,position,wholeLength, (MotcleAndLexieLink.InMotclePosition) previous);
        }
        
        private FicheAndLexieLink.InfichePosition createFicheAndLexieLink(String s, int position, int wholeLength,FicheAndLexieLink.InfichePosition previousPosition) {
            CollationUnit collatationUnit = currentCollatedStringMap.collateString(s);
            String collatedLexie = collatationUnit.getCollatedString().intern();
            CollationUnitPosition collationUnitPosition = new CollationUnitPosition(collatationUnit,position,wholeLength);
            FicheAndLexieLink ficheAndLexieLink = (FicheAndLexieLink) tempMap.get(collatedLexie);
            if (ficheAndLexieLink == null) {
                ScrutariLexieUnit lexieUnit = getScrutariLexieUnit(collatedLexie,collatationUnit.getSourceString());
                ficheAndLexieLink = new FicheAndLexieLink(lexieUnit,currentCodeFiche,currentCodeCorpus,currentFieldNumber,collationUnitPosition,previousPosition);
                lexieUnit.addFicheAndLexieLink(ficheAndLexieLink);
                tempMap.put(collatedLexie,ficheAndLexieLink);
                return ficheAndLexieLink.getBestPosition();
            } else {
                return ficheAndLexieLink.add(currentFieldNumber, collationUnitPosition,previousPosition);
            }
        }
        
        private MotcleAndLexieLink.InMotclePosition createMotcleAndLexieLink(String s, int position, int  wholeLength,MotcleAndLexieLink.InMotclePosition previous) {
            CollationUnit collationUnit = currentCollatedStringMap.collateString(s);
            String collatedLexie = collationUnit.getCollatedString().intern();
            CollationUnitPosition collationUnitPosition = new CollationUnitPosition(collationUnit,position,wholeLength);
            MotcleAndLexieLink motcleAndLexieLink = (MotcleAndLexieLink) tempMap.get(collatedLexie);
            if (motcleAndLexieLink == null) {
                ScrutariLexieUnit lexieUnit = getScrutariLexieUnit(collatedLexie,collationUnit.getSourceString());
                motcleAndLexieLink = new MotcleAndLexieLink(lexieUnit,currentCodeMotcle,currentCodeThesaurus,collationUnitPosition, previous);
                lexieUnit.addMotcleAndLexieLink(motcleAndLexieLink);
                tempMap.put(collatedLexie,motcleAndLexieLink);
                return motcleAndLexieLink.getBestPosition();
            } else {
                return motcleAndLexieLink.add(collationUnitPosition,previous);
            }
        }
        
        private ScrutariLexieUnit getScrutariLexieUnit(String collatedLexie,String sourceString) {
            ScrutariLexieUnit scrutariLexieUnit = (ScrutariLexieUnit) currentCollatedStringMap.getValueByCollatedKey(collatedLexie);
            if (scrutariLexieUnit == null) {
                scrutariLexieUnit = new ScrutariLexieUnit(collatedLexie);
                currentCollatedStringMap.putValueByCollatedKey(collatedLexie,scrutariLexieUnit);
            }
            scrutariLexieUnit.setCanonicalLexie(sourceString);
            return scrutariLexieUnit;
        }
        
        private class InternalLexieParserListener implements LexieParserListener{
            Object previous;
            int wholeLength;
            String source;
            
            InternalLexieParserListener(String source) {
                wholeLength = source.length();
                this.source = source;
            }
            
            void parse() {
                LexieParser.parse(source,this);
            }
            
            public void flushLexie(String lexie, int lexieStartIndex) {
                if (currentWordFilter.acceptLexie(lexie)) {
                    previous = createLexie(lexie,lexieStartIndex, wholeLength, previous);
                }
            }
            
            public void checkSpecialChar(char carac) {
                
            }
            
        }
    }*/
}
