/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.scrutari.lexie.OccurrenceList;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.*;
import java.util.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.scrutari.data.*;
import com.sleepycat.je.*;
import com.sleepycat.bind.tuple.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.primitives.*;
import java.util.*;
import net.mapeadores.scrutari.db.lexiebuilder.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieQuery {
    LexieManager2 lexieManager;
    
    LexieQuery(LexieManager2 lexieManager) {
        this.lexieManager = lexieManager;
    }
    
    public List getScrutariLexieUnitList(int langInteger) {
        List list = new ArrayList();
        try {
            DatabaseEntry secondaryKey = new DatabaseEntry();
            IntegerBinding.intToEntry(langInteger,secondaryKey);
            DatabaseEntry value = new DatabaseEntry();
            SecondaryCursor cursor = lexieManager.langIndex.openSecondaryCursor(null, null);
            OperationStatus op = cursor.getSearchKey(secondaryKey,value,null);
            if (op == OperationStatus.SUCCESS) {
                list.add(ScrutariLexieUnit.fromDatabaseEntry(value));
                while(cursor.getNextDup(new DatabaseEntry(), value, null) == OperationStatus.SUCCESS) {
                    list.add(ScrutariLexieUnit.fromDatabaseEntry(value));
                }
            }
            cursor.close();
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
        return list;
    }
    
    public BestPosition[] getBestPositionArray(long lexieId, int code) {
        DatabaseEntry data = getBestPositionData(lexieId, code);
        if (data == null) return null;
        BestPosition[] array = LexieUtils.fromDatabaseEntry(data);
        if (array.length == 0) return null;
        return array;
    }
    
    /**
     * code est un code de corpus ou de thésaurus
     */
    public OccurrenceList[] getOccurrenceListArray(long lexieId, int code) {
        DatabaseEntry data = getOccurrenceListArrayData(lexieId,code);
        if (data == null) return null;
        return OccurrenceList.fromArrayDatabaseEntry(data);
    }
    
    public FicheLexification getFicheLexification(int codefiche) {
        DatabaseEntry data = getLexificationData(codefiche);
        if (data == null) return null;
        return FicheLexification.fromDatabaseEntry(data);
    }
    
    public MotcleLexification getMotcleLexification(int codemotcle) {
        DatabaseEntry data = getLexificationData(codemotcle);
        if (data == null) return null;
        return MotcleLexification.fromDatabaseEntry(data);
    }
    
    private DatabaseEntry getBestPositionData(long lexieId, int code) {
        DatabaseEntry key = LexieLinkKey.toDatabaseEntry(lexieId,code);
        return getData(lexieManager.bestPositionDB,key);
    }
    
    private DatabaseEntry getOccurrenceListArrayData(long lexieId, int code) {
        DatabaseEntry key = LexieLinkKey.toDatabaseEntry(lexieId,code);
        return getData(lexieManager.occurrenceListDB,key);
    }
    
    
    private DatabaseEntry getLexificationData(int code) {
        DatabaseEntry key = new DatabaseEntry();
        IntegerBinding.intToEntry(code,key);
        return getData(lexieManager.lexificationDB,key);
    }
    
    private DatabaseEntry getData(Database database,DatabaseEntry key) {
        DatabaseEntry data = new DatabaseEntry();
        try {
            OperationStatus op = database.get(null,key,data,null);
            if (op == OperationStatus.SUCCESS) {
                return data;
            } else {
                return null;
            }
        } catch(DatabaseException de) {
            throw new NestedLibraryException(de);
        }
    }
    
}
