/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db.lexiebuilder;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;

/**
 *
 * @author Vincent Calame
 */
public class FicheLexieBuilder extends AbstractLexieBuilder implements FieldNumbers {
    private int currentCodecorpus = -1;
    private int currentCodefiche;
    private int currentFieldNumber;
   
    
    
    public FicheLexieBuilder(LexieUnitBuilder lexieUnitBuilder) {
        super(lexieUnitBuilder);
    }
    
    public int getCurrentCodeCorpus() {
        return currentCodecorpus;
    }
    
    public void setCurrentCodecorpus(int codecorpus) {
        this.currentCodecorpus = codecorpus;
    }
    
    public boolean isOnFiche() {
        return true;
    }
    
     protected int getCurrentFieldNumber() {
         return currentFieldNumber;
     }
     
     protected int getCurrentCode() {
         return currentCodefiche;
     }
    
    
    public FicheLexification initFicheData(FicheData ficheData) {
        setCurrentLangInteger(ficheData.getLangInteger());
        FicheLexification ficheLexification = new FicheLexification();
        currentCodefiche = ficheData.getCodeFiche();
        
        currentFieldNumber = TITRE_FIELDNUMBER;
        tokenize(ficheData.getTitre());
        flushLexification(ficheLexification);
        
        currentFieldNumber =  SOUSTITRE_FIELDNUMBER;
        tokenize(ficheData.getSoustitre());
        flushLexification(ficheLexification);
        
        for (int i=1; i <= ficheData.getComplementMaxNumber();i++) {
            String complement = ficheData.getComplementByNumber(i);
            currentFieldNumber = i;
            tokenize(complement);
            flushLexification(ficheLexification);
        }
        
        flushLexieOccurrenceBuilderMap();
        return ficheLexification;
    }
    
    private void flushLexification(FicheLexification ficheLexification) {
        Lexification lexification = flushLexification();
        if (lexification != null) {
            ficheLexification.putLexification(currentFieldNumber, lexification);
        }
    }
    
    

    
}
