/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db.lexiebuilder;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.*;
import java.util.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.scrutari.data.*;
import com.sleepycat.je.*;
import com.sleepycat.bind.tuple.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.primitives.*;
import java.util.*;
import net.mapeadores.scrutari.db.lexiebuilder.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieUnitBuilder {
    private LinkedIntHashMap incrementMap = new LinkedIntHashMap();
    private LinkedIntHashMap mapOfMap = new LinkedIntHashMap();
    private Map lexieUnitMap = new LinkedHashMap();
    
    
    public LexieUnitBuilder() {
    }
    
    public Collection getLexieUnitList() {
        return lexieUnitMap.values();
    }
    
    public LexieOccurence getLexieOccurence(int langInteger,String sourceLexie, boolean inFiche) {
        CollatedKeyMap map = getOrCreateCollatedStringMap(langInteger);
        CollationUnit collationUnit = map.collateString(sourceLexie);
        String collatedString = collationUnit.getCollatedString();
        Long lg = (Long) map.getValueByCollatedKey(collatedString);
        ScrutariLexieUnit scrutariLexieUnit = null;
        if (lg == null) {
            lg = new Long(getNewLexieId(langInteger));
            scrutariLexieUnit = new ScrutariLexieUnit(lg.longValue(),collatedString);
            scrutariLexieUnit.setCanonicalLexie(collationUnit.getSourceString());
            lexieUnitMap.put(lg,scrutariLexieUnit);
            map.putValueByCollatedKey(collatedString, lg);
        }
        else {
            scrutariLexieUnit = (ScrutariLexieUnit) lexieUnitMap.get(lg);
        }
        if (inFiche) {
            scrutariLexieUnit.setHasFicheLink(true);
        }
        else {
            scrutariLexieUnit.setHasMotcleLink(true);
        }
        return new LexieOccurence(lg.longValue(),collationUnit);
    }
    
    public LinkedIntHashMap getMapOfCollatedLexieMap() {
        return mapOfMap;
    }
    
    private SortedCollatedKeyMap getOrCreateCollatedStringMap(int langInt) {
        SortedCollatedKeyMap collatedStringMap = (SortedCollatedKeyMap) mapOfMap.get(langInt);
        if (collatedStringMap == null) {
            collatedStringMap = new SortedCollatedKeyMap(langInt);
            mapOfMap.put(langInt,collatedStringMap);
            incrementMap.put(langInt, new Integer(1));
        }
        return collatedStringMap;
    }
    
    private long getNewLexieId(int langInteger) {
        Integer itg = (Integer)incrementMap.get(langInteger);
        int newid = 1;
        if (itg != null) {
            newid = itg.intValue();
        }
        long l = PrimUtils.toLong(langInteger, newid);
        newid++;
        incrementMap.put(langInteger,new Integer(newid));
        return l;
    }
    
    
    
    
}
