/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db.lexiebuilder;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class MotcleLexieBuilder extends AbstractLexieBuilder {
    private int currentCodethesaurus = -1;
    private int currentCodemotcle;
    
    
    public MotcleLexieBuilder(LexieUnitBuilder lexieUnitBuilder) {
        super(lexieUnitBuilder);
    }
    
    public int getCurrentCodethesaurus() {
        return currentCodethesaurus;
    }
    
    public void setCurrentCodethesaurus(int codethesaurus) {
        this.currentCodethesaurus = codethesaurus;
    }
    
    protected int getCurrentFieldNumber() {
         return FieldNumbers.NONE_FIELDNUMBER;
     }
    
    protected int getCurrentCode() {
         return currentCodemotcle;
     }
    
    public MotcleLexification initMotcleData(MotcleData motcleData) {
        currentCodemotcle = motcleData.getCodeMotcle();
        MotcleLexification motcleLexification = new MotcleLexification();
        int libelleCount = motcleData.getLibelleCount();
        for (int i = 0; i < libelleCount;i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            setCurrentLangInteger(libelle.getLangInteger());
            tokenize(libelle.getLibelleString());
            flushLexification(motcleLexification);
        }
        flushLexieOccurrenceBuilderMap();
        return motcleLexification;
    }
    
    public boolean isOnFiche() {
        return false;
    }
    
    private void flushLexification(MotcleLexification motcleLexification) {
        Lexification lexification = flushLexification();
        if (lexification != null) {
            motcleLexification.putLexification(getCurrentLangInteger(), lexification);
        }
    }
   

    
}
