/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db.util;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;

/**
 * Propose des méthodes statitiques utilitaires pour récupérer des Datarmations sur les bases.
 *
 * @author Vincent Calame
 */
public class DataUtils {
    static final char SEUIL1 = '\u0080';
    static final char SEUIL2 = '\u0800';
    
    /**
     * Retourne le nombre de caractères supplémentaires introduit par le codage UTF8 du caractère en argument.
     * Par exemple, la méthode retournera 0  pour le caractère « a » qui en UTF-8 s'écrit « a » et elle retournera 1 pour le caractère « é » qui en UTF-8 s'écrit « Ã© ».
     */
    public static int getDecalageUTF8(char c) {
        if (c < SEUIL1) return 0;
        if (c < SEUIL2) return 1;
        return 2;
    }
    
    private DataUtils() {
    }
    
    
    static public String getHref(FicheData ficheData, DataManager dataManager, int langUiInteger) {
        BaseData baseData = dataManager.getBaseDataByCode(dataManager.getCorpusDataByCode(ficheData.getCodeCorpus()).getCodeBase());
        int checkedLangInteger = baseData.checkLangUi(langUiInteger);
        return ficheData.getHref(checkedLangInteger);
    }
    
 
    static public int getFicheCountByCorpus(DataManager dataManager,int[] codescorpus, int[] intlangList) {
        int resultat = 0;
        if (intlangList != null) {
            int taille = intlangList.length;
            if (codescorpus == null) {
                for(int i = 0; i < dataManager.getCorpusCount();i++) {
                    CorpusData corpusData = dataManager.getCorpusDataByIndex(i);
                    for (int k = 0; k < taille;k++) {
                        resultat = resultat + corpusData.getFicheCountByLang(intlangList[k]);
                    }
                }
            } else {
                for (int i = 0; i < codescorpus.length;i++) {
                    CorpusData corpusData = dataManager.getCorpusDataByCode(codescorpus[i]);
                    for (int k = 0; k < taille;k++) {
                        resultat = resultat + corpusData.getFicheCountByLang(intlangList[k]);
                    }
                }
            }
        } else {
            if (codescorpus == null) {
                for(int i = 0; i < dataManager.getCorpusCount();i++) {
                    CorpusData corpusData = dataManager.getCorpusDataByIndex(i);
                    resultat = resultat + corpusData.getFicheCount();
                }
            } else {
                for (int i = 0; i < codescorpus.length;i++) {
                    CorpusData corpusData = dataManager.getCorpusDataByCode(codescorpus[i]);
                    if (corpusData != null) {
                        resultat = resultat + corpusData.getFicheCount();
                    }
                }
            }
        }
        return resultat;
    }
    
    static public int getFicheCountByBase(DataManager dataManager,int[] codebaseArray, int[] intlangList) {
        int resultat = 0;
        
        if (intlangList != null) {
            int length = intlangList.length;
            if (codebaseArray == null) {
                for(int i = 0; i < dataManager.getBaseCount();i++) {
                    BaseData basData = dataManager.getBaseDataByIndex(i);
                    for (int j = 0; j < basData.getCorpusCount();j++) {
                        CorpusData corpusData =  dataManager.getCorpusDataByCode(basData.getCodeCorpus(j));
                        for (int k = 0; k < length;k++) {
                            resultat = resultat + corpusData.getFicheCountByLang(intlangList[k]);
                        }
                    }
                }
            } else {
                for (int i = 0; i < codebaseArray.length;i++) {
                    BaseData basData = dataManager.getBaseDataByCode(codebaseArray[i]);
                    if (basData != null) {
                        for (int j = 0; j < basData.getCorpusCount();j++) {
                            CorpusData corpusData = dataManager.getCorpusDataByCode(basData.getCodeCorpus(j));
                            for (int k = 0; k < length;k++) {
                                resultat = resultat +  corpusData.getFicheCountByLang(intlangList[k]);
                            }
                        }
                    }
                }
            }
        } else {
            if (codebaseArray == null) {
                for(int i = 0; i < dataManager.getBaseCount();i++) {
                    BaseData baseData = dataManager.getBaseDataByIndex(i);
                    for (int j = 0; j < baseData.getCorpusCount();j++) {
                        CorpusData corpusData = dataManager.getCorpusDataByCode(baseData.getCodeCorpus(j));
                        resultat = resultat + corpusData.getFicheCount();
                    }
                }
            } else {
                for (int i = 0; i < codebaseArray.length;i++) {
                    BaseData basData = dataManager.getBaseDataByCode(codebaseArray[i]);
                    if (basData != null) {
                        for (int j = 0; j < basData.getCorpusCount();j++) {
                            CorpusData corpusData = dataManager.getCorpusDataByCode(basData.getCodeCorpus(j));
                            resultat = resultat + corpusData.getFicheCount();
                        }
                    }
                }
            }
        }
        return resultat;
    }
    
    static public int getMotcleCountByBase(DataManager dataManager,int[] codebaseArray) {
        int resultat = 0;
        if (codebaseArray == null) {
            for(int i = 0; i < dataManager.getBaseCount();i++) {
                BaseData baseData = dataManager.getBaseDataByIndex(i);
                for (int j = 0; j < baseData.getThesaurusCount();j++) {
                    ThesaurusData corpusData = dataManager.getThesaurusDataByCode(baseData.getCodeThesaurus(j));
                    resultat = resultat + corpusData.getMotcleCount();
                }
            }
        } else {
            for (int i = 0; i < codebaseArray.length;i++) {
                BaseData basData = dataManager.getBaseDataByCode(codebaseArray[i]);
                if (basData != null) {
                    for (int j = 0; j < basData.getCorpusCount();j++) {
                        ThesaurusData corpusData = dataManager.getThesaurusDataByCode(basData.getCodeThesaurus(j));
                        resultat = resultat + corpusData.getMotcleCount();
                    }
                }
            }
        }
        return resultat;
    }
    
    /*static public int getLexieCount(LexieManager lexieManager) {
        int result = 0;
        for(int i = 0;i < lexieManager.getUsedLangCount();i++) {
            result = result + lexieManager.getLexieCountInUsedLang(i);
        }
        return result;
    }
     
    static public IntList getAllLangIntegerList(ScrutariDB generalDatabase) {
        AllLangBundle allLangBundle = new AllLangBundle();
        DataManager dataManager = generalDatabase.getDataManager();
        for (int i = 0; i < dataManager.getBaseCount();i++) {
            BaseData baseData = dataManager.getBaseDataByIndex(i);
            for (int j = 0; j < baseData.getLangDataCount();j++) {
                BaseData.BaseLangData langData = baseData.getLangDataByIndex(j);
                allLangBundle.put(langData.getLangInteger());
            }
        }
        for (int i = 0; i < dataManager.getCorpusCount();i++) {
            CorpusData corpusData = dataManager.getCorpusDataByIndex(i);
            for (int j = 0; j < corpusData.getLangDataCount();j++) {
                CorpusData.CorpusLangData langData = corpusData.getLangDataByIndex(j);
                allLangBundle.put(langData.getLangInteger());
            }
        }
        for (int i = 0; i < dataManager.getThesaurusCount();i++) {
            ThesaurusData thesaurusData = dataManager.getThesaurusDataByIndex(i);
            for (int j = 0; j < thesaurusData.getThesaurusLangDataCount();j++) {
                ThesaurusData.ThesaurusLangData langData = thesaurusData.getThesaurusLangDataByIndex(j);
                allLangBundle.put(langData.getLangInteger());
            }
        }
        int[] usedLangIntegerList = generalDatabase.getLexieManager().getUsedLangIntegerArray();
        for (int i = 0; i < usedLangIntegerList.length;i++) {
            allLangBundle.put(usedLangIntegerList[i]);
        }
        return allLangBundle.getResult();
    }
     
    static private class AllLangBundle {
        ArrayIntList result = new ArrayIntList();
        IntHashMap intHashMap = new IntHashMap();
     
        AllLangBundle() {
     
        }
     
        void put(int langInteger) {
            if (!intHashMap.containsKey(langInteger)) {
                result.add(langInteger);
                intHashMap.put(langInteger,Boolean.TRUE);
            }
        }
     
        IntList getResult() {
            return result;
        }
    }*/
    
}
