/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.db.util;

import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.util.text.*;

import net.mapeadores.util.collections.*;


/**
 *
 * @author Vincent Calame
 */
public class LibelleUtils {
    
    private LibelleUtils() {
    }
    
    public static String getLibelleString(CorpusData corpusData,int fieldNumber,int langInteger,String defaultString) {
        CorpusData.CorpusLangData corpusLangData = corpusData.getLangDataByLang(langInteger);
        String res = getLibelleString(corpusLangData,fieldNumber);
        if (res != null) return res;
        int count = corpusData.getLangDataCount();
        for (int i = 0; i < count;i++) {
            corpusLangData = corpusData.getLangDataByIndex(i);
            res = getLibelleString(corpusLangData,fieldNumber);
            if (res != null) return res;
        }
        return defaultString;
    }
    
    private static String getLibelleString(CorpusData.CorpusLangData corpusLangData,int fieldNumber) {
        if (corpusLangData == null) return null;
        Libelle lib = corpusLangData.getLibelle(fieldNumber);
        if (lib == null) return null;
        return lib.getLibelleString();
    }
    
    public static String getLibelleString(BaseData baseData,short type,int langInteger,String defaultString) {
        BaseData.BaseLangData baseLangData = baseData.getLangDataByLang(langInteger);
        String res = getLibelleString(baseLangData,type);
        if (res != null) return res;
        int count = baseData.getLangDataCount();
        for (int i = 0; i < count;i++) {
            baseLangData = baseData.getLangDataByIndex(i);
            res = getLibelleString(baseLangData,type);
            if (res != null) return res;
        }
        return defaultString;
    }
    
    private static String getLibelleString(BaseData.BaseLangData baseLangData,short type) {
        if (baseLangData == null) return null;
        Libelle lib = baseLangData.getLibelle(type);
        if (lib == null) return null;
        return lib.getLibelleString();
    }
    
    public static String getLibelleString(ThesaurusData thesaurusData,int langInteger,String defaultString) {
        ThesaurusData.ThesaurusLangData thesaurusLangData = thesaurusData.getThesaurusLangDataByLang(langInteger);
        String res = getLibelleString(thesaurusLangData);
        if (res != null) return res;
        int count = thesaurusData.getThesaurusLangDataCount();
        for (int i = 0; i < count;i++) {
            thesaurusLangData = thesaurusData.getThesaurusLangDataByIndex(i);
            res = getLibelleString(thesaurusLangData);
            if (res != null) return res;
        }
        return defaultString;
    }
    
    private static String getLibelleString(ThesaurusData.ThesaurusLangData thesaurusLangData) {
        if (thesaurusLangData == null) return null;
        Libelle lib = thesaurusLangData.getLibelle();
        if (lib == null) return null;
        return lib.getLibelleString();
    }
}
