/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.data;

import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class BaseData {
    public final static short INTITULE_SHORT = 1;
    public final static short INTITULE_LONG = 2;
    private int codebase;
    private int[] codesCorpus;
    private int[] codesThesaurus;
    private LinkedIntHashMap intLangMap;
    private int[] langsUI;
    private String baseIconPart1;
    private String baseIconPart2;
    private BaseURI baseURI;
    
    
    public BaseData(BaseURI baseURI,int codebase,int[] codesCorpus, int[] codesThesaurus,LinkedIntHashMap intLangMap,int[] langsUI,String baseIcon) {
        this.codesCorpus = codesCorpus;
        this.codesThesaurus = codesThesaurus;
        this.codebase = codebase;
        this.intLangMap = intLangMap;
        this.langsUI = langsUI;
        this.baseURI = baseURI;
        parseBaseIcon(baseIcon);
    }
    
    public int getCodeBase() {
        return codebase;
    }
    
    public int getCorpusCount() {
        return codesCorpus.length;
    }
    
    public int getCodeCorpus(int i) {
        return codesCorpus[i];
    }
    
    public int getThesaurusCount() {
        return codesThesaurus.length;
    }
    public int getCodeThesaurus(int i) {
        return codesThesaurus[i];
    }
    
    public int getLangUiCount() {
        return langsUI.length;
    }
    
    public int getLangUi(int i) {
        return langsUI[i];
    }
    
    public BaseURI getBaseURI() {
        return baseURI;
    }
    
    public String getBaseIcon(int langUI) {
        if (baseIconPart2 == null) return baseIconPart1;
        StringBuffer buf = new StringBuffer(baseIconPart1.length() + baseIconPart2.length() + 10);
        buf.append(baseIconPart1);
        LangInteger.appendString(buf,langUI);
        buf.append(baseIconPart2);
        return buf.toString();
    }
    
    public int checkLangUi(int langInteger) {
        int length = langsUI.length;
        if (length == 0) return langInteger;
        for(int i = 0; i < length;i++) {
            if (langInteger == langsUI[i]) {
                return langInteger;
            }
        }
        return langsUI[0];
    }
    
    public BaseLangData getLangDataByLang(int langInt) {
        return (BaseLangData) intLangMap.get(langInt);
    }
    
    /**
     * Retourne null ou une chaîne de longueur non nulle et non composée d'espaces blancs.
     */
    public String getLibelleStringByLang(short type,int langInteger) {
        BaseLangData baseLangInfo = getLangDataByLang(langInteger);
        if (baseLangInfo == null) return null;
        Libelle lib = baseLangInfo.getLibelle(type);
        if (lib == null) return null;
        return lib.getLibelleString();
    }
    
    public BaseLangData getLangDataByIndex(int i) {
        return (BaseLangData) intLangMap.getValueAt(i);
    }
    
    public int getLangDataCount() {
        return intLangMap.size();
    }
    
    public static class BaseLangData {
        int langInt;
        InternalLibelle shortLibelle;
        InternalLibelle longLibelle;
        
        BaseLangData(int langInt) {
            this.langInt = langInt;
        }
        
        public Libelle getLibelle(short type) {
            switch(type) {
                case INTITULE_SHORT :
                    return shortLibelle;
                case INTITULE_LONG :
                    return longLibelle;
            }
            return null;
        }
        
        public String getLibelleString(short type) {
            Libelle libelle = getLibelle(type);
            if (libelle == null) return null;
            return libelle.getLibelleString();
        }
        
        public int getLangInteger() {
            return langInt;
        }
        
        void setLibelle(short type,String s) {
            switch(type) {
                case INTITULE_SHORT :
                    shortLibelle  = new InternalLibelle(s);
                    break;
                case INTITULE_LONG :
                    longLibelle  = new InternalLibelle(s);
                    break;
            }
        }
        
        
        class InternalLibelle implements Libelle {
            String lib;
            
            InternalLibelle(String lib) {
                this.lib = lib;
            }
            
            public int getLangInteger() {
                return langInt;
            }
            
            public String getLibelleString() {
                return lib;
            }
            
            public boolean isVo() {
                return true;
            }
        }
    }
    
    private void parseBaseIcon(String baseIcon) {
        if (baseIcon == null) return;
        int idx = baseIcon.indexOf("$LANGUI");
        if (idx == -1) {
            baseIconPart1 = baseIcon;
            baseIconPart2 = null;
        } else {
            baseIconPart1 = baseIcon.substring(0,idx);
            baseIconPart2 = baseIcon.substring(idx + 7);
        }
    }
    
    public static class  Builder {
        private BaseURI baseURI;
        private ArrayList corpusDataBuilderList = new ArrayList();
        private ArrayList thesaurusDataBuilderList = new ArrayList();
        private ArrayList shortLibelleList = new ArrayList();
        private ArrayList longLibelleList = new ArrayList();
        private LinkedIntHashMap langsUiMap = new LinkedIntHashMap();
        private String baseIcon;
        
        public Builder(BaseURI baseURI) {
            this.baseURI = baseURI;
        }
        public void addLibelle(short type,Libelle libelle) {
            switch(type) {
                case BaseData.INTITULE_SHORT :
                    shortLibelleList.add(libelle);
                    break;
                case BaseData.INTITULE_LONG :
                    longLibelleList.add(libelle);
                    break;
            }
        }
        
        public void addLangUI(int langInteger) {
            langsUiMap.put(langInteger,Boolean.TRUE);
        }
        
        public BaseURI getBaseURI() {
            return baseURI;
        }
        
        public void setBaseIcon(String icon) {
            this.baseIcon = icon;
        }
        
        public void addCorpusDataBuilder(CorpusData.Builder builder) {
            corpusDataBuilderList.add(builder);
        }
        
        public void addThesaurusDataBuilder(ThesaurusData.Builder builder) {
            thesaurusDataBuilderList.add(builder);
        }
        
        public List getCorpusDataBuilderList() {
            return Collections.unmodifiableList(corpusDataBuilderList);
        }
        
        public List getThesaurusDataBuilderList() {
            return Collections.unmodifiableList(thesaurusDataBuilderList);
        }
        
        public BaseData toBaseData(ScrutariURICoder uriCoder) {
            int codebase = uriCoder.getOrCreateCode(baseURI);
            LinkedIntHashMap linkedHashMap = new LinkedIntHashMap();
            for (int i = 0; i < shortLibelleList.size();i++) {
                Libelle libelle = (Libelle) shortLibelleList.get(i);
                putLibelle(linkedHashMap,libelle,BaseData.INTITULE_SHORT);
            }
            for (int i = 0; i < longLibelleList.size();i++) {
                Libelle libelle = (Libelle) longLibelleList.get(i);
                putLibelle(linkedHashMap,libelle,BaseData.INTITULE_LONG);
            }
            int codesCorpusSize = corpusDataBuilderList.size();
            int[] codesCorpus = new int[codesCorpusSize];
            for (int i = 0; i < codesCorpusSize;i++) {
                CorpusData.Builder buf = (CorpusData.Builder) corpusDataBuilderList.get(i);
                int code = uriCoder.getOrCreateCode(buf.getCorpusURI());
                codesCorpus[i] = code;
            }
            int codesThesaurusSize = thesaurusDataBuilderList.size();
            int[] codesThesaurus = new int[codesThesaurusSize];
            for (int i = 0; i < codesThesaurusSize;i++) {
                ThesaurusData.Builder buf = (ThesaurusData.Builder) thesaurusDataBuilderList.get(i);
                int code = uriCoder.getOrCreateCode(buf.getThesaurusURI());
                codesThesaurus[i] = code;
            }
            int size = langsUiMap.size();
            int[] langsUI = new int[size];
            for(int i = 0; i < size;i++) {
                langsUI[i] = langsUiMap.getKeyAt(i);
            }
            return new BaseData(baseURI,codebase,codesCorpus,codesThesaurus,linkedHashMap,langsUI,baseIcon);
        }
        
        /**
         * S'assure que le libelle n'est pas nul ou de longueur nulle après un trim()
         */
        private void putLibelle(LinkedIntHashMap linkedHashMap,Libelle libelle, short type) {
            String s = libelle.getLibelleString();
            if (s == null) return;
            s = s.trim();
            if (s.length() == 0) return;
            int langInt = libelle.getLangInteger();
            BaseData.BaseLangData langInfo = (BaseData.BaseLangData) linkedHashMap.get(langInt);
            if (langInfo == null) {
                langInfo = new BaseData.BaseLangData(langInt);
                linkedHashMap.put(langInt,langInfo);
            }
            langInfo.setLibelle(type,s);
        }
    }
    
}
