/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.data;

import java.io.Serializable;

import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.text.*;
import java.util.*;

/**
 *
 * @author  Vincent Calame
 */
public final class FicheData implements Serializable {
    private int codefiche;
    private int codecorpus;
    private int codebase;
    private int langInteger;
    private int dateInteger;
    private String titre;
    private String soustitre;
    private String hrefPart1;
    private String hrefPart2;
    private String[] complements;
    private short type;
    private FicheURI ficheURI;
    private String ficheIcon;
    
    /*
     * Constructeur utilisé par FicheDataTupleBinding pour l'insertion dans la base de Berkeley.
     */
    private FicheData() {
    }
    
    
    private FicheData(FicheURI ficheURI,int codefiche,int codecorpus,int codebase,int langInteger,int dateInteger,String titre,String soustitre,String href, String[] complements, short type, String ficheIcon) {
        this.codecorpus = codecorpus;
        this.codefiche = codefiche;
        this.codebase = codebase;
        this.langInteger = langInteger;
        this.dateInteger = dateInteger;
        this.titre = titre;
        this.soustitre = soustitre;
        parseHref(href);
        this.complements = complements;
        this.type = type;
        this.ficheURI = ficheURI;
        this.ficheIcon = ficheIcon;
    }

    public int getCodeFiche() {
        return codefiche;
    }
    
    public int getCodeCorpus() {
        return codecorpus;
    }
    
    public int getCodeBase() {
        return codebase;
    }
    
    public int getLangInteger() {
        return langInteger;
    }
    
    public int getComplementMaxNumber() {
        if (complements == null) return 0;
        return complements.length;
    }
    
    public FicheURI getFicheURI() {
        return ficheURI;
    }
    
    /**
     * Peut avoir la valeur nulle mais n'est pas de longueur nulle ou composé d'espace blanc.
     */
    public String getTitre() {
        return titre;
    }
    
    /**
     * Peut avoir la valeur nulle mais n'est pas de longueur nulle ou composé d'espace blanc.
     */
    public String getSoustitre() {
        return soustitre;
    }
    
    public String getHref(int langUI) {
        if (hrefPart2 == null) return hrefPart1;
        StringBuffer buf = new StringBuffer(hrefPart1.length() + hrefPart2.length() + 10);
        buf.append(hrefPart1);
        LangInteger.appendString(buf,langUI);
        buf.append(hrefPart2);
        return buf.toString();
    }
    
    
    public String getRawHref() {
        if (hrefPart2 == null) {
            return hrefPart1;
        }
        return hrefPart1 + "$LANGUI" + hrefPart2;
    }
    
    public int getDateInteger() {
        return dateInteger;
    }
    
    public short getType() {
        return type;
    }
    
    public String getFicheIcon() {
        return ficheIcon;
    }
    
    /**
     * Attention, les compléments sont numérotés à partir de 1.
     */
    public String getComplementByNumber(int i) {
        if (complements == null) throw new IndexOutOfBoundsException();
        return complements[i-1];
    }
    
    public DatabaseEntry toDatabaseEntry() {
        DatabaseEntry data = new DatabaseEntry();
        FicheDataTupleBinding bind = new FicheDataTupleBinding();
        bind.objectToEntry(this,data);
        return data;
    }
    
    
    public static FicheData fromDatabaseEntry(DatabaseEntry data) {
        FicheDataTupleBinding bind = new FicheDataTupleBinding();
        return (FicheData) bind.entryToObject(data);
    }
    
    /*public static int getDateInteger(TupleInput tupleInput) {
        URITupleBinding.entryToScrutariURI(tupleInput);
        tupleInput.readInt();
        tupleInput.readInt();
        tupleInput.readInt();
        return tupleInput.readInt();
    }*/
    
    
    
    static private final class FicheDataTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            FicheData ficheData = new FicheData();
            ficheData.ficheURI = (FicheURI) URITupleBinding.entryToScrutariURI(tupleInput);
            ficheData.codefiche = tupleInput.readInt();
            ficheData.codecorpus = tupleInput.readInt();
            ficheData.langInteger = tupleInput.readInt();
            ficheData.dateInteger = tupleInput.readInt();
            ficheData.codebase = tupleInput.readInt();
            ficheData.type = tupleInput.readShort();
            ficheData.titre = tupleInput.readString();
            ficheData.soustitre = tupleInput.readString();
            ficheData.hrefPart1 = tupleInput.readString();
            ficheData.hrefPart2 = tupleInput.readString();
            ficheData.ficheIcon = tupleInput.readString();
            int length = tupleInput.readInt();
            if (length > 0) {
                String[] comp = new String[length];
                for (int i = 0; i < length;i++) {
                    comp[i] = tupleInput.readString();
                }
                ficheData.complements = comp;
            }
            return ficheData;
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            FicheData ficheData = (FicheData) obj;
            URITupleBinding.scrutariURIToEntry(ficheData.ficheURI,tupleOutput);
            tupleOutput.writeInt(ficheData.codefiche);
            tupleOutput.writeInt(ficheData.codecorpus);
            tupleOutput.writeInt(ficheData.langInteger);
            tupleOutput.writeInt(ficheData.dateInteger);
            tupleOutput.writeInt(ficheData.codebase);
            tupleOutput.writeShort(ficheData.type);
            tupleOutput.writeString(ficheData.titre);
            tupleOutput.writeString(ficheData.soustitre);
            tupleOutput.writeString(ficheData.hrefPart1);
            tupleOutput.writeString(ficheData.hrefPart2);
            tupleOutput.writeString(ficheData.ficheIcon);
            if (ficheData.complements == null) tupleOutput.writeInt(-1);
            else {
                int length = ficheData.complements.length;
                tupleOutput.writeInt(length);
                for (int i = 0; i < length;i++) {
                    tupleOutput.writeString(ficheData.complements[i]);
                }
            }
        }
    }
    
    private void parseHref(String href) {
        if (href == null) return;
        int idx = href.indexOf("$LANGUI");
        if (idx == -1) {
            hrefPart1 = href;
            hrefPart2 = null;
        } else {
            hrefPart1 = href.substring(0,idx);
            hrefPart2 = href.substring(idx + 7);
        }
    }
    
    public static class Builder {
        private FicheURI ficheURI;
        private int langInteger = LangInteger.UNDETERMINED;
        private int dateInteger = IntDate.NULL;
        private int complementMaxNumber = 0;
        private String titre;
        private String soustitre;
        private String href;
        private String ficheIcon;
        private short type = TypeConstants.UNKNOWN_TYPE;
        private ArrayList complementsList;
        
        /**
         * Construit une instance de FicheInfoBuffer pour la fiche d'URI <code>ficheURI</code> en précisant le
         * nombre maximum de champs complémentaires qu'est susceptible de contenir la fiche.
         */
        public Builder(FicheURI ficheURI, int complementMaxNumber) {
            this.ficheURI = ficheURI;
            this.complementMaxNumber = complementMaxNumber;
            if (complementMaxNumber > 0) complementsList = new ArrayList();
        }
        
        public void setType(short type) {
            this.type = type;
        }
        
        /**
         * Modifie l'entier de la langue de la fiche
         */
        public void setLangInteger(int langInteger) {
            this.langInteger = langInteger;
        }
        
        /**
         * Modifie l'entier de la date de la fiche.
         */
        public void setDateInteger(int dateInteger) {
            this.dateInteger = dateInteger;
        }
        
        /**
         * Modifie le titre de la fiche.
         */
        public void setTitre(String titre) {
            this.titre = titre;
        }
        
        /**
         * Modifie le sous-titre de la fiche
         */
        public void setSoustitre(String soustitre) {
            this.soustitre = soustitre;
        }
        
        /**
         * Modifie l'hyperlien de la fiche.
         */
        public void setHref(String href) {
            this.href = href;
        }
        
        public void setFicheIcon(String ficheIcon) {
            this.ficheIcon = ficheIcon;
        }
        
        /**
         * Retourne l'entier de la langue de la fiche.
         */
        public int getLangInteger() {
            return langInteger;
        }
        
        /**
         * Retourne l'entier de la date de la fiche.
         */
        public int getDateInteger() {
            return dateInteger;
        }
        
        /**
         * Retourne l'URI de la fiche.
         */
        public FicheURI getFicheURI() {
            return ficheURI;
        }
        
        /**
         * Ajoute la valeur d'un champ complémentaire. L'ordre d'ajout des champs détermine l'ordre
         * final des champs dans l'instance de FicheData issue de l'objet FicheInfoBuffer.
         * Cette méthode retourne false si le nombre maximum de champs est atteint.
         */
        public boolean addComplement(String s) {
            if (complementMaxNumber == 0) return false;
            if (complementsList.size() == complementMaxNumber)  return false;
            complementsList.add(s);
            return true;
        }
        
        /**
         * Construit une instance de FicheData à partir des données de l'objet FicheInfoBuffer.
         */
        public FicheData toFicheData(ScrutariURICoder uriCoder, short defaultType, String hrefParent) {
            int codefiche = uriCoder.getOrCreateCode(ficheURI);
            CorpusURI corpusURI = ficheURI.getCorpusURI();
            int codecorpus = uriCoder.getOrCreateCode(corpusURI);
            int codebase = uriCoder.getOrCreateCode(corpusURI.getBaseURI());
            titre = StringUtils.nullifie(titre);
            soustitre =  StringUtils.nullifie(soustitre);
            href = StringUtils.nullifie(href);
            ficheIcon = StringUtils.nullifie(ficheIcon);
            if (type == TypeConstants.UNKNOWN_TYPE) type = defaultType;
            if ((href != null) && (hrefParent != null)) {
                if ((!(href.startsWith("http://"))) || (!(href.startsWith("https://")))) {
                    href = hrefParent + href;
                }
            }
            String[] complements = null;
            if (complementMaxNumber > 0) {
                complements = new String[complementMaxNumber];
                for (int i = 0; i < complementsList.size();i++) {
                    complements[i] =  StringUtils.nullifie((String) complementsList.get(i));
                }
            }
            return new FicheData(ficheURI,codefiche,codecorpus, codebase,langInteger,dateInteger,titre,soustitre,href,complements, type, ficheIcon);
        }
        
    }
    
    
}
