/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.data;


import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.collections.crossing.CrossingMap;
import java.util.*;
import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;

/**
 *
 * @author Vincent Calame
 */
public class IndexationData implements CrossingMap.Crossing, CrossingMap.CrossingItem {
    private int codecorpus;
    private int codethesaurus;
    private int codemotcle;
    private int codefiche;
    private float ponderation;
    
    private IndexationData() {
        
    }
    
    private IndexationData(int codefiche,int codecorpus,int codemotcle,int codethesaurus,float ponderation) {
        this.codecorpus = codecorpus;
        this.codefiche = codefiche;
        this.codethesaurus = codethesaurus;
        this.codemotcle = codemotcle;
        this.ponderation = ponderation;
    }
    
    public int getCodeMotcle() {
        return codemotcle;
    }
    
    public int getCodeThesaurus() {
        return codethesaurus;
    }
    
    public int getCodeFiche() {
        return codefiche;
    }
    
    public int getCodeCorpus() {
        return codecorpus;
    }
    
    public float getPonderation() {
        return ponderation;
    }
    
    public int getIndex1() {
        return codefiche;
    }
    
    public int getIndex2() {
        return codemotcle;
    }
    
    public DatabaseEntry toDatabaseEntry() {
        DatabaseEntry data = new DatabaseEntry();
        IndexationDataTupleBinding bind = new IndexationDataTupleBinding();
        bind.objectToEntry(this,data);
        return data;
    }
    
    
    public static IndexationData fromDatabaseEntry(DatabaseEntry data) {
        IndexationDataTupleBinding bind = new IndexationDataTupleBinding();
        return (IndexationData) bind.entryToObject(data);
    }
    
    public static class Builder {
        private IndexationKey indexationKey;
        private int poids=1;
        
        public Builder(IndexationKey indexationKey, int poids) {
            this.indexationKey = indexationKey;
            this.poids = poids;
        }
        
        public int getPoids() {
            return poids;
        }
        
        /**
         * Peut retourner null.
         */
        public IndexationData toIndexationData(ScrutariURICoder uriCodeMap, int poidsMax) {
            int codefiche = uriCodeMap.getCode(indexationKey.getFicheURI());
            if (codefiche == -1) {
                return null;
            }
            int codecorpus = uriCodeMap.getOrCreateCode(indexationKey.getFicheURI().getCorpusURI());
            int codemotcle = uriCodeMap.getCode(indexationKey.getMotcleURI());
            if (codemotcle == -1) {
                return null;
            }
            int codethesaurus = uriCodeMap.getOrCreateCode(indexationKey.getMotcleURI().getThesaurusURI());
            float ponderation= 0;
            if (poids > 1) {
                ponderation = ((float)poids)/((float)(poidsMax + 1));
            }
            return new IndexationData(codefiche,codecorpus,codemotcle,codethesaurus,ponderation);
        }
        
    }
    
    static private final class IndexationDataTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            IndexationData indexationData = new IndexationData();
            indexationData.codefiche = tupleInput.readInt();
            indexationData.codemotcle = tupleInput.readInt();
            indexationData.codecorpus = tupleInput.readInt();
            indexationData.codethesaurus = tupleInput.readInt();
            indexationData.ponderation = tupleInput.readFloat();
            return indexationData;
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            IndexationData indexationData = (IndexationData) obj;
            tupleOutput.writeInt(indexationData.codefiche);
            tupleOutput.writeInt(indexationData.codemotcle);
            tupleOutput.writeInt(indexationData.codecorpus);
            tupleOutput.writeInt(indexationData.codethesaurus);
            tupleOutput.writeFloat(indexationData.ponderation);
        }
    }
    
    public static class BuilderList {
        ArrayList indexationInfoBufferList = new ArrayList();
        int poidsMax = 1;
        
        public BuilderList() {
        }
        
        public void addIndexationDataBuilder(IndexationData.Builder indexationInfoBuffer) {
            int wg = indexationInfoBuffer.getPoids();
            poidsMax = Math.max(wg,poidsMax);
            indexationInfoBufferList.add(indexationInfoBuffer);
        }
        
        public int getPoidsMax() {
            return poidsMax;
        }
        
        public List getIndexationDataBuilderList() {
            return Collections.unmodifiableList(indexationInfoBufferList);
        }
        
    }
    
}
