/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.data;


import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class ThesaurusData {
    private int codethesaurus;
    private int codebase;
    private LinkedIntHashMap intLangMap;
    private int[] codesMotscles;
    private ThesaurusURI thesaurusURI;
    
    public ThesaurusData(ThesaurusURI thesaurusURI,int codethesaurus,int codebase,int[] codesMotscles,LinkedIntHashMap intLangMap) {
        this.codethesaurus = codethesaurus;
        this.codebase = codebase;
        this.intLangMap = intLangMap;
        this.codesMotscles = codesMotscles;
        this.thesaurusURI = thesaurusURI;
    }
    
    public int getCodeThesaurus() {
        return codethesaurus;
    }
    
    public int getCodeBase() {
        return codebase;
    }
    
    public ThesaurusURI getThesaurusURI() {
        return thesaurusURI;
    }
    
    public int getMotcleCount() {
        return codesMotscles.length;
    }
    
    public int getCodeMotcle(int i) {
        return codesMotscles[i];
    }
    
    public ThesaurusLangData getThesaurusLangDataByLang(int langInteger) {
        return (ThesaurusLangData) intLangMap.get(langInteger);
    }
    
    /**
     * Retourne null ou une chaîne de longueur non nulle et non composée d'espaces blancs.
     */
    public String getLibelleStringByLang(int langInteger) {
        ThesaurusLangData thesaurusLangInfo = getThesaurusLangDataByLang(langInteger);
        if (thesaurusLangInfo == null) return null;
        Libelle lib =  thesaurusLangInfo.getLibelle();
        if (lib == null) return null;
        return lib.getLibelleString();
    }
    
    public int getThesaurusLangDataCount() {
        return intLangMap.size();
    }
    
    public ThesaurusLangData getThesaurusLangDataByIndex(int index) {
        return (ThesaurusLangData) intLangMap.getValueAt(index);
    }
    
    public static class ThesaurusLangData {
        int langInt;
        InternalLibelle thesaurusLibelle;
        
        ThesaurusLangData(int langInt) {
            this.langInt = langInt;
        }
        
        
        public Libelle getLibelle() {
            return thesaurusLibelle;
        }
        
        public int getLangInteger() {
            return langInt;
        }
        
        /**
         * Appelé uniquement par ThesaurusInfoBuffer qui s'est assuré que s n'était pas nul ou de longueur nulle après un trim()
         */
        void setLibelle(String s) {
            thesaurusLibelle  = new InternalLibelle(s);
        }
        
        class InternalLibelle implements Libelle {
            String lib;
            
            InternalLibelle(String lib) {
                this.lib = lib;
            }
            
            public int getLangInteger() {
                return langInt;
            }
            
            public String getLibelleString() {
                return lib;
            }
            
            public boolean isVo() {
                return true;
            }
            
        }
    }
    
    public static class Builder {
        ThesaurusURI thesaurusURI;
        ArrayList libelleList = new ArrayList();
        ArrayList motcleInfoBufferList = new ArrayList();
        
        public Builder(ThesaurusURI thesaurusURI) {
            this.thesaurusURI = thesaurusURI;
        }
        
        public ThesaurusURI getThesaurusURI() {
            return thesaurusURI;
        }
        
        public List getMotcleDataBuilderList() {
            return Collections.unmodifiableList(motcleInfoBufferList);
        }
        
        public void addLibelle(Libelle libelle) {
            if (libelle == null) throw new IllegalArgumentException("libelle is null");
            libelleList.add(libelle);
        }
        
        public void addMotcleDataBuilder(MotcleData.Builder motcleDataBuilder) {
            if (motcleDataBuilder == null) throw new IllegalArgumentException("motcleInfoBuffer is null");
            motcleInfoBufferList.add(motcleDataBuilder);
        }
        
        
        public ThesaurusData toThesaurusData(ScrutariURICoder uriCoder) {
            int codethesaurus = uriCoder.getOrCreateCode(thesaurusURI);
            int codebase = uriCoder.getOrCreateCode(thesaurusURI.getBaseURI());
            LinkedIntHashMap linkedHashMap = new LinkedIntHashMap();
            for (int i = 0; i < libelleList.size();i++) {
                Libelle libelle = (Libelle) libelleList.get(i);
                String s = libelle.getLibelleString();
                if (s == null) continue;
                s = s.trim();
                if (s.length() == 0) continue;
                int langInt = libelle.getLangInteger();
                ThesaurusData.ThesaurusLangData langInfo = (ThesaurusData.ThesaurusLangData) linkedHashMap.get(langInt);
                if (langInfo == null) {
                    langInfo = new ThesaurusData.ThesaurusLangData(langInt);
                    linkedHashMap.put(langInt,langInfo);
                }
                langInfo.setLibelle(s);
            }
            int size = motcleInfoBufferList.size();
            int[] codesMotscles = new int[size];
            for (int i = 0; i < size;i++) {
                MotcleData.Builder motcleInfoBuffer = (MotcleData.Builder) motcleInfoBufferList.get(i);
                codesMotscles[i] = uriCoder.getOrCreateCode(motcleInfoBuffer.getMotcleURI());
            }
            return new ThesaurusData(thesaurusURI,codethesaurus,codebase,codesMotscles,linkedHashMap);
        }
        
    }
    
}
