/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.data;

import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.exceptions.*;

/**
 *
 * @author Vincent Calame
 */
public class URITupleBinding extends TupleBinding {
    
    public URITupleBinding() {
    }
    
    public Object entryToObject(TupleInput tupleInput) {
        return entryToScrutariURI(tupleInput);
    }
    
    public static ScrutariURI entryToScrutariURI(TupleInput tupleInput) {
        short type = tupleInput.readShort();
        switch(type) {
            case ScrutariURI.BASEURI_TYPE :
                return toBaseURI(tupleInput);
            case ScrutariURI.CORPUSURI_TYPE :
                return toCorpusURI(tupleInput);
            case ScrutariURI.THESAURUSURI_TYPE :
                return toThesaurusURI(tupleInput);
            case ScrutariURI.FICHEURI_TYPE :
                return toFicheURI(tupleInput);
            case ScrutariURI.MOTCLEURI_TYPE :
                return toMotcleURI(tupleInput);
            default :
                throw new SwitchException("tupleInput.readShort() = " + type);
                
        }
    }
    
    public void objectToEntry(Object obj, TupleOutput tupleOutput) {
        scrutariURIToEntry((ScrutariURI) obj, tupleOutput);
    }
    
    public static void scrutariURIToEntry(ScrutariURI scrutariURI, TupleOutput tupleOutput) {
        tupleOutput.writeShort(scrutariURI.getType());
        if (scrutariURI instanceof BaseURI) {
            toEntry((BaseURI) scrutariURI, tupleOutput);
        } else if (scrutariURI instanceof CorpusURI) {
            toEntry((CorpusURI) scrutariURI, tupleOutput);
        } else if (scrutariURI instanceof ThesaurusURI) {
            toEntry((ThesaurusURI) scrutariURI, tupleOutput);
        } else if (scrutariURI instanceof FicheURI) {
            toEntry((FicheURI) scrutariURI, tupleOutput);
        } else if (scrutariURI instanceof MotcleURI) {
            toEntry((MotcleURI) scrutariURI, tupleOutput);
        } else {
            throw new SwitchException("obj instanceof = " + scrutariURI.getClass().getName());
        }
    }
    
    public static DatabaseEntry toDatabaseEntry(ScrutariURI scrutariURI) {
        DatabaseEntry data = new DatabaseEntry();
        URITupleBinding bind = new URITupleBinding();
        bind.objectToEntry(scrutariURI,data);
        return data;
    }
    
    private static void toEntry(BaseURI baseURI, TupleOutput tupleOutput) {
        tupleOutput.writeString(baseURI.getAuthorityUUID());
        tupleOutput.writeString(baseURI.getBaseName());
    }
    
    private static void toEntry(CorpusURI corpusURI,TupleOutput tupleOutput) {
        toEntry(corpusURI.getBaseURI(), tupleOutput);
        tupleOutput.writeString(corpusURI.getCorpusName());
    }
    
    private static void toEntry(ThesaurusURI thesaurusURI,TupleOutput tupleOutput) {
        toEntry(thesaurusURI.getBaseURI(), tupleOutput);
        tupleOutput.writeString(thesaurusURI.getThesaurusName());
    }
    
    private static void toEntry(FicheURI ficheURI, TupleOutput tupleOutput) {
        toEntry(ficheURI.getCorpusURI(), tupleOutput);
        tupleOutput.writeString(ficheURI.getFicheId());
    }
    
    private static void toEntry(MotcleURI motcleURI, TupleOutput tupleOutput) {
        toEntry(motcleURI.getThesaurusURI(), tupleOutput);
        tupleOutput.writeString(motcleURI.getMotcleId());
    }
    
    
    
    private static BaseURI toBaseURI(TupleInput tupleInput) {
        String authorityUUID = tupleInput.readString();
        String baseName = tupleInput.readString();
        return new BaseURI(authorityUUID, baseName);
    }
    
    private static CorpusURI toCorpusURI(TupleInput tupleInput) {
        BaseURI baseURI = toBaseURI(tupleInput);
        String corpusName = tupleInput.readString();
        return new CorpusURI(baseURI, corpusName);
    }
    
    private static ThesaurusURI toThesaurusURI(TupleInput tupleInput) {
        BaseURI baseURI = toBaseURI(tupleInput);
        String thesaurusName = tupleInput.readString();
        return new ThesaurusURI(baseURI, thesaurusName);
    }
    
    private static FicheURI toFicheURI(TupleInput tupleInput) {
        CorpusURI corpusURI = toCorpusURI(tupleInput);
        String ficheId = tupleInput.readString();
        return new FicheURI(corpusURI, ficheId);
    }
    
    private static MotcleURI toMotcleURI(TupleInput tupleInput) {
        ThesaurusURI thesaurusURI = toThesaurusURI(tupleInput);
        String motcleId = tupleInput.readString();
        return new MotcleURI(thesaurusURI, motcleId);
    }
}
