/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;
import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class BestPosition implements FieldNumbers {
    private int code;
    private int bestFieldNumber = NONE_FIELDNUMBER;
    private CollationUnitPosition bestCollationUnitPosition;
    private int wholeLength;
    
    public BestPosition() {
    }
    
    public int getCode() {
        return code;
    }
    
    public void setCode(int code) {
        this.code = code;
    }
    
    /**
     * N'est applicable que lorsqu'il s'agit d'une fiche.
     */
    public int getBestFieldNumber() {
        return bestFieldNumber;
    }
    
    public void setBestFieldNumber(int bestFieldNumber) {
        this.bestFieldNumber = bestFieldNumber;
    }
    
    public CollationUnitPosition getBestCollationUnitPosition() {
        return bestCollationUnitPosition;
    }
    
    public void setBestCollationUnitPosition(CollationUnitPosition bestCollationUnitPosition) {
        this.bestCollationUnitPosition = bestCollationUnitPosition;
    }
    
    public int getWholeLength() {
        return wholeLength;
    }

    public void setWholeLength(int wholeLength) {
        this.wholeLength = wholeLength;
    }
    
    public DatabaseEntry toDatabaseEntry() {
        DatabaseEntry data = new DatabaseEntry();
        InternalTupleBinding bind = new InternalTupleBinding();
        bind.objectToEntry(this,data);
        return data;
    }
    
    
    public static BestPosition fromDatabaseEntry(DatabaseEntry data) {
        InternalTupleBinding bind = new InternalTupleBinding();
        return (BestPosition) bind.entryToObject(data);
    }
    
    public static void toEntry(BestPosition l, TupleOutput tupleOutput) {
        tupleOutput.writeInt(l.getCode());
        tupleOutput.writeInt(l.getBestFieldNumber());
        tupleOutput.writeInt(l.getWholeLength());
        LexieUtils.writeCollationUnitPosition(l.getBestCollationUnitPosition(),tupleOutput);
    }
    
    public static BestPosition toObject(TupleInput tupleInput) {
        BestPosition l = new BestPosition();
        l.setCode(tupleInput.readInt());
        l.setBestFieldNumber(tupleInput.readInt());
        l.setWholeLength(tupleInput.readInt());
        l.setBestCollationUnitPosition(LexieUtils.readCollationUnitPosition(tupleInput));
        return l;
    }
    
    static private final class InternalTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            return toObject(tupleInput);
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            BestPosition l = (BestPosition) obj;
            toEntry(l,tupleOutput);
        }
        
    }

    
}
