/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;
import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public final class LexieLinkKey {
    private long lexieId;
    private int code;
    
    public LexieLinkKey(long lexieId, int code) {
        this.lexieId = lexieId;
        this.code = code;
    }
    
    public DatabaseEntry toDatabaseEntry() {
        DatabaseEntry data = new DatabaseEntry();
        InternalTupleBinding bind = new InternalTupleBinding();
        bind.objectToEntry(this,data);
        return data;
    }
    
    public static DatabaseEntry toDatabaseEntry(long lexieId, int code) {
        LexieLinkKey l = new LexieLinkKey(lexieId,code);
        return l.toDatabaseEntry();
    }
    
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if (!(obj instanceof LexieLinkKey)) return false;
        LexieLinkKey other = (LexieLinkKey) obj;
        return (this.code == other.code) && (this.lexieId == other.lexieId);
    }
    
    public int hashCode() {
        return code + (int) lexieId;
    }
    
    
    public static LexieLinkKey fromDatabaseEntry(DatabaseEntry data) {
        InternalTupleBinding bind = new InternalTupleBinding();
        return (LexieLinkKey) bind.entryToObject(data);
    }
    
    static private final class InternalTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            long lexieId = tupleInput.readLong();
            int code = tupleInput.readInt();
            return new LexieLinkKey(lexieId, code);
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            LexieLinkKey l = (LexieLinkKey) obj;
            tupleOutput.writeLong(l.getLexieId());
            tupleOutput.writeInt(l.getCode());
        }
    }

    public long getLexieId() {
        return lexieId;
    }

    public int getCode() {
        return code;
    }
    
}
