/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;
import java.io.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieUtils {
    
    private LexieUtils() {
    }
    
    public static DatabaseEntry toDatabaseEntry(BestPosition[] array) {
        DatabaseEntry data = new DatabaseEntry();
        ArrayTupleBinding bind = new ArrayTupleBinding();
        bind.objectToEntry(array,data);
        return data;
    }
    
     public static BestPosition[] fromDatabaseEntry(DatabaseEntry data) {
         ArrayTupleBinding bind = new ArrayTupleBinding();
         return (BestPosition[]) bind.entryToObject(data);
     }
    
    private static void writeBestPositionArray(BestPosition[] array,TupleOutput tupleOutput) {
        int length = array.length;
        tupleOutput.writeInt(length);
        for(int i = 0; i < length; i++) {
            BestPosition.toEntry(array[i],tupleOutput);
        }
    }
    
    private static BestPosition[] readBestPositionArray(TupleInput tupleInput) {
        int length = tupleInput.readInt();
        BestPosition[] array = new BestPosition[length];
        for(int i = 0; i < length; i++) {
            array[i] = BestPosition.toObject(tupleInput);
        }
        return array;
    }
    
    public static void writeCollationUnitPosition(CollationUnitPosition collationUnitPosition,TupleOutput tupleOutput) {
        SubstringPosition sp = collationUnitPosition.getPositionInWholeSource();
        tupleOutput.writeInt(sp.getBeginIndex());
        tupleOutput.writeInt(sp.getLength());
        int[] array = collationUnitPosition.getCorrespondances();
        int length = array.length;
        tupleOutput.writeInt(length);
        for(int i = 0; i < length; i++) {
            tupleOutput.writeInt(array[i]);
        }
    }
    
    public static CollationUnitPosition readCollationUnitPosition(TupleInput tupleInput) {
        int beginIndex = tupleInput.readInt();
        int length = tupleInput.readInt();
        SubstringPosition sp = new SubstringPosition(beginIndex,length);
        int al = tupleInput.readInt();
        int[] correspondances = new int[al];
        for(int i = 0; i < al; i++) {
            correspondances[i] = tupleInput.readInt();
        }
        return new CollationUnitPosition(sp,correspondances);
        
    }
    
    static private final class ArrayTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            return readBestPositionArray(tupleInput);
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            BestPosition[] l = (BestPosition[]) obj;
            writeBestPositionArray(l,tupleOutput);
        }
        
    }
}
