/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;

/**
 *
 * @author Vincent Calame
 */
public class Lexification {
    private long[] lexieIdArray;
    private CollationUnitPosition[] collationUnitPositionArray;
    private int wholeLength;
    
    public Lexification() {
    }
    
    public int getCount() {
        return lexieIdArray.length;
    }
    
    public long getLexieId(int i) {
        return lexieIdArray[i];
    }
    
    public CollationUnitPosition getCollationUnitPosition(int i) {
        return collationUnitPositionArray[i];
    }
    
    public void setLexieIdArray(long[] lexieIdArray) {
        this.lexieIdArray = lexieIdArray;
    }

    public void setCollationUnitPositionArray(CollationUnitPosition[] collationUnitPositionArray) {
        this.collationUnitPositionArray = collationUnitPositionArray;
    }
    
    public int getWholeLength() {
        return wholeLength;
    }

    public void setWholeLength(int wholeLength) {
        this.wholeLength = wholeLength;
    }
    
    public static Lexification readTuple(TupleInput tupleInput) {
        Lexification lexification = new Lexification();
        lexification.setWholeLength(tupleInput.readInt());
        int length = tupleInput.readInt();
        long[] lexieIdArray = new long[length];
        CollationUnitPosition[] collationUnitPositionArray = new CollationUnitPosition[length];
        for(int i = 0; i < length; i++) {
            lexieIdArray[i] = tupleInput.readLong();
            collationUnitPositionArray[i] = LexieUtils.readCollationUnitPosition(tupleInput);
        }
        lexification.setLexieIdArray(lexieIdArray);
        lexification.setCollationUnitPositionArray(collationUnitPositionArray);
        return lexification;
    }
    
    public static void writeTuple(Lexification lexification, TupleOutput tupleOutput) {
        tupleOutput.writeInt(lexification.getWholeLength());
        int count = lexification.getCount();
        tupleOutput.writeInt(count);
        for(int i = 0; i < count; i++) {
            tupleOutput.writeLong(lexification.getLexieId(i));
            LexieUtils.writeCollationUnitPosition(lexification.getCollationUnitPosition(i),tupleOutput);
        }
    }
    
}
