/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import net.mapeadores.util.text.collation.CollationUnitPosition;


/**
 *
 * @author Vincent Calame
 */
public class MotcleAndLexieLink {
    private int codemotcle;
    private int codethesaurus;
    private CollationUnitPosition wordPosition;
    private ScrutariLexieUnit scrutariLexieUnit;
    private InMotclePosition bestPosition;
    private InMotclePosition[] positionArray;
    
    public MotcleAndLexieLink(ScrutariLexieUnit scrutariLexieUnit,int codemotcle,int codethesaurus,CollationUnitPosition wordPosition, InMotclePosition previousPosition) {
        this.codemotcle = codemotcle;
        this.codethesaurus = codethesaurus;
        this.scrutariLexieUnit = scrutariLexieUnit;
        InMotclePosition inmotclePosition = new InMotclePosition(wordPosition,previousPosition);
        bestPosition = inmotclePosition;
        positionArray = new InMotclePosition[1];
        positionArray[0] = inmotclePosition;
    }
    
    public int getCodeMotcle() {
        return codemotcle;
    }
    
    public int getCodeThesaurus() {
        return codethesaurus;
    }
    
    public CollationUnitPosition getCollationUnitPosition() {
        return bestPosition.getCollationUnitPosition();
    }
    
    public String getCollatedLexie() {
        return scrutariLexieUnit.getCollatedLexie();
    }
    
    
    public InMotclePosition[] getInMotclePositionArray() {
        return positionArray;
    }
    
    public InMotclePosition getBestPosition() {
        return bestPosition;
    }
        
    public InMotclePosition add(CollationUnitPosition wordPosition,InMotclePosition previousPosition) {
        InMotclePosition inmotclePosition = new InMotclePosition(wordPosition,previousPosition);
        if (inmotclePosition.compareTo(bestPosition) < 0) {
            bestPosition = inmotclePosition;
        }
        int length = positionArray.length;
        InMotclePosition[] temp = new InMotclePosition[length + 1];
        System.arraycopy(positionArray, 0, temp, 0, length);
        temp[length] = inmotclePosition;
        positionArray = temp;
        return inmotclePosition;
    }
    
    public class InMotclePosition {
        private CollationUnitPosition collationUnitPosition;
        private InMotclePosition previousPosition;
        
        public InMotclePosition(CollationUnitPosition collationUnitPosition, InMotclePosition previousPosition) {
            this.collationUnitPosition = collationUnitPosition;
            this.previousPosition = previousPosition;
        }
        
        public CollationUnitPosition getCollationUnitPosition() {
            return collationUnitPosition;
        }
        
        public InMotclePosition getPrevious() {
            return previousPosition;
        }
        
        public MotcleAndLexieLink getMotcleAndLexieLink() {
            return MotcleAndLexieLink.this;
        }
        
        public int compareTo(InMotclePosition other) {
            return collationUnitPosition.compareTo(other.collationUnitPosition);
        }
    }

    
    
}
