/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;

/**
 *
 * @author Vincent Calame
 */
public class OccurrenceList {
    private int code;
    private int[] fieldNumberArray;
    private int[] lexieIndexArray;
    
    public OccurrenceList() {
    }
    
    public int getCode() {
        return code;
    }
    
    public void setCode(int code) {
        this.code = code;
    }
    
    public void setFieldNumberArray(int[] fieldNumberArray) {
        this.fieldNumberArray = fieldNumberArray;
    }
    
    public void setLexieIndexArray(int[] lexieIndexArray) {
        this.lexieIndexArray = lexieIndexArray;
    }
    
    public int getOccurenceCount() {
        return lexieIndexArray.length;
    }
    
    public int getFieldNumber(int i) {
        return fieldNumberArray[i];
    }
    
    public int getLexieIndex(int i) {
        return lexieIndexArray[i];
    }
    
    public boolean withFieldNumber() {
        return (fieldNumberArray != null);
    }
    
    public DatabaseEntry toDatabaseEntry() {
        DatabaseEntry data = new DatabaseEntry();
        InternalTupleBinding bind = new InternalTupleBinding();
        bind.objectToEntry(this,data);
        return data;
    }
    
    public static DatabaseEntry toArrayDatabaseEntry(OccurrenceList[] occurrenceListArray) {
        DatabaseEntry data = new DatabaseEntry();
        ArrayInternalTupleBinding bind = new ArrayInternalTupleBinding();
        bind.objectToEntry(occurrenceListArray,data);
        return data;
        
    }
    
    public static OccurrenceList[] fromArrayDatabaseEntry(DatabaseEntry data) {
        ArrayInternalTupleBinding bind = new ArrayInternalTupleBinding();
        return (OccurrenceList[]) bind.entryToObject(data);
    }
    
    public static OccurrenceList fromDatabaseEntry(DatabaseEntry data) {
        InternalTupleBinding bind = new InternalTupleBinding();
        return (OccurrenceList) bind.entryToObject(data);
    }
    
    public static OccurrenceList[] readArrayTuple(TupleInput tupleInput) {
        int length = tupleInput.readInt();
        OccurrenceList[] array = new OccurrenceList[length];
        for(int i = 0; i < length; i++) {
            array[i] = readTuple(tupleInput);
        }
        return array;
    }
    
    public static OccurrenceList readTuple(TupleInput tupleInput) {
        OccurrenceList occurrenceList = new OccurrenceList();
        occurrenceList.code = tupleInput.readInt();
        int length = tupleInput.readInt();
        boolean withFieldNumber = tupleInput.readBoolean();
        int[] lexieIndexArray = new int[length];
        int[] fieldNumberArray = null;
        if (withFieldNumber) {
            fieldNumberArray = new int[length];
        }
        for(int i = 0; i < length; i++) {
            lexieIndexArray[i] = tupleInput.readInt();
            if (withFieldNumber) {
                fieldNumberArray[i] = tupleInput.readInt();
            }
        }
        occurrenceList.setLexieIndexArray(lexieIndexArray);
        if (withFieldNumber) occurrenceList.setFieldNumberArray(fieldNumberArray);
        return occurrenceList;
    }
    public static void writeArrayTuple(OccurrenceList[] occurrenceListArray, TupleOutput tupleOutput) {
        int length = occurrenceListArray.length;
        tupleOutput.writeInt(length);
        for(int i = 0; i < length; i++) {
            writeTuple(occurrenceListArray[i], tupleOutput);
        }
    }
    
    public static void writeTuple(OccurrenceList occurrenceList, TupleOutput tupleOutput) {
        tupleOutput.writeInt(occurrenceList.getCode());
        int length = occurrenceList.getOccurenceCount();
        tupleOutput.writeInt(length);
        boolean withFieldNumber = occurrenceList.withFieldNumber();
        tupleOutput.writeBoolean(withFieldNumber);
        for(int i = 0; i < length; i++) {
            tupleOutput.writeInt(occurrenceList.getLexieIndex(i));
            if (withFieldNumber) tupleOutput.writeInt(occurrenceList.getFieldNumber(i));
        }
    }
    
    static private final class InternalTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            return readTuple(tupleInput);
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            writeTuple((OccurrenceList) obj, tupleOutput);
            
        }
        
    }
    
    static private final class ArrayInternalTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            return readArrayTuple(tupleInput);
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            writeArrayTuple((OccurrenceList[]) obj, tupleOutput);
            
        }
        
    }

}
