/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.lexie;

import net.mapeadores.util.text.lexie.*;
import net.mapeadores.util.primitives.*;
import com.sleepycat.bind.tuple.*;
import com.sleepycat.je.*;

/**
 * Point particulier des unités lexicales : elles correspondent à une langue donnée. 
 * 
 * @author Vincent Calame
 */
public class ScrutariLexieUnit implements LexieUnit {
    long lexieId;
    private String collatedLexie;
    private String canonicalLexie;
    private boolean hasFicheLink = false;
    private boolean hasMotcleLink = false;
    

    public ScrutariLexieUnit(long lexieId,String collatedLexie) {
        this.lexieId = lexieId;
        this.collatedLexie = collatedLexie;
    }
    
    public int getLangInteger() {
        return PrimUtils.getInt1(lexieId);
    }
    
    public String getCollatedLexie() {
        return collatedLexie;
        
    }
    
    public long getLexieId() {
        return lexieId;
    }
    
    public String getCanonicalLexie() {
        return canonicalLexie;
    }
    
    public void setCanonicalLexie(String canonicalLexie) {
        this.canonicalLexie = canonicalLexie;
    }
    
    public boolean hasMotcleAndLexieLink() {
        return hasMotcleLink;
    }
    
    public boolean hasFicheAndLexieLink() {
        return hasFicheLink;
    }
    
    public void setHasFicheLink(boolean hasFicheLink) {
        this.hasFicheLink = hasFicheLink;
    }

    public void setHasMotcleLink(boolean hasMotcleLink) {
        this.hasMotcleLink = hasMotcleLink;
    }
    
    public DatabaseEntry toDatabaseEntry() {
        DatabaseEntry data = new DatabaseEntry();
        InternalTupleBinding bind = new InternalTupleBinding();
        bind.objectToEntry(this,data);
        return data;
    }
    
    public static ScrutariLexieUnit fromDatabaseEntry(DatabaseEntry data) {
        InternalTupleBinding bind = new InternalTupleBinding();
        return (ScrutariLexieUnit) bind.entryToObject(data);
    }
    
    static private final class InternalTupleBinding extends TupleBinding {
        
        public Object entryToObject(TupleInput tupleInput) {
            long lexieId = tupleInput.readLong();
            String collatedLexie = tupleInput.readString();
            ScrutariLexieUnit u = new ScrutariLexieUnit(lexieId, collatedLexie);
            u.setCanonicalLexie(tupleInput.readString());
            u.setHasFicheLink(tupleInput.readBoolean());
            u.setHasMotcleLink(tupleInput.readBoolean());
            return u;
        }
        
        public void objectToEntry(Object obj, TupleOutput tupleOutput) {
            ScrutariLexieUnit u = (ScrutariLexieUnit) obj;
            tupleOutput.writeLong(u.lexieId);
            tupleOutput.writeString(u.getCollatedLexie());
            tupleOutput.writeString(u.getCanonicalLexie());
            tupleOutput.writeBoolean(u.hasFicheLink);
            tupleOutput.writeBoolean(u.hasMotcleLink);
        }
    }

}
