/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.searchengine;

import net.mapeadores.util.text.SearchSequence;

/**
 *
 * @author Vincent Calame
 */
public interface SearchResult {
    /**
     * Retourne le nombre total de mots-clés qui contiennent au moins un mot de
     * la séquence de recherche.
     */
    public int getAllMotcleCount();

    /**
     * Retourne le nombre total de mots-clés qui contiennent tous les mots de la
     * séquence de recherche.
     */
    public int getCompleteMotcleCount();

    /**
     * Retourne le nombre de fiches dans le résultat de recherche.
     */
    public int getFicheCount();

    /**
     * Retourne le nombre maximum de fiches qui auraient pu se trouver dans
     * le résultat de recherche. Autrement dit, c'est le nombre de fiches qui répondent aux
     * options de recherche (limitation à certaines bases ou corpus) sur lesquelles ont a fait
     * réellement une recherche de présence de mots.
     */
    public int getFicheMaximum();

    /**
     * Retourne la i-ième instance de FicheResult appartenant au résultat.
     * 0 =< i < getFicheCount().
     */
    public FicheSearchResultInfo getFicheResult(int i);

    /**
     * Retourne l'identifiant de la recherche.
     */
    public String getIdentifiant();

    /**
     * Retourne la i-ième instance de MotcleResult appartenant au résultat.
     * 0 =< i < getAllMotcleCount().
     */
    public MotcleSearchResultInfo getMotcleResult(int i);

    /**
     * Retourne le nombre total de mots-clés qui contiennt au moins un mais pas tous
     * les mots de la séquence de recherche. Est égal à getAllMotcleCount() - getCompleteMotcleCount()
     */
    public int getPartialMotcleCount();

    /**
     * Retourne les options de recherche qui ont permis de filtrer les fiches.
     */
    public SearchOptions getSearchOptions();

    /**
     * Retourne la séquence de recherche à l'origine de la recherche.
     */
    public SearchSequence getSearchSequence();
    
}
