/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.searchengine.inner;

import java.io.*;
import net.mapeadores.scrutari.searchengine.beans.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.scrutari.searchengine.pertinence.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import java.util.*;


/**
 * Classe de construction d'un résultat d'une recherche.
 *
 * @author  Vincent Calame
 */
public class SearchResultBuilder {
    String identifiant;
    SearchSequence sequence;
    SearchOptions options;
    int allMotcleCount = 0;
    int totalmotsclescomplets = 0;
    int ficheMaximum = 0;
    FicheSearchResultInfoBean[] ficheResults;
    MotcleSearchResultInfoBean[] motcleResults;
    int pas = 0;
    int pasmotcle = 0;

    public SearchResultBuilder(String identifiant, SearchSequence sequence,SearchOptions options) {
        this.identifiant = identifiant;
        this.sequence = sequence;
        this.options = options;
    }
    
    public SearchResultBean toBean() {
        SearchResultBean searchResultBean = new SearchResultBean();
        searchResultBean.setIdentifiant(identifiant);
        searchResultBean.setSearchSequence(sequence);
        searchResultBean.setSearchOptions(options);
        searchResultBean.setFicheMaximum(ficheMaximum);
        searchResultBean.setCompleteMotcleCount(totalmotsclescomplets);
        searchResultBean.setFicheSearchResultInfoBeanArray(ficheResults);
        searchResultBean.setMotcleSearchResultInfoBeanArray(motcleResults);
        return searchResultBean;
    }
    
    public void initFicheCount(int ficheCount) {
        ficheResults = new FicheSearchResultInfoBean[ficheCount];
        pas = 0;
    }
    
    public void initMotcleCount(int i) {
        allMotcleCount = i;
        motcleResults = new MotcleSearchResultInfoBean[i];
        pasmotcle = 0;
    }
    
    /*
     * Précise le nombre total de mots-clés complets.
     */
    public void setCompleteMotcleCount(int i) {
        totalmotsclescomplets = i;
    }
    
    public void setFicheMaximum(int ficheMaximum) {
        this.ficheMaximum = ficheMaximum;
    }
    
    /*
     * Ajoute une nouvelle instance de FicheResult à partir de l'objet FichePertinence en
     * argument.
     */
    public void addFicheResult(FichePertinence fichePertinence) {
        ficheResults[pas] = toBean(fichePertinence);
        pas++;
    }
    
    /*
     * Ajoute une nouvelle instance de MotcleResult à partir de l'objet MotclePertinence en
     * argument.
     */
    public void addMotcleResult(IndexationPertinence indexationPertinence) {
        motcleResults[pasmotcle] = toBean(indexationPertinence);
        pasmotcle++;
    }

    
    
    private MotcleSearchResultInfoBean toBean(IndexationPertinence indexationPertinence) {
        MotcleSearchResultInfoBean bean = new MotcleSearchResultInfoBean();
        MotclePertinence motclePertinence = indexationPertinence.getMotclePertinence();
        bean.setCode(motclePertinence.getCodeMotcle());
        bean.setMotcleSearchTokenPositionListBeanArray(getMotcleSearchTokenPositionListBeanArray(motclePertinence));
        bean.setAcceptedCodeficheArray(getAcceptedCodeficheArray(indexationPertinence));
        bean.setFicheMaximum(indexationPertinence.getFicheMaximum());
        return bean;
    }
    
    
    private int[] getAcceptedCodeficheArray(IndexationPertinence indexationPertinence) {
        int acceptedFicheCount = indexationPertinence.getAcceptedFicheCount();
        int[] acceptedCodeficheArray = new int[acceptedFicheCount];
        for(int i = 0; i < acceptedFicheCount;i++) {
            acceptedCodeficheArray[i] = indexationPertinence.getAcceptedFiche(i).getCodeFiche();
        }
        return acceptedCodeficheArray;
    }
    
    
    private MotcleSearchTokenPositionListBean[] getMotcleSearchTokenPositionListBeanArray(MotclePertinence motclePertinence) {
        LinkedIntHashMap langMap = new LinkedIntHashMap();
        int concernedLangCount = motclePertinence.getConcernedLangCount();
        for(int i = 0; i < concernedLangCount;i++) {
            int concernedLangInteger = motclePertinence.getConcernedLangInteger(i);
            SubstringPosition[] array = motclePertinence.getSubstringPositionByTokenArray(i);
            int tokenCount = array.length;
            SearchTokenPositionListBuilder builder = (SearchTokenPositionListBuilder) langMap.get(concernedLangInteger);
            if (builder == null) {
                builder = new SearchTokenPositionListBuilder(tokenCount);
                langMap.put(concernedLangInteger, builder);
            }
            for (int tokenIndex = 0; tokenIndex < tokenCount;tokenIndex++) {
                SubstringPosition substringPosition = array[tokenIndex];
                if (substringPosition != null) builder.add(tokenIndex,substringPosition);
            }
        }
        int size = langMap.size();
        MotcleSearchTokenPositionListBean[] beanArray = new MotcleSearchTokenPositionListBean[size];
        for(int i = 0; i < size; i++) {
            int langInteger = langMap.getKeyAt(i);
            SearchTokenPositionListBuilder builder = (SearchTokenPositionListBuilder) langMap.getValueAt(i);
            MotcleSearchTokenPositionListBean bean = new MotcleSearchTokenPositionListBean();
            bean.setLangInteger(langInteger);
            bean.setComplete(builder.isComplete());
            bean.setSearchTokenPositionBeanArray(builder.getSearchTokenPositionBeanArray());
            beanArray[i] = bean;
        }
        return beanArray;
    }
    
    
    private FicheSearchResultInfoBean toBean(FichePertinence currentPertinence) {
        FicheSearchResultInfoBean currentBean = new FicheSearchResultInfoBean();
        currentBean.setCode(currentPertinence.getCodeFiche());
        currentBean.setCodemotcleArray(getCodemotcleArray(currentPertinence));
        currentBean.setFicheSearchTokenPositionListBeanArray(getFicheSearchTokenPositionListBeanArray(currentPertinence));
        return currentBean;
    }
    
    private int[] getCodemotcleArray(FichePertinence fichePertinence) {
        if (fichePertinence.getMotcleCompletudeMask() < 1) return null;
        MotclePertinence[] pertinenceArray = fichePertinence.getMotclePertinenceByTokenArray();
        int length = pertinenceArray.length;
        int[] codesmotscles = new int[length];
        if (length == 1) {
            codesmotscles[0] = pertinenceArray[0].getCodeMotcle();
        } else {
            int p = 0;
            for (int i = 0; i < length;i++) {
                if (pertinenceArray[i] != null) {
                    int codemotcle = pertinenceArray[i].getCodeMotcle();
                    boolean here = false;
                    for (int j = 0; j < p;j++) {
                        if (codesmotscles[j] == codemotcle) {
                            here = true;
                            break;
                        }
                    }
                    if (!here) {
                        codesmotscles[p] = codemotcle;
                        p++;
                    }
                }
            }
            if (p < length) {
                int[] nv = new int[p];
                System.arraycopy(codesmotscles,0, nv, 0, p);
                codesmotscles = nv;
            }
        }
        return codesmotscles;
    }
    
    private FicheSearchTokenPositionListBean[] getFicheSearchTokenPositionListBeanArray(FichePertinence fichePertinence) {
        int[] fieldNumbersByToken = fichePertinence.getFieldNumberByTokenArray();
        SubstringPosition[] substringPositionByToken = fichePertinence.getSubstringPositionByTokenArray();
        int tokenCount = fieldNumbersByToken.length;
        
        LinkedIntHashMap intHashMap = new LinkedIntHashMap();
        
        for (int tokenIndex = 0; tokenIndex < tokenCount;tokenIndex++) {
            SubstringPosition substringPosition = substringPositionByToken[tokenIndex];
            if (substringPosition == null) continue;
            int fieldNumber = fieldNumbersByToken[tokenIndex];
            SearchTokenPositionListBuilder builder = (SearchTokenPositionListBuilder) intHashMap.get(fieldNumber);
            if (builder == null) {
                builder = new SearchTokenPositionListBuilder(tokenCount);
                intHashMap.put(fieldNumber, builder);
            }
            builder.add(tokenIndex,substringPosition);
        }
        int size = intHashMap.size();
        FicheSearchTokenPositionListBean[] beanArray = new FicheSearchTokenPositionListBean[size];
        for(int i = 0; i < size; i++) {
            int fieldNumber = intHashMap.getKeyAt(i);
            SearchTokenPositionListBuilder builder = (SearchTokenPositionListBuilder) intHashMap.getValueAt(i);
            FicheSearchTokenPositionListBean bean = new FicheSearchTokenPositionListBean();
            bean.setFieldNumber(fieldNumber);
            bean.setComplete(builder.isComplete());
            bean.setSearchTokenPositionBeanArray(builder.getSearchTokenPositionBeanArray());
            beanArray[i] = bean;
        }
        return beanArray;
    }
    
    
    
    
    private class SearchTokenPositionListBuilder {
        private SubstringPosition[] positionArray;
        private int[] tokenIndices;
        private int size = 0;
        
        SearchTokenPositionListBuilder(int tokenCount) {
            positionArray = new SubstringPosition[tokenCount];
            tokenIndices = new int[tokenCount];
        }
        
        void add(int tokenIndex, SubstringPosition substringPosition) {
            boolean insert = false;
            for (int i = 0; i < size; i++) {
                if (substringPosition.compareTo(positionArray[i]) < 0) {
                    System.arraycopy(positionArray, i, positionArray,i+1, size -i);
                    System.arraycopy(tokenIndices, i, tokenIndices,i+1, size -i);
                    tokenIndices[i] = tokenIndex;
                    positionArray[i] = substringPosition;
                    insert = true;
                    break;
                }
            }
            if (!insert) {
                tokenIndices[size] = tokenIndex;
                positionArray[size] = substringPosition;
            }
            size++;
        }
        
        SearchTokenPositionBean[] getSearchTokenPositionBeanArray() {
            SearchTokenPositionBean[] beanArray = new SearchTokenPositionBean[size];
            for(int i = 0; i < size; i++) {
                SearchTokenPositionBean bean = new SearchTokenPositionBean();
                bean.setSearchTokenIndex(tokenIndices[i]);
                bean.setSubstringPosition(positionArray[i]);
                beanArray[i] = bean;
            }
            return beanArray;
        }
        
        boolean isComplete() {
            return (size == positionArray.length);
        }
        
    }
    
    
    
    
    
    
    
}
