/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.searchengine.pertinence;

import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class IndexationPertinence {
    private MotclePertinence motclePertinence;
    private ArrayList fichePertinenceList = new ArrayList();
    private FichePertinence[] fichePertinenceArray;
    private int acceptedFicheCount;
    
    public IndexationPertinence(MotclePertinence motclePertinence) {
        this.motclePertinence = motclePertinence;
    }

    public MotclePertinence getMotclePertinence() {
        return motclePertinence;
    }
    
    /**
     * Retourne le nombre total de fiches finalement acceptées indexées par le mot-clé.
     */
    public int getAcceptedFicheCount() {
        return acceptedFicheCount;
    }
    
    /**
     * Retourne la i-ième fiche acceptée indexée par le mot-clé.
     * Ces fiches sont classées par ordre de pertinence.
     */
    public FichePertinence getAcceptedFiche(int i) {
        return fichePertinenceArray[i];
    }
    
    /**
     * Retourne le nombre maximum de fiches susceptibles d'être associées suivant 
     * les options de recherche. Autrement dit, c'est le nombre total de fiches
     * indexées par le mot-clé dans les corpus et bases sélectionnées.
     */
    public int getFicheMaximum() {
        return fichePertinenceArray.length;
    }
    
    /*
     * Permet d'indiquer l'indexation d'une fiche par un mot-clé.
     */
    public void addFichePertinence(FichePertinence fichePertinence) {
        fichePertinenceList.add(fichePertinence);
    }
    
     /*
     * Trie les fiches par pertinence. Appelé en fin de traitement (puisque cela se base sur la valeur de pertinence).
     */
    public void sortPertinenceFiche() {
        int size = fichePertinenceList.size();
        FichePertinence[] array = new FichePertinence[size];
        int acceptIndex = 0;
        int refuseIndex = size -1;
        for(int i = 0; i < size;i++) {
            FichePertinence fichePertinence = (FichePertinence) fichePertinenceList.get(i);
            if (fichePertinence.getPertinenceValue() < 1) {
                array[refuseIndex] = fichePertinence;
                refuseIndex--;
            }
            else {
                array[acceptIndex] = fichePertinence;
                acceptIndex++;
            }
        }
        if (acceptIndex > 1) {
            Arrays.sort(array,0,acceptIndex,FichePertinence.ACCEPTED_COMPARATOR);
        }
        acceptedFicheCount = acceptIndex;
        fichePertinenceArray = array;     
        fichePertinenceList = null;
    }

    
}
