/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.searchengine.seeker;

import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieSeeker {
    LexieManager2 lexieManager;
    SearchEngine searchEngine;
    LinkedIntHashMap ficheMap = new LinkedIntHashMap();
    LinkedIntHashMap motcleMap = new LinkedIntHashMap();
    SimpleTokenSeeker simpleTokenSeeker;
    MultiTokenSeeker subtokenSeeker;
    
    public LexieSeeker(SearchEngine searchEngine,LexieManager2 lexieManager) {
        this.lexieManager = lexieManager;
        this.searchEngine = searchEngine;
        simpleTokenSeeker = new SimpleTokenSeeker(this);
        subtokenSeeker = new MultiTokenSeeker(this);
    }
    
    public void seek(SearchSequence searchSequence, SearchOptions searchOptions) {
        int[] langIntegerArray = searchEngine.getLangIntegerArray();
        for (int k = 0;k < langIntegerArray.length;k++) {
            int langInteger = langIntegerArray[k];
            simpleTokenSeeker.setLangInteger(langInteger);
            subtokenSeeker.setLangInteger(langInteger);
            SearchSequence locSequence = searchSequence.filter(LocalisationLexieFilters.getLexieFilter(langInteger),searchOptions.getDefaultSearchType());
            ficheMap.clear();
            motcleMap.clear();
            for (int i = 0; i < locSequence.getTokenCount();i++) {
                SearchSequence.Token token = locSequence.getToken(i);
                if (token.isMultiToken()) {
                    SearchSequence.MultiToken multiToken = (SearchSequence.MultiToken) token;
                    int count = multiToken.getSubtokenCount();
                    if (count > 1) {
                        subtokenSeeker.setToken(multiToken);
                        subtokenSeeker.seek();
                    } else if (count == 1) {
                        simpleTokenSeeker.setToken(multiToken.getSubtokenArray()[0]);
                        simpleTokenSeeker.seek();
                    }
                } else {
                    simpleTokenSeeker.setToken((SearchSequence.SimpleToken) token);
                    simpleTokenSeeker.seek();
                }
            }
            if (locSequence.hasNonPertinentTokens()) {
                int[] np = locSequence.getNonPertinentIndexArray();
                int npLength = np.length;
                int ficheMapSize = ficheMap.size();
                for(int i = 0; i < ficheMapSize;i++) {
                    int codefiche = ficheMap.getKeyAt(i);
                    int codecorpus = ((Integer) ficheMap.getValueAt(i)).intValue();
                    for(int j=0; j < npLength;j++) {
                        searchEngine.addFichePertinence(codefiche,codecorpus, np[j], FicheAndLexieLink.NOPERTINENT_FIELDNUMBER, null);
                    }
                }
                int motcleMapSize = motcleMap.size();
                for(int i = 0; i < motcleMap.size();i++) {
                    int codemotcle = motcleMap.getKeyAt(i);
                    int codethesaurus = ((Integer) motcleMap.getValueAt(i)).intValue();
                    for(int j=0; j < npLength;j++) {
                        searchEngine.addMotclePertinence(codemotcle,codethesaurus, np[j], langInteger, null, 0.0f);
                    }
                }
            }
        }
    }
    
    public SearchEngine getSearchEngine() {
        return searchEngine;
    }
    
    public LexieManager2 getLexieManager() {
        return lexieManager;
    }
    
    public void addFiche(int codefiche, int codecorpus) {
        ficheMap.put(codefiche, new Integer(codecorpus));
    }
    
    public void addMotcle(int codemotcle, int codethesaurus) {
        motcleMap.put(codemotcle, new Integer(codethesaurus));
    }
    
    
    public List search(String searchText,int langInteger,int type) {
        CollatedKeyMap collatedStringMap = lexieManager.getCollatedLexieMapByLang(langInteger);
        if (collatedStringMap == null) return Collections.EMPTY_LIST;
        return collatedStringMap.search(searchText,type);
    }
    
    public String toCollatedKey(int langInteger, String lexie) {
        CollatedKeyMap map = lexieManager.getCollatedLexieMapByLang(langInteger);
        return map.toCollatedKey(lexie);
    }
    
    
}
