/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.searchengine.seeker;

import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.langinteger.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class MultiTokenSeeker {
    private LexieSeeker lexieSeeker;
    private LexieQuery lexieQuery;
    private SearchSequence.MultiToken multiToken;
    PotentialLexieSequence potentialLexieSequence;
    int langInteger;
    
    MultiTokenSeeker(LexieSeeker lexieSeeker) {
        this.lexieSeeker = lexieSeeker;
        this.lexieQuery = lexieSeeker.getLexieManager().getLexieQuery();
    }
    
    
    public void setLangInteger(int langInteger) {
        this.langInteger = langInteger;
    }
    
    public void setToken(SearchSequence.MultiToken multiToken) {
        this.multiToken = multiToken;
    }
    
    
    void seek() {
        SearchSequence.SimpleToken[] subtokenArray = multiToken.getSubtokenArray();
        potentialLexieSequence = new PotentialLexieSequence(subtokenArray);
        if (!potentialLexieSequence.isInit()) return;
        int length = potentialLexieSequence.getFirstLexieCount();
        for(int i = 0; i < length; i++) {
            scanOccurenceList(i);
        }
    }
    
    private void scanOccurenceList(int firstTokenLexieIndex) {
        SearchEngine searchEngine = lexieSeeker.getSearchEngine();
        int[] codecorpusArray = searchEngine.getCodescorpusArray();
        int  codecorpusLength = codecorpusArray.length;
        long lexieId = potentialLexieSequence.getFirstLexie(firstTokenLexieIndex);
        for (int j = 0; j < codecorpusLength;j++) {
            int codecorpus = codecorpusArray[j];
            OccurrenceList[] array = lexieQuery.getOccurrenceListArray(lexieId, codecorpus);
            if (array == null) continue;
            int length = array.length;
            for(int k= 0; k < length; k++) {
                scanFicheLexification(firstTokenLexieIndex,codecorpus,array[k]);
            }
        }
        if (searchEngine.isWithMotscles()) {
            int[] codesthesaurusArray = searchEngine.getCodesthesaurusArray();
            int codethesaurusLength = codesthesaurusArray.length;
            for (int j = 0; j < codethesaurusLength;j++) {
                int codethesaurus = codesthesaurusArray[j];
                OccurrenceList[] array = lexieQuery.getOccurrenceListArray(lexieId, codethesaurus);
                 if (array == null) continue;
                int length = array.length;
                for(int k= 0; k < length; k++) {
                    scanMotcleLexification(firstTokenLexieIndex,codethesaurus,array[k]);
                }
            }
        }
    }
    
    private void scanFicheLexification(int firstTokenLexieIndex,int codecorpus,OccurrenceList occurrenceList) {
        SearchEngine searchEngine = lexieSeeker.getSearchEngine();
        int codefiche = occurrenceList.getCode();
        int count = occurrenceList.getOccurenceCount();
        FicheLexification ficheLexification = lexieQuery.getFicheLexification(codefiche);
        for(int i = 0; i < count; i++) {
            int fieldNumber = occurrenceList.getFieldNumber(i);
            Lexification lexification = ficheLexification.getLexification(fieldNumber);
            int indexInLexification = occurrenceList.getLexieIndex(i);
            SubstringPosition substringPosition = potentialLexieSequence.testLexification(firstTokenLexieIndex,lexification, indexInLexification);
            if (substringPosition != null) {
                searchEngine.addFichePertinence(codefiche,codecorpus, multiToken.getTokenIndex(), fieldNumber, substringPosition);
                lexieSeeker.addFiche(codefiche,codecorpus);
            }
        }
    }
    
    private void scanMotcleLexification(int firstTokenLexieIndex,int codethesaurus,OccurrenceList occurrenceList) {
        SearchEngine searchEngine = lexieSeeker.getSearchEngine();
        int codemotcle = occurrenceList.getCode();
        int count = occurrenceList.getOccurenceCount();
        MotcleLexification ficheLexification = lexieQuery.getMotcleLexification(codemotcle);
        for(int i = 0; i < count; i++) {
            Lexification lexification = ficheLexification.getLexification(langInteger);
            int indexInLexification = occurrenceList.getLexieIndex(i);
            SubstringPosition substringPosition = potentialLexieSequence.testLexification(firstTokenLexieIndex,lexification, indexInLexification);
            if (substringPosition != null) {
                float coverRate = ((float) substringPosition.getLength()) / (lexification.getWholeLength());
                searchEngine.addMotclePertinence(codemotcle,codethesaurus, multiToken.getTokenIndex(), langInteger, substringPosition, coverRate);
                lexieSeeker.addMotcle(codemotcle,codethesaurus);
            }
        }
    }
    
    private class PotentialLexieSequence {
        long[][] lexieIdArrays;
        SubstringPosition[] firstSearchTextePositionArray;
        SubstringPosition[] lastSearchTextePositionArray;
        
        PotentialLexieSequence(SearchSequence.SimpleToken[] subtokenArray) {
            int length = subtokenArray.length;
            lexieIdArrays = new long[length][];
            for(int i = 0; i < length; i++) {
                SearchSequence.SimpleToken token = subtokenArray[i];
                System.out.println("token = " + token.toString(false));
                System.out.println("langue = " + LangInteger.toString(langInteger));
                List list = lexieSeeker.search(token.toString(false),langInteger, token.getSearchType());
                System.out.println("size = " + list.size());
                int size = list.size();
                if (size == 0) {
                    lexieIdArrays = null;
                    return;
                }
                long[] array = new long[size];
                SubstringPosition[] searchTextePositionArray = null;
                if (i == 0) {
                    firstSearchTextePositionArray = new SubstringPosition[size];
                    searchTextePositionArray = firstSearchTextePositionArray;
                } else if (i == (length-1)) {
                    lastSearchTextePositionArray = new SubstringPosition[size];
                    searchTextePositionArray = lastSearchTextePositionArray;
                }
                for(int j = 0; j < size; j++) {
                    SearchResultItem searchResultItem = (SearchResultItem) list.get(j);
                    Long lexieId = (Long) searchResultItem.getValue();
                    array[j] = lexieId.longValue();
                    if (searchTextePositionArray != null) {
                        searchTextePositionArray[j] = searchResultItem.getSearchTextPositionInCollatedKey();
                    }
                }
                lexieIdArrays[i] = array;
            }
            
        }
        
        public boolean isInit() {
            return (lexieIdArrays != null);
        }
        
        public int getFirstLexieCount() {
            return lexieIdArrays[0].length;
        }
        
        public long getFirstLexie(int index) {
            return lexieIdArrays[0][index];
        }
        
        public SubstringPosition testLexification(int firstTokenLexieIndex,Lexification lexification, int index) {
            int length = lexieIdArrays.length;
            if ((index + length) > lexification.getCount()) return null;
            int lastTokenLexieIndex = -1;
            for(int i = 1; i < length; i++) {
                long lexieId = lexification.getLexieId(index + i);
                int test = test(lexieId, i);
                if (test == -1) return null;
                else if (i == (length-1)) {
                    lastTokenLexieIndex = test;
                }
            }
            SubstringPosition firstSubstringPosition = lexification.getCollationUnitPosition(index).convertToPositionInWholeSource(firstSearchTextePositionArray[firstTokenLexieIndex]);
            SubstringPosition lastSubstringPosition = lexification.getCollationUnitPosition(index + length -1).convertToPositionInWholeSource(lastSearchTextePositionArray[lastTokenLexieIndex]);
            SubstringPosition finalSubstringPosition = SubstringPosition.merge(firstSubstringPosition,lastSubstringPosition);
            return finalSubstringPosition;
        }
        
        private int test(long lexieId,int tokenIndex) {
            long[] potentialArray = lexieIdArrays[tokenIndex];
            int length = potentialArray.length;
            for(int i = 0; i < length; i++) {
                if (potentialArray[i] == lexieId) {
                    return i;
                }
            }
            return -1;
        }
        
    }
    
    /*private class FicheResult {
        private SubstringPosition substringPosition;
        private int fieldNumber;
     
        public SubstringPosition getSubstringPosition() {
            return substringPosition;
        }
     
        public void setSubstringPosition(SubstringPosition substringPosition) {
            this.substringPosition = substringPosition;
        }
     
        public int getFieldNumber() {
            return fieldNumber;
        }
     
        public void setFieldNumber(int fieldNumber) {
            this.fieldNumber = fieldNumber;
        }
     
        public int compareTo(FicheResult other) {
            if (other.fieldNumber > this.fieldNumber) return -1;
            if (other.fieldNumber < this.fieldNumber) return 1;
            return substringPosition.compareTo(other.substringPosition);
        }
    }*/
    
}
