/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.searchengine.seeker;

import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.lexie.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.text.collation.*;
import net.mapeadores.util.text.collation.map.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.util.localisation.*;
import net.mapeadores.util.collections.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class SimpleTokenSeeker {
    private LexieSeeker lexieSeeker;
    private SearchEngine searchEngine;
    private int langInteger;
    private SearchSequence.SimpleToken simpleToken;
    
    public SimpleTokenSeeker(LexieSeeker lexieSeeker) {
        this.lexieSeeker = lexieSeeker;
    }
    
    void seek() {
        List list = lexieSeeker.search(simpleToken.toString(false),langInteger, simpleToken.getSearchType());
        for(int j = 0; j < list.size();j ++) {
            SearchResultItem searchResultItem = (SearchResultItem) list.get(j);
            Long lexieId = (Long) searchResultItem.getValue();
            add(lexieId.longValue(), searchResultItem.getSearchTextPositionInCollatedKey());
        }
    }
    
    private void add(long lexieId, SubstringPosition searchTextePositionInCollatedKey) {
        LexieManager2 lexieManager = lexieSeeker.getLexieManager();
        int tokenIndex = simpleToken.getTokenIndex();
        SearchEngine searchEngine = lexieSeeker.getSearchEngine();
        int[] codecorpusArray = searchEngine.getCodescorpusArray();
        int  codecorpusLength = codecorpusArray.length;
        for (int j = 0; j < codecorpusLength;j++) {
            int codecorpus = codecorpusArray[j];
            BestPosition[] array = lexieManager.getLexieQuery().getBestPositionArray(lexieId,codecorpus);
            if (array == null) continue;
            int length = array.length;
            for(int k = 0; k < length; k++) {
                BestPosition p = array[k];
                int codefiche = p.getCode();
                CollationUnitPosition bestCollationUnitPosition = p.getBestCollationUnitPosition();
                int bestFieldNumber = p.getBestFieldNumber();
                SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                searchEngine.addFichePertinence(codefiche,codecorpus, tokenIndex, bestFieldNumber, sourcePosition);
                lexieSeeker.addFiche(codefiche,codecorpus);
            }
        }
        if (searchEngine.isWithMotscles()) {
            int[] codesthesaurusArray = searchEngine.getCodesthesaurusArray();
            int codethesaurusLength = codesthesaurusArray.length;
            for (int j = 0; j < codethesaurusLength;j++) {
                int codethesaurus = codesthesaurusArray[j];
                BestPosition[] array = lexieManager.getLexieQuery().getBestPositionArray(lexieId,codethesaurus);
                if (array == null) continue;
                int length = array.length;
                for(int k = 0; k < length; k++) {
                    BestPosition p = array[k];
                    int codemotcle = p.getCode();
                    CollationUnitPosition bestCollationUnitPosition = p.getBestCollationUnitPosition();
                    SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                    float coverRate = ((float) sourcePosition.getLength())/(p.getWholeLength());
                    searchEngine.addMotclePertinence(codemotcle,codethesaurus,tokenIndex, langInteger,sourcePosition,coverRate);
                    lexieSeeker.addMotcle(codemotcle, codethesaurus);
                }
            }
        }
    }
    
    
   /* void addScrutariLexieUnit(ScrutariLexieUnit scrutariLexieUnit,SubstringPosition colllatedWordSubstringPosition) {
        int tokenIndex = simpleToken.getTokenIndex();
        SearchEngine searchEngine = lexieSeeker.getSearchEngine();
        int[] codescorpusArray = searchEngine.getCodescorpusArray();
        for (int j = 0; j < codescorpusArray.length;j++) {
            int codecorpus = codescorpusArray[j];
            List liste = scrutariLexieUnit.getFicheAndLinkList(codecorpus);
            for (Iterator it = liste.iterator();it.hasNext();) {
                FicheAndLexieLink ficheWord = (FicheAndLexieLink) it.next();
                int codefiche = ficheWord.getCodeFiche();
                CollationUnitPosition bestCollationUnitPosition = ficheWord.getCollationUnitPosition();
                int bestFieldNumber = ficheWord.getFieldNumber();
                SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(colllatedWordSubstringPosition);
                searchEngine.addFichePertinence(codefiche,codecorpus, tokenIndex, bestFieldNumber, sourcePosition);
                lexieSeeker.addFiche(codefiche,codecorpus);
            }
        }
        if (searchEngine.isWithMotscles()) {
            int[] codesthesaurusArray = searchEngine.getCodesthesaurusArray();
            for (int j = 0; j < codesthesaurusArray.length;j++) {
                int codethesaurus = codesthesaurusArray[j];
                List liste = scrutariLexieUnit.getMotcleAndLinkList(codethesaurus);
                for (Iterator it = liste.iterator();it.hasNext();) {
                    MotcleAndLexieLink motcleWord = (MotcleAndLexieLink) it.next();
                    int codemotcle = motcleWord.getCodeMotcle();
                    CollationUnitPosition bestCollationUnitPosition = motcleWord.getCollationUnitPosition();
                    SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(colllatedWordSubstringPosition);
                    searchEngine.addMotclePertinence(codemotcle,codethesaurus,tokenIndex, langInteger,sourcePosition,bestCollationUnitPosition.getCoverRate());
                    lexieSeeker.addMotcle(codemotcle, codethesaurus);
                }
            }
        }
    }*/
    
    
    public void setToken(SearchSequence.SimpleToken simpleToken) {
        this.simpleToken = simpleToken;
    }
    
    public void setLangInteger(int langInteger) {
        this.langInteger = langInteger;
    }
    
}
