/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.uri;


import java.net.*;
/**
 *
 * @author  Vincent Calame
 */
public final class BaseURI extends ScrutariURI implements Comparable  {
    private String authorityUUID;
    private String baseName;
    
    public BaseURI(String authorityUUID, String baseName) {
        this.authorityUUID = authorityUUID;
        this.baseName = baseName;
    }
    
    public String getAuthorityUUID() {
        return authorityUUID;
    }
    
    public String getBaseName() {
        return baseName;
    }
    
    public int hashCode() {
        return authorityUUID.hashCode() + baseName.hashCode();
    }
    
    public int compareTo(Object o) {
        BaseURI autreuid = (BaseURI) o;
        int resultat = baseName.compareTo(autreuid.baseName);
        if (resultat == 0) resultat = authorityUUID.compareTo(autreuid.authorityUUID);
        return resultat;
    }
    
    public boolean equals(Object o) {
        if (o == this) return true;
        if (!(o instanceof BaseURI)) return false;
        return (compareTo(o) == 0);
    }
    
    public short getType() {
        return BASEURI_TYPE;
    }
    
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("base:/");
        buf.append(getAuthorityUUID());
        buf.append('/');
        buf.append(getBaseName());
        return buf.toString();
    }
    
}
