/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.uri;


/**
 *
 * @author  Vincent Calame
 */
public final class CorpusURI extends ScrutariURI implements Comparable {
    BaseURI baseURI;
    String corpusName;
    
    public CorpusURI(String authorityUUID,String nombase, String corpusName) {
        this.corpusName = corpusName;
        this.baseURI = new BaseURI(authorityUUID,nombase);
    }
    
    public CorpusURI(BaseURI baseURI, String corpusName) {
        this.corpusName = corpusName;
        this.baseURI = baseURI;
    }
    
    public BaseURI getBaseURI() {
        return baseURI;
    }
    
    public String getCorpusName() {
        return corpusName;
    }
    
    public int hashCode() {
        return corpusName.hashCode() + baseURI.hashCode();
    }
    
    public int compareTo(Object o) {
        CorpusURI uidautre = (CorpusURI) o;
        int resultat = corpusName.compareTo(uidautre.corpusName);
        if (resultat == 0) {
            resultat = baseURI.compareTo(uidautre.baseURI);
        }
        return resultat;
    }
    
    public boolean equals(Object o) {
        if (o == this) return true;
        if (!(o instanceof CorpusURI)) return false;
        return (compareTo(o) == 0);
    }
    
    public short getType() {
        return CORPUSURI_TYPE;
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("corpus:/");
        buf.append(baseURI.getAuthorityUUID());
        buf.append('/');
        buf.append(baseURI.getBaseName());
        buf.append('/');
        buf.append(corpusName);
        return buf.toString();
    }
}
