/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.uri;

import java.util.*;
import java.io.Serializable;

/**
 *
 * @author  Vincent Calame
 */
public class DefaultURICoder implements Serializable {
    ScrutariURI[] uriArray = new ScrutariURI[2048];
    private int baseCount = 0;
    private int corpusCount = 0;
    private int thesaurusCount = 0;
    private int motcleCount = 0;
    private int ficheCount = 0;
    private int size = 0;
    HashMap mapUID = new HashMap();
    
    
    public DefaultURICoder() {
    }
    
    public int getBaseURICount() {
        return baseCount;
    }
    
    public int getCorpusURICount() {
        return corpusCount;
    }
    
    public int getThesaurusURICount() {
        return thesaurusCount;
    }
    
    public int getMotcleURICount() {
        return motcleCount;
    }
    
    public int getFicheURICount() {
        return ficheCount;
    }
    
    public int getOrCreateCode(BaseURI baseURI) {
        int code = getCode(baseURI);
        if (code > -1) return code;
        return addBaseURI(baseURI);
    }
    
     public int getOrCreateCode(CorpusURI corpusURI) {
        int code = getCode(corpusURI);
        if (code > -1) return code;
        return addCorpusURI(corpusURI);
    }
     
     public int getOrCreateCode(FicheURI ficheURI) {
        int code = getCode(ficheURI);
        if (code > -1) return code;
        return addFicheURI(ficheURI);
    }
     
     public int getOrCreateCode(ThesaurusURI thesaurusURI) {
         int code = getCode(thesaurusURI);
        if (code > -1) return code;
        return addThesaurusURI(thesaurusURI);
    }
     
     public int getOrCreateCode(MotcleURI motcleURI) {
        int code = getCode(motcleURI);
        if (code > -1) return code;
        return addMotcleURI(motcleURI);
    }
     
     public int getCode(BaseURI baseURI) {
        Integer itg = (Integer) mapUID.get(baseURI);
        if (itg != null) return itg.intValue();
        else return -1;
    }
    
     public int getCode(CorpusURI corpusURI) {
        Integer itg = (Integer) mapUID.get(corpusURI);
        if (itg != null) return itg.intValue();
        else return -1;
    }
     
     public int getCode(FicheURI ficheURI) {
        Integer itg = (Integer) mapUID.get(ficheURI);
        if (itg != null) return itg.intValue();
        else return -1;
    }
     
     public int getCode(MotcleURI motcleURI) {
        Integer itg = (Integer) mapUID.get(motcleURI);
        if (itg != null) return itg.intValue();
        else return -1;
    }
     
     public int getCode(ThesaurusURI thesaurusURI) {
        Integer itg = (Integer) mapUID.get(thesaurusURI);
        if (itg != null) return itg.intValue();
        else return -1;
    }
    
    
    /*
     * Retourne -1 si le codage n'existe pas.
     */
    public int getCorpusURICode(int codebase,String corpusName) {
        if ((codebase < 0) || (codebase >= size)) return -1;
         Integer itg = (Integer) mapUID.get(new CorpusURI((BaseURI) uriArray[codebase],corpusName));
        if (itg != null) return itg.intValue();
        return -1;
    }
    
    /*
     * Retourne -1 si le codage n'existe pas.
     */
    public int getThesaurusURICode(int codebase,String thesaurusName) {
        if ((codebase < 0) || (codebase >= size)) return -1;
         Integer itg = (Integer) mapUID.get(new ThesaurusURI((BaseURI) uriArray[codebase],thesaurusName));
        if (itg != null) return itg.intValue();
        return -1;
    }
    
    

    
    public boolean containsMapeadoresURI(ScrutariURI mapeadoresURI) {
        return mapUID.containsKey(mapeadoresURI);
    }
    
    public ScrutariURI getMapeadoresURI(int code) {
        if (code >= size) return null;
        return uriArray[code];
    }
     
    public BaseURI getBaseURI(int code) {
        return (BaseURI) uriArray[code];
    }
    public ThesaurusURI getThesaurusUID(int code) {
        return (ThesaurusURI) uriArray[code];
    }
    public CorpusURI getCorpusURI(int code) {
        return (CorpusURI) uriArray[code];
    }
    public FicheURI getFicheURI(int code) {
        return (FicheURI) uriArray[code];
    }
    public MotcleURI getMotcleURI(int code) {
        return (MotcleURI) uriArray[code];
    }
    
    private synchronized int addBaseURI(BaseURI baseURI) {
        checkCapacity();
        int code = size;
        uriArray[code] = baseURI;
        mapUID.put(baseURI,new Integer(code));
        baseCount ++;
        size++;
        return code;
    }
    
    private synchronized int addCorpusURI(CorpusURI corpusURI) {
        checkCapacity();
        int code = size;
        uriArray[code] = corpusURI;
        mapUID.put(corpusURI,new Integer(code));
        corpusCount ++;
        size++;
        return code;
    }
    
    private synchronized int addThesaurusURI(ThesaurusURI thesaurusURI) {
        checkCapacity();
        int code = size;
        uriArray[code] = thesaurusURI;
        mapUID.put(thesaurusURI,new Integer(code));
        thesaurusCount ++;
        size++;
        return code;
    }

    
    private synchronized int addMotcleURI(MotcleURI motcleURI) {
        checkCapacity();
        int code = size;
        uriArray[code] = motcleURI;
        mapUID.put(motcleURI,new Integer(code));
        motcleCount ++;
        size++;
        return code;
    }

    
    private synchronized int addFicheURI(FicheURI ficheURI) {
        checkCapacity();
        int code = size;
        uriArray[code] = ficheURI;
        mapUID.put(ficheURI,new Integer(code));
        ficheCount ++;
        size++;
        return code;
    }
    
    private synchronized void checkCapacity() {
        if (uriArray.length == size) {
            ScrutariURI[] newArray = new ScrutariURI[size*2];
            System.arraycopy(uriArray,0,newArray,0, size);
            uriArray = newArray;
        }
    }
    
}
