/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.uri;
import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;

/**
 *
 * @author  Vincent Calame
 */
public final class FicheURI extends ScrutariURI implements Comparable {
    private CorpusURI corpusURI;
    private String ficheId;
    
    public FicheURI(CorpusURI corpusURI, String ficheId) {
        this.corpusURI = corpusURI;
        this.ficheId = ficheId;
    }
    
    public FicheURI(String authorityUUID,String nombase, String corpusName, String ficheId) {
        this.ficheId = ficheId;
        this.corpusURI = new CorpusURI(authorityUUID,nombase,corpusName);
    }
    
    public String getFicheId() {
        return ficheId;
    }
    
    public CorpusURI getCorpusURI() {
        return corpusURI;
    }
    
    public int hashCode() {
        return ficheId.hashCode() + corpusURI.hashCode();
    }
    
    public int compareTo(Object o) {
        FicheURI uidautre = (FicheURI) o;
        int resultat = ficheId.compareTo(uidautre.ficheId);
        if (resultat == 0) {
            resultat = corpusURI.compareTo(uidautre.corpusURI);
        }
        return resultat;
    }
    
    public boolean equals(Object o) {
        if (o == this) return true;
        if (!(o instanceof FicheURI)) return false;
        return (compareTo(o) == 0);
    }
    
    public short getType() {
        return FICHEURI_TYPE;
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("fiche:/");
        BaseURI baseUID = corpusURI.getBaseURI();
        buf.append(baseUID.getAuthorityUUID());
        buf.append('/');
        buf.append(baseUID.getBaseName());
        buf.append('/');
        buf.append(corpusURI.getCorpusName());
        buf.append('/');
        buf.append(ficheId);
        return buf.toString();
    }
    
}
