/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.uri;
import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;

/**
 *
 * @author  Vincent Calame
 */
public final class MotcleURI extends ScrutariURI implements Comparable {
    private ThesaurusURI thesaurusURI;
    private String motcleId;
    
    public MotcleURI(ThesaurusURI thesaurusURI, String motcleId) {
        this.thesaurusURI = thesaurusURI;
        this.motcleId = motcleId;
    }
    
    public MotcleURI(String authorityUUID,String nombase, String thesaurusName, String motcleId) {
        this.motcleId = motcleId;
        this.thesaurusURI = new ThesaurusURI(authorityUUID,nombase,thesaurusName);
    }
    
    public String getMotcleId() {
        return motcleId;
    }
    
    public ThesaurusURI getThesaurusURI() {
        return thesaurusURI;
    }
    
    public int hashCode() {
        return motcleId.hashCode() + thesaurusURI.hashCode();
    }
    
    public int compareTo(Object o) {
        MotcleURI uidautre = (MotcleURI) o;
        int resultat = motcleId.compareTo(uidautre.motcleId);
        if (resultat == 0) {
            resultat = thesaurusURI.compareTo(uidautre.thesaurusURI);
        }
        return resultat;
    }
    
    public boolean equals(Object o) {
        if (o == this) return true;
        if (!(o instanceof MotcleURI)) return false;
        return (compareTo(o) == 0);
    }
    
    public short getType() {
        return MOTCLEURI_TYPE;
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        BaseURI baseURI = thesaurusURI.getBaseURI();
        buf.append("motcle:/");
        buf.append(baseURI.getAuthorityUUID());
        buf.append('/');
        buf.append(baseURI.getBaseName());
        buf.append('/');
        buf.append(thesaurusURI.getThesaurusName());
        buf.append('/');
        buf.append(motcleId);
        return buf.toString();
    }
    
}
