/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.scrutari.uri;
import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;

/**
 *
 * @author  Vincent Calame
 */
public final class ThesaurusURI extends ScrutariURI implements Comparable {
    BaseURI baseURI;
    String thesaurusName;
    
    public ThesaurusURI(String authorityUUID,String nombase, String thesaurusName) {
        this.thesaurusName = thesaurusName;
        this.baseURI = new BaseURI(authorityUUID,nombase);
    }
    
    public ThesaurusURI(BaseURI baseURI, String thesaurusName) {
        this.thesaurusName = thesaurusName;
        this.baseURI = baseURI;
    }
    
    public BaseURI getBaseURI() {
        return baseURI;
    }
    
    public String getThesaurusName() {
        return thesaurusName;
    }
    
    public int hashCode() {
        return thesaurusName.hashCode() + baseURI.hashCode();
    }
    
    public int compareTo(Object o) {
        ThesaurusURI uidautre = (ThesaurusURI) o;
        int resultat = thesaurusName.compareTo(uidautre.thesaurusName);
        if (resultat == 0) {
            resultat = baseURI.compareTo(uidautre.baseURI);
        }
        return resultat;
    }
    
    public boolean equals(Object o) {
        if (o == this) return true;
        if (!(o instanceof ThesaurusURI)) return false;
        return (compareTo(o) == 0);
    }
    
    public short getType() {
        return THESAURUSURI_TYPE;
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("thesaurus:/");
        buf.append(baseURI.getAuthorityUUID());
        buf.append('/');
        buf.append(baseURI.getBaseName());
        buf.append('/');
        buf.append(thesaurusName);
        return buf.toString();
    }
    
}
