/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.uri;

import java.net.URI;
import java.net.URISyntaxException;

/**
 *
 * @author Vincent Calame
 */
public class URIParser {
    
    private URIParser() {
    }
    
    public static ScrutariURI parse(String s) throws URIParseException {
        try {
            URI uri = new URI(s);
            String scheme = uri.getScheme();
            if (scheme == null) throw new URIParseException(URIParseException.MISSINGSCHEME_EXCEPTION);
            if (uri.isOpaque()) throw new URIParseException(URIParseException.OPAQUEURI_EXCEPTION);
            String path = uri.getRawPath();
            if (path == null) throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION);
            String tokens[] = path.split("/");
            int size = tokens.length - 1;
            if ((size < 2) || (size > 4)) throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION);
            if (scheme.equals("fiche")) {
                if (size != 4) throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION);
                return new FicheURI(tokens[1],tokens[2],tokens[3],tokens[4]);
            } else if (scheme.equals("corpus")) {
                if (size != 3) throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION);
                return new CorpusURI(tokens[1],tokens[2],tokens[3]);
            } else if (scheme.equals("thesaurus")) {
                if (size != 3) throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION);
                return new ThesaurusURI(tokens[1],tokens[2],tokens[3]);
            } else if (scheme.equals("motcle")) {
                if (size != 4) throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION);
                return new MotcleURI(tokens[1],tokens[2],tokens[3],tokens[4]);
            } else if (scheme.equals("base")) {
                if (size != 2) throw new URIParseException(URIParseException.MALFORMEDPATH_EXCEPTION);
                return new BaseURI(tokens[1],tokens[2]);
            } else {
                throw new URIParseException(URIParseException.UNKNOWNSCHEME_EXCEPTION);
            }
        } catch(URISyntaxException urise) {
            throw new URIParseException(URIParseException.URISYNTAX_EXCEPTION, urise.getMessage());
        }
    }
    
}
