/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.xml.xmlpull;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.uri.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.logging.*;
import org.xmlpull.v1.*;
import java.io.*;
import net.mapeadores.util.xml.xmlpull.*;
import net.mapeadores.scrutari.xml.*;
import net.mapeadores.scrutari.uri.*;

/**
 *
 * @author Vincent Calame
 */
public class IndexationGroupHandler extends DefaultElementHandler implements ErrorKeys {
    CorpusURI corpusURI;
    ThesaurusURI thesaurusURI;
    private Parameters parameters;
    private IndexationData.BuilderList indexationDataBuilderList;
    
    
    public IndexationGroupHandler(Map attributeMap, Parameters parameters) {
        String corpuspath = getAttributeString(attributeMap,"corpus-path");
        String thesauruspath = getAttributeString(attributeMap,"thesaurus-path");
        boolean error = false;
        if (corpuspath == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGCORPUSPATH);
        }
        if (thesauruspath == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGTHESAURUSPATH);
        }
        if (error) throw new IllegalArgumentException();
        this.parameters = parameters;
        BaseURI baseURI = parameters.getBaseURI();
        corpusURI = (CorpusURI) parameters.intern(new CorpusURI(baseURI, corpuspath));
        thesaurusURI = (ThesaurusURI) parameters.intern(new ThesaurusURI(baseURI,thesauruspath));
        indexationDataBuilderList = parameters.getIndexationDataBuilderList();
    }
    
    public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
        if (tagname.equals("indexation")) {
            boolean error = false;
            String ficheid = getAttributeString(attributeMap,"fiche-id");
            if (ficheid == null) {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGFICHEID);
                error = true;
            }
            String motcleid = getAttributeString(attributeMap,"motcle-id");
            if (motcleid == null) {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_MISSINGMOTCLEID);
                error = true;
            }
            if (!error) {
                FicheURI ficheURI = (FicheURI) parameters.intern(new FicheURI(corpusURI, ficheid));
                MotcleURI motcleURI = (MotcleURI) parameters.intern(new MotcleURI(thesaurusURI,motcleid));
                IndexationKey indexationKey = new IndexationKey(ficheURI, motcleURI);
                if (!parameters.testDoublon(indexationKey)) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_INDEXATION_DOUBLEINDEXATION);
                    error = true;
                    return null;
                }
                int poids = 1;
                String poidsString = getAttributeString(attributeMap, "poids");
                if (poidsString == null) {
                    poidsString = getAttributeString(attributeMap, "weight");
                }
                if (poidsString != null) {
                    try {
                        poids = Integer.parseInt(poidsString);
                        if (poids < 1) poids = 1;
                    } catch(NumberFormatException nfe) {
                        
                    }
                }
                IndexationData.Builder indexationDataBuilder = new IndexationData.Builder(indexationKey, poids);
                indexationDataBuilderList.addIndexationDataBuilder(indexationDataBuilder);
            }
        }
        return null;
    }

    public IndexationData.BuilderList getIndexationDataBuilderList() {
        return indexationDataBuilderList;
    }

}
