/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.scrutari.xml.xmlpull;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.uri.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.logging.*;
import org.xmlpull.v1.*;
import java.io.*;
import net.mapeadores.util.xml.xmlpull.*;
import net.mapeadores.scrutari.xml.*;
import net.mapeadores.scrutari.uri.*;

/**
 *
 * @author Vincent Calame
 */
public class ThesaurusHandler extends DefaultElementHandler implements ErrorKeys {
     private Parameters parameters;
     private ThesaurusURI thesaurusURI;
     private ThesaurusData.Builder thesaurusDataBuilder;
     private boolean metadataDone = false;
     private MotcleHandler motcleHandler = new MotcleHandler();
     
    public ThesaurusHandler(Map attributeMap, Parameters parameters) {
        String thesaurusName = getAttributeString(attributeMap,"thesaurus-name");
        if (thesaurusName == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_MISSINGTHESAURUSNAME);
            throw new IllegalArgumentException();
        }
        this.parameters = parameters;
        thesaurusURI = new ThesaurusURI(parameters.getBaseURI(), thesaurusName);
        if (!parameters.testDoublon(thesaurusURI)) {
           parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_DOUBLETHESAURUSNAME,thesaurusURI);
           throw new IllegalArgumentException();
        }
        thesaurusURI = (ThesaurusURI) parameters.intern(thesaurusURI);
        thesaurusDataBuilder = new ThesaurusData.Builder(thesaurusURI);
    }

    public ThesaurusData.Builder getThesaurusDataBuilder() {
        if (!metadataDone) return null;
        return thesaurusDataBuilder;
    }
    
    public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("thesaurus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_MISSINGMETADATA,thesaurusURI);
                return null;
            }
        } else {
            if (tagname.equals("motcle")) {
                String motcleId = getAttributeString(attributeMap,"motcle-id");
                if (motcleId == null) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_MOTCLE_MISSINGMOTCLEID);
                    return null;
                }
                MotcleURI motcleURI = new MotcleURI(thesaurusURI, motcleId);
                if (!parameters.testDoublon(motcleURI)) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_MOTCLE_DOUBLEMOTCLEID,motcleURI);
                    return null;
                }
                motcleURI = (MotcleURI) parameters.intern(motcleURI);
                motcleHandler.setMotcleURI(motcleURI);
                return motcleHandler;
            }
            else {
                return null;
            }
        }
    }
    
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof MotcleHandler) {
            MotcleData.Builder motcleDataBuilder = ((MotcleHandler) elementHandler).getMotcleDataBuilder();
            thesaurusDataBuilder.addMotcleDataBuilder(motcleDataBuilder);
        }
    }

    
    private class MetadataHandler extends DefaultElementHandler {
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("intitule-thesaurus")) {
                return new IntituleHandler();
            } else {
                return null;
            }
        }
    }
    
    
    private class IntituleHandler extends DefaultElementHandler {
        
        IntituleHandler() {

        }
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch(LangIntegerException lie) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_THESAURUS_LANGERROR,thesaurusURI);
                    return null;
                }
            }
            return null;
        }
        
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof Libelle) {
                thesaurusDataBuilder.addLibelle((Libelle) elementHandler);
            }
        }
    }
    
    private class MotcleHandler extends DefaultElementHandler {
        private MotcleURI motcleURI;
        private MotcleData.Builder motcleDataBuilder;
        
        MotcleHandler() {
            
        }

        public void setMotcleURI(MotcleURI motcleURI) {
            this.motcleURI = motcleURI;
            motcleDataBuilder = new MotcleData.Builder(motcleURI);
        }

        public MotcleData.Builder getMotcleDataBuilder() {
            return motcleDataBuilder;
        }
        
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch(LangIntegerException lie) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_MOTCLE_LANGERROR,motcleURI);
                    return null;
                }
            }
            return null;
        }
        
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof Libelle) {
                motcleDataBuilder.addLibelle((Libelle) elementHandler);
            }
        }
    }

    
}
