/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.admin;

import net.mapeadores.scrutari.searchengine.*;
import fr.exemole.sctserver.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.util.collections.*;
import fr.exemole.sctserver.request.*;
import net.mapeadores.util.conditions.*;

/**
 *
 * @author Vincent Calame
 */
public class OptionsManager {
    private SctServer sctServer;
    private InternalGlobalSearchOptions internalGlobalSearchOptions = new InternalGlobalSearchOptions();
    private boolean withClasses = false;
    private CorpusClasse[] corpusClasseArray;
    private SearchLogDeny[] searchLogDenyArray;
    
    public OptionsManager(SctServer sctServer) {
        this.sctServer = sctServer;
    }
    
    
    public void reloadCorpusClasseArray(CorpusClasse[] corpusClasseArray) {
        withClasses = false;
        this.corpusClasseArray = null;
        ScrutariURICoder scrutariCoder = sctServer.getScrutariDB().getDataManager().getScrutariURICoder();
        int defaultClasseValue = 0;
        IntHashMap codeMap = null;
        if (corpusClasseArray != null) {
            this.corpusClasseArray = corpusClasseArray;
            withClasses = true;
            codeMap = new IntHashMap();
            int length = corpusClasseArray.length;
            for(int i = 0;i < length; i++) {
                CorpusClasse corpusClasse = corpusClasseArray[i];
                String name = corpusClasse.getName();
                if (name == null) {
                    defaultClasseValue = corpusClasse.getClasseValue();
                } else {
                    int count = corpusClasse.getCorpusURICount();
                    for(int j = 0; j < count; j++) {
                        CorpusURI corpusURI = corpusClasse.getCorpusURI(j);
                        int codecorpus = scrutariCoder.getCode(corpusURI);
                        if (codecorpus > 0) {
                            codeMap.put(codecorpus,  corpusClasse);
                        }
                    }
                }
            }
        }
        internalGlobalSearchOptions.reload(codeMap, defaultClasseValue);
    }
    
    public void reloadSearchLogDenyArray(SearchLogDeny[] searchLogDenyArray) {
        this.searchLogDenyArray = null;
        if (searchLogDenyArray != null) {
            if (searchLogDenyArray.length != 0) {
                this.searchLogDenyArray = searchLogDenyArray;
            }
        }
    }
    
    public GlobalSearchOptions getGlobalSearchOptions() {
        return internalGlobalSearchOptions;
    }
    
    public int getCorpusClasseCount() {
        if (!withClasses) return 0;
        return corpusClasseArray.length;
    }
    
    public int getSearchLogDenyCount() {
        if (searchLogDenyArray == null) return 0;
        return searchLogDenyArray.length;
    }
    
    public CorpusClasse getCorpusClasse(int i) {
        return corpusClasseArray[i];
    }
    
    public SearchLogDeny getSearchLogDeny(int i) {
        return searchLogDenyArray[i];
    }
    
    public boolean filterLog(RequestInfo requestInfo) {
        if (!requestInfo.withLog()) return false;
        if (searchLogDenyArray == null) return true;
        int length = searchLogDenyArray.length;
        if (length == 0) return true;
        for(int i = 0; i< length; i++) {
            SearchLogDeny logDeny = searchLogDenyArray[i];
            ContentCondition ipContentCondition = logDeny.getContentCondition(SearchLogDeny.IP_DENY);
            if (ipContentCondition != null) {
                boolean test = Conditions.accept(ipContentCondition, requestInfo.getRemoteAddress(),null);
                if (!test) continue;
            }
            ContentCondition siteContentCondition = logDeny.getContentCondition(SearchLogDeny.SITE_DENY);
            if (siteContentCondition != null) {
                boolean test = Conditions.accept(siteContentCondition, requestInfo.getSite(), null);
                if (!test) continue;
            }
            ContentCondition refContentCondition = logDeny.getContentCondition(SearchLogDeny.REF_DENY);
            if (refContentCondition != null) {
                boolean test = Conditions.accept(refContentCondition, requestInfo.getReferrer(), null);
                if (!test) continue;
            }
            return false;
        }
        return true;
    }
    
    
    private class InternalGlobalSearchOptions implements GlobalSearchOptions {
        private IntHashMap codeMap;
        private int defautClassValue;
        
        public int getClasseValue(int codecorpus) {
            if (!withClasses) return 0;
            CorpusClasse corpusClasse = (CorpusClasse) codeMap.get(codecorpus);
            if (corpusClasse == null) {
                return defautClassValue;
            } else return corpusClasse.getClasseValue();
        }
        
        private void reload(IntHashMap codeMap, int defautClassValue) {
            this.defautClassValue = defautClassValue;
            this.codeMap = codeMap;
        }
    }
    
    
    
}
