/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.admin;

import java.net.*;
import java.util.*;
import fr.exemole.sctserver.storage.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.scrutari.uri.*;

/**
 *
 * @author Vincent Calame
 */
public class SourceInfoManager {
    private InnerSourceInfo[] sourceInfoArray;
    
    public SourceInfoManager() {

    }
    
    public int getSourceCount() {
        if (sourceInfoArray == null) return 0;
        return sourceInfoArray.length;
    }
    
    public ScrutariInfo getSourceInfo(int i) {
        return sourceInfoArray[i];
    }
    
    public void reloadInfoUrlArray(UrlKey[] urlKeyArray) {
        if (urlKeyArray == null) {
            sourceInfoArray = null;
            return;
        }
        int length = urlKeyArray.length;
        InnerSourceInfo[] newArray = new InnerSourceInfo[length];
        for(int i = 0; i < length; i++) {
            UrlKey urlKey = urlKeyArray[i];
            InnerSourceInfo sourceInfo = getInnerSourceInfo(urlKey.getKeyString());
            if (sourceInfo != null) {
                newArray[i] = sourceInfo;
            } else {
                newArray[i] = new InnerSourceInfo(urlKey);
            }
        }
        this.sourceInfoArray = newArray;
    }
    
    public boolean setState(String keyString, short state) {
        InnerSourceInfo sourceInfo = getInnerSourceInfo(keyString);
        if (sourceInfo != null) {
            return sourceInfo.setState(state);
        }
        else {
            return false;
        }
    }
    

    public boolean setLastUpdate(String keyString, int lastUpdate) {
        InnerSourceInfo sourceInfo = getInnerSourceInfo(keyString);
        if (sourceInfo != null) {
            return sourceInfo.setLastUpdate(lastUpdate);
        }
        else {
            return false;
        }
    }
    
    public void setScrutariDataUrl(String keyString, URL scrutariDataUrl) {
        InnerSourceInfo sourceInfo = getInnerSourceInfo(keyString);
        if (sourceInfo != null) {
            sourceInfo.setScrutariDataUrl(scrutariDataUrl);
        }
    }
    
    public void setBaseURI(String keyString, BaseURI baseURI) {
        InnerSourceInfo sourceInfo = getInnerSourceInfo(keyString);
        if (sourceInfo != null) {
            sourceInfo.setBaseURI(baseURI);
        }
    }
    
    private InnerSourceInfo getInnerSourceInfo(String keyString) {
        if (sourceInfoArray == null) return null;
        int length = sourceInfoArray.length;
        for(int i = 0; i < length; i++) {
            InnerSourceInfo sourceInfo = sourceInfoArray[i];
            if (sourceInfo.getKeyString().equals(keyString)) {
                return sourceInfo;
            }
        }
        return null;
    }
   
    private class InnerSourceInfo implements ScrutariInfo {
        private URL url;
        private short state;
        private int lastUpdate;
        private URL scrutariDataURL;
        private BaseURI baseURI;
        private String keyString;
        
        
        InnerSourceInfo(UrlKey urlKey) {
            this.url = urlKey.getUrl();
            this.keyString = urlKey.getKeyString();
            this.state = ScrutariInfo.UNKNOWN_STATE;
            this.lastUpdate = IntDate.NULL;
        }
        
        public String getKeyString() {
            return keyString;
        }
    
        public URL getUrl() {
            return url;
        }
        
        public URL getScrutariDataUrl() {
            return scrutariDataURL;
        }
        
        private void setScrutariDataUrl(URL scrutariDataURL) {
            this.scrutariDataURL = scrutariDataURL;
        }

        public short getState() {
            return state;
        }
        
        public int getLastUpdate() {
            return lastUpdate;
        }

        private boolean setState(short state) {
            if (this.state == state) return false;
            this.state = state;
            return true;
        }

        private boolean setLastUpdate(int lastUpdate) {
            if (this.lastUpdate == lastUpdate) return false;
            this.lastUpdate = lastUpdate;
            return true;
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

        private void setBaseURI(BaseURI baseURI) {
            this.baseURI = baseURI;
        }
    }
    
}
