/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.admin;

import java.net.*;

/**
 *
 * @author Vincent Calame
 */
public class UrlKey {
    private URL url;
    private String keyString;
    
    public UrlKey(URL url) {
        if (url == null) throw new IllegalArgumentException("url is null");
        this.url = url;
        this.keyString = toKey(url);
    }

    public URL getUrl() {
        return url;
    }

    public String getKeyString() {
        return keyString;
    }
    
    public String toString() {
        return keyString;
    }
    
    public int hashCode() {
        return keyString.hashCode();
    }
    
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if (!(obj instanceof UrlKey)) return false;
        UrlKey other = (UrlKey) obj;
        return (this.keyString.equals(other.keyString));
    }
    
    private static String toKey(URL url) {
        String key = url.toString();
        int length = key.length();
        int start = 0;
        if (key.startsWith("http://")) {
            start = 7;
        }
        StringBuffer keyBuffer = new StringBuffer(length);
        for(int i = start; i < length; i++) {
            char carac = key.charAt(i);
            switch(carac) {
                case ':' :
                case '?' :
                case '%' :
                case '/' :
                case '=' :
                case '&' :
                case ';' :
                case ',' :
                case ' ' :
                case '+' :
                case '.' :
                    keyBuffer.append('_');
                    break;
                default :
                    keyBuffer.append(carac);
            }
        }
        return keyBuffer.toString();
    }
    
}
