/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.conf.xml.*;
import org.w3c.dom.*;
import net.mapeadores.util.logging.*;
import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import fr.exemole.sctserver.*;
import net.mapeadores.scrutari.uri.*;
import java.util.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class CorpusClassesConf {
    private CorpusClassesLog processLog;
    private CorpusClasse[] corpusClasseArray;
    private ProcessURI processURI;
    
    private CorpusClassesConf() {
    }
    
    public ProcessLog getProcessLog() {
        return processLog;
    }
    
    public CorpusClasse[] getCorpusClasseArray() {
        return corpusClasseArray;
    }
    
    public static CorpusClassesConf load(File f) throws IOException, ParserConfigurationException {
        CorpusClassesConf conf = new CorpusClassesConf();
        conf.init(f);
        return conf;
    }
    
    public ProcessURI getProcessURI() {
        return processURI;
    }
    
    private void init(File f) throws IOException, ParserConfigurationException {
        corpusClasseArray = null;
        processURI = ProcessURI.fromFile(f);
        processLog = new CorpusClassesLog(processURI );
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch(SAXException saxe) {
            processLog.saxException(saxe);
            return;
        }
        CorpusClasseBuilder builder = new CorpusClasseBuilder();
        CorpusClassesDOMReader domReader = new CorpusClassesDOMReader(builder, processLog);
        domReader.readCorpusClasses(document.getDocumentElement());
        corpusClasseArray = builder.toCorpusClasseArray();
    }
    
     private static class CorpusClasseBuilder implements CorpusClassesDOMReader.Builder {
        LinkedHashMap map = new LinkedHashMap();
        HashMap uriMap = new HashMap();
        
        public CorpusClasse addClasse(String name) {
            InternalCorpusClasse corpusClasse = (InternalCorpusClasse) map.get(name);
            if (!map.containsKey(name)) {
                corpusClasse =  new InternalCorpusClasse(name);
                map.put(name, corpusClasse);
            }
            return corpusClasse;
        }
        
        public CorpusClasse getCorpusClasse(String name) {
            return (CorpusClasse) map.get(name);
        }
        
        public void setLibelle(CorpusClasse corpusClasse, String libelle, int langInteger) {
            ((InternalCorpusClasse) corpusClasse).internalSetLibelle(libelle, langInteger);
        }
        
        public boolean addURI(CorpusClasse corpusClasse, CorpusURI corpusURI) {
            InternalCorpusClasse internalCorpusClasse = (InternalCorpusClasse) corpusClasse;
            if (uriMap.containsKey(corpusURI)) return false;
            uriMap.put(corpusURI, internalCorpusClasse);
            internalCorpusClasse.addURI(corpusURI);
            return true;
        }

        private void flush() {
             if (!map.containsKey(null)) {
                map.put(null, new InternalCorpusClasse(null));
            }
            int defaultClassValue = 0;
            int classeValue = map.size();
            for(Iterator it = map.entrySet().iterator(); it.hasNext();) {
                Map.Entry entry = (Map.Entry) it.next();
                String name = (String) entry.getKey();
                InternalCorpusClasse corpusClasse = (InternalCorpusClasse) entry.getValue();
                int cv = classeValue*10;
                classeValue = classeValue -1;
                corpusClasse.setClasseValue(cv);
                if (name == null) defaultClassValue = cv;
            }
        }

        private InternalCorpusClasse[] toCorpusClasseArray() {
            int size = map.size();
            if (size == 0) return null;
            flush();
            InternalCorpusClasse[] result = new InternalCorpusClasse[size];
            int p = 0;
            for(Iterator it = map.values().iterator(); it.hasNext();) {
               result[p] = (InternalCorpusClasse) it.next();
               p++;
            }
            return result;
        }
        
    }
    
    private static class CorpusClassesLog extends MutableProcessLog implements CorpusClassesDOMReader.ErrorHandler {
        
        public CorpusClassesLog(ProcessURI processURI) {
            setCurrentURI(processURI);
        }
        
        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }
        
        /*public void configException(Exception e) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(e.getMessage());
        }*/
        
        public void unknownTagWarning(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("unknownTag");
            logItem.setObject(tagName);
        }
        
        public void emptyAttributeError(String tagName, String attributeName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyAttribute");
            logItem.setObject(tagName + "@" + attributeName);
        }
    
        public void emptyElementError(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyElement");
            logItem.setObject(tagName);
        }
        
        public void uriSyntaxError(String uriString) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("uriSyntaxError");
            logItem.setObject(uriString);
        }
        
        public void unknownCorpusClasseError(String corpusClasseName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("unknownCorpusClasse");
            logItem.setObject(corpusClasseName);
        }
        
        /*public void unwknownCorpusURI(CorpusURI corpusURI) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("unwknownCorpusURI");
            logItem.setObject(corpusURI);
        }*/
    }
    
    private static class InternalCorpusClasse extends AbstractLibelleHolder implements CorpusClasse {
        private String name;
        private List uriList = new ArrayList();
        private int classeValue;
        
        private InternalCorpusClasse(String name) {
            super((name == null)?"_default":name);
            this.name = name;
        }
        
        public String getName() {
            return name;
        }
        
        public int getCorpusURICount() {
            return uriList.size();
        }
    
        public CorpusURI getCorpusURI(int i) {
            return (CorpusURI) uriList.get(i);
        }
        
         private void internalSetLibelle(String nvlibelle, int langInteger) {
            setLibelle(nvlibelle, langInteger);
        }
        
         private void addURI(CorpusURI corpusURI) {
            uriList.add(corpusURI);
        }

        public int getClasseValue() {
            return classeValue;
        }

         private void setClasseValue(int classeValue) {
            this.classeValue = classeValue;
        }
    }
    
}
