/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.conf.xml.*;
import org.w3c.dom.*;
import net.mapeadores.util.logging.*;
import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import fr.exemole.sctserver.*;
import net.mapeadores.scrutari.uri.*;
import java.util.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.conditions.*;


/**
 *
 * @author Vincent Calame
 */
public class SearchLogFilterConf {
    private SearchLogFilterLog processLog;
    private SearchLogDeny[] searchLogDenyArray;
    private ProcessURI processURI;
    
    private SearchLogFilterConf() {
    }
    
    public ProcessLog getProcessLog() {
        return processLog;
    }
    
    public SearchLogDeny[] getSearchLogDenyArray() {
        return searchLogDenyArray;
    }
    
    public static SearchLogFilterConf load(File f) throws IOException, ParserConfigurationException {
        SearchLogFilterConf conf = new SearchLogFilterConf();
        conf.init(f);
        return conf;
    }
    
    public ProcessURI getProcessURI() {
        return processURI;
    }
    
    private void init(File f) throws IOException, ParserConfigurationException {
        searchLogDenyArray = null;
        processURI = ProcessURI.fromFile(f);
        processLog = new SearchLogFilterLog(processURI );
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch(SAXException saxe) {
            processLog.saxException(saxe);
            return;
        }
        SearchLogFilterBuilder builder = new SearchLogFilterBuilder();
        SearchLogFilterDOMReader domReader = new SearchLogFilterDOMReader(builder, processLog);
        domReader. readSearchLogFilter(document.getDocumentElement());
        searchLogDenyArray = builder.toSearchLogDenyArray();
    }
    
    private static class SearchLogFilterBuilder implements SearchLogFilterDOMReader.Builder {
        List denyList = new ArrayList();
        
        public void addDeny(ContentCondition ipCondition, ContentCondition siteCondition, ContentCondition refCondition) {
            denyList.add(new InternalSearchLogDeny(ipCondition, siteCondition, refCondition));
        }
        
        public SearchLogDeny[] toSearchLogDenyArray() {
            int size = denyList.size();
            if (size == 0) return null;
            SearchLogDeny[] sld = new SearchLogDeny[size];
            for(int i = 0; i < size; i++) {
                sld[i] = (SearchLogDeny) denyList.get(i);
            }
            return sld;
        }
    }
    
    private static class SearchLogFilterLog extends MutableProcessLog implements SearchLogFilterDOMReader.ErrorHandler {
        
        public SearchLogFilterLog(ProcessURI processURI) {
            setCurrentURI(processURI);
        }
        
        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }
        
        public void unknownTagWarning(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("unknownTag");
            logItem.setObject(tagName);
        }
        
        public void noAttributeWarning(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("noAttribute");
            logItem.setObject(tagName);
        }
    }
    
    private static class InternalSearchLogDeny implements SearchLogDeny {
        private ContentCondition ipCondition;
        private ContentCondition siteCondition;
        private ContentCondition refCondition;
        
        InternalSearchLogDeny(ContentCondition ipCondition, ContentCondition siteCondition, ContentCondition refCondition) {
            this.ipCondition = ipCondition;
            this.siteCondition = siteCondition;
            this.refCondition = refCondition;
        }
        
        public ContentCondition getContentCondition(short type) {
            switch(type) {
                case SearchLogDeny.IP_DENY :
                    return ipCondition;
                case SearchLogDeny.SITE_DENY :
                    return siteCondition;
                case SearchLogDeny.REF_DENY :
                    return refCondition;
                default :
                    throw new IllegalArgumentException("Wrong type : " + type);
            }
        }
    }
    
}
