/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.conf.xml.*;
import org.w3c.dom.*;
import net.mapeadores.util.logging.*;
import java.io.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import fr.exemole.sctserver.*;
import net.mapeadores.scrutari.uri.*;
import java.util.*;
import fr.exemole.sctserver.admin.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.conditions.*;


/**
 *
 * @author Vincent Calame
 */
public class SourcesConf {
    private SourcesLog processLog;
    private UrlKey[] infoUrlKeyArray = null;
    private ProcessURI processURI;
    
    public SourcesConf() {
    }
    
    public ProcessLog getProcessLog() {
        return processLog;
    }
    
    public ProcessURI getProcessURI() {
        return processURI;
    }
    
    public UrlKey[] getInfoUrlKeyArray() {
        return infoUrlKeyArray;
    }
    
    public static SourcesConf load(File f) throws IOException, ParserConfigurationException {
        SourcesConf conf = new SourcesConf();
        conf.init(f);
        return conf;
    }
    
    private void init(File f) throws IOException, ParserConfigurationException {
        infoUrlKeyArray = null;
        processURI = ProcessURI.fromFile(f);
        processLog = new SourcesLog(processURI );
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document = null;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch(SAXException saxe) {
            processLog.saxException(saxe);
            return;
        }
        SourcesBuilder builder = new SourcesBuilder();
        SourcesDOMReader domReader = new SourcesDOMReader(builder, processLog);
        domReader.readSources(document.getDocumentElement());
        infoUrlKeyArray = builder.toInfoUrlKeyArray();
    }
    
    private static class SourcesBuilder implements SourcesDOMReader.Builder {
        LinkedHashSet set = new LinkedHashSet();
        
        public boolean addInfoUrl(UrlKey urlKey) {
            if (set.contains(urlKey)) return false;
            set.add(urlKey);
            return true;
        }
        
        private UrlKey[] toInfoUrlKeyArray() {
            int size = set.size();
            if (size == 0) return null;
            UrlKey[] infoUrlKeyArray = new UrlKey[size];
            int p = 0;
            for(Iterator it = set.iterator(); it.hasNext();) {
                infoUrlKeyArray[p] = (UrlKey) it.next();
                p++;
            }
            return infoUrlKeyArray;
        }
    }
    
    private static class SourcesLog extends MutableProcessLog implements SourcesDOMReader.ErrorHandler {
        
        public SourcesLog(ProcessURI processURI) {
            setCurrentURI(processURI);
        }
        
        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }
        
        public void unknownTagWarning(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("unknownTag");
            logItem.setObject(tagName);
        }
        
        public void emptyElementError(String tagName) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("emptyElement");
            logItem.setObject(tagName);
        }
        
        public void malformedUrlError(String uriString,String message) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.SEVERE_CATEGORY);
            logItem.setKey("malformedUrl");
            logItem.setObject(uriString + " (" + message + ")");
        }
        
        public void doublonUrlWarning(String urlString, String urlKey) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SctServer.WARNING_CATEGORY);
            logItem.setKey("doublonUrl");
            logItem.setObject(urlString + " (" + urlKey + ")");
        }
    }
    
}
