/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.conf.xml;

import fr.exemole.sctserver.admin.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.scrutari.uri.*;

/**
 *
 * @author Vincent Calame
 */
public class CorpusClassesDOMReader {
    private Builder builder;
    private ErrorHandler errorHandler;
    
    public CorpusClassesDOMReader(Builder builder, ErrorHandler errorHandler) {
        this.builder = builder;
        this.errorHandler = errorHandler;
    }
    
    public void readCorpusClasses(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if ((tagName.equals("classes-defs")) || (tagName.equals("classe-def-list"))) {
                    addDefs(element);
                }
            }
        }
        for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("corpus-list")) {
                    addCorpusList(element);
                }
                else if ((!tagName.equals("classes-defs")) && (!tagName.equals("classe-def-list"))) {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }
    
    private void addDefs(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("classe-def")) {
                    addDef(element);
                }
                else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }
    
    
    private void addDef(Element element_xml) {
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "name");
            return;
        }
        if (name.equals("_default")) name= null;
        CorpusClasse corpusClasse = builder.addClasse(name);
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("lib")) {
                    try {
                        int langInteger = LangInteger.parse(element.getAttribute("xml:lang"));
                        String lib = XMLUtils.getData(element);
                        builder.setLibelle(corpusClasse, lib, langInteger);
                    }
                    catch(LangIntegerException lie) {
                        
                    }
                }
                else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
        
    }
    
    private void addCorpusList(Element element_xml) {
        String name = element_xml.getAttribute("classe-name");
        if (name.length() == 0) {
            errorHandler.emptyAttributeError(element_xml.getTagName(), "classe-name");
            return;
        }
        CorpusClasse corpusClasse = builder.getCorpusClasse(name);
        if (corpusClasse == null) {
            errorHandler.unknownCorpusClasseError(name);
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("uri")) {
                    String uriString = XMLUtils.getData(element);
                    if (uriString.length() == 0) {
                        errorHandler.emptyElementError("uri");
                        continue;
                    }
                    try {
                        ScrutariURI scrutariURI = URIParser.parse(uriString);
                        if (scrutariURI instanceof CorpusURI) {
                            builder.addURI(corpusClasse, (CorpusURI) scrutariURI);
                        }
                    }
                    catch(URIParseException uripe) {
                        errorHandler.uriSyntaxError(uriString);
                    }
                }
                else {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }
    
    public static interface Builder {
        
        public CorpusClasse addClasse(String name);
        
        public CorpusClasse getCorpusClasse(String name);
        
        public void setLibelle(CorpusClasse corpusClasse, String libelle, int langInteger);
     
        public boolean addURI(CorpusClasse corpusClasse, CorpusURI corpusURI);
    }
    
    public static interface ErrorHandler {
        
        public void unknownTagWarning(String tagName);
        
        public void emptyAttributeError(String tagName, String attributeName);
    
        public void emptyElementError(String tagName);
        
        public void uriSyntaxError(String uriString);
        
        public void unknownCorpusClasseError(String corpusClasseName);
    }
    
}
