/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.conf.xml;

import fr.exemole.sctserver.admin.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.conditions.*;

/**
 *
 * @author Vincent Calame
 */
public class SearchLogFilterDOMReader {
    private ErrorHandler errorHandler;
    private Builder builder;
    
    public SearchLogFilterDOMReader(Builder builder, ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }
    
    public void readSearchLogFilter(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("deny")) {
                    addDeny(element);
                } else  {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }
    
    private void addDeny(Element element_xml) {
        ContentCondition ipCondition = Conditions.parseContentCondition(element_xml.getAttribute("ip"));
        ContentCondition siteCondition = Conditions.parseContentCondition(element_xml.getAttribute("site"));
        ContentCondition refCondition = Conditions.parseContentCondition(element_xml.getAttribute("ref"));
        if ((ipCondition == null) && (refCondition == null) && (siteCondition == null)) {
            errorHandler.noAttributeWarning("deny");
            return;
        }
        builder.addDeny(ipCondition, siteCondition, refCondition);
    }
    
    
    
    public static interface ErrorHandler {
        
        public void unknownTagWarning(String tagName);
        
        public void noAttributeWarning(String tagName);
    }
    
    public static interface Builder {
        
        public void addDeny(ContentCondition ipCondition, ContentCondition siteCondition, ContentCondition refCondition);
    }
    
}
