/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.conf.xml;

import fr.exemole.sctserver.admin.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.conditions.*;
import java.net.*;

/**
 *
 * @author Vincent Calame
 */
public class SourcesDOMReader {
    private ErrorHandler errorHandler;
    private Builder builder;
        
    public SourcesDOMReader(Builder builder, ErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.builder = builder;
    }
    
    public void readSources(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("info-url")) {
                    addInfoUrl(element);
                } else  {
                    errorHandler.unknownTagWarning(tagName);
                }
            }
        }
    }
    
    public void addInfoUrl(Element element_xml) {
        String urlString = XMLUtils.getData(element_xml);
        if (urlString.length() == 0) {
            errorHandler.emptyElementError("info-url");
            return;
        }
        try {
            URL url = new URL(urlString);
            UrlKey urlKey = new UrlKey(url);
            boolean done = builder.addInfoUrl(urlKey);
            if (!done) {
                errorHandler.doublonUrlWarning(urlString,urlKey.getKeyString());
            }
        }
        catch(MalformedURLException mue) {
            errorHandler.malformedUrlError(urlString, mue.getMessage());
        }
    }
    
    public static interface ErrorHandler {
        
        public void unknownTagWarning(String tagName);
        
        public void emptyElementError(String tagName);
        
        public void malformedUrlError(String urlString,String message);
        
        public void doublonUrlWarning(String urlString, String urlKey);
    }
    
    public static interface Builder {
        
        public boolean addInfoUrl(UrlKey urlKey);
        
    }
    
}
