/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.simple.catalog;

import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.uri.BaseURI;

/**
 *
 * @author Vincent Calame
 */
public class BaseDataRequest extends SimpleRequest {
    private int langUi;
    private BaseData baseData;
    
    public BaseDataRequest(ScrutariDB scrutariDB,int langUi) {
        super(scrutariDB);
        this.langUi = langUi;
    }
    
    public BaseDataRequest(ScrutariDB scrutariDB,int langUi, BaseData baseData) {
        super(scrutariDB);
        this.langUi = langUi;
        this.baseData = baseData;
    }
    
    public void printResponse(PrintWriter pw) throws IOException {
        if (baseData != null) {
            printResponse(pw,baseData);
        }
        else {
            DataManager dataManager = scrutariDB.getDataManager();
            for (int i = 0; i < dataManager.getBaseCount();i++) {
                BaseData currentBaseData = dataManager.getBaseDataByIndex(i);
                printResponse(pw,currentBaseData);
            }
        }
    }
    
    private void printResponse(PrintWriter pw, BaseData currentBaseData) {
        int checkedLangInteger = currentBaseData.checkLangUi(langUi);
        BaseURI baseURI = currentBaseData.getBaseURI();
        String name = baseURI.getBaseName();
        pw.print("#b");
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(currentBaseData.getCodeBase());
        pw.print('\n');
        pw.print("auuid=");
        pw.print(baseURI.getAuthorityUUID());
        pw.print('\n');
        pw.print("nmbdf=");
        pw.print(baseURI.getBaseName());
        pw.print('\n');
        pw.print("cacrp=");
        for (int i = 0; i < currentBaseData.getCorpusCount();i++) {
            if (i > 0) pw.print(',');
            pw.print(currentBaseData.getCodeCorpus(i));
        }
        pw.print('\n');
        pw.print("caths=");
        for (int i = 0; i < currentBaseData.getThesaurusCount();i++) {
            if (i > 0) pw.print(',');
            pw.print(currentBaseData.getCodeThesaurus(i));
        }
        pw.print('\n');
        pw.print("bfico=");
        String baseIcon = currentBaseData.getBaseIcon(checkedLangInteger);
        if (baseIcon != null) {
            pw.print(baseIcon);
        }
        pw.print('\n');
        pw.print("insht");
        pw.print('=');
        pw.print(LibelleUtils.getLibelleString(currentBaseData,BaseData.INTITULE_SHORT,checkedLangInteger,name));
        pw.print('\n');
        pw.print("inlng");
        pw.print('=');
        pw.print(LibelleUtils.getLibelleString(currentBaseData,BaseData.INTITULE_LONG,checkedLangInteger,name));
        pw.print('\n');
    }
    
}
