/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.simple.catalog;

import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.uri.CorpusURI;

/**
 *
 * @author Vincent Calame
 */
public class CorpusDataRequest extends SimpleRequest {
    private int langUi;
    private CorpusData corpusData;
    
    public CorpusDataRequest(ScrutariDB scrutariDB,int langUi) {
        super(scrutariDB);
        this.langUi = langUi;
    }
    
    public CorpusDataRequest(ScrutariDB scrutariDB,int langUi,CorpusData corpusData) {
        super(scrutariDB);
        this.langUi = langUi;
        this.corpusData = corpusData;
    }
    
    public void printResponse(PrintWriter pw) throws IOException {
        if (corpusData != null) {
            printCorpus(pw,corpusData);
        }
        else {
            DataManager dataManager = scrutariDB.getDataManager();
            for (int i = 0; i < dataManager.getCorpusCount();i++) {
                CorpusData currentCorpusData = dataManager.getCorpusDataByIndex(i);
                printCorpus(pw,currentCorpusData);
            }
        }
    }
    
    void printCorpus(PrintWriter pw, CorpusData currentCorpusData) {
        BaseData currentBaseData = scrutariDB.getDataManager().getBaseDataByCode(currentCorpusData.getCodeBase());
        int checkedLangInteger = currentBaseData.checkLangUi(langUi);
        int codecrp = currentCorpusData.getCodeCorpus();
        int totalcomplements = currentCorpusData.getComplementMaxNumber();
        CorpusURI corpusURI = currentCorpusData.getCorpusURI();
        String name = corpusURI.getCorpusName();
        pw.print("#c");
        pw.print('\n');
        pw.print("cdcrp=");
        pw.print(codecrp);
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(currentCorpusData.getCodeBase());
        pw.print('\n');
        pw.print("nmcrp=");
        pw.print(corpusURI.getCorpusName());
        pw.print('\n');
        pw.print("ttcmp=");
        pw.print(totalcomplements);
        pw.print('\n');
        pw.print("incrp=");
        pw.print(LibelleUtils.getLibelleString(currentCorpusData,CorpusData.INTITULECORPUS_FIELDNUMBER,checkedLangInteger,name));
        pw.print('\n');
        pw.print("infch=");
        pw.print(LibelleUtils.getLibelleString(currentCorpusData,CorpusData.INTITULEFICHE_FIELDNUMBER,checkedLangInteger,name));
        pw.print('\n');
        for (int i = 1; i <= totalcomplements;i++) {
            pw.print("inc");
            if (i < 10) pw.print('0');
            pw.print(i);
            pw.print('=');
            pw.print(LibelleUtils.getLibelleString(currentCorpusData,i,checkedLangInteger,"c" + i));
            pw.print('\n');
        }
    }
    
}
