/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.simple.catalog;

import fr.exemole.sctserver.request.simple.*;
import java.io.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.collections.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.scrutari.lexie.*;

/**
 *
 * @author Vincent Calame
 */
public class FicheResultRequest extends SimpleRequest implements FieldNumbers {
    private int langUi;
    private SearchResult searchResult;
    //private FicheSearchResultInfo ficheResult;
    private boolean decalageutf8;
    private int plageStart=0;
    private int plageLength=-1;
    private GlobalSearchOptions globalSearchOptions;
    
    public FicheResultRequest(ScrutariDB scrutariDB,SearchResult searchResult) {
        super(scrutariDB);
        this.langUi = searchResult.getSearchOptions().getLangUi();
        this.searchResult = searchResult;
    }
    
    public FicheResultRequest(ScrutariDB scrutariDB,SearchResult searchResult, int plageStart, int plageLength) {
        this(scrutariDB,searchResult);
        this.plageStart = plageStart;
        this.plageLength = plageLength;
    }
    
    /*public FicheResultRequest(ScrutariDB scrutariDB,FicheSearchResultInfo ficheResult,int langUi) {
        super(scrutariDB);
        this.langUi = langUi;
        this.ficheResult = ficheResult;
    }*/
    
    public void setDecalageUTF8(boolean decalageutf8) {
        this.decalageutf8 = decalageutf8;
    }
    
    public void setGlobalSearchOptions(GlobalSearchOptions globalSearchOptions) {
        this.globalSearchOptions = globalSearchOptions;
    }
    
    public void printResponse(PrintWriter pw) throws IOException {
        /*if (ficheResult != null) {
            printFicheResult(pw,ficheResult);
        }
        else {*/
            int count = searchResult.getFicheCount();
            int min = (plageLength==-1)?count:Math.min(count, plageStart+plageLength);
            for(int i =  plageStart;i < min;i++) {
                printFicheResult(pw,searchResult.getFicheResult(i));
            }
        //}
    }
    
    private void printFicheResult(PrintWriter pw,FicheSearchResultInfo currentFicheResult) {
        DataManager dataManager = scrutariDB.getDataManager();
        int codefiche = currentFicheResult.getCode();
        FicheData ficheData = dataManager.getFicheDataByCode(codefiche);
        int codecorpus = ficheData.getCodeCorpus();
        int codebase = ficheData.getCodeBase();
        BaseData baseInfo = dataManager.getBaseDataByCode(codebase);
        pw.print("#f");
        pw.print('\n');
        pw.print("cdfch=");
        pw.print(codefiche);
        pw.print('\n');
        pw.print("cdcrp=");
        pw.print(codecorpus);
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(codebase);
        pw.print('\n');
        FicheURI ficheURI = ficheData.getFicheURI();
        pw.print("idcrp=");
        pw.print(ficheURI.getFicheId());
        pw.print('\n');
        pw.print("clcrp=");
        if (globalSearchOptions != null) {
            pw.print(globalSearchOptions.getClasseValue(codecorpus));
        }
        else {
            pw.print('0');
        }
        pw.print('\n');
        String ficheIcon = ficheData.getFicheIcon();
        pw.print("fcico=");
        if (ficheIcon != null) {
            pw.print(ficheIcon);
        }
        pw.print('\n');
        String titre = ficheData.getTitre();
        String soustitre = ficheData.getSoustitre();
        if (titre == null) titre="?";
        if (soustitre == null) soustitre="";
        if (decalageutf8) {
            int[] decalagetitre = new int[titre.length()];
            int[] decalagesoustitre = new int[soustitre.length()];
            IntHashMap decalagescomplement = null;
            int decalage = 0;
            pw.print("titre=");
            for (int i = 0; i < titre.length();i++) {
                char c = titre.charAt(i);
                pw.print(c);
                decalage = decalage + DataUtils.getDecalageUTF8(c);
                decalagetitre[i] = decalage;
            }
            pw.print('\n');
            pw.print("ssttr=");
            decalage = 0;
            for (int i = 0; i < soustitre.length();i++) {
                char c = soustitre.charAt(i);
                pw.print(c);
                decalage = decalage + DataUtils.getDecalageUTF8(c);
                decalagesoustitre[i] = decalage;
            }
            pw.print('\n');
            printPosition(pw, TITRE_FIELDNUMBER, currentFicheResult.getSubstringPositionList(TITRE_FIELDNUMBER),decalagetitre);
            printPosition(pw, SOUSTITRE_FIELDNUMBER, currentFicheResult.getSubstringPositionList(SOUSTITRE_FIELDNUMBER),decalagesoustitre);
            for (int p = 1; p <= ficheData.getComplementMaxNumber();p++) {
                SearchTokenPositionList positionList = currentFicheResult.getSubstringPositionList(p);
                if (positionList != null) {
                    pw.print("cmp");
                    if (p < 10) pw.print('0');
                    pw.print(p);
                    pw.print('=');
                    String strg_complement = ficheData.getComplementByNumber(p);
                    int[] decalagecomp = new int[strg_complement.length()];
                    decalage = 0;
                    for (int j= 0; j < strg_complement.length();j++) {
                        char c = strg_complement.charAt(j);
                        pw.print(c);
                        decalage = decalage + DataUtils.getDecalageUTF8(c);
                        decalagecomp[j] = decalage;
                    }
                    pw.print('\n');
                    printPosition(pw,p,positionList,decalagecomp);
                }
            }
        } else {
            pw.print("titre=");
            pw.print(titre);
            pw.print('\n');
            pw.print("ssttr=");
            pw.print(soustitre);
            pw.print('\n');
            for (int p = 1; p <= ficheData.getComplementMaxNumber();p++) {
                SearchTokenPositionList positionList = currentFicheResult.getSubstringPositionList(p);
                if (positionList != null) {
                    pw.print("cmp");
                    if (p < 10) pw.print('0');
                    pw.print(p);
                    pw.print('=');
                    pw.print(ficheData.getComplementByNumber(p));
                    pw.print('\n');
                }
            }
        }
        pw.print("hreff=");
        int checkedLangInteger = baseInfo.checkLangUi(langUi);
        String href = ficheData.getHref(checkedLangInteger);
        if (href != null) pw.print(href);
        pw.print('\n');
        int dateInteger = ficheData.getDateInteger();
        if (dateInteger != IntDate.NULL) {
            pw.print("datan=");
            pw.print(IntDate.getAnnee(dateInteger));
            pw.print('\n');
            short typedate = IntDate.getDateIntegerType(dateInteger);
            switch(typedate) {
                case IntDate.TYPE_ANNEE :
                    pw.print("dattp=a\n");
                    break;
                case IntDate.TYPE_SEMESTRE :
                    pw.print("dattp=s\n");
                    pw.print("datsm=");
                    pw.print(IntDate.getSemestre(dateInteger));
                    pw.print('\n');
                    break;
                case IntDate.TYPE_TRIMESTRE :
                    pw.print("dattp=t\n");
                    pw.print("dattm=");
                    pw.print(IntDate.getTrimestre(dateInteger));
                    pw.print('\n');
                    break;
                case IntDate.TYPE_MOIS :
                    pw.print("dattp=m\n");
                    pw.print("datms=");
                    pw.print(IntDate.getMois(dateInteger));
                    pw.print('\n');
                    break;
                case IntDate.TYPE_JOUR :
                    pw.print("dattp=j\n");
                    pw.print("datms=");
                    pw.print(IntDate.getMois(dateInteger));
                    pw.print('\n');
                    pw.print("datjr=");
                    pw.print(IntDate.getJour(dateInteger));
                    pw.print('\n');
                    break;
            }
        } else {
            pw.print("dattp=o\n");
        }
        int motcleCount = currentFicheResult.getMotcleCount();
        if (motcleCount > 0) {
            for (int i = 0;i < motcleCount;i++) {
                int codemotcle = currentFicheResult.getMotcleCode(i);
                pw.print("mtc");
                if (i < 10) pw.print('0');
                pw.print(i);
                pw.print('=');
                pw.print(codemotcle);
                pw.print('\n');
            }
        }
    }
    
    private void printPosition(PrintWriter pw,int fieldNumber,SearchTokenPositionList positionList, int[] decalages) {
        if (positionList == null) return;
        for (int k = 0; k < positionList.size();k++) {
            SubstringPosition substringPosition = positionList.getSearchTokenPosition(k);
            int tokenIndex = positionList.getSearchTokenIndexForPosition(k);
            int positiondebut = substringPosition.getBeginIndex();
            int decalagedebut = 0;
            if (positiondebut > 0) {
                decalagedebut = decalages[positiondebut-1];
            }
            pw.print("loc");
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            printFieldNumber(pw,fieldNumber);
            pw.print("pos");
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(positiondebut + decalagedebut);
            pw.print('\n');
            pw.print("lng");
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            int longueur = substringPosition.getLength();
            longueur = longueur + decalages[positiondebut-1 + longueur]- decalagedebut;
            pw.print(longueur);
            pw.print('\n');
        }
    }
    
    /* 
     Absence de position lorsqu'il n'y a pas de décalage
     
     private void printPosition(PrintWriter pw,int fieldNumber,SearchResult.SubstringPositionList positionList) {
        if (positionList == null) return;
        for (int k = 0; k < positionList.size();k++) {
            SubstringPosition substringPosition = positionList.getSubstringPosition(k);
            int tokenIndex = positionList.getTokenIndex(k);
            pw.print("loc");
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            printFieldNumber(pw,fieldNumber);
            pw.print("pos");
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(substringPosition.getBeginIndex());
            pw.print('\n');
            pw.print("lng");
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(substringPosition.getLength());
            pw.print('\n');
        }
    }*/
    
    private void printFieldNumber(PrintWriter pw,int fieldNumber) {
        if (fieldNumber  == TITRE_FIELDNUMBER) {
            pw.print("titre");
        } else if (fieldNumber  == SOUSTITRE_FIELDNUMBER) {
            pw.print("ssttr");
        } else if (fieldNumber == NONE_FIELDNUMBER) {
            pw.print("");
        } else if (fieldNumber == NOPERTINENT_FIELDNUMBER) {
            pw.print("");
        } else if (fieldNumber < 10) {
            pw.print("cmp0");
            pw.print(fieldNumber);
        } else {
            pw.print("cmp");
            pw.print(fieldNumber);
        }
        pw.print('\n');
    }

   
    
}
