/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.simple.catalog;

import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.searchengine.MotcleSearchResultInfo;
import net.mapeadores.scrutari.searchengine.SearchResult;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;

/**
 *
 * @author Vincent Calame
 */
public class MotcleResultRequest extends SimpleRequest {
    private SearchResult searchResult;
    private boolean decalageutf8;
    private int uiLangInteger;
    
    public MotcleResultRequest(ScrutariDB scrutariDB,SearchResult searchResult) {
        super(scrutariDB);
        this.searchResult = searchResult;
    }
    
    public void setDecalageUTF8(boolean decalageutf8) {
        this.decalageutf8 = decalageutf8;
    }
    
    /**
     * Précise la langue de préférence
     */
    public void setUiLangInteger(int uiLangInteger) {
        this.uiLangInteger = uiLangInteger;
    }
    
    public void printResponse(PrintWriter pw) throws IOException {
        int count = searchResult.getAllMotcleCount();
        for (int i = 0; i < count;i++) {
            printMotcleResult(pw,searchResult.getMotcleResult(i));
        }
    }
    
    private void printMotcleResult(PrintWriter pw,MotcleSearchResultInfo motcleSearchResultInfo) {
        DataManager dataManager = scrutariDB.getDataManager();
        int codemotcle = motcleSearchResultInfo.getCode();
        MotcleData motcleData = dataManager.getMotcleDataByCode(codemotcle);
        int codethesaurus = motcleData.getCodeThesaurus();
        pw.print("#m");
        pw.print('\n');
        pw.print("cdmtc=");
        pw.print(codemotcle);
        pw.print('\n');
        pw.print("cdths=");
        pw.print(codethesaurus);
        pw.print('\n');
        ThesaurusData thesaurusData= dataManager.getThesaurusDataByCode(codethesaurus);
        pw.print("cdbf=");
        pw.print(thesaurusData.getCodeBase());
        pw.print('\n');
        String uilib = "";
        StringBuffer bufcodes = new StringBuffer();
        int libcount = motcleData.getLibelleCount();
        for (int i = 0; i < libcount;i++) {
            Libelle lib = motcleData.getLibelleByIndex(i);
            int langInteger = lib.getLangInteger();
            SearchTokenPositionList positionList = motcleSearchResultInfo.getSearchTokentPositionListByLang(langInteger);
            if (positionList != null) {
                if (bufcodes.length() > 0) bufcodes.append(',');
                String langString = LangInteger.toString(langInteger);
                bufcodes.append(langString);
                if (decalageutf8) {
                    printWithDecalage(pw, lib,positionList, langString);
                    
                } else {
                    printWithoutDecalage(pw, lib,positionList, langString);
                }
            } else if (langInteger == uiLangInteger) {
                uilib = lib.getLibelleString();
            }
        }
        pw.print("lglib=");
        pw.print(bufcodes.toString());
        pw.print('\n');
        pw.print("uilib=");
        pw.print(uilib);
        pw.print('\n');
    }
    
    private void printWithDecalage(PrintWriter pw,Libelle lib,SearchTokenPositionList positionList, String langString) {
        pw.print("lib");
        pw.print(langString);
        pw.print('=');
        String libelle = lib.getLibelleString();
        int[] decalagelibelle = new int[libelle.length()];
        int decalage = 0;
        for (int j = 0; j < libelle.length();j++) {
            char c = libelle.charAt(j);
            pw.print(c);
            decalage = decalage + DataUtils.getDecalageUTF8(c);
            decalagelibelle[j] = decalage;
        }
        pw.print('\n');
        for (int k = 0; k < positionList.size();k++) {
            SubstringPosition substringPosition = positionList.getSearchTokenPosition(k);
            int tokenIndex = positionList.getSearchTokenIndexForPosition(k);
            int positiondebut = substringPosition.getBeginIndex();
            int decalagedebut = 0;
            if (positiondebut > 0) decalagedebut = decalagelibelle[positiondebut -1];
            pw.print("p");
            pw.print(langString);
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(positiondebut + decalagedebut);
            pw.print('\n');
            int longueur = substringPosition.getLength();
            longueur = longueur + decalagelibelle[positiondebut-1 + longueur] - decalagedebut;
            pw.print("l");
            pw.print(langString);
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(longueur);
            pw.print('\n');
        }
        
    }
    
    private void printWithoutDecalage(PrintWriter pw,Libelle lib,SearchTokenPositionList positionList, String langString) {
        pw.print("lib");
        pw.print(langString);
        pw.print('=');
        String libelle = lib.getLibelleString();
        pw.print(libelle);
        pw.print('\n');
        for (int k = 0; k < positionList.size();k++) {
            SubstringPosition substringPosition = positionList.getSearchTokenPosition(k);
            int tokenIndex = positionList.getSearchTokenIndexForPosition(k);
            pw.print("p");
            pw.print(langString);
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(substringPosition.getBeginIndex());
            pw.print('\n');
            pw.print("l");
            pw.print(langString);
            if (tokenIndex < 10) pw.print('0');
            pw.print(tokenIndex);
            pw.print('=');
            pw.print(substringPosition.getLength());
            pw.print('\n');
        }
    }
}
