/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.simple.catalog;

import fr.exemole.sctserver.request.simple.SimpleRequest;
import java.io.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.util.*;
import net.mapeadores.scrutari.uri.ThesaurusURI;

/**
 *
 * @author Vincent Calame
 */
public class ThesaurusDataRequest extends SimpleRequest {
    private int langUi;
    private ThesaurusData thesaurusData;
    
    public ThesaurusDataRequest(ScrutariDB scrutariDB,int langUi) {
        super(scrutariDB);
        this.langUi = langUi;
    }
    
    public ThesaurusDataRequest(ScrutariDB scrutariDB,int langUi,ThesaurusData thesaurusData) {
        super(scrutariDB);
        this.langUi = langUi;
        this.thesaurusData = thesaurusData;
    }
    
    public void printResponse(PrintWriter pw) throws IOException {
        if (thesaurusData != null) {
            printThesaurus(pw, thesaurusData);
        }
        else {
            DataManager dataManager = scrutariDB.getDataManager();
            for (int i = 0; i < dataManager.getThesaurusCount();i++) {
                ThesaurusData currentThesaurusData = dataManager.getThesaurusDataByIndex(i);
               printThesaurus(pw,currentThesaurusData);
            }
        }
    }
    
    private void printThesaurus(PrintWriter pw, ThesaurusData currentThesaurusData) {
        BaseData currentBaseData = scrutariDB.getDataManager().getBaseDataByCode(currentThesaurusData.getCodeBase());
        int checkedLangInteger = currentBaseData.checkLangUi(langUi);
        int codeths = currentThesaurusData.getCodeThesaurus();
        ThesaurusURI thesaurusURI = currentThesaurusData.getThesaurusURI();
        pw.print("#t");
        pw.print('\n');
        pw.print("cdths=");
        pw.print(codeths);
        pw.print('\n');
        pw.print("cdbdf=");
        pw.print(currentThesaurusData.getCodeBase());
        pw.print('\n');
        pw.print("nmths=");
        pw.print(thesaurusURI.getThesaurusName());
        pw.print('\n');
        pw.print("inths=");
        pw.print(LibelleUtils.getLibelleString(currentThesaurusData,checkedLangInteger,thesaurusURI.getThesaurusName()));
        ThesaurusData.ThesaurusLangData thesaurusLangInfo = currentThesaurusData.getThesaurusLangDataByLang(checkedLangInteger);
        pw.print('\n');
    }
    
}
