/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.uri;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.DataManager;
import net.mapeadores.scrutari.uri.BaseURI;
import net.mapeadores.scrutari.uri.CorpusURI;
import net.mapeadores.scrutari.uri.FicheURI;
import net.mapeadores.scrutari.uri.ScrutariURI;
import net.mapeadores.scrutari.uri.MotcleURI;
import net.mapeadores.scrutari.uri.ThesaurusURI;
import net.mapeadores.scrutari.uri.URIParseException;
import net.mapeadores.scrutari.uri.URIParser;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.IOException;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.Libelle;

/**
 *
 * @author Vincent Calame
 */
public class URIRequest {
    public final static short NOMOREAVAILABLE_TYPE = -3;
    public final static short MALFORMED_TYPE = -2;
    public final static short NOTFOUND_TYPE = -1;
    public final static short FICHE_TYPE = 1;
    public final static short CORPUS_TYPE = 2;
    public final static short THESAURUS_TYPE = 3;
    public final static short MOTCLE_TYPE = 4;
    public final static short BASE_TYPE = 5;
    short type;
    ScrutariURI mapeadoresURI;
    DataManager dataManager;
    int code;
    Object DataObject;
    
    public URIRequest(String param_uri,ScrutariDB scrutariDB) {
        dataManager = scrutariDB.getDataManager();
        ScrutariURICoder uriCoder = dataManager.getScrutariURICoder();
        try {
            ScrutariURI scrutariURI = URIParser.parse(param_uri);
            if (scrutariURI instanceof FicheURI) {
                type = FICHE_TYPE;
                code = uriCoder.getCode(scrutariURI);
                if (code < 0) type = NOTFOUND_TYPE;
                else {
                    DataObject= dataManager.getFicheDataByCode(code);
                    if (DataObject == null) type = NOMOREAVAILABLE_TYPE;
                }
            }
            else if (scrutariURI instanceof CorpusURI) {
                type = CORPUS_TYPE;
                code = uriCoder.getCode(scrutariURI);
                if (code < 0) type = NOTFOUND_TYPE;
                else {
                    DataObject = dataManager.getCorpusDataByCode(code);
                    if (DataObject == null) type = NOMOREAVAILABLE_TYPE;
                }
            }
            else if (scrutariURI instanceof MotcleURI) {
                type = MOTCLE_TYPE;
                code = uriCoder.getCode(scrutariURI);
                if (code < 0) type = NOTFOUND_TYPE;
                else {
                    DataObject = dataManager.getMotcleDataByCode(code);
                    if (DataObject == null) type = NOMOREAVAILABLE_TYPE;
                }
            }
            else if (scrutariURI instanceof ThesaurusURI) {
                type = THESAURUS_TYPE;
                code = uriCoder.getCode(scrutariURI);
                if (code < 0) type = NOTFOUND_TYPE;
                else {
                    DataObject = dataManager.getThesaurusDataByCode(code);
                    if (DataObject == null) type = NOMOREAVAILABLE_TYPE;
                }
            }
            else if (scrutariURI instanceof BaseURI) {
                type = BASE_TYPE;
                code = uriCoder.getCode(scrutariURI);
                if (code < 0) type = NOTFOUND_TYPE;
                else {
                    DataObject = dataManager.getBaseDataByCode(code);
                    if (DataObject == null) type = NOMOREAVAILABLE_TYPE;
                }
            }
            else {
                type = NOTFOUND_TYPE;
            }
        }
        catch(URIParseException uriParseException) {
            type = MALFORMED_TYPE;
        }
    }
    
    public void printResponse(HttpServletResponse reponse) throws IOException {
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8'?>");
        switch(type) {
            case NOMOREAVAILABLE_TYPE :
                pw.print("<noresult cause=\"noMoreAvailable\"/>");
                break;
            case MALFORMED_TYPE :
                pw.print("<noresult cause=\"malformedURI\"/>");
                break;
            case NOTFOUND_TYPE :
                pw.print("<noresult cause=\"notfoundURI\"/>");
                break;
            case FICHE_TYPE :
                printFicheURI(pw,(FicheData)DataObject);
                break;
            case CORPUS_TYPE :
                printCorpusURI(pw,(CorpusData)DataObject);
                break;
            case MOTCLE_TYPE :
                printMotcleURI(pw,(MotcleData)DataObject);
                break;
           case BASE_TYPE :
                printBaseURI(pw,(BaseData)DataObject);
                break;
        }
    }
    
    void printCorpusURI(PrintWriter pw,CorpusData corpusData) {
        pw.print("<corpus>");
        pw.print("<count-Data fiche-count=\"");
        pw.print(String.valueOf(corpusData.getFicheCount()));
        pw.print("\" complement-count=\"");
        pw.print(String.valueOf(corpusData.getComplementMaxNumber()));
        pw.print("\"/>");
        int numbermax = corpusData.getComplementMaxNumber();
        int count = corpusData.getLangDataCount();
        for (int i = 0; i < count;i++) {
            CorpusData.CorpusLangData langData = corpusData.getLangDataByIndex(i);
            pw.print("<lang-Data lang=\"");
            LangInteger.printString(pw, langData.getLangInteger());
            pw.print("\">");
            printTag("fiche-count", String.valueOf(langData.getFicheCount()),pw,true);
            printTag("intitule-corpus", langData.getLibelleString(CorpusData.INTITULECORPUS_FIELDNUMBER),pw,true);
            printTag("intitule-fiche", langData.getLibelleString(CorpusData.INTITULEFICHE_FIELDNUMBER),pw,true);
            for (int k = 1; k <= numbermax;k++) {
                printTag("intitule-complement", langData.getLibelleString(k),pw,true);
            }
            pw.print("</lang-Data>");
        }
        /*printTag("titre",ficheData.getTitre(),pw);
        printTag("soustitre",ficheData.getSoustitre(),pw);
        printTag("href",ficheData.getHref(),pw);
        int maxnumber = ficheData.getComplementMaxNumber();
        if (maxnumber > 0) {
            for(int i = 1;i <= maxnumber;i++) {
                printTag("complement",ficheData.getComplementByNumber(i),pw);
            }
        }*/
        pw.print("</corpus>");
    }
    
    void printFicheURI(PrintWriter pw, FicheData ficheData) {
        pw.print("<fiche>");
        printTag("titre",ficheData.getTitre(),pw,true);
        printTag("soustitre",ficheData.getSoustitre(),pw,true);
        printTag("href",ficheData.getRawHref(),pw,true);
        int maxnumber = ficheData.getComplementMaxNumber();
        if (maxnumber > 0) {
            for(int i = 1;i <= maxnumber;i++) {
                printTag("complement",ficheData.getComplementByNumber(i),pw,true);
            }
        }
        pw.print("</fiche>");
    }
    
    void printMotcleURI(PrintWriter pw, MotcleData motcleData) {
        pw.print("<motcle>");
        int count = motcleData.getLibelleCount();
        for (int i = 0; i < count;i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            pw.print("<lang-Data lang=\"");
            LangInteger.printString(pw, libelle.getLangInteger());
            pw.print("\">");
            printTag("lib",libelle.getLibelleString(),pw,true);
            pw.print("</lang-Data>");
        }
        pw.print("</motcle>");
    }
    
    private void printTag(String tagname,String value,PrintWriter pw, boolean emptyValue) {
        if ((value == null) || (value.length() == 0)) {
            if (emptyValue) {
                pw.print("<");
                pw.print(tagname);
                pw.print("/>");
            }
        }
        else {
            pw.print("<");
            pw.print(tagname);
            pw.print(">");
            XMLUtils.escapeXML(value,pw);
            pw.print("</");
            pw.print(tagname);
            pw.print(">");
        }
    }
    
    void printBaseURI(PrintWriter pw,BaseData baseData) {
        pw.print("<base>");
        int count = baseData.getLangDataCount();
        for (int i = 0; i < count;i++) {
            BaseData.BaseLangData langData = baseData.getLangDataByIndex(i);
            pw.print("<lang-Data lang=\"");
            LangInteger.printString(pw, langData.getLangInteger());
            pw.print("\">");
            printTag("intitule-short", langData.getLibelleString(BaseData.INTITULE_SHORT),pw,true);
            printTag("intitule-long", langData.getLibelleString(BaseData.INTITULE_LONG),pw,true);
            pw.print("</lang-Data>");
        }
        pw.print("</base>");
    }
    
    
    
}
