/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.util.DataUtils;
import net.mapeadores.scrutari.db.DataManager;
import net.mapeadores.scrutari.searchengine.*;
import net.mapeadores.scrutari.uri.*;
import net.mapeadores.scrutari.lexie.*;
import javax.servlet.http.*;
import java.io.*;
import net.mapeadores.util.xml.XMLUtils;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.config.ConfigException;
import net.mapeadores.util.primitives.*;

/**
 * Classe produisant l'arbre XML des fiches.
 *
 * @author Vincent Calame
 */
public class FichesRequest implements FieldNumbers {
    private SearchResult searchResult;
    private DataManager dataManager;
    private int langUi;
    
    /**
     * Construit une nouvelle instance de FichesRequest à partir de la recherche d'identifiant searchId.
     */
    public FichesRequest(String searchid,SctServer rchfServer) throws XmlRequestException {
        if (searchid == null) throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.MISSING_SEARCHID_PARAMETER);
        try {
            searchResult = rchfServer.getSearchResult(searchid);
        }
        catch(ConfigException ce) {
            if (ce.hasSource()) throw new XmlRequestException(ce.getSource());
            else throw new XmlRequestException(ce);
        }
        if (searchResult == null) throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.WRONG_SEARCHID_PARAMETER,searchid);
        dataManager = rchfServer.getScrutariDB().getDataManager();
        langUi = searchResult.getSearchOptions().getLangUi();
    }
    
    /**
     * Écrit l'arbre XML des fiches comme réponse à une requête de servlet.
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE fiches SYSTEM \"dtd/fiches.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<fiches id=\"fiches\" search-id=\"");
        pw.print(searchResult.getIdentifiant());
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        /*pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(searchResult.getFicheMaximum()));
        pw.print("\" all-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getAllMotcleCount()));
        pw.print("\" complete-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getCompleteMotcleCount()));
        pw.print("\" partial-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getPartialMotcleCount()));*/
        pw.print("\"");
        pw.print(">");
        int count = searchResult.getFicheCount();
        for (int i = 0; i < count;i++) {
            printFicheResult(pw, searchResult.getFicheResult(i),tabulations+1);    
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</fiches>");
        
    }
    
    /*
     * Écrit les informations liées à une fiche d'un résultat. Méthode appelée par printResponse.
     */
    private void printFicheResult(PrintWriter pw,FicheSearchResultInfo ficheResult,int tabulations) {
        int code = ficheResult.getCode();
        String code_string = String.valueOf(code);
        FicheData ficheData = dataManager.getFicheDataByCode(code);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<fiche id=\"");
        pw.print(code_string);
        pw.print("\" fiche-code=\"");
        pw.print(code_string);
        pw.print("\" corpus-code=\"");
        pw.print(String.valueOf(ficheData.getCodeCorpus()));
            pw.print("\" lang=\"");
            pw.print(XmlRequestUtils.getLangString(ficheData.getLangInteger()));
        int dateInteger = ficheData.getDateInteger();
        if (dateInteger != IntDate.NULL) {
            pw.print("\" date=\"");
            pw.print(IntDate.toString(dateInteger));
        }
        String href = DataUtils.getHref(ficheData,dataManager, langUi);
        if (href != null) {
            pw.print("\" href=\"");
            XMLUtils.escapeXML(href,pw);
        }
        pw.print("\"");
        pw.print(">");
        String titre = ficheData.getTitre();
        if (titre != null) {
            SearchTokenPositionList positionList = ficheResult.getSubstringPositionList(TITRE_FIELDNUMBER);
            XMLUtils.printTabs(pw, tabulations+1);
            pw.print("<titre id=\"");
            pw.print(code_string);
            pw.print("_titre\" role=\"");
            if (positionList == null) pw.print("0");
            else pw.print("1");
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw,titre,positionList);
            pw.print("</titre>");
        }
        String soustitre = ficheData.getSoustitre();
        if (soustitre != null) {
            SearchTokenPositionList positionList = ficheResult.getSubstringPositionList(SOUSTITRE_FIELDNUMBER);
            XMLUtils.printTabs(pw, tabulations+1);
            pw.print("<soustitre id=\"");
            pw.print(code_string);
            pw.print("_soustitre\" role=\"");
            if (positionList == null) pw.print("0");
            else pw.print("1");
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw,soustitre,positionList);
            pw.print("</soustitre>");
        }
        pw.print("<numero id=\"");
        pw.print(code_string);
        pw.print("_numero\">");
        FicheURI ficheURI = ficheData.getFicheURI();
        pw.print(ficheURI.getFicheId());
        pw.print("</numero>");
        for (int p = 1; p <= ficheData.getComplementMaxNumber();p++) {
            String complement = ficheData.getComplementByNumber(p);
            if (complement != null) {
                SearchTokenPositionList positionList = ficheResult.getSubstringPositionList(p);
                XMLUtils.printTabs(pw, tabulations+1);
                pw.print("<complement id=\"");
                pw.print(code_string);
                pw.print("_complement_");
                pw.print(String.valueOf(p));
                pw.print("\" complement-number=\"");
                pw.print(String.valueOf(p));
                pw.print("\" role=\"");
                if (positionList == null) pw.print("0");
                else pw.print("1");
                pw.print("\">");
                XmlRequestUtils.printPositionList(pw,complement,positionList);
                pw.print("</complement>");
            }
        }
        for(int i = 0; i < ficheResult.getMotcleCount();i++) {
            int codeMotcle = ficheResult.getMotcleCode(i);
            XMLUtils.printTabs(pw, tabulations+1);
            pw.print("<mtcl motcle-code=\"");
            pw.print(String.valueOf(codeMotcle));
            pw.print("\"/>");
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</fiche>");
    }
    
}
