/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.scrutari.data.*;
import java.util.ArrayList;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.DataManager;

/**
 *
 * @author Vincent Calame
 */
public class LangInfo {
    int langInteger;
    int ficheCount = 0;
    ArrayList intituleInfoList = new ArrayList();
    
    public LangInfo(int langInteger,ScrutariDB generalDatabase) {
        this.langInteger = langInteger;
        DataManager infoDatabase = generalDatabase.getDataManager();
        
        for (int i = 0; i < infoDatabase.getBaseCount();i++) {
            BaseData baseInfo = infoDatabase.getBaseDataByIndex(i);
            createIntituleInfo(baseInfo.getLibelleStringByLang(baseInfo.INTITULE_SHORT, langInteger), "short", "base", baseInfo.getCodeBase());
            createIntituleInfo(baseInfo.getLibelleStringByLang(baseInfo.INTITULE_LONG, langInteger), "long", "base", baseInfo.getCodeBase());
        }
        for (int i = 0; i < infoDatabase.getCorpusCount();i++) {
            CorpusData corpusInfo = infoDatabase.getCorpusDataByIndex(i);
            ficheCount = ficheCount + corpusInfo.getFicheCountByLang(langInteger);
            createIntituleInfo(corpusInfo.getLibelleStringByLang(CorpusData.INTITULECORPUS_FIELDNUMBER, langInteger), "corpus", "corpus", corpusInfo.getCodeCorpus());
            createIntituleInfo(corpusInfo.getLibelleStringByLang(CorpusData.INTITULEFICHE_FIELDNUMBER, langInteger), "fiche", "corpus", corpusInfo.getCodeCorpus());
            for (int num = 1; num <= corpusInfo.getComplementMaxNumber();num++) {
                createIntituleInfo(corpusInfo.getLibelleStringByLang(num, langInteger), corpusInfo.getCodeCorpus(),num);
            }
        }
        for (int i = 0; i < infoDatabase.getThesaurusCount();i++) {
            ThesaurusData thesaurusInfo = infoDatabase.getThesaurusDataByIndex(i);
            createIntituleInfo( thesaurusInfo.getLibelleStringByLang(langInteger), "thesaurus", "thesaurus", thesaurusInfo.getCodeThesaurus());
        }
    }
    
    public int getIntituleInfoCount() {
        return intituleInfoList.size();
    }
    
    public IntituleInfo getIntituleInfo(int index) {
        return (IntituleInfo) intituleInfoList.get(index);
    }
    
    public int getFicheCount() {
        return ficheCount;
    }
    
    void createIntituleInfo(String intitule,String name,String codename,int code) {
        if (intitule == null) return;
        intituleInfoList.add(new IntituleInfo(name,intitule,code,codename));
    }
    
    void createIntituleInfo(String intitule,int code, int complementNumber) {
        if (intitule == null) return;
        intituleInfoList.add(new IntituleInfo("complement",intitule,code,"corpus",complementNumber));
    }
    
    public class IntituleInfo {
        String name;
        String intitule;
        String codename;
        int complementNumber = 0;
        int code;
        
        IntituleInfo(String name,String intitule,int code,String codename) {
            this.name = name;
            this.intitule = intitule;
            this.code = code;
            this.codename = codename;
        }
        
        IntituleInfo(String name,String intitule,int code,String codename, int complementNumber) {
            this.name = name;
            this.intitule = intitule;
            this.code = code;
            this.codename = codename;
            this.complementNumber = complementNumber;
        }
        
        public String getName() {
            return name;
        }
        
        public int getCode() {
            return code;
        }
        
        public String getCodeName() {
            return codename;
        }
        
        public String getIntitule() {
            return intitule;
        }
        
        public boolean hasComplementNumber() {
            return (complementNumber != 0);
        }
        
        public int getComplementNumber() {
            return complementNumber;
        }
        
    }
    
}
