/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.scrutari.data.*;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.IOException;
import net.mapeadores.scrutari.db.stats.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.util.xml.XMLUtils;

/**
 *
 * @author Vincent Calame
 */
public class MetadataRequest {
    DataManager dataManager;
    ScrutariDB scrutariDB;
        
    public MetadataRequest(ScrutariDB scrutariDB) {
        dataManager = scrutariDB.getDataManager();
        this.scrutariDB = scrutariDB;
    }
    
    public void printResponse(HttpServletResponse reponse) throws IOException {
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE metadata SYSTEM \"dtd/metadata.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<metadata id=\"metadata\"");
        printGlobalCountStatsAttribute(pw);
        pw.print(">");
        
        for (int i = 0;i < dataManager.getBaseCount();i++) {
            printBase(pw,dataManager.getBaseDataByIndex(i),tabulations+1);
        }
        LangStats langStats = scrutariDB.getLangStats();
        int[] langArray = langStats.getAllLangArray();
        int length = langArray.length;
        for (int i = 0; i < length;i++) {
            printLangInfo(pw,langArray[i],tabulations+1);
        }     
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</metadata>");
        
    }
    
    private void printBase(PrintWriter pw,BaseData baseData,int tabulations) {
        CountStats countStats = scrutariDB.getCountStats();
        int codebase = baseData.getCodeBase();
        BaseCountStats baseCountInfo = countStats.getBaseCountStats(codebase);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<base id=\"");
        pw.print(String.valueOf(codebase));
        pw.print("\" base-code=\"");
        pw.print(String.valueOf(codebase));
        pw.print("\"");
        printBaseCountAttribute(pw, baseCountInfo);
        pw.print(">");
        int corpusCount = baseData.getCorpusCount();
        for (int i = 0; i < corpusCount;i++) {
            int codecorpus = baseData.getCodeCorpus(i);
            CorpusCountStats corpusCountStats = countStats.getCorpusCountStats(codecorpus);
            printCorpus(pw,codecorpus,corpusCountStats,tabulations+1);
        }
        int thesaurusCount = baseData.getThesaurusCount();
        for (int i = 0; i < thesaurusCount;i++) {
            int codethesaurus = baseData.getCodeThesaurus(i);
            ThesaurusCountStats thesaurusCountStats = countStats.getThesaurusCountStats(codethesaurus);
            printThesaurus(pw,codethesaurus, thesaurusCountStats,tabulations+1);
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</base>");
    }
    
    private void printCorpus(PrintWriter pw,int codecorpus, CorpusCountStats corpusCountInfo,int tabulations) {
         XMLUtils.printTabs(pw, tabulations);
         pw.print("<corpus id=\"");
        pw.print(String.valueOf(codecorpus));
        pw.print("\" corpus-code=\"");
        pw.print(String.valueOf(codecorpus));
        pw.print("\" complement-maxnumber=\"");
        pw.print(String.valueOf(corpusCountInfo.getComplementMaxNumber()));
        pw.print("\"");
        printCorpusCountAttribute(pw, corpusCountInfo);
        pw.print("/>");
    }
    
    private void printThesaurus(PrintWriter pw,int codethesaurus, ThesaurusCountStats thesaurusCountInfo,int tabulations) {
         XMLUtils.printTabs(pw, tabulations);
         pw.print("<thesaurus id=\"");
        pw.print(String.valueOf(codethesaurus));
        pw.print("\" thesaurus-code=\"");
        pw.print(String.valueOf(codethesaurus));
        pw.print("\"");
        printThesaurusCountAttribute(pw, thesaurusCountInfo);
        pw.print("/>");
    }
    
    private void printCorpusCountAttribute(PrintWriter pw, CorpusCountStats countInfo) {
        pw.print(" fiche-count=\"");
        pw.print(String.valueOf(countInfo.getFicheCount()));
        pw.print("\" indexation-count=\"");
        pw.print(String.valueOf(countInfo.getIndexationCount()));
        pw.print("\" motcle-count=\"");
        pw.print(String.valueOf(countInfo.getMotcleCount()));
        pw.print("\"");
    }
    
    private void printThesaurusCountAttribute(PrintWriter pw, ThesaurusCountStats countInfo) {
        pw.print(" motcle-count=\"");
        pw.print(String.valueOf(countInfo.getMotcleCount()));
        pw.print("\" indexation-count=\"");
        pw.print(String.valueOf(countInfo.getIndexationCount()));
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(countInfo.getFicheCount()));
        pw.print("\"");
    }
    
    
    private void printBaseCountAttribute(PrintWriter pw, BaseCountStats baseCountInfo) {
        pw.print(" corpus-count=\"");
        pw.print(String.valueOf(baseCountInfo.getCorpusCount()));
        pw.print("\" thesaurus-count=\"");
        pw.print(String.valueOf(baseCountInfo.getThesaurusCount()));
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(baseCountInfo.getFicheCount()));
        pw.print("\" motcle-count=\"");
        pw.print(String.valueOf(baseCountInfo.getMotcleCount()));
         pw.print("\" indexation-count=\"");
        pw.print(String.valueOf(baseCountInfo.getIndexationCount()));
        pw.print("\"");
    }
    
    
    private void printGlobalCountStatsAttribute(PrintWriter pw) {
        CountStats countStats = scrutariDB.getCountStats();
        pw.print(" base-count=\"");
        pw.print(String.valueOf(countStats.getBaseCount()));
        pw.print("\" corpus-count=\"");
        pw.print(String.valueOf(countStats.getCorpusCount()));
        pw.print("\" thesaurus-count=\"");
        pw.print(String.valueOf(countStats.getThesaurusCount()));
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(countStats.getFicheCount()));
        pw.print("\" motcle-count=\"");
        pw.print(String.valueOf(countStats.getMotcleCount()));
        pw.print("\" indexation-count=\"");
        pw.print(String.valueOf(countStats.getIndexationCount()));
        pw.print("\" token-count=\"");
        pw.print(String.valueOf(countStats.getLexieCount()));
        pw.print("\"");
    }

    void printLangInfo(PrintWriter pw,int langInteger, int tabulations) {
        LangInfo langInfo = new LangInfo(langInteger,scrutariDB);
        XMLUtils.printTabs(pw, tabulations);
        String strg_lang = XmlRequestUtils.getLangString(langInteger);
        pw.print("<lang-info id=\"");
        pw.print(strg_lang);
        pw.print("\" lang=\"");
        pw.print(strg_lang);
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(langInfo.getFicheCount()));
        pw.print("\">");
        for(int i = 0; i < langInfo.getIntituleInfoCount();i++) {
            printIntitule(pw, langInfo.getIntituleInfo(i),strg_lang,tabulations+1);
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</lang-info>");
    }
    
    private void printIntitule(PrintWriter pw,LangInfo.IntituleInfo intitule, String lang,int tabulations) {
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<intitule id=\"");
        pw.print(intitule.getName());
        pw.print("_");
        pw.print(String.valueOf(intitule.getCode()));
        pw.print("_");
        if (intitule.hasComplementNumber()) {
            pw.print(String.valueOf(intitule.getComplementNumber()));
            pw.print("_");
        }
        pw.print(lang);
        pw.print("\" type=\"");
        pw.print(intitule.getName());
        pw.print("\" ");
        pw.print(intitule.getCodeName());
        pw.print("-code=\"");
        pw.print(String.valueOf(intitule.getCode()));
        if (intitule.hasComplementNumber()) {
            pw.print("\" complement-number=\"");
            pw.print(String.valueOf(intitule.getComplementNumber()));
        }
        pw.print("\">");
        XMLUtils.escapeXML(intitule.getIntitule(),pw);
        pw.print("</intitule>");
    }

}
