/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.searchengine.*;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import net.mapeadores.util.xml.XMLUtils;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.config.ConfigException;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.text.*;


/**
 * Classe produisant l'arbre XML des mots-clés.
 *
 * @author Vincent Calame
 */
public class MotsclesRequest {
    SearchResult searchResult;
    DataManager DataDatabase;
    
    /**
     * Construit une nouvelle instance de MotsclesRequest à partir de la recherche d'identifiant searchId.
     */
    public MotsclesRequest(String searchid,SctServer rechercheDeFiches) throws XmlRequestException {
        if (searchid == null) throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.MISSING_SEARCHID_PARAMETER);
        try {
            searchResult = rechercheDeFiches.getSearchResult(searchid);
        }
        catch(ConfigException ce) {
            if (ce.hasSource()) throw new XmlRequestException(ce.getSource());
            else throw new XmlRequestException(ce);
        }
        if (searchResult == null) throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.WRONG_SEARCHID_PARAMETER,searchid);
        DataDatabase = rechercheDeFiches.getScrutariDB().getDataManager();
    }
    
    /**
     * Écrit l'arbre XML des mots-clés comme réponse à une requête de servlet.
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE motscles SYSTEM \"dtd/motscles.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<motscles id=\"motscles\" search-id=\"");
        pw.print(searchResult.getIdentifiant());
        /*pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(searchResult.getFicheMaximum()));*/
        pw.print("\" all-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getAllMotcleCount()));
        /*pw.print("\" complete-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getCompleteMotcleCount()));
        pw.print("\" partial-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getPartialMotcleCount()));*/
        pw.print("\"");
        pw.print(">");
        int count = searchResult.getAllMotcleCount();
        for (int i = 0; i < count;i++) {
            printMotcleResult(pw, searchResult.getMotcleResult(i),tabulations+1);    
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</motscles>");
        
    }
    
    /*
     * Écrit les Datarmations liées à un mot-clé d'un résultat. Méthode appelée par printResponse.
     */
    private void printMotcleResult(PrintWriter pw,MotcleSearchResultInfo motcleResult,int tabulations) {
        int code = motcleResult.getCode();
        int ficheCount = motcleResult.getAcceptedFicheCount();
        MotcleData motcleData = DataDatabase.getMotcleDataByCode(code);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<motcle id=\"");
        pw.print(String.valueOf(code));
        pw.print("\" motcle-code=\"");
        pw.print(String.valueOf(code));
        pw.print("\" thesaurus-code=\"");
        pw.print(String.valueOf(motcleData.getCodeThesaurus()));
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(ficheCount));
        pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(motcleResult.getFicheMaximum()));
        pw.print("\"");
        pw.print(">");
        int libcount = motcleData.getLibelleCount();
        for (int i = 0; i < libcount;i++) {
            Libelle libelle = motcleData.getLibelleByIndex(i);
            int langInteger = libelle.getLangInteger();
            String strg_libelle = libelle.getLibelleString();
            SearchTokenPositionList positionList = motcleResult.getSearchTokentPositionListByLang(langInteger);
            String strg_lang = XmlRequestUtils.getLangString(langInteger);
            XMLUtils.printTabs(pw, tabulations+1);
            pw.print("<lib id=\"");
            pw.print(String.valueOf(code));
            pw.print("_");
            pw.print(strg_lang);
            pw.print("\" lang=\"");
            pw.print(strg_lang);
            pw.print("\" role=\"");
            if (positionList == null) pw.print("0");
            else pw.print("1");
            pw.print("\">");
            XmlRequestUtils.printPositionList(pw,strg_libelle,positionList);
            XMLUtils.printTabs(pw, tabulations+1);
            pw.print("</lib>");
        }
        
        for (int i = 0; i < ficheCount;i++) {
            int codeFiche = motcleResult.getAcceptedFicheCode(i);
            FicheData ficheData = DataDatabase.getFicheDataByCode(codeFiche);
            XMLUtils.printTabs(pw, tabulations+1);
            pw.print("<fch fiche-code=\"");
            pw.print(String.valueOf(codeFiche));
            int dateInteger = ficheData.getDateInteger();
            if (dateInteger != IntDate.NULL) {
                pw.print("\" date=\"");
                pw.print(IntDate.toString(dateInteger));
            }
            pw.print("\"/>");
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</motcle>");
    }
}
