/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.xml;

import net.mapeadores.scrutari.searchengine.util.*;
import net.mapeadores.scrutari.data.*;
import net.mapeadores.scrutari.db.*;
import net.mapeadores.scrutari.searchengine.*;
import org.w3c.dom.*;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.IOException;
import net.mapeadores.util.xml.XMLUtils;
import fr.exemole.sctserver.*;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import fr.exemole.sctserver.request.*;

/**
 *
 * @author Vincent Calame
 */
public class SearchRequest {
    SearchOptions searchOptions;
    SearchSequence searchSequence;
    SearchResult searchResult;
    
    /**
     * Construit une nouvelle instance à partir du paramètre xml
     */
    public SearchRequest(String xml,SctServer sctServer,RequestInfo requestInfo) throws XmlRequestException {
        if (xml == null) throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.MISSING_XML_PARAMETER);
        LinkedIntHashMap baseCodeMap = new LinkedIntHashMap();
        LinkedIntHashMap corpusCodeMap = new LinkedIntHashMap();
        LinkedIntHashMap langIntMap = new LinkedIntHashMap();
        int langUi = LangInteger.FRENCH;
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        DataManager dataManager = scrutariDB.getDataManager();
        String source = null;
        Document doc = XmlRequestUtils.getDocument(xml);
        Element racine = doc.getDocumentElement();
        NodeList nodeList = racine.getChildNodes();
        for (int i = 0; i < nodeList.getLength();i++) {
            Node node = nodeList.item(i);
            if (node instanceof Element) {
                Element el = (Element) node;
                String tagname = el.getTagName();
                if (tagname.equals("source")) {
                    source = XMLUtils.getData(el);
                } else if (tagname.equals("base")) {
                    try {
                        int code = Integer.parseInt(el.getAttribute("base-code"));
                        baseCodeMap.put(code, Boolean.TRUE);
                    } catch(NumberFormatException nfe) {
                        
                    }
                } else if (tagname.equals("corpus")) {
                    try {
                        int code = Integer.parseInt(el.getAttribute("corpus-code"));
                        CorpusData corpusInfo = dataManager.getCorpusDataByCode(code);
                        if (corpusInfo != null) corpusCodeMap.put(code, Boolean.TRUE);
                    } catch(NumberFormatException nfe) {
                        
                    }
                } else if (tagname.equals("lang")) {
                    String strg_lang = el.getAttribute("lang");
                    try {
                        int langInteger = LangInteger.parse(strg_lang);
                        langIntMap.put(langInteger,Boolean.TRUE);
                    } catch(LangIntegerException ile) {

                    }
                } else if (tagname.equals("lang-ui")) {
                    String strg_langui = el.getAttribute("lang");
                    try {
                        langUi = LangInteger.parse(strg_langui);
                    } catch(LangIntegerException ile) {

                    }
                }
            }
        }
        for(int i = 0; i < baseCodeMap.size();i++) {
            BaseData baseInfo = dataManager.getBaseDataByCode(baseCodeMap.getKeyAt(i));
            if (baseInfo != null) {
                int count = baseInfo.getCorpusCount();
                for (int j = 0; j < count;j++) {
                    corpusCodeMap.put(baseInfo.getCodeCorpus(j), Boolean.TRUE);
                }
            }
        }
        searchSequence = SearchSequence.createSearchSequence(source);
        if (searchSequence == null) throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.MISSING_SOURCE_ELEMENT);
        if (searchSequence.getTokenCount() == 0) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE,XmlRequestException.POOR_SOURCE_ELEMENT, searchSequence.getSource());
        }
        int[] codecorpusArray = (corpusCodeMap.size() > 0)?corpusCodeMap.getKeyArray():null;
        int[] langIntegerArray = (langIntMap.size() > 0)?langIntMap.getKeyArray():null;
        searchOptions = new SearchOptions();
        searchOptions.setLangUi(langUi);
        searchOptions.setLangIntegerArray(langIntegerArray);
        searchOptions.setCorpusCodeArray(codecorpusArray);
        if (codecorpusArray != null) {
            searchOptions.setThesaurusCodeArray(SearchOptionsUtils.getCodethesaurusArray(scrutariDB,codecorpusArray));
        }
        searchOptions.setDefaultSearchType(TextConstants.STARTSWITH);
        searchResult = sctServer.newSearch(searchSequence,searchOptions,requestInfo);
    }
    
    /**
     * Écrit la réponse à la requête : il s'agit de l'arbre xml « search ».
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE search SYSTEM \"dtd/search.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<search id=\"search\" search-id=\"");
        pw.print(searchResult.getIdentifiant());
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(searchResult.getFicheMaximum()));
        pw.print("\" all-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getAllMotcleCount()));
        pw.print("\" complete-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getCompleteMotcleCount()));
        pw.print("\" partial-motcle-count=\"");
        pw.print(String.valueOf(searchResult.getPartialMotcleCount()));
        pw.print("\"");
        pw.print(">");
        printSequence(pw, searchSequence,tabulations+1);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</search>");
        
    }
    
    /**
     * Écrit le format xml décrivant la séquence.
     */
    public void printSequence(PrintWriter pw,SearchSequence searchSequence,int tabulations) {
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<sequence id=\"sequence\" token-count=\"");
        pw.print(String.valueOf(searchSequence.getTokenCount()));
        pw.print("\">");
        XMLUtils.printTabs(pw, tabulations+1);
        pw.print("<source id=\"source\" clean=\"");
        XMLUtils.escapeXML(searchSequence.toString(),pw);
        pw.print("\">");
        XMLUtils.escapeXML(searchSequence.getSource(),pw);
        pw.print("</source>");
        for (int i = 0; i < searchSequence.getTokenCount();i++) {
            SearchSequence.Token token = searchSequence.getToken(i);
            XMLUtils.printTabs(pw, tabulations+1);
            if (token.isMultiToken()) {
                pw.print("<token token-type=\"multi\" id=\"token_");
                pw.print(String.valueOf(i));
                pw.print("\" token-index=\"");
                pw.print(String.valueOf(i));
                pw.print("\">");
                SearchSequence.SimpleToken[] array = ((SearchSequence.MultiToken) token).getSubtokenArray();
                for(int j = 0; j < array.length;j++) {
                    SearchSequence.SimpleToken simpleToken = array[j];
                    XMLUtils.printTabs(pw, tabulations+2);
                    pw.print("<subtoken search-type=\"");
                    pw.print(TextConstants.toString(simpleToken.getSearchType()));
                    pw.print("\">");
                    XMLUtils.escapeXML(simpleToken.toString(false),pw);
                    pw.print("</subtoken>");
                }
                XMLUtils.printTabs(pw, tabulations+1);
                pw.print("</token>");
            }
            else {
                SearchSequence.SimpleToken simpleToken = (SearchSequence.SimpleToken) token;
                pw.print("<token token-type=\"simple\" id=\"token_");
                pw.print(String.valueOf(i));
                pw.print("\" token-index=\"");
                pw.print(String.valueOf(i));
                pw.print("\" search-type=\"");
                pw.print(TextConstants.toString(simpleToken.getSearchType()));
                pw.print("\">");
                XMLUtils.escapeXML(simpleToken.toString(false),pw);
                pw.print("</token>");
            }
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</sequence>");
    }
}
