/* RechercheDeFiches - Article Search Engine
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.xml;

import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.xml.XMLUtils;
import java.io.*;
import org.xml.sax.SAXParseException;

/**
 *
 * @author Vincent Calame
 */
public class XmlRequestException extends Exception {
    public static final short RCHF_SOURCE = 1;
    public static final short CLIENT_SOURCE = 2;
    public static final String MISSING_CMD_PARAMETER = "MissingCmdParameter";
    public static final String MISSING_XML_PARAMETER = "MissingXmlParameter";
    public static final String MISSING_SEARCHID_PARAMETER = "MissingSearchidParameter";
    public static final String WRONG_CMD_PARAMETER = "WrongCmdParameter";
    public static final String WRONG_SEARCHID_PARAMETER = "WrongSearchidParameter";
    public static final String XML_PARAMETER_PARSE = "XmlParameterParse";
    public static final String MISSING_SOURCE_ELEMENT = "MissingSourceElement";
    public static final String POOR_SOURCE_ELEMENT = "PoorSourceElement";
    public static final String EXCEPTION = "Exception";
    short sourceType;
    String errorType;
    String errorDetail;
    Exception e;
    
    public XmlRequestException(short sourceType,String errorType) {
        super(errorType);
        this.errorType = errorType;
        this.sourceType = sourceType;
    }
    
    public XmlRequestException(short sourceType,String errorType, String errorDetail) {
        super(errorType);
        this.errorType = errorType;
        this.errorDetail = errorDetail;
        this.sourceType = sourceType;
    }
    
    public XmlRequestException(Exception e) {
        super(e);
        errorType = EXCEPTION;
        sourceType = RCHF_SOURCE;
        errorDetail = e.getClass().getName();
    }
    
    public XmlRequestException(SAXParseException saxPe) {
        super(XML_PARAMETER_PARSE);
        sourceType = CLIENT_SOURCE;
        errorType = XML_PARAMETER_PARSE;
        StringBuffer buf = new StringBuffer();
        buf.append(saxPe.getLocalizedMessage());
        buf.append(" ");
        String publicId = saxPe.getPublicId();
        if (publicId != null) {
            buf.append("publicId=");
            buf.append(publicId);
            buf.append(" ");
        }
        String systemId = saxPe.getSystemId();
        if (systemId != null) {
            buf.append("systemId=");
            buf.append(systemId);
            buf.append(" ");
        }
        buf.append("line=");
        buf.append(saxPe.getLineNumber());
        buf.append(" ");
        buf.append("column=");
        buf.append(saxPe.getColumnNumber());
        errorDetail = buf.toString();
    }
    
    public void printErrorResponse(HttpServletResponse reponse) throws IOException {
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        pw.print("<!DOCTYPE error SYSTEM \"dtd/error.dtd\">");
        pw.print("<error source=\"");
        switch(sourceType) {
            case RCHF_SOURCE :
                pw.print("rchf");
                break;
            case CLIENT_SOURCE :
                pw.print("client");
                break;
        }
        pw.print("\" type=\"");
        pw.print(errorType);
        if (errorDetail != null) {
            pw.print("\" detail=\"");
            XMLUtils.escapeXML(errorDetail,pw);
        }
        pw.print("\"/>");
    }

    
}
