/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package fr.exemole.sctserver.request.xml;

import javax.xml.parsers.*;
import net.mapeadores.scrutari.searchengine.SearchTokenPositionList;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXParseException;
import java.io.*;
import org.w3c.dom.*;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.xml.*;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.text.SubstringPosition;

/**
 * Série de méthodes statiques utilisées pour l'écriture et la lecture des arbres Xml.
 *
 * @author Vincent Calame
 */
public class XmlRequestUtils {

    /**
     * Parse la chaîne xml en instance de org.w3c.dom.Document.
     */
    public static Document getDocument(String xml) throws XmlRequestException {
        try {
            return XMLUtils.getDocument(xml);
        }
        catch(XMLException xe) {
            throw new XmlRequestException(xe.getException());
        }
    }
    
    public static InputStream getDtdInputStream(String name) {
        return XmlRequestUtils.class.getResourceAsStream("dtd/" + name);
    }
    
    /**
     * Écrit la chaîne s en insérant des éléments <code>&lt;s token-index=""&gt;</code> suivant les positions
     * contenues dans l'instance de SearchResult.SubstringPositionList. Cette liste ne doit pas contenir de
     * positions qui vont au delà de la taille de la chaîne <code>s</code>.
     */
    public static void printPositionList(PrintWriter pw,String s,SearchTokenPositionList positionList) {
        if (positionList == null) {
            XMLUtils.escapeXML(s,pw);
            return;
        }
        PositionListWriter writer = new PositionListWriter(pw, s, positionList);
        writer.print();
    }
    
    /**
     * Retourne la chaîne représentant la langue. Si (langInteger == IntLang.CODELANGUE_NULL), retourne "x-unknown" sinon retourne le code ISO.
     */
    public static String getLangString(int langInteger) {
        return (langInteger == LangInteger.UNDETERMINED)?"x-unknown":LangInteger.toString(langInteger, '-');
    }
    
    /*
     * Classe utilisée par la méthode printPositionList.
     */
    private static class PositionListWriter {
        SearchTokenPositionList positionList;
        PrintWriter pw;
        String s;
        int openIndex;
        int closeIndex;
        int currentPosition;
        
        /*
         * Constructeur unique.
         */
        PositionListWriter(PrintWriter pw,String s,SearchTokenPositionList positionList) {
            this.pw = pw;
            this.positionList = positionList;
            this.s = s;
            currentPosition = 0;
            openIndex = positionList.getSearchTokenPosition(0).getBeginIndex();
            closeIndex = positionList.getSearchTokenPosition(0).getEndIndex();
        }
        
        /*
         * Lancement de l'écriture.
         */
        void print() {
                int length = s.length();
                for (int i = 0; i < length;i++) {
                if (i == openIndex) {
                    pw.print("<s token-index=\"");
                    pw.print(String.valueOf(positionList.getSearchTokenIndexForPosition(currentPosition)));
                    pw.print("\">");
                }
                char carac = s.charAt(i);
                XMLUtils.escapeXML(carac,pw);
                if (i == closeIndex) {
                    pw.print("</s>");
                    increment();
                }
            }
        }
        
        /*
         * gère les éventuels recouvrements de position. Cela peut aller jusqu'à générer un élément s vide (<s token-index=""/>)
         * si une position est complètement incluse dans une autre.
         */
        void increment() {
            if (currentPosition == (positionList.size() -1)) {
                openIndex = -1;
                closeIndex = -1;
                return;
            }
            currentPosition++;
            SubstringPosition sp = positionList.getSearchTokenPosition(currentPosition);
            if (sp.getBeginIndex() > closeIndex) {
                openIndex = sp.getBeginIndex();
                closeIndex = sp.getEndIndex();
                return;
            }
            openIndex = -1;
            pw.print("<s token-index=\"");
            pw.print(String.valueOf(positionList.getSearchTokenIndexForPosition(currentPosition)));
            if (sp.getEndIndex() <= closeIndex) {
                pw.print("\"/>");
                increment();
            }
            else {
                pw.print("\">");
                closeIndex = sp.getEndIndex();
            }
        }
    }
}
