/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.servlets;

import javax.servlet.http.*;
import org.w3c.dom.*;
import net.mapeadores.util.xml.*;
import java.util.*;
import net.mapeadores.util.text.UTF8Bridge;

/**
 *
 * @author Vincent Calame
 */
public class DefaultRequestMap implements RequestMap {
    protected HttpServletRequest request;
    protected UTF8Bridge utf8Bridge;
    private Map<String, Object> multiPartValueMap;
    
    public DefaultRequestMap(HttpServletRequest request)  {
        this(request, null);
    }
    
    public DefaultRequestMap(HttpServletRequest request, MultiPartParser multiPartParser) {
        this.request = request;
        utf8Bridge = new UTF8Bridge(request.getCharacterEncoding());
        if (multiPartParser != null) {
            if (multiPartParser.isMultiPartContentRequest(request)) {
                this.multiPartValueMap = multiPartParser.parseMultiPartContentRequest(request);
            }
        }
    }
    
    public FileValue getFileValue(String name) {
        if (multiPartValueMap != null) {
            Object obj = multiPartValueMap.get(name);
            if (obj instanceof FileValue) return (FileValue) obj;
            else return null;
        }
        else {
            return null;
        }
    }

    public HttpServletRequest getRequest() {
        return request;
    }
    
    public String getParameter(String name) {
        if (multiPartValueMap != null) {
            Object obj = multiPartValueMap.get(name);
            if (obj instanceof String) return (String) obj;
            else return null;
        }
        else {
            return utf8Bridge.toUTF8(request.getParameter(name));
        }
    }
    
    public String[] getParameterValues(String name) {
        String[] paramValues = request.getParameterValues(name);
        if (paramValues == null) return null;
        int length = paramValues.length;
        String[] result = new String[length];
        for(int i = 0; i < length; i++) {
            result[i] = utf8Bridge.toUTF8(paramValues[i]);
        }
        return result;
    }
    
    public Set<String> getParameterNameSet() {
        Set<String> hashSet = new HashSet<String>();
        for(Enumeration enumeration = request.getParameterNames();enumeration.hasMoreElements();) {
            String paramName = (String) enumeration.nextElement();
            hashSet.add(paramName);
        }
        return hashSet;
    }
    
}
