/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.servlets;

import javax.servlet.http.*;
import javax.servlet.*;
import java.io.*;
import net.mapeadores.util.io.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class DocStreamPrinter implements ResponsePrinter {
    private String servletPath = "";
    private String pathInfo = "";
    
    public DocStreamPrinter(HttpServletRequest request) {
        String s1 = request.getServletPath();
        if ((s1 != null) && (s1.length() > 0)) {
            servletPath = s1.substring(1);
        }
        String s2 = request.getPathInfo();
         if ((s2 != null) && (s2.length() > 0)) {
            pathInfo = s2.substring(1);
        }
    }
    
    /**
     * Appelé par printResponse
     */
    public abstract DocStream getDocStream(String servletPath, String pathInfo);
    
    /**
     * Nettoyé de "/" à l'avant
     */
    public String getServletPath() {
        return servletPath;
    }
    
    public String getPathInfo() {
        return pathInfo;
    }
    
    public void printResponse(HttpServletResponse response) throws IOException {
        DocStream docStream = getDocStream(servletPath,pathInfo);
        if (docStream == null) {
            if (pathInfo.length() == 0) {
                response.sendError(HttpServletResponse.SC_FORBIDDEN,servletPath);
            }
            else {
                response.sendError(HttpServletResponse.SC_NOT_FOUND,servletPath + "/" + pathInfo);
            }
            
            return;
        }
        InputStream inputStream = docStream.getInputStream();
        if (inputStream == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND,servletPath + "/" + pathInfo);
            return;
        }
        String contentType = docStream.getMimeType();
        String charset = docStream.getCharset();
        if (charset != null) contentType = contentType + ";charset=" + charset;
        response.setContentType(contentType);
        int length = docStream.getLength();
        if (length > -1) response.setContentLength(length);
        OutputStream output = response.getOutputStream();
        IOUtils.copy(inputStream,output);
        inputStream.close();
        output.close();
    }
    
}
