/* OutilCarto - Concept Mapping Tool
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.awt;

import java.awt.Point;
import java.awt.Dimension;

/**
 * Point considéré dans le repère cartésien classique, c'est à dire où l'axe des y est orienté vers le haut (alors que dans le repère utilisé dans java.awt à l'axe des y orienté vers le bas).
 *
 * @author  Vincent Calame
 */
public class CartesianPoint extends Point {
    
    public CartesianPoint(int x, int y) {
        super(x,y);
    }
    
    public CartesianPoint() {
        super();
    }
    
    public CartesianPoint(CartesianPoint cartesianPoint) {
        super(cartesianPoint);
    }
    
    public CartesianPoint(Point P, Point O) {
       super(P.x-O.x,O.y - P.y);
    }
    
    /**
     * Convertit des coordonnées polaires en coordonnées cartésiennes.
     */
    static public CartesianPoint convertToCartesianPoint(final double theta,final double r) {
        int x = (int) Math.round(r*Math.cos(theta));
        int y = (int) Math.round(r*Math.sin(theta));
        return new CartesianPoint(x,y);
    }
    
    /**
     *Calcule l'angle entre (O<I>A</I>) et l'axe des x.
     *C'est le theta des coordonnèes polaires. Il est compris entre 0 et 2pi
     */
    public static double angle(CartesianPoint C,CartesianPoint A) {
        return angle(A.x - C.x,A.y - C.y);
    }
    
    public static double angle(CartesianPoint A) {
       return angle(A.x,A.y);
    }
    
    /**
     * Retourne un angle dontla valeur est comprise entre 0 et 2π
     */
    public static double angle(int x, int y) {
        double theta = Math.acos(x/norme(x,y));
        if (y < 0) theta = 2*Math.PI - theta;
        return theta;
    }
    
    /**
     *Calcule la norme du point <I>A</I>.
     */
    public static double norme(CartesianPoint A) {
        return  norme(A.x,A.y);
    }
    
    public static double norme(int x, int y) {
        return  Math.sqrt((double) (x*x+y*y));
    }
    
    public static double norme(double x, double y) {
        return  Math.sqrt(x*x+y*y);
    }
    
    public static double norme(CartesianPoint C,CartesianPoint A) {
        return  norme(A.x - C.x,A.y - C.y);
    }
    
    public static CartesianPoint getCartesianPointOnEllipse(CartesianPoint C,double angle,Dimension ab) {
        double r = getNormeOnEllipse(angle,ab);
        int x = (int) Math.round(Math.cos(angle) * r);
        int y = (int) Math.round(Math.sin(angle) * r);
        return new CartesianPoint(x + C.x,y+C.y);
    }

    
    public static double getNormeOnEllipse(final double angle,Dimension ab) {
        double costheta = Math.cos(angle);
        double sintheta = Math.sin(angle);
        int acarre = ab.width * ab.width;
        int bcarre = ab.height * ab.height;
        double r = Math.sqrt(1/((costheta*costheta/acarre) + (sintheta*sintheta/bcarre)));

        return r;
    }
    
    public Point toGrapicPoint(Point O) {
        return new Point(x + O.x,O.y - y);
    }
    
}
