/* OutilCarto - Concept Mapping Tool
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.awt;

import java.awt.Dimension;
import javax.swing.SwingConstants;


/**
 *
 * @author  Vincent Calame
 */
public class CartesianUtils implements SwingConstants {
    public final static double deuxpi = 2*Math.PI;
    
    /** Creates a new instance of CartesUtils */
    private CartesianUtils() {
    }
    
    /**
     * Convertit en angle en une valeur entre 0 et 2π
     */
    public static double modulo2pi(double angle) {
        return (angle - (Math.floor(angle/deuxpi) * deuxpi));
    }
    
    public static CartesianPoint getAngleIntersection(double angle,int a, int b) {
        double sinAngle = Math.sin(angle);
        double cosAngle = Math.cos(angle);
        double sinDim = b/CartesianPoint.norme(a,b);
        int x = 0;
        int y = 0;
        if (Math.abs(sinAngle) > Math.abs(sinDim)) {
            if (sinAngle > 0) {
                y = b;
            }
            else y = -b;
            x = (int) Math.round(y*cosAngle/sinAngle);
        }
        else {
            if (cosAngle > 0) {
                x = a;
            }
            else {
                x = -a;
            }
            y = (int) Math.round(x*sinAngle/cosAngle);
        }
        return new CartesianPoint(x,y);
    }
    
    public static Intersection getAngleIntersection(double angle,int east, int west,int north, int south) {
        if ((east <= 0) || (north <= 0) || (south >= 0) || (west >= 0)) throw new IllegalArgumentException();
        double sinAngle = Math.sin(angle);
        double cosAngle = Math.cos(angle);
        int x = 0;
        int y = 0;
        int refx = (cosAngle >= 0)?east:west;
        int refy = (sinAngle >=0)?north:south;
        int intersectedDirection = 0;
        double sinDim = refy/CartesianPoint.norme(refx,refy);
        if (Math.abs(sinAngle) > Math.abs(sinDim)) {
            intersectedDirection = (sinAngle > 0)?NORTH:SOUTH;
            y = refy;
            x = (int) Math.round(y*cosAngle/sinAngle);
        }
        else {
            intersectedDirection = (cosAngle > 0)?EAST:WEST;
            x = refx;
            y = (int) Math.round(x*sinAngle/cosAngle);
        }
        return new Intersection(intersectedDirection,x,y);
    }
    
    public static class Intersection {
        CartesianPoint cartesianPoint;
        int intersectedDirection;
        
        Intersection(int intersectedDirection,int x, int y) {
            this.intersectedDirection = intersectedDirection;
            cartesianPoint = new CartesianPoint(x,y);
        }
        
        public CartesianPoint getCartesianPoint() {
            return cartesianPoint;
        }
        
        public int getIntersectedDirection() {
            return intersectedDirection;
        }
        
        
    }
    
}
