/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections;

/**
 * Implémentation de l'interface {@link StableIntList} qui utilise un objet <code>IntHashMap</code> pour retrouver rapidement l'index d'un entier de la liste.
 * C'est l'équivalent de {@link MappedStableList} pour l'interface {@link StableIntList}.
 *
 * @author  Vincent Calame
 */
public class HashIntTicketAllocator extends AbstractStableIntList implements IntTicketAllocator {
    int[] valeurs;
    IntHashMap map = new IntHashMap();
    int size = 0;
    
    /**  Constructs an empty list*/
    public HashIntTicketAllocator() {
        valeurs = new int[64];
    }
    
    /*
     * Returns the value of the element at the specified position within me.
     */
    public int get(int index) {
        if ((index < 0) || (index >= size)) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        return valeurs[index];
    }
    
    /**
    * Returns the index of the first and sole occurrence of the specified element within me, or -1 if I do not contain the element.
    *
    */
     public int indexOf(int valeur) {
        Integer itg = (Integer) map.get(valeur);
        if (itg == null) return -1;
        return itg.intValue();
    }
     
    /**
    * Returns the index of the last occurrence of the specified element within me, or -1 if I do not contain the element.
     * Comme il n'y a pas de doublons, le résultat est le même que <code>indexOf(valeur]</code>
     */
    public int lastIndexOf(int valeur) {
        return indexOf(valeur);
    }
     /**
     * Returns the number of elements I contain.
     **/
    public int size() {
        return size;
    }
    
    /**
     * Équivalent de <code>size()</code> puisqu'aucun index ne peut être désactivé.
     */
    public int getEnabledIndexCount() {
        return size();
    }
    
    /**
     * Retourne toujours <em>true<:em>;
     */
    public boolean isEnabledIndex(int index) {
        return true;
    }

    /**
     * Rajoute un objet dans la liste et renvoie le nouvel index. Si l'objet est présent, retourne l'index actuel.
     * L'objet <code>obj</code> ne peut pas être <em>obj</em>.
     */
    public int allocateTicket(int valeur)  {
        Integer itg = (Integer) map.get(valeur);
        if (itg != null) return itg.intValue();
        map.put(valeur,new Integer(size));
        checkLength();
        valeurs[size] = valeur;
        size++;
        return size-1;
    }
    
    /*
     * Augmente la taille du tableau objects si nécessaire.
     */
    private void checkLength() {
        if (size == valeurs.length) {
            int taille = valeurs.length;
            int[] result = new int[taille*2];
            for (int i = 0; i < taille;i++) {
                result[i] = valeurs[i];
            }
            valeurs = result;
        }
        
    }
    
}
