/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections;

/**
 * Une interface qui permet un double accès à des objets par des clés entières et une position d'index de liste.
 * <code>IndexAndIntKeyMap</code> n'est pas une extension de l'interface <code>Map</code> du <em>Java Collection Framework</em>.
 ** <p>Cette interface est utilisée pour des classes qui indexe des objets avec des valeurs entières (typiquement des identifiants comme dans les bases de données relationnelles)
 ** et qui veluent un accès similaire à celle d'une liste.<:p>
 **<p> Elle repose sur l'idée que les clés entières sont elles-mêmes regroupées dans une instance de {@link StableIntList}.
 ** C'est cette instance qui détermine les valeurs d'index pour les objets.</p>
 ** Il y a donc un dialogue entre l'interface <code>IndexAndIntKeyMap</code> et l'objet renvoyé par <code>getIntKeyList()</code>.
 ** Il peut même s'agir du même objet.
 *
 * @author  Vincent Calame - Exemole
 */
public interface IndexAndIntKeyMap {

    /**
     * Retourne l'instance <code>StableIntList</code> qui assure la correspondance entre valeur d'index et clés entières.
     */
    public StableIntList getIntKeyList();
    
    /**
     * Retourne la clé entière correspondant à l'index. Envoi l'exception DisabledIndexException si l'index a été déactivé.
     */
    public int getKeyByIndex(int index) throws DisabledIndexException;
    
    /**
     * Retourne l'index de la clé entière. Retourne -1 si la clé n'existe pas, c'est à dire si elle n'a jamais été entrée ou si elle a été désactivée antérieurment.
     */
    public int getIndexByKey(int key);
    
    /**
     * Retourne la valeur à l'index <code>index</code>.
     * Retourne <em>null</em> si l'index n'existe pas ou a été désactivé.
     */
    public Object getValueByIndex(int index);
    
    /**
     * Retourne la valeur correspondant à la clé entière <code>key</code>. Retourne <em>null</em> s'il n'y a pas de valeur pour cette clé.
     **/
    public Object getValueByKey(int key);
    
    
    /**
     * Équivalent de getIntKeyList().size();
     */
    public int length();
    
   /**
    * Équivalent de getIntKeyList().getEnabledIndexCount();
    */
    public int getEnabledIndexCount();
  

    
}
