/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.collections;

import org.apache.commons.collections.primitives.IntList;
import org.apache.commons.collections.primitives.ArrayIntList;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;

/**
 * Série d'utilitaires consacrés aux listes d'entiers, en particulier dans les cas où il s'agit de dates ou de langues.
 *
 * @author Vincent Calame
 */
public class IntListUtils {
    
    private IntListUtils() {
    }

    
    public static String langIntListToString(IntList langIntList,char c) {
        int size = langIntList.size();
        if (size == 0) return "";
        if (size == 1) return LangInteger.toString(langIntList.get(0));
        StringBuffer buf = new StringBuffer();
        LangInteger.appendString(buf,langIntList.get(0));
        for (int i = 1;i < size;i++) {
            buf.append(c);
            LangInteger.appendString(buf,langIntList.get(i));
        }
        return buf.toString();
    }
    
    public static ArrayIntList toIntList(String s, char c) {
        ArrayIntList arrayIntList = new ArrayIntList();
        String[] token = s.split(String.valueOf(c));
        if (token.length == 1) {
            try {
                arrayIntList.add(Integer.parseInt(token[0]));
            }
            catch(NumberFormatException nfe) {
            }
        }
        else {
            IntHashMap existing = new IntHashMap();
            for (int i = 0; i < token.length;i++) {
                try {
                    int val = Integer.parseInt(token[i]);
                    if (!existing.containsKey(val)) {
                        arrayIntList.add(val);
                        existing.put(val,Boolean.TRUE);
                    }
                }
                catch(NumberFormatException nfe) {
                }
            }
        }
        return arrayIntList;
    }
    
    public static ArrayIntList toLangIntList(String s, char c) {
        ArrayIntList arrayIntList = new ArrayIntList();
        String[] token = s.split(String.valueOf(c));
        if (token.length == 1) {
            try {
                arrayIntList.add(LangInteger.parse(token[0]));
            }
            catch(LangIntegerException nfe) {
            }
        }
        else {
            IntHashMap existing = new IntHashMap();
            for (int i = 0; i < token.length;i++) {
                try {
                    int val = LangInteger.parse(token[i]);
                    if (!existing.containsKey(val)) {
                        arrayIntList.add(val);
                        existing.put(val,Boolean.TRUE);
                    }
                }
                catch(LangIntegerException nfe) {
                }
            }
        }
        return arrayIntList;
    }
    
}
