/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections;

import java.util.*;

/**
 * Contient des méthodes statiques de traitement des listes comme java.util.Collection.
 * @author  Vincent Calame
 */
public class ListUtils {
    
    private ListUtils() {
    }
    
    /**
     * Équivalent à list.remove(obj) sauf qu'au lieu de renvoyer un booléen. La méthode se fonde sur l'interface ListIterator.
     * Retourne -1 si l'objet n'a pas été trouvé.
     */
    public static int removeFromList(List liste,Object obj) {
        for (ListIterator it = liste.listIterator();it.hasNext();) {
            int idx = it.nextIndex();
            Object o = it.next();
            if (o.equals(obj)) {
                it.remove();
                return idx;
            }
        }
        return -1;
    }
    
    /**
     * Équivalent ) list.add(index,obj) mais ne renvoie pas d'exception si la valeur d'index est incorrecte. Si elle est incorrecte, l'objet est insérer à la fin de la liste.
     */
    public static int addInList(List liste, int index,Object obj) {
        try {
            liste.add(index,obj);
            return index;
        }
        catch(IndexOutOfBoundsException ioobe) {
            liste.add(obj);
            return liste.size() - 1;
        }
    }
}
