/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.collections;

import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class SetFilter {
    Set initialSet;
    Set acceptedSet;
    Set rejectedSet;
    
    public SetFilter(Set initialSet) {
        this.initialSet = initialSet;
        acceptedSet = new HashSet(initialSet);
        rejectedSet = new HashSet();
    }
    
    public SetFilter(Set initialSet, Set potentiels) {
        this.initialSet = initialSet;
        acceptedSet = new HashSet(initialSet);
        acceptedSet.retainAll(potentiels);
        rejectedSet = new HashSet(initialSet);
        rejectedSet.removeAll(potentiels);
    }
    
    public SetFilter(Set initialSet, Set acceptedSet, Set rejectedSet) {
        this.initialSet = initialSet;
        this.acceptedSet = acceptedSet;
        this.rejectedSet = rejectedSet;
    }
    
    public Set getInitialSet() {
        return initialSet;
    }
    
    public Set getAcceptedSet() {
        return acceptedSet;
    }
    
    public Set getRejectedSet() {
        return rejectedSet;
    }
    
    public void reject(Object obj) {
        boolean here = acceptedSet.remove(obj);
        if (here) rejectedSet.add(obj);
    }
    
    public void accept(Object obj) {
        boolean here = rejectedSet.remove(obj);
        if (here) acceptedSet.add(obj);
    }
    
    public void reject(int i) {
        reject(new Integer(i));
    }
    
    public void accept(int i) {
        accept(new Integer(i));
    }
}
