/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.collections;

import org.apache.commons.collections.primitives.IntList;

/**
 * Équivalent de {@link StableList} appliqué à l'interface <code>IntList</code> de Commons-Primitives du Apache Jakarta Project.
 * Toutes les caractériques de <code>StableList</code> s'applique à <code>StableIntList</code>.
 * La seule différence c'est que la désactivation d'un index ne peut pas être signifié par le retour d'une valeur <code>nulle</code>.
 *
 * @author  Vincent Calame - Exemole
 */
public interface StableIntList extends IntList {

     /**
     * Retourne <em>true</em> si l'entier situé à l'index <em>index</em> est disponible ou non.
     * La valeur renvoyée par <code>get(index)</code> est quelconque.
     */
    public boolean isEnabledIndex(int index);
    
    /**
     * Retourne le nombre d'index activés. Doit retourner la même valeur que la méthode <code>size()</code> de l'interface <code>List</code> si aucun index n'est désactivé.
     */
    public int getEnabledIndexCount();
   
    
}
