/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.collections.crossing;

import net.mapeadores.util.collections.crossing.CrossingMap;

/**
 *Encapsule un objet Crossing pour conserver son lien avec un numéro d'index et le relier aux autres Crossing dépendant de ce numéro d'index.
 *
 * @author Vincent Calame
 */
public class CrossingEntry {
         int otherIndex;
        CrossingMap.Crossing crossing;
        SameIndexBag sameIndexBag;
        CrossingEntry nextCrossingEntry;
        CrossingEntry previousCrossingEntry;
        
        /*Construit le premier crossing de la série */
        CrossingEntry(CrossingMap.Crossing crossing, int otherIndex,SameIndexBag sameIndexBag) {
            this.otherIndex = otherIndex;
            this.sameIndexBag= sameIndexBag;
            this.crossing = crossing;
        }
        
        /*Construit un crossing rattaché au précédent */
        CrossingEntry(CrossingMap.Crossing crossing,int otherIndex,CrossingEntry previousCrossingEntry) {
            this.otherIndex = otherIndex;
            this.previousCrossingEntry = previousCrossingEntry;
            this.sameIndexBag = previousCrossingEntry.sameIndexBag;
            this.crossing = crossing;
        }
        
        /*Rajoute un nouveau crossing à la suite */
        void append(CrossingMap.Crossing crossing, int newOtherIndex) {
            nextCrossingEntry = new CrossingEntry(crossing,newOtherIndex,this);
        }
        
        public CrossingMap.Crossing getCrossing() {
            return crossing; 
        }
        
        public int getOtherIndex() {
            return otherIndex;
        }
        
        public CrossingEntry next() {
            return nextCrossingEntry;
        }
        
        public CrossingEntry previous() {
            return previousCrossingEntry;
        }
        
        public SameIndexBag getSameIndexBag() {
            return sameIndexBag;
        }
    
}
