/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.conditions;

import java.util.*;
import net.mapeadores.util.exceptions.*;

/**
 *
 * @author Vincent Calame
 */
public class ConditionGroup implements ConditionsConstants{
    private short mode;
    private short state = U;
    ArrayList excludingList = new ArrayList();
    ArrayList includingList = new ArrayList();
    
    public ConditionGroup(short mode) {
        this.mode = mode;
    }
    
    public ConditionGroup(ConditionGroupBean bean) {
        this.mode = bean.getMode();
        ContentConditionBean[] contentConditionBeanArray = bean.getContentConditionBeanArray();
        if (contentConditionBeanArray != null) {
            for (int i = 0; i < contentConditionBeanArray.length;i++) {
                if (contentConditionBeanArray[i] != null) addContentCondition(contentConditionBeanArray[i]);
            }
        }
    }
    
    public short getMode() {
        return mode;
    }
    
    public short getState() {
        return state;
    }
    
    public void addContentCondition(ContentConditionBean contentConditionBean) {
        short conditionType = contentConditionBean.getConditionType();
        short addingState = Conditions.getState(conditionType);
        state = Conditions.merge(mode,state,addingState);
        if (!Conditions.isPartState(state)) {
            excludingList.clear();
            includingList.clear();
        }
        else {
            if (Conditions.isExcludingPartConditionType(conditionType)) {
                excludingList.add(new ContentConditionBean(contentConditionBean)); //@todo cover
            }
            else if (Conditions.isIncludingPartConditionType(conditionType)) {
                includingList.add(new ContentConditionBean(contentConditionBean)); //@todo cover
            }
        }
    }
    
    public ContentConditionBean[] getExcludingContentConditionBeanArray() {
        return getContentConditionBeanArray(excludingList);
    }
    
    public ContentConditionBean[] getIncludingContentConditionBeanArray() {
        return getContentConditionBeanArray(includingList);
    }
    
    private ContentConditionBean[] getContentConditionBeanArray(List list) {
        int size = list.size();
        ContentConditionBean[] result = new ContentConditionBean[size];
        for(int i = 0; i < size;i++) {
            result[i] = new ContentConditionBean((ContentConditionBean)list.get(i));
        }
        return result;
    }
    
    /**
     * Retourne lui-même si son statut est P, si PA une instance de ConditionGroup
     * une instance 
     */
    public ConditionGroup toPartConditionGroup() {
        if (state == P) return this;
        if (state == PA) {
            ConditionGroup cg = new ConditionGroup(mode);
            cg.state = P;
            cg.excludingList.addAll(excludingList);
            cg.includingList.addAll(includingList);
            return cg;
        }
        return null;
    }
    
    public ConditionGroupBean toConditionGroupBean() {
        return new ConditionGroupBean(mode,getContentConditionArray());
    }
    
    public ContentConditionBean[] getContentConditionArray() {
        switch(state) {
            case A :
                 return getA();
            case T :
                 return getT();
            case P :
                return getP();
            case PA :
                return getPA();
            case X :
                return getX();
            case U :
            case N :
                return new ContentConditionBean[0];
            default :
                throw new SwitchException("state =" + state);
        }
    }
    
    private ContentConditionBean[] getA() {
        ContentConditionBean[] array = new ContentConditionBean[1];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        return array;
    }
    
    private ContentConditionBean[] getT() {
        ContentConditionBean[] array = new ContentConditionBean[1];
        array[0] = new ContentConditionBean(EST_NON_VIDE, "");
        return array;
    }
    
    private ContentConditionBean[] getP() {
        int includingSize = includingList.size();
        int excludingSize = excludingList.size();
        ContentConditionBean[] array = new ContentConditionBean[includingSize + excludingSize];
        for (int i = 0; i< includingSize;i++) {
            array[i] = new ContentConditionBean((ContentConditionBean) includingList.get(i));
        }
        for (int i = 0; i< excludingSize;i++) {
            array[i + includingSize] = new ContentConditionBean((ContentConditionBean) excludingList.get(i));
        }
        return array;
    }
    
    private ContentConditionBean[] getPA() {
        int excludingSize = excludingList.size();
        if (excludingSize > 0) return getP();
        int includingSize = includingList.size();
        ContentConditionBean[] array = new ContentConditionBean[1 + includingSize];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        for (int i = 0; i< includingSize;i++) {
            array[i + 1] = new ContentConditionBean((ContentConditionBean) includingList.get(i));
        }
        return array;
    }
    
    private ContentConditionBean[] getX() {
        ContentConditionBean[] array = new ContentConditionBean[2];
        array[0] = new ContentConditionBean(EST_VIDE, "");
        array[1] = new ContentConditionBean(EST_NON_VIDE, "");
        return array;
    }
}
