/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.conditions;

import org.w3c.dom.*;
import net.mapeadores.util.conditions.*;
import java.util.*;
import net.mapeadores.util.xml.*;
import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class ConditionsXMLStorage {
    
    private ConditionsXMLStorage() {
    }
    
    public static void appendConditionGroup(XMLWriter xmlWriter, ConditionGroup conditionGroup) throws IOException {
        xmlWriter.startOpenTag("condition-group");
        xmlWriter.addAttribute("mode", Conditions.modeToString(conditionGroup.getMode()));
        xmlWriter.endOpenTag();
        ContentConditionBean[] array = conditionGroup.getContentConditionArray();
        for(int i = 0; i < array.length; i++) {
            ContentConditionBean contentCondition = array[i];
            xmlWriter.startOpenTag("condition");
            xmlWriter.addAttribute("type",Conditions.typeToString(contentCondition.getConditionType()));
            xmlWriter.endOpenTag(false);
            xmlWriter.escape(contentCondition.getText());
            xmlWriter.closeTag("condition", false, false);
        }
        xmlWriter.closeTag("condition-group");
    }
    
    public static ConditionGroup getConditionGroup(Element element_xml) {
        String mode_strg = element_xml.getAttribute("mode");
        short mode = (mode_strg.equals("and"))?ConditionGroup.MODE_AND:ConditionGroup.MODE_OR;
        String raw = element_xml.getAttribute("raw");
        if (raw.length() > 0) {
            return Conditions.parseConditionGroup(raw,mode);
        }
        ConditionGroup conditionGroup = new ConditionGroup(mode);
        NodeList liste = element_xml.getChildNodes();
         for (int i = 0; i < liste.getLength(); i ++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) nd;
                String tagname = el.getTagName();
                if (tagname.equals("condition")) {
                    ContentConditionBean ccb = getContentConditionBean(el);
                    if (ccb != null) conditionGroup.addContentCondition(ccb);
                }
            }
         }
        return conditionGroup;
    }
        
   private static ContentConditionBean getContentConditionBean(Element element_xml) {
        String text = XMLUtils.getData(element_xml);
        String type = element_xml.getAttribute("type");
        short conditionType = 0;
        if (type.length() == 0) {
            conditionType = Conditions.CONTIENT;
        }
        else {
            try {
                conditionType = Conditions.typeToShort(type);
            }
            catch(IllegalArgumentException iae) {
                return null;
            }
        }
        return new ContentConditionBean(conditionType,text);
        
   }
   
   
   
   

}
