/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.conditions;

import java.io.Serializable;

/**
 *
 * @author Vincent Calame
 */
public final class ContentConditionBean implements ContentCondition, Serializable {
    private short conditionType = Conditions.EST_NON_VIDE;
    private String text = "";
    
    
    public ContentConditionBean() {
    }
    
    /**
     *
     *@throws IllegalArgumentException si text est nul
     */
    public ContentConditionBean(short conditionType,String text) {
        setConditionType(conditionType);
        setText(text);
    }
    
    public ContentConditionBean(ContentConditionBean contentConditionBean) {
        this.conditionType = contentConditionBean.conditionType;
        this.text = contentConditionBean.text;
    }
    
    public short getConditionType() {
        return conditionType;
    }
    
    public void setConditionType(short conditionType) {
        this.conditionType = conditionType;
    }
    
    public String getText() {
        return text;
    }
    
    /**
     *
     *@throws IllegalArgumentException si text est nul
     */
    public void setText(String text) {
        if (text == null) throw new IllegalArgumentException("text is null");
        this.text = text;
    }
    
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if (!(obj instanceof ContentConditionBean)) return false;
        ContentConditionBean other = (ContentConditionBean) obj;
        return ((other.conditionType == this.conditionType) && (other.text.equals(this.text)));
    }
    
    public int hashCode() {
        return conditionType + text.hashCode();
    }
    
    
}
