/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.conf;

import org.w3c.dom.*;
import net.mapeadores.util.xml.*;

/**
 *
 * @author Vincent Calame
 */
public class ConfDOMReader {
    private ConfDOMErrorHandler errorHandler;
    private Conf conf;
    
    
    public ConfDOMReader(Conf conf, ConfDOMErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
        this.conf = conf;
    }
    
    public void readConf(Element element_xml) {
        DOMReader.readChildren(element_xml, new ConfHandler());
    }
    
    private class ConfHandler implements ElementHandler{
        
         public void readElement(Element element) {
             String tagName = element.getTagName();
             if (tagName.equals("param")) {
                 String name = element.getAttribute("name");
                 if (name.length() == 0) {
                     errorHandler.missingParamNameAttributeError();
                     return;
                 }
                 String value = element.getAttribute("value");
                 short result = conf.set(name,value);
                 if (result == Conf.UNKNOWNKEY_RESULT) {
                     errorHandler.unknownKeyError(name);
                 } else if (result == Conf.BADVALUE_RESULT) {
                     errorHandler.badValueError(name, value);
                 }
             }
             else {
                errorHandler.unknownTagWarning(tagName);
            }
         }
    }
    
}
