/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.css;

import java.io.*;
import java.awt.Color;
import net.mapeadores.util.xml.XMLUtils;

/**
 *
 * @author Vincent Calame
 */
public class CssWriter {
    Writer writer;
    boolean onSelector = false;
    
    public CssWriter(Writer writer) {
        this.writer = writer;
    }
    
    public void startSelector(String selector) throws IOException {
        closeCurrent();
        onSelector = true;
        writer.write(selector);
        writer.write("  {\n");
    }
    
    public void startIdSelector(String id) throws IOException {
        startSelector("#"+id);
    }
    
    public void startClassSelector(String className) throws IOException {
        startSelector("."+className);
    }
    
    public void startTagClassSelector(String tagName, String className) throws IOException {
        startSelector(tagName + "."+className);
    }
    
    public void addProperty(String propertyName, String propertyValue) throws IOException {
        writer.write("\t");
        writer.write(propertyName);
        writer.write(": ");
        writer.write(propertyValue);
        writer.write(";\n");
    }
    
    public void addProperty(String propertyName,Color color) throws IOException {
        if (color == null) addProperty(propertyName,"none");
        else addProperty(propertyName,XMLUtils.toHexFormat(color));
    }
    
    public void flush() throws IOException{
        closeCurrent();
        writer.flush();
    }
    
    private void closeCurrent() throws IOException {
        if (onSelector) {
            writer.write("}\n\n");
            onSelector = false;
        }        
    }
    
}
