/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.exceptions;

import javax.xml.transform.*;
import org.xml.sax.*;

/**
 *
 * @author Vincent Calame
 */
public class NestedTransformerException extends RuntimeException {
    TransformerException transformerException;
    
    public NestedTransformerException(TransformerException te) {
        super(te);
        this.transformerException = te;
    }
    
    public String getMessage() {
        return getTransformerMessage(transformerException);
    }
    
    public TransformerException geTransformerException() {
        return transformerException;
    }
    
    public TransformerException getTransformerException() {
        return transformerException;
    }
    
    private String getTransformerMessage(TransformerException te) {
        Throwable e = te.getCause();
        if ((e != null) && (e instanceof TransformerException)) {
            return getTransformerMessage((TransformerException) e);
        }
        StringBuffer buf = new StringBuffer();
        SourceLocator sourceLocator = te.getLocator();
        if (sourceLocator != null) {
            ExceptionsUtils.append(buf,sourceLocator.getPublicId(),sourceLocator.getSystemId(), sourceLocator.getLineNumber(),sourceLocator.getColumnNumber());
        }
        else if ((e != null) && (e instanceof SAXParseException)) {
            SAXParseException spe = (SAXParseException) e;
            ExceptionsUtils.append(buf,spe);
        }
        if (e != null) {
            buf.append(e.getLocalizedMessage());
        }
        else {
            buf.append(te.getMessageAndLocation());
        }
        return buf.toString();
    }
    
    private void append(StringBuffer buf, String publicId, String systemId,int lineNumber,int columnNumber) {
            if (publicId != null) {
                buf.append(publicId);
                buf.append(":");
            }
            if (systemId != null) {
                buf.append(systemId);
                buf.append(":");
            }
            buf.append("line#");
            buf.append(lineNumber);
            buf.append(":col#");
            buf.append(columnNumber);
            buf.append(": ");
    }
    
}
