/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.format;

import java.text.*;
import net.mapeadores.util.exceptions.*;

/**
 *
 * @author Vincent Calame
 */
public class Calcul {
    public final static short DIVISION = 1;
    public final static short ADDITION = 2;
    public final static short SOUSTRACTION = 3;
    public final static short MULTIPLICATION = 4;
    private short type;
    private boolean isEntier = false;
    private int valeurEntiere;
    private float valeur;
    
    Calcul(short type, float valeur) {
        this.type = type;
        this.valeur = valeur;
        isEntier = false;
        valeurEntiere = 0;
    }
    
    Calcul(short type, int valeurEntiere) {
        this.type = type;
        this.valeurEntiere = valeurEntiere;
        valeur = (float) valeurEntiere;
        isEntier = true;
    }
    
    public static Calcul parse(String s) throws ParseException {
        s = s.trim();
        if (s.length() < 2) {
            throw new ParseException("Too short", 0);
        }
        short type = 0;
        char carac = s.charAt(0);
        switch(carac) {
            case  '+' :
                type = ADDITION;
                break;
            case '-' :
                type = SOUSTRACTION;
                break;
            case '*' :
                type = MULTIPLICATION;
                break;
            case '/' :
                type = DIVISION;
                break;
            default :
                throw new ParseException("Missing operator", 0);
        }
        String numero = s.substring(1).trim();
        try {
            int valeurEntiere = Integer.parseInt(numero);
            return new Calcul(type, valeurEntiere);
        }
        catch(NumberFormatException nfe) {
            try {
                float valeur = Float.parseFloat(numero);
                return new Calcul(type, valeur);
            }
            catch(NumberFormatException nfe2) {
                throw new ParseException("Not a number", 1);
            }
        }
    }
    
    public double execute(double d) {
        switch(type) {
            case ADDITION :
                return d + valeur;
            case SOUSTRACTION :
                return d - valeur;
            case MULTIPLICATION :
                return d * valeur;
            case DIVISION:
                return d / valeur;
            default :
                throw new SwitchException("unknwon type = " + type);
        }
    }
    
    public float execute(float d) {
        switch(type) {
            case ADDITION :
                return d + valeur;
            case SOUSTRACTION :
                return d - valeur;
            case MULTIPLICATION :
                return d * valeur;
            case DIVISION:
                return d / valeur;
            default :
                throw new SwitchException("unknwon type = " + type);
        }
    }
    
    public long execute(long l) {
        if (!isEntier) return (long) execute((double) l);
        switch(type) {
            case ADDITION :
                return l + valeurEntiere;
            case SOUSTRACTION :
                return l - valeurEntiere;
            case MULTIPLICATION :
                return l *valeurEntiere;
            case DIVISION:
                return l / valeurEntiere;
            default :
                throw new SwitchException("unknwon type = " + type);
        }
        
    }
    
    public int execute(int i) {
        if (!isEntier) return (int) execute((float) i);
        switch(type) {
            case ADDITION :
                return i + valeurEntiere;
            case SOUSTRACTION :
                return i - valeurEntiere;
            case MULTIPLICATION :
                return i *valeurEntiere;
            case DIVISION:
                return i / valeurEntiere;
            default :
                throw new SwitchException("unknwon type = " + type);
        }
    }
    
}
