/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.io;

import javax.xml.transform.*;
import org.xml.sax.*;
import net.mapeadores.util.exceptions.*;

/**
 *
 * @author Vincent Calame
 */
public class TransformerErrorListener implements ErrorListener {
    private ErrorListener customErrorListener;
    private StringBuffer errorBuffer = new StringBuffer();
    private int fatalErrorCode = 0;
    
    public TransformerErrorListener() {
        
    }
    
    public void setCustomErrorListener(ErrorListener customErrorListener) {
        this.customErrorListener = customErrorListener;
    }
    
    
    
    public void checkFatalTransformerException(TransformerException te) {
        if (te.hashCode() != fatalErrorCode) addError(te);
    }
    
    public boolean hasError() {
        return (errorBuffer.length() != 0);
    }
    
    public String getErrorMessage() {
        return errorBuffer.toString();
    }
    
    public void error(TransformerException transformerException) throws TransformerException {
        addError(transformerException);
        if (customErrorListener != null) customErrorListener.error(transformerException);
        else throw transformerException;
    }
    
    public void fatalError(TransformerException transformerException) throws TransformerException {
        addError(transformerException);
        fatalErrorCode = transformerException.hashCode();
        if (customErrorListener != null) customErrorListener.fatalError(transformerException);
        else throw transformerException;
    }
    
    public void warning(TransformerException transformerException) throws TransformerException {
        if (customErrorListener != null) customErrorListener.warning(transformerException);
        else throw transformerException;
    }
    
    private void addError(TransformerException transformerException) {
        String message = getTransformerMessage(transformerException);
        if (errorBuffer.length() != 0) {
            errorBuffer.append(" / ");
        }
        errorBuffer.append(message);
    }
    
     private String getTransformerMessage(TransformerException te) {
        Throwable e = te.getCause();
        if ((e != null) && (e instanceof TransformerException)) {
            return getTransformerMessage((TransformerException) e);
        }
        StringBuffer buf = new StringBuffer();
        SourceLocator sourceLocator = te.getLocator();
        if (sourceLocator != null) {
            ExceptionsUtils.append(buf,sourceLocator.getPublicId(),sourceLocator.getSystemId(), sourceLocator.getLineNumber(),sourceLocator.getColumnNumber());
        }
        else if ((e != null) && (e instanceof SAXParseException)) {
            SAXParseException spe = (SAXParseException) e;
            ExceptionsUtils.append(buf,spe);
        }
        if (e != null) {
            buf.append(e.getLocalizedMessage());
        }
        else {
            buf.append(te.getMessageAndLocation());
        }
        return buf.toString();
    }
    
}
