/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.langinteger;

import java.io.*;
import java.util.*;
import net.mapeadores.util.collections.IntHashMap;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.primitives.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class CountryUtils {
    private static IntHashMap geolocMap;
    private static TreeMap iso3166CodeMap;
    private static HashMap iso3166ConversionMap;
    private static Set iso3166CodeSet;
    static {
        iso3166CodeMap = new TreeMap();
        try {
            String ligne = null;
            InputStream is = CountryUtils.class.getResourceAsStream("iso3166.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is,"UTF-8"));
            while ((ligne = bufReader.readLine()) != null) {
                ligne = ligne.trim();
                if (ligne.length() > 0) {
                    int code = LangInteger.parse("zxx-"+ligne,true);
                    iso3166CodeMap.put(ligne,new Integer(code));
                }
            }
            InputStream is2 = CountryUtils.class.getResourceAsStream("iso3166-complement.csv");
            BufferedReader bufReader2 = new BufferedReader(new InputStreamReader(is2,"UTF-8"));
            while ((ligne = bufReader2.readLine()) != null) {
                int idx = ligne.indexOf(',');
                if (idx != -1) {
                    ligne = ligne.substring(0,idx);
                }
                ligne = ligne.trim();
                if (ligne.length() > 0) {
                    int code = LangInteger.parse("zxx-"+ligne,true);
                    iso3166CodeMap.put(ligne,new Integer(code));
                }
            }
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
        catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
        iso3166CodeSet = Collections.unmodifiableSet(iso3166CodeMap.keySet());
    }
    static {
        String ligne = null;
        iso3166ConversionMap = new HashMap();
        try {
            InputStream is = CountryUtils.class.getResourceAsStream("iso3166-conversion.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is,"UTF-8"));
            while ((ligne = bufReader.readLine()) != null) {
                int idx = ligne.indexOf(',');
                if (idx > 0) {
                    String code1 = ligne.substring(0,idx).trim();
                    String code2 = ligne.substring(idx+1).trim();
                    if (code1.length() > 0) {
                        int code = LangInteger.parse("zxx-"+code2,true);
                        iso3166ConversionMap.put(code1,new Integer(code));
                    }
                }
            }
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
        catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
    }
    static {
        geolocMap = new IntHashMap();
        try {
            InputStream is = CountryUtils.class.getResourceAsStream("geoloc.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is,"UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx1 = ligne.indexOf(',');
                    int idx2 = ligne.indexOf(',', idx1+1);
                    if ((idx1 != -1) && (idx2 != -1)) {
                        String countryCode = ligne.substring(0, idx1).trim();
                        int countryInteger = parseCountryCode(countryCode);
                        DegreDecimal latitude = new DegreDecimal(StringUtils.parseDecimal(ligne.substring(idx1+1, idx2).trim()));
                        DegreDecimal longitude = new DegreDecimal(StringUtils.parseDecimal(ligne.substring(idx2+1).trim()));
                        geolocMap.put(countryInteger, new GeoPoint(latitude, longitude));
                    }
                }
            }
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
        catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
        catch(NumberFormatException nfe) {
             throw new InternalResourceException(nfe);
        }
    }
    
    private CountryUtils() {
    }
    
    public static GeoPoint getGeolocalisation(int countryCode) {
        return (GeoPoint) geolocMap.get(countryCode);
    }
    
    /**
     * Classé dans l'ordre alphabétique.
     */
    public static Set getIsoCodeSet() {
        return iso3166CodeSet;
    }
    
    public static class GeoPoint {
        private DegreDecimal latitude;
        private DegreDecimal longitude;

        private GeoPoint(DegreDecimal latitude, DegreDecimal longitude) {
            this.latitude = latitude;
            this.longitude = longitude;
        }
        
        public DegreDecimal getLatitude() {
            return latitude;
        }

        public DegreDecimal getLongitude() {
            return longitude;
        }
        
    }
    
    public static int parseCountryCode(String countryCode) throws LangIntegerException {
        countryCode = countryCode.toUpperCase();
        Integer itg = (Integer) iso3166CodeMap.get(countryCode);
        if (itg == null) {
            itg = (Integer) iso3166ConversionMap.get(countryCode);
        }
        if (itg != null) {
             return itg.intValue();
        }
        else {
            throw new LangIntegerException(countryCode);
        }
    }
    
}
