/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2006-2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.langinteger;

import java.util.*;
import net.mapeadores.util.collections.IntHashMap;
import java.io.*;
import net.mapeadores.util.exceptions.*;

/**
 *
 * @author Vincent Calame
 */
public class LangIntegerLocalisation {
    private static HashMap localeMap = new HashMap();
    private static IntHashMap langvoMap;
    /*
     *Initialisation des langues pour lequel il existe un libellé pour tous les codes,
     * pour les autres langues on utilisera les libellés fournis par la machine virtuelle pour les codes à deux lettres et l'anglais pour les codes à trois lettres
     */
    static {
        init("en");
        init("fr");
    }
    static {
        langvoMap = new IntHashMap();
        try {
            InputStream is = LangIntegerInitializer.class.getResourceAsStream("langvo.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is,"UTF-8"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    int idx = ligne.indexOf(',');
                    if (idx != -1) {
                        int langInteger = LangInteger.parse(ligne.substring(0, idx));
                        langvoMap.put(langInteger, ligne.substring(idx+1));
                    }
                }
            }
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
        catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
    }
    
    private LangIntegerLocalisation() {
    }
    
    public static String getVOLibelle(int langInteger) {
        String lib = (String) langvoMap.get(langInteger);
        if (lib == null) {
            lib = getLangLibelle(langInteger,Locale.getDefault());
        }
        return lib;
    }
    
    public static String getLangLibelle(int langInteger,Locale locale) {
        return getLangLibelle(langInteger, locale, true);
    }
    
    public static String getLangLibelle(int langInteger,Locale locale, boolean onlyMain) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getLangLibelle(langInteger, onlyMain);
    }
    
    public static String getCountryLibelle(int langInteger,Locale locale) {
        return getCountryLibelle(langInteger, locale, true);
    }
    
    public static String getCountryLibelle(int langInteger,Locale locale, boolean onlyMain) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getCountryLibelle(langInteger, onlyMain);
    }
    
    public static SortedMap getCountryMap(Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getCountrySortedMap();
    }
    
    public static SortedMap getLanguageMap(Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getLanguageSortedMap();
    }
    
    public static Integer getLangInteger(String lib,Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem.getLangInteger(lib);
    }
    
    public static String getCardinalDirection(short direction, boolean onlyInitiale,Locale locale) {
        LocaleItem localeItem = getLocaleItem(locale);
        return localeItem. getCardinalDirection(direction, onlyInitiale);
    }
    
   
    public static LocaleItem getLocaleItem(Locale locale) {
        LocaleItem localeItem = (LocaleItem) localeMap.get(locale);
        if (localeItem != null) return localeItem;
        synchronized(localeMap) {
            localeItem = (LocaleItem) localeMap.get(locale);
            if (localeItem != null) return localeItem;
            String language = locale.getLanguage();
            if (language.length() == 0) {
                Locale defaultLocale = new Locale("en");
                localeItem = (LocaleItem) localeMap.get(defaultLocale);
                if (localeItem == null) {
                    localeItem = new LocaleItem(defaultLocale);
                    localeMap.put(defaultLocale,localeItem);
                }
                return localeItem;
            }
            else {
                Locale otherLocale = new Locale(language);
                localeItem = (LocaleItem) localeMap.get(otherLocale);
                if (localeItem == null) {
                    localeItem = new LocaleItem(locale);
                    localeMap.put(locale,localeItem);
                }
                return localeItem;
            }
        }
    }
    
    private static LocaleItem init(String langString) {
        LocaleItem localeItem = new LocaleItem(langString);
        localeMap.put(localeItem.getLocale(),localeItem);
        return localeItem;
    }
    
}
