/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.langinteger;

import java.io.*;
import java.util.*;
import net.mapeadores.util.exceptions.*;
import net.mapeadores.util.collections.*;

/**
 *
 * @author Vincent Calame
 */
public class LangUtils {
    private static TreeMap iso369Alpha3CodeMap;
    private static TreeMap iso369MixedCodeMap;
    private static Set iso369Alpha3CodeSet;
    private static Set iso369MixedCodeSet;
    private static IntHashMap conversionMap;
    static {
        conversionMap = new IntHashMap();
        HashMap iso3to2Map = new HashMap();
        try {
            InputStream is = LangIntegerInitializer.class.getResourceAsStream("iso639-3to2.csv");
            BufferedReader bufReader = new BufferedReader(new InputStreamReader(is,"ISO-8859-1"));
            String ligne = null;
            while ((ligne = bufReader.readLine()) != null) {
                if (ligne.length() > 0) {
                    String alpha3 = ligne.substring(0, 3);
                    String alpha2 = ligne.substring(4,6);
                    iso3to2Map.put(alpha3, alpha2);
                    conversionMap.put(getIso3(alpha3), new Integer(getIso2(alpha2)));
                }
            }
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
        catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
        iso369Alpha3CodeMap = new TreeMap();
        iso369MixedCodeMap = new TreeMap();
        try {
            String ligne = null;
            InputStream is2 = LangUtils.class.getResourceAsStream("iso639-alpha3-2.csv");
            BufferedReader bufReader2 = new BufferedReader(new InputStreamReader(is2,"UTF-8"));
            while ((ligne = bufReader2.readLine()) != null) {
                String codeAlpha3 = ligne.trim();
                if (codeAlpha3.length() == 3) {
                    Integer langInteger = null;
                    String codeAlpha2 = (String) iso3to2Map.get(codeAlpha3);
                    if (codeAlpha2 != null) {
                        langInteger = new Integer(getIso2(codeAlpha2));
                        iso369MixedCodeMap.put(codeAlpha2,langInteger);
                    }
                    else {
                        langInteger = new Integer(getIso3(codeAlpha3));
                        iso369MixedCodeMap.put(codeAlpha3,langInteger);
                    }
                    iso369Alpha3CodeMap.put(codeAlpha3,langInteger);
                }
            }
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
        catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
        iso369Alpha3CodeSet = Collections.unmodifiableSet(iso369Alpha3CodeMap.keySet());
        iso369MixedCodeSet = Collections.unmodifiableSet(iso369MixedCodeMap.keySet());
        try {
            InputStream is3 = LangIntegerInitializer.class.getResourceAsStream("iso639-conversion.csv");
            BufferedReader bufReader3 = new BufferedReader(new InputStreamReader(is3,"ISO-8859-1"));
            String ligne = null;
            while ((ligne = bufReader3.readLine()) != null) {
                int idx = ligne.indexOf(',');
                if (idx > 1) {
                    String code1 = ligne.substring(0,idx).trim();
                    String code2 = ligne.substring(idx+1).trim();
                    conversionMap.put(getCode(code1), new Integer(getCode(code2)));
                }
            }
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
        catch(LangIntegerException lie) {
            throw new InternalResourceException(lie);
        }
    }
    
    private LangUtils() {
    }
    
    public static Set geIsoAlpha3CodeSet() {
        return iso369Alpha3CodeSet;
    }
    
    public static Set getIsoMixedCodeSet() {
        return iso369MixedCodeSet;
    }
    
    public static int checkCode(int code) {
        Integer itg = (Integer) conversionMap.get(code);
        if (itg != null) return itg.intValue();
        else return code;
    }
    
    private static int getCode(String s) throws LangIntegerException{
        int length = s.length();
        if (length == 2) {
            return getIso2(s);
        }
        else if (length == 3) {
            return getIso3(s);
        }
        else {
            throw new LangIntegerException("wrong code : " + s);
        }
    }
    
    private static int getIso2(String s) throws LangIntegerException {
        int langInteger = 0x80000000;
        langInteger = putChar(langInteger, s.charAt(0), 1);
        langInteger = putChar(langInteger, s.charAt(1), 2);
        return langInteger;
    }
    
    private static int getIso3(String s) throws LangIntegerException {
        int langInteger = getIso2(s);
        langInteger = putChar(langInteger, s.charAt(2), 3);
        return langInteger;
    }
    
    private static int putChar(int langInteger, char c, int position) throws LangIntegerException {
        return putCinqbits(langInteger, toCinqbits(c), position);
    }
    
    static int putCinqbits(int langInteger, int cinqbits, int position) {
        switch(position) {
            case 1 :
                return (langInteger | (cinqbits << 25));
            case 2 :
                return (langInteger | (cinqbits << 20));
            case 3 :
                return (langInteger | (cinqbits << 15));
            case 4 :
                return (langInteger | (cinqbits << 10));
            case 5 :
                return (langInteger | (cinqbits << 5));
            case 6 :
                return (langInteger | cinqbits);
            default:
                throw new IllegalArgumentException();
        }
    }
    
    static int toCinqbits(char c) throws LangIntegerException {
        if ((c >= 65) && (c<=90)) {
            return (int) (c - 64);
        } else if ( (c>= 97) && (c<=122)) {
            return (int) (c - 96);
        }
        throw new LangIntegerException();
    }
    
}
