/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.localisation;

import java.util.*;
import net.mapeadores.util.langinteger.*;
import java.text.*;

/**
 *
 * @author Vincent Calame
 */
public class LocalisationLists {
    private int[] workingLangArray;
    private Locale[] formatLocaleArray;
    private int[] formatLocaleIntegerArray;
    private DecimalFormatSymbols[] decimalFormatSymbolsArray;
    
    /*public LocalisationLists() {
    }*/
    
    private LocalisationLists(int[] workingLangArray,Locale[] formatLocaleArray,int[] formatLocaleIntegerArray) {
        this.workingLangArray = workingLangArray;
        this.formatLocaleArray = formatLocaleArray;
        this.formatLocaleIntegerArray = formatLocaleIntegerArray;
        int length = formatLocaleArray.length;
        decimalFormatSymbolsArray = new DecimalFormatSymbols[length];
        for (int i = 0; i < length; i++) {
            decimalFormatSymbolsArray[i] = new DecimalFormatSymbols(formatLocaleArray[i]);
        }
    }
    
    public int getWorkingLangCount() {
        return workingLangArray.length;
    }
    
    public int getWorkingLangInteger(int i) {
        return workingLangArray[i];
    }
    
    public int getFormatLocaleCount() {
        return formatLocaleArray.length;
    }
    
    public Locale getFormatLocale(int i) {
        return formatLocaleArray[i];
    }
    
    public int getFormatLocaleInteger(int i) {
        return formatLocaleIntegerArray[i];
    }
    
    public DecimalFormatSymbols getDecimalFormatSymbols(int i) {
        return decimalFormatSymbolsArray[i];
    }
    
    public static LocalisationLists newInstance(LocalisationInfo localisationInfo) {
        int[] wl = new int[1];
        Locale[] loc = new Locale[1];
        int[] loci = new int[1];
        wl[0] = localisationInfo.getWorkingLangInteger();
        loc[0] = localisationInfo.getFormatLocale();
        loci[0] = localisationInfo.getFormatLocaleInteger();
        return new LocalisationLists(wl,loc,loci);
    }
    
    public static LocalisationLists newInstance(int workingLangInteger) {
        int[] wl = new int[1];
        Locale[] loc = new Locale[1];
        int[] loci = new int[1];
        wl[0] = workingLangInteger;
        loc[0] = LangInteger.getLocale(workingLangInteger);
        loci[0] = LangInteger.fromLocale(loc[0]);
        return new LocalisationLists(wl,loc,loci);
    }
    
    public static LocalisationLists newInstance(int[] workingLangIntegerArray) {
        int length = workingLangIntegerArray.length;
        int[] wl = new int[length];
        Locale[] loc = new Locale[length];
        int[] loci = new int[length];
        for(int i = 0; i < length; i++) {
            int langInteger = workingLangIntegerArray[i];
            Locale locale = LangInteger.getLocale(langInteger);
            wl[i] = langInteger;
            loc[i] = locale;
            loci[i] = LangInteger.fromLocale(locale);
        }
        return new LocalisationLists(wl,loc,loci);
    }
    
}
