/* Mapeadores Libraries - Open processURI libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.logging;

import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractProcessLog implements ProcessLog {
    private LinkedHashMap uriLogMap = new LinkedHashMap();
    private Set unmodifiableSet = Collections.unmodifiableSet(uriLogMap.keySet());
    
    public AbstractProcessLog() {
    }
    
    public Set getProcessSourceURISet() {
        return unmodifiableSet;
    }
    
    public List getProcessLogItemList(String processSourceURI) {
        URILog sourceLog = (URILog) uriLogMap.get(processSourceURI);
        if (sourceLog == null) {
            return Collections.EMPTY_LIST;
        }
        return sourceLog.getLogItemList();
    }
    
    public boolean hasLog() {
        for(Iterator it = uriLogMap.values().iterator();it.hasNext();) {
            URILog uriLog = (URILog) it.next();
            if (uriLog.hasLog()) return true;
        }
        return false;
    }
    
    public boolean hasLog(String processURIString) {
        URILog uriLog =  getURILog(processURIString);
        if (uriLog == null) {
            return false;
        }
        return uriLog.hasLog();
    }
    
    protected URILog getURILog(String processURIString) {
        return (URILog) uriLogMap.get(processURIString);
    }
    
    protected void putURILog(URILog uriLog) {
        uriLogMap.put(uriLog.processURI.getProcessURIString(),uriLog);
    }
    
    public class URILog {
        private List liste;
        protected ProcessURI processURI;
        
        public URILog(ProcessURI processURI) {
            this.processURI = processURI;
            liste = new ArrayList();
        }
        
        public boolean hasLog() {
            return !liste.isEmpty();
        }
        
        public List getLogItemList() {
            return liste;
        }
        
        public void add(ProcessLogItem logItem) {
            liste.add(logItem);
        }
        
        public ProcessLogItem newProcessLogItem() {
            return new ProcessLogItem(processURI.getProcessURIString());
        }
    }
    
}
