/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.logging;

import java.util.logging.*;

import java.util.ArrayList;
import java.io.*;
import net.mapeadores.util.html.HTMLUtils;

/**
 *
 * @author Vincent Calame
 */
public class BufferLogHandler extends Handler {
    ArrayList logRecordList = new ArrayList();
    Logger logger;
    
    public BufferLogHandler() {
        setFormatter(new InternalFormatter());
    }
    
    public void setLogger(Logger logger) {
        if (this.logger != null) {
            this.logger.removeHandler(this);
        }
        this.logger = logger;
        logger.addHandler(this);
    }
    
    public void close() throws SecurityException {
        logRecordList.clear();
        logger.removeHandler(this);
    }
    
    public void flush() {
    }
    
    public void publish(LogRecord logRecord) {
        Level current = this.getLevel();
        if (current != null) {
            Level level = logRecord.getLevel();
            if (level.intValue() >= current.intValue()) {
                logRecordList.add(logRecord);
            }
        } else logRecordList.add(logRecord);
    }
    
    public int bufferSize() {
        return logRecordList.size();
    }
    
    public Logger getLogger() {
        return logger;
    }
    
    public void print(PrintWriter printWriter, boolean escapeHTML) {
        Formatter formatter = getFormatter();
        if (formatter == null) {
            formatter = new InternalFormatter();
        }
        for (int i = 0; i < logRecordList.size();i++) {
            LogRecord logRecord = (LogRecord) logRecordList.get(i);
            if (escapeHTML) HTMLUtils.printPreformattedHTML(printWriter,formatter.format(logRecord));
            else printWriter.println(formatter.format(logRecord));
        }
        logRecordList.clear();
    }
    
    class InternalFormatter extends Formatter {
        
        public String format(LogRecord logRecord) {
            StringBuffer buf = new StringBuffer();
            buf.append('[');
            buf.append(logRecord.getLevel().getLocalizedName());
            buf.append("] ");
            buf.append(logRecord.getMessage());
            Object[] objs = logRecord.getParameters();
            if (objs != null) {
                for(int i = 0; i < objs.length;i++) {
                    buf.append(" - ");
                    buf.append(objs[i].toString());
                }
            }
            buf.append('\n');
            return buf.toString();
        }
    }
}
