/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.logging;

import net.mapeadores.util.localisation.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class CommandMessage {
    public static final short ERROR_MESSAGE = 1;
    public static final short DONE_MESSAGE = 2;
    public static final short MULTIERROR_MESSAGE = 3;
    public static final short RESULTLOG_MESSAGE = 4;
    private short type;
    private ResourceItem keyResourceItem;
    private ProcessLog processLog;
    private List multiErrorList;
    
    private CommandMessage(short type, String key, Object value) {
        if (key == null) throw new IllegalArgumentException("key is null");
        this.type = type;
        keyResourceItem = new ResourceItem(key,value);
    }
    
    private CommandMessage(ResourceItem resourceItem, List multiErrorList) {
        this.type = MULTIERROR_MESSAGE;
        this.keyResourceItem = resourceItem;
        this.multiErrorList = multiErrorList;
    }
    
    private CommandMessage(short type, ResourceItem resourceItem) {
        if (resourceItem == null) throw new IllegalArgumentException("keyResourceItem is null");
        this.type = type;
        this.keyResourceItem = resourceItem;
    }
    
    private CommandMessage(ResourceItem titleKeyResourceItem,ProcessLog processLog) {
        this.type = RESULTLOG_MESSAGE;
        this.processLog = processLog;
        this.keyResourceItem = titleKeyResourceItem;
    }
    
    public short getType() {
        return type;
    }
    
   
    public boolean isErrorMessage() {
        return ((type == ERROR_MESSAGE) || (type == MULTIERROR_MESSAGE));
    }
    
    
    public ResourceItem getKeyResourceItem() {
        return keyResourceItem;
    }
    
    public List getMultiErrorList() {
        return multiErrorList;
    }
    
    public ProcessLog getProcessLog() {
        return processLog;
    }
    
    public static CommandMessage error(String key, Object value) {
        return new CommandMessage(ERROR_MESSAGE,key,value);
    }
    
    public static CommandMessage error(ResourceItem keyResourceItem) {
        return new CommandMessage(ERROR_MESSAGE,keyResourceItem);
    }
    
    /**
     * la liste doit contenir des instances de RessourceItem
     */
    public static CommandMessage multiError(ResourceItem titleKeyResourceItem, List ressourceItemList) {
        if (titleKeyResourceItem == null) throw new IllegalArgumentException("titleKeyResourceItem is null");
        if (ressourceItemList == null) throw new IllegalArgumentException("errorMap is null");
        return new CommandMessage(titleKeyResourceItem,ressourceItemList);
    }
    
    public static CommandMessage done(String key, Object value) {
        return new CommandMessage(DONE_MESSAGE,key,value);
    }
    
    public static CommandMessage done(ResourceItem keyResourceItem) {
        return new CommandMessage(DONE_MESSAGE,keyResourceItem);
    }
    
    public static CommandMessage log(ResourceItem titleKeyResourceItem,ProcessLog processLog) {
        if (titleKeyResourceItem == null) throw new IllegalArgumentException("titleKeyResourceItem is null");
        if (processLog == null) throw new IllegalArgumentException("processLog is null");
        return new CommandMessage(titleKeyResourceItem,processLog);
    }
}
