/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.logging;

import java.util.logging.*;
import java.util.*;
import java.text.*;
import net.mapeadores.util.text.*;
import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class LocalizedFormatter extends java.util.logging.Formatter {
    private static final DateFormat jour = new SimpleDateFormat("yyyy-MM-dd");
    private static final DateFormat heure = new SimpleDateFormat("HH:mm:ss");
    private Date dat = new Date();
    private Locale locale;
    
    public LocalizedFormatter(Locale locale) {
        this.locale = locale;
    }
    
    public synchronized String format(LogRecord logRecord) {
        dat.setTime(logRecord.getMillis());
	StringBuffer buf = new StringBuffer();
        appendLetter(buf, logRecord.getLevel());
        buf.append("[");
        buf.append(jour.format(dat));
        buf.append("][");
        buf.append(heure.format(dat));
        buf.append("]");
        appendMessage(buf,logRecord);
	buf.append('\n');
	if (logRecord.getThrown() != null) {
	    try {
	        StringWriter sw = new StringWriter();
	        PrintWriter pw = new PrintWriter(sw);
	        logRecord.getThrown().printStackTrace(pw);
	        pw.close();
		buf.append(sw.toString());
	    } catch (Exception ex) {
	    }
	}
	return buf.toString();
    }
    
     void appendMessage(StringBuffer buf,LogRecord logRecord) {
	String key = logRecord.getMessage();
	ResourceBundle resourceBundle = logRecord.getResourceBundle();
        if (resourceBundle == null) {
            appendSimple(buf,logRecord);
        }
        else if (resourceBundle != null) {
	        try {
                    String s = StringUtils.getMessage(resourceBundle,key,logRecord.getParameters(), locale);
                    buf.append("[");
	            buf.append(key);
                    buf.append("] ");
                    buf.append(s);
                    
	        } catch (Exception ex) {
		     appendSimple(buf,logRecord);
                    return;
		}
	}
    }
     
     void appendSimple(StringBuffer buf,LogRecord logRecord) {
         buf.append(" ");
         buf.append(logRecord.getMessage());
         Object[] objs = logRecord.getParameters();
            if (objs != null) {
                for(int i = 0; i < objs.length;i++) {
                    buf.append(" - ");
                    buf.append(objs[i].toString());
                }
            }
     }
     
     void appendLetter(StringBuffer buf, Level level) {
         if (level.equals(Level.SEVERE)) {
             buf.append("s");
         }
         else if (level.equals(Level.WARNING)) {
             buf.append("w");
         }
         else if (level.equals(Level.INFO)) {
             buf.append("i");
         }
         else if (level.equals(Level.CONFIG)) {
             buf.append("c");
         }
         else {
             buf.append("f");
         }
     }
    
}
