/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.logging;

import java.util.logging.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class LoggingUtils {
    public static ProcessLog EMPTY_PROCESSLOG = new EmptyProcessLog();
    
    private LoggingUtils() {
    }
    
    public static void logWithThrowable(Logger logger,String key, Throwable throwable,Object parameter) {
        Object[] objs  = null;
        if (parameter != null) {
            objs = new Object[2];
            objs[0] = parameter;
            objs[1] = throwable;
        }
        else {
           objs = new Object[1];
            objs[0] = throwable;
        }
        logger.log(Level.SEVERE,key,objs);
    }
    
    public static void logWithThrowable(Logger logger,String key, Throwable throwable,Object parameter1,Object parameter2) {
        Object[] objs  = new Object[3];
        objs[0] = parameter1;
        objs[1] = parameter2;
        objs[2] = throwable;
        logger.log(Level.SEVERE,key,objs);
    }
    
    public static void logWithThrowable(Logger logger,Level level,String key, Throwable throwable,Object[] parameters) {
        Object[] objs  = new Object[parameters.length+1];
        System.arraycopy(parameters,0, objs,0,parameters.length);
        objs[parameters.length] = throwable;
        logger.log(level,key,objs);
    }
    
    private static class EmptyProcessLog implements ProcessLog {
        
        public boolean hasLog() {
            return false;
        }
    
        public boolean hasLog(String processSourceURI) {
            return false;
        }
    
        public List getProcessLogItemList(String processSourceURI) {
            return Collections.EMPTY_LIST;
        }
    
        public Set getProcessSourceURISet() {
            return Collections.EMPTY_SET;
        }
        
    }
    
}
