/* Mapeadores Libraries - Open processSourceURI libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.logging;

/**
 *
 * @author Vincent Calame
 */
public class ProcessLogItem {
    private int lineNumber = -1;
    private String category;
    private String key;
    private Object object;
    private String processSourceURI;
    private long generationTime;
    
    public ProcessLogItem(String processSourceURI) {
        this.processSourceURI = processSourceURI;
        this.generationTime = System.currentTimeMillis();
    }
    
    public String getProcessSourceURI() {
        return processSourceURI;
    }
    
    /**
     * Retourne le moment où ProcessLogItem a été généré.
     */
    public long getGenerationTime() {
        return generationTime;
    }
    
    public int getLineNumber() {
        return lineNumber;
    }
    
    public void setLineNumber(int lineNumber) {
        this.lineNumber = lineNumber;
    }
    
    public Object getObject() {
        return object;
    }
    
    public void setObject(Object object) {
        this.object = object;
    }
    
    public String getKey() {
        return key;
    }
    
    public void setKey(String key) {
        this.key = key;
    }
    
    public String getCategory() {
        return category;
    }
    
    public void setCategory(String category) {
        this.category = category;
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        if (lineNumber != -1) {
            buf.append("@");
            buf.append(lineNumber);
            buf.append(": ");
        }
        if (category != null) {
            buf.append("[");
            buf.append(category);
            buf.append("] ");
        }
        buf.append(key);
        if (object != null) {
            buf.append(" = ");
            if (object instanceof Object[]) {
                Object[] array = (Object[]) object;
                int length = array.length;
                for(int i = 0; i < length; i++) {
                    if (i > 0) buf.append(" | ");
                    buf.append(array[i].toString());
                }
            }
            else {
                buf.append(object.toString());
            }
        }
        return buf.toString();
    }
}
