/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.primitives;

/**
 *
 * @author  Vincent Calame
 */
public class BitmaskCounter {
    int[] counts = new int[32];
    int noemptybitmask = 0;
    
    public BitmaskCounter() {
    }
    
    public BitmaskCounter(BitmaskCounter maskCounter) {
        this.noemptybitmask = maskCounter.noemptybitmask;
        System.arraycopy(maskCounter.counts, 0, counts, 0, 32);
    }
    
    public boolean isEmpty() {
        return (noemptybitmask == 0);
    }
    
    /**
     * Équivalent de ((bitmask & getNoEmptyBitmask()) == 0)
     */
    public boolean isEmptyForBitmask(int bitmask) {
        return ((bitmask & noemptybitmask) == 0);
    }
    
    public int getNoEmptyBitmask() {
        return noemptybitmask;
    }
    
    /**
     * est égal à ~getNoEmptyBitmask
     */
    public int getEmptyBitmask() {
        return (~noemptybitmask);
    }
    
    public int getCountAt(byte rang) {
        return counts[rang];
    }
    /*
    public void increaseCountAt(byte rang) {
        counts[rang]++;
        noemptybitmask = noemptybitmask | BitmaskUtils.getBitmask(rang);
    }
    
    public void decreaseCountAt(byte rang) {
        counts[rang]--;
        if (counts[rang] <= 0) {
            counts[rang] = 0;
            noemptybitmask = (noemptybitmask & (~BitmaskUtils.getBitmask(rang)));
        }
    }
    */
    
    public int getCount(int bitmask) {
        if ((noemptybitmask & bitmask) == 0) return 0;
        int count = 0;
        for (int i = 0; i < 32;i++) {
            int puissance = BitmaskUtils.getBitValue(i);
            if ((bitmask & puissance) != 0) {
                count  = count + counts[i];
            }
        }
        return count;
    }
    
    public void increaseCount(int bitmask) {
        if (bitmask == 0) return;
        for (int i = 0; i < 32;i++) {
            int puissance = BitmaskUtils.getBitValue(i);
            if ((bitmask & puissance) != 0) {
                counts[i] ++;
            }
        }
        noemptybitmask = noemptybitmask | bitmask;
    }
    
    public void decreaseCount(int bitmask) {
        if (bitmask == 0) return;
        for (int i = 0; i < 32;i++) {
            int puissance = BitmaskUtils.getBitValue(i);
            if ((bitmask & puissance) != 0) {
                counts[i] --;
                if (counts[i] <= 0) {
                    counts[i] = 0;
                    noemptybitmask = (noemptybitmask & (~puissance));
                }
            }
        }
    }
    
}
