/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.primitives;

import java.io.*;
import java.text.*;

/**
 *
 * @author Vincent Calame
 */
public final class Decimal implements Serializable {
    private static final long serialVersionUID = 1L;
    private long partieEntiere;
    private int partieDecimale;
    private byte zeroLength;
    
    public Decimal(long partieEntiere, byte zeroLength, int partieDecimale) {
        if (partieEntiere < 0) {
            if (partieDecimale > 0) partieDecimale = - partieDecimale;
        } else if (partieEntiere > 0) {
            if (partieDecimale < 0) partieDecimale = - partieDecimale;
        }
        if (zeroLength < 0) zeroLength = (byte) (- zeroLength);
        this.partieEntiere = partieEntiere;
        this.zeroLength = zeroLength;
        this.partieDecimale = partieDecimale;
    }
    
    public Decimal(double d) {
        this.partieEntiere = (long) d;
        double d2 = Math.abs(d - partieEntiere);
        if (d2 == 0) {
            partieDecimale = 0;
            zeroLength = 0;
        }
        else {
            int zl = 0;
            while(true) {
                double d3 = d2 * 10;
                if (d3 >= 1) {
                    break;
                }
                d2 = d3;
                zl++;
            }
            this.zeroLength = (byte) zl;
            int partE = (int) Math.round(d2*1000000);
            if (partE == 1000000) partE = 999999;
            while(true) {
                int pe2 = partE / 10;
                if ((pe2*10) == partE) {
                    partE = pe2;
                }
                else {
                    break;
                }
            }
            if (partieEntiere < 0) partE = - partE;
            this.partieDecimale = partE;
        }
    }
    
    public int hashCode() {
        return (int) partieEntiere;
    }
    
    
    public long getPartieEntiere() {
        return partieEntiere;
    }
    
    public double toDouble() {
        return Double.parseDouble(this.toString());
    }
    
    /**
     * Négative si le nombre est négatif.
     */
    public int getPartieDecimale() {
        return partieDecimale;
    }
    
    public String toString() {
        return toString(partieEntiere,zeroLength, partieDecimale);
    }
    
    public String toString(DecimalFormatSymbols symbols) {
        return toString(symbols.getDecimalSeparator(), symbols.getGroupingSeparator());
    }
    
    public String toString(char decimalSeparator, char groupingSeparator) {
        StringBuffer buf = new StringBuffer();
        append(buf,partieEntiere,groupingSeparator,false,partieDecimale);
        if ((partieDecimale != 0) || (zeroLength != 0))  {
            buf.append(decimalSeparator);
            appendDecimal(buf,zeroLength, partieDecimale);
        }
        return buf.toString();
    }
    
    public String toStringWithBlank(char decimalSeparator) {
        StringBuffer buf = new StringBuffer();
        append(buf,partieEntiere,' ', true,partieDecimale);
        if ((partieDecimale != 0) || (zeroLength != 0)) {
            buf.append(decimalSeparator);
            appendDecimal(buf,zeroLength, partieDecimale);
            buf.append(' ');
        }
        return buf.toString();
    }
    
    public byte getZeroLength() {
        return zeroLength;
    }
    
    public String getPartieEntiereString() {
        if (partieEntiere == 0) {
            if (partieDecimale < 0) {
                return "-0";
            } else return "0";
        }
        return Long.toString(partieEntiere);
    }
    
    public String getPartieDecimaleString() {
        return getPartieDecimaleString(zeroLength, partieDecimale);
    }
    
    private static void append(StringBuffer buf, long l, char groupingSeparator, boolean atLast, int partieDecimale) {
        if (l == Long.MIN_VALUE) {
            buf.append("- 9 223 372 036 854 775 808");
            return;
        }
        if (l < 0) {
            l = -l;
            buf.append("- ");
        } else if ((l == 0) && (partieDecimale < 0)) {
            buf.append("- ");
        }
        String s = Long.toString(l);
        int length = s.length();
        if (length <= 3) {
            buf.append(s);
            if (atLast) buf.append(groupingSeparator);
            return;
        }
        int reste = length % 3;
        if (reste > 0) {
            buf.append(s.charAt(0));
            if (reste > 1) buf.append(s.charAt(1));
            buf.append(groupingSeparator);
        }
        int p = 0;
        for(int i = reste; i < length; i++) {
            buf.append(s.charAt(i));
            p++;
            if (p == 3) {
                if ((i < (length -1)) || (atLast)) buf.append(groupingSeparator);
                p = 0;
            }
        }
    }
    
    public static String toString(long partieEntiere,  byte zeroLength, int partieDecimale) {
        if ((partieDecimale == 0) && (zeroLength == 0)) {
            return Long.toString(partieEntiere);
        }
        StringBuffer buf = new StringBuffer();
        if ((partieEntiere == 0) && (partieDecimale < 0)) {
            buf.append("-0");
        } else buf.append(Long.toString(partieEntiere));
        buf.append('.');
        appendDecimal(buf, zeroLength, partieDecimale);
        return buf.toString();
    }
    
    public static String getPartieDecimaleString(byte zeroLength, int partieDecimale) {
        StringBuffer buf = new StringBuffer();
        appendDecimal(buf, zeroLength, partieDecimale);
        return buf.toString();
    }
    
    public static float toFloat(byte zeroLength, int partieDecimale) {
        StringBuffer buf = new StringBuffer();
        buf.append("0.");
        appendDecimal(buf, zeroLength, partieDecimale);
        return Float.parseFloat(buf.toString());
    }
    
    
    private static void appendDecimal(StringBuffer buf, byte zeroLength, int partieDecimale) {
        for(int i = 0; i < zeroLength; i++) {
            buf.append('0');
        }
        if (partieDecimale != 0) buf.append(Integer.toString(Math.abs(partieDecimale)));
    }
    
    public Decimal abs() {
        if ((partieEntiere < 0) || (partieDecimale < 0)) {
            return new Decimal(-partieEntiere, zeroLength, -partieDecimale);
        } else {
            return this;
        }
    }
    
    public boolean isZero() {
        return ((partieEntiere == 0) && (partieDecimale == 0));
    }
    
    public boolean isPositif() {
        return ((partieEntiere >= 0) && (partieDecimale >= 0));
    }
    
    
    
    public static Decimal add(Decimal decimal1, Decimal decimal2) {
        if (decimal1.isZero()) return decimal2;
        if (decimal2.isZero()) return decimal1;
        
        Decimal abs1 = decimal1.abs();
        Decimal abs2 = decimal2.abs();
        boolean positif = decimal1.isPositif();
        boolean minus = (positif != decimal2.isPositif());
        if (minus) {
            int comp = abs1.compareAbs(abs2);
            if (comp > 0)  {
                return minusAbs(abs1, abs2, positif);
            } else if (comp < 0) {
                return minusAbs(abs2, abs1, !positif);
            } else {
                return new Decimal(0, (byte) 0, 0);
            }
        } else {
            return addAbs(abs1, abs2, positif);
        }
    }
    
    private int compareAbs(Decimal other) {
        if (this.partieEntiere < other.partieEntiere) {
            return -1;
        } else if (this.partieEntiere > other.partieEntiere) {
            return 1;
        } else if (this.zeroLength > other.zeroLength) {
            return -1;
        } else if (this.zeroLength < other.zeroLength) {
            return 1;
        } else {
            String s1 = String.valueOf(this.partieDecimale);
            String s2 = String.valueOf(other.partieDecimale);
            return s1.compareTo(s2);
        }
    }
    
    private static Decimal addAbs(Decimal decimal1, Decimal decimal2, boolean positif) {
        long partieEntiere = decimal1.partieEntiere + decimal2.partieEntiere;
        StringBuffer decString1 = new StringBuffer();
        decString1.append('1');
        StringBuffer decString2 = new StringBuffer();
        decString2.append('1');
        appendDecimal(decString1, decimal1.zeroLength, decimal1.partieDecimale);
        appendDecimal(decString2, decimal2.zeroLength, decimal2.partieDecimale);
        int diff = decString1.length() - decString2.length();
        if (diff < 0) {
            diff = - diff;
            for(int i = 0; i < diff; i++) {
                decString1.append('0');
            }
        } else {
            for(int i = 0; i < diff; i++) {
                decString2.append('0');
            }
        }
        int length = decString1.length();
        int dec1 = Integer.parseInt(decString1.toString());
        int dec2 = Integer.parseInt(decString2.toString());
        int dec = dec1 + dec2;
        String decString = String.valueOf(dec);
        if (decString.charAt(0) == '3') {
            partieEntiere = partieEntiere + 1;
        }
        dec = Integer.parseInt(decString.substring(1));
        byte zerobyte = 0;
        for(int i = 1; i < decString.length(); i++) {
            if (decString.charAt(i) == '0') {
                zerobyte++;
            } else {
                break;
            }
        }
        if (!positif) {
            partieEntiere = -partieEntiere;
            dec = - dec;
        }
        return new Decimal(partieEntiere, zerobyte, dec);
    }
    
    
    
    private static Decimal minusAbs(Decimal decimal1, Decimal decimal2, boolean positif) {
        long partieEntiere = decimal1.partieEntiere - decimal2.partieEntiere;
        StringBuffer decString1 = new StringBuffer();
        decString1.append('3');
        StringBuffer decString2 = new StringBuffer();
        decString2.append('1');
        appendDecimal(decString1, decimal1.zeroLength, decimal1.partieDecimale);
        appendDecimal(decString2, decimal2.zeroLength, decimal2.partieDecimale);
        int diff = decString1.length() - decString2.length();
        if (diff < 0) {
            diff = - diff;
            for(int i = 0; i < diff; i++) {
                decString1.append('0');
            }
        } else {
            for(int i = 0; i < diff; i++) {
                decString2.append('0');
            }
        }
        int length = decString1.length();
        int dec1 = Integer.parseInt(decString1.toString());
        int dec2 = Integer.parseInt(decString2.toString());
        int dec = dec1 - dec2;
        String decString = String.valueOf(dec);
        if (decString.charAt(0) == '1') {
            partieEntiere = partieEntiere -1;
        }
        dec = Integer.parseInt(decString.substring(1));
        byte zerobyte = 0;
        for(int i = 1; i < decString.length(); i++) {
            if (decString.charAt(i) == '0') {
                zerobyte++;
            } else {
                break;
            }
        }
        if (!positif) {
            partieEntiere = -partieEntiere;
            dec = - dec;
        }
        return new Decimal(partieEntiere, zerobyte, dec);
    }
    /*
    public static Decimal add(Decimal decimal1, Decimal decimal2) {
        if (decimal1.isZero()) return decimal2;
        if (decimal2.isZero()) return decimal1;
     
        boolean positif = decimal1.isPositif();
        boolean minus = (positif != decimal2.isPositif());
     
        StringBuffer decString1 = new StringBuffer();
        decString1.append('1');
        StringBuffer decString2 = new StringBuffer();
        decString2.append('1');
        appendDecimal(decString1, decimal1.zeroLength, decimal1.partieDecimale);
        appendDecimal(decString2, decimal2.zeroLength, decimal2.partieDecimale);
        int diff = decString1.length() - decString2.length();
        if (diff < 0) {
            diff = - diff;
            for(int i = 0; i < diff; i++) {
                decString1.append('0');
            }
        }
        else {
            for(int i = 0; i < diff; i++) {
                decString2.append('0');
            }
        }
        int length = decString1.length();
        int dec1 = Integer.parseInt(decString1.toString());
        int dec2 = Integer.parseInt(decString2.toString());
        if (minus) {
            long partieEntiere = Math.abs(decimal1.partieEntiere) - Math.abs(decimal2.partieEntiere);
            int dec = dec1 - dec2;
            if (partieEntiere == 0) {
     
            }
            if (dec == 0) {
                return new Decimal(partieEntiere, (byte) (length - 1), 0);
            }
            else if (dec < 0) {
                if (partieEntiere > 0) partieEntiere = partieEntiere -1;
                if ()
            }
            else {
     
            }
            return null;
        }
        else {
            long partieEntiere = decimal1.partieEntiere + decimal2.partieEntiere;
            int dec = dec1 + dec2;
            String decString = String.valueOf(dec);
            if (decString.charAt(0) == '3') {
                if (positif) partieEntiere = partieEntiere + 1;
                else partieEntiere = partieEntiere - 1;
            }
            if (!positif) dec = - dec;
            byte zerobyte = 0;
            for(int i = 1; i < decString.length(); i++) {
                if (decString.charAt(i) == '0') {
                    zerobyte++;
                }
                else {
                    break;
                }
            }
            return new Decimal(partieEntiere, zerobyte, dec);
        }
    }*/
    
   /* private static void minus() {
    
    }
    
    public int compareTo(Object obj) {
        Decimal other = (Decimal) obj;
        if (this.partieEntiere < other.partieEntiere) {
            return -1;
        }
        else if (this.partieEntiere > other.partieEntiere) {
            return 1;
        }
        else if (this.zeroLength > other.zeroLength) {
            return -1;
        }
    }
    
    private static class DecimalAddOperator {
        Decimal decimal1;
        Decimal decimal2;
    
        DecimalAddOperator(Decimal dec1, Decimal dec2) {
            Decimal dect1 = dec1.abs();
            Decimal dect2 = dec2.abs();
            if (dect1.partieEntiere > dct2.partieEntiere) {
    
            }
        }
    
        void init(Decimal dec1, Decimal dec2) {
    
        }
    }
    
    private static class DecimalPart {
        boolean positif;
        int diviseur;
        int partieDecimale;
    
        DecimalPart(Decimal decimal) {
            partieDecimale = decimal.partieDecimale;
            String s = String.valueOf(partieDecimale);
            int etendue = s.length() + zeroLength;
            diviseur = 1;
            for(int i = 0; i < etendue; i++) {
                diviseur = diviseur*10;
            }
        }
    
        DecimalPart add(DecimalPart other) {
            int nouvellepart = this.partiedecimale
        }
    }
    
    private DecimalPart[] getDecimalPart(Decimal dec)
    */
    
}