/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.primitives;

import java.io.*;

/**
 *La partie décimale n'a jamais plus de six chiffres. La partie entière est comprise entre 180 et -179
 *
 * @author Vincent Calame
 */
public final class DegreDecimal implements Serializable {
    private static final long serialVersionUID = 1L;
    private int partieEntiere;
    private int partieDecimale;
    private byte zeroLength;
    
    public DegreDecimal(int partieEntiere, byte zeroLength, int partieDecimale) {
        this.partieEntiere = modulo(partieEntiere);
        if (partieEntiere < 0) {
            if (partieDecimale > 0) partieDecimale = - partieDecimale;
        }
        else if (partieEntiere > 0) {
            if (partieDecimale < 0) partieDecimale = - partieDecimale;
        }
        if (zeroLength < 0) zeroLength = (byte) (- zeroLength);
        this.zeroLength = zeroLength;
        this.partieDecimale = partieDecimale;
        check();
    }
    
    public DegreDecimal(DegreSexagesimal degreSexagesimal) {
        int partieEntiere = degreSexagesimal.getDegre();
        int seconde = ((degreSexagesimal.getMinute())* 60) + degreSexagesimal.getSeconde();
        int mm = Math.round((((float) seconde)*2500) /9);
        if (mm == 0) {
            this.zeroLength = 6;
        }
        else {
            String s = String.valueOf(mm);
            this.zeroLength = (byte) (6 - s.length());
        }
        if (degreSexagesimal.isPositif()) {
            this.partieEntiere = -partieEntiere;
            this.partieDecimale = - mm;
        }
        else {
            this.partieEntiere = partieEntiere;
            this.partieDecimale = mm;
        }
    }
    
    public DegreDecimal(Decimal decimal) {
        this.partieEntiere = modulo(decimal.getPartieEntiere());
        this.zeroLength = decimal.getZeroLength();
        this.partieDecimale = decimal.getPartieDecimale();
        check();
    }
    
    public boolean isPositif() {
        return ((partieEntiere >= 0) && (partieDecimale >= 0));
    }
    
    private int modulo(long partieEntiere) {
        long modulo = (partieEntiere) / 360;
        int result = (int) (partieEntiere - (modulo * 360));
        return result;
    }
    
    private void check() {
        checkDecimale();
        checkDegre();
    }
    
    private void checkDegre() {
        if (Math.abs(partieEntiere) < 180) {
            return;
        }
        if ((partieEntiere == 180) && (partieDecimale == 0)) return;
        Decimal newDecimal;
        if (partieEntiere > 180) {
            newDecimal = Decimal.add(new Decimal(-360,(byte)0,0), new Decimal(partieEntiere, zeroLength, partieDecimale));
        }
        else {
            newDecimal = Decimal.add(new Decimal(360,(byte)0,0), new Decimal(partieEntiere, zeroLength, partieDecimale));
        }
        this.partieEntiere = (int) newDecimal.getPartieEntiere();
        this.partieDecimale = newDecimal.getPartieDecimale();
        this.zeroLength = newDecimal.getZeroLength();
    }
    
    private void checkDecimale() {
        if (partieDecimale == 0) {
            zeroLength = 0;
            return;
        }
        if (zeroLength > 6) {
            zeroLength = 0;
            partieDecimale = 0;
        }
        int dec = Math.abs(partieDecimale);
        boolean negatif = (partieDecimale < 0);
        while(true) {
            if (dec % 10 == 0) {
                dec = dec / 10;
            }
            else {
                break;
            }
        }
        String s = String.valueOf(dec);
        if (zeroLength == 6) {
            if (s.charAt(0) > '5') {
                zeroLength = 5;
                partieDecimale = (negatif)?-1:1;
            }
            else {
                zeroLength = 0;
                partieDecimale = 0;
            }
            return;
        }
        int p =  6 - zeroLength;
        if (s.length() <= p) return;
        dec = Integer.parseInt(s.substring(0,p));
        if (s.charAt(p) > '5') {
            boolean neufS = true;
            for(int  i = 0; i < p; i++) {
                if (s.charAt(i) != '9') {
                    neufS = false;
                }
            }
            if (!neufS) dec = dec + 1;
            else if (zeroLength > 0) {
                zeroLength --;
                dec = 1;
            }
            else {
                dec = 0;
                if (partieEntiere > 0) {
                    partieEntiere = partieEntiere + 1;
                }
                else if (partieEntiere < 0) {
                    partieEntiere = partieEntiere - 1;
                }
                else {
                    if (negatif) {
                        partieEntiere = -1;
                    }
                    else {
                        partieEntiere = 1;
                    }
                }
            }
        }
        partieDecimale = (negatif)?-dec:dec;
    }
    
    public Decimal toDecimal() {
        return new Decimal(partieEntiere, zeroLength, partieDecimale);
    }
    
    public String toString() {
        return Decimal.toString(partieEntiere,zeroLength, partieDecimale);
    }

    public int getPartieEntiere() {
        return partieEntiere;
    }

    public int getPartieDecimale() {
        return partieDecimale;
    }

    public byte getZeroLength() {
        return zeroLength;
    }
    
    public int hashCode() {
        return (int) partieEntiere;
    }
    
    public DegreSexagesimal toDegreSexagesimal() {
        return DegreSexagesimal.fromDegreDecimal(this);
    }
    
}
