/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.primitives;

import java.text.*;

/**
 *
 * @author Vincent Calame
 */
public final class DegreSexagesimal {
    private boolean positif = true;
    private int degre;
    private int minute;
    private int seconde;
    
    public int getDegre() {
        return degre;
    }
    
    void setDegre(int degre) {
        this.degre = degre;
    }
    
    public int getMinute() {
        return minute;
    }
    
    void setMinute(int minute) {
        this.minute = minute;
    }
    
    public int getSeconde() {
        return seconde;
    }
    
    void setSeconde(int seconde) {
        this.seconde = seconde;
    }
    
    public boolean isPositif() {
        return positif;
    }
    
    void setPositif(boolean positif) {
        this.positif = positif;
    }
    
    public static DegreSexagesimal fromDegreDecimal(DegreDecimal degreDecimal) {
        DegreSexagesimal degreSexagesimal = new DegreSexagesimal();
        degreSexagesimal.setPositif(degreDecimal.isPositif());
        degreSexagesimal.setDegre(Math.abs(degreDecimal.getPartieEntiere()));
        float f = Decimal.toFloat(degreDecimal.getZeroLength(), degreDecimal.getPartieDecimale());
        float minF = (f * 60);
        int min = (int) Math.floor(minF);
        degreSexagesimal.setMinute(min);
        float secF = (minF - min) * 60;
        int  sec = (int) Math.round(secF);
        if (sec == 60) {
            sec = 0;
            min = min+1;
            if (min == 60) {
                min = 59;
                sec = 59;
            }
        }
        degreSexagesimal.setSeconde(sec);
        return degreSexagesimal;
    }
    
    public String toString() {
        return toString(true,"");
    }
    
    public String toString(boolean withSign, String separator) {
        StringBuffer buf = new StringBuffer();
        if (withSign) {
            if (!positif) buf.append("-");
        }
        buf.append(degre);
        buf.append('°');
        buf.append(separator);
        if (minute < 10) buf.append('0');
        buf.append(minute);
        buf.append('\u2032');
        buf.append(separator);
        if (seconde < 10) buf.append('0');
        buf.append(seconde);
        buf.append('\u2033');
        return buf.toString();
    }
    
    public DegreSexagesimal getOppose() {
        if ((minute == 0) && (seconde == 0) && ((degre == 0) || (degre == 180))) {
            return this;
        }
        DegreSexagesimal sexa = new DegreSexagesimal();
        sexa.setPositif(!positif);
        sexa.setDegre(degre);
        sexa.setMinute(minute);
        sexa.setSeconde(seconde);
        return sexa;
    }
    
    public static DegreSexagesimal parse(String s) throws ParseException {
        s = s.trim();
        int length = s.length();
        if (length == 0) {
            throw new ParseException("empty", 0);
        }
        char carac = s.charAt(0);
        boolean positif = true;
        if (carac == '-') {
            positif = false;
            s = s.substring(1).trim();
            length = s.length();
            if (length == 0) {
                throw new ParseException("no digit", 0);
            }
        }
        int degre = 0;
        int minute = 0;
        int seconde = 0;
        int nextIndex = 0;
        boolean valide = false;
        boolean debut = true;
        for(int i = 0; i < length; i++) {
            nextIndex = i + 1;
            int integer = toInt(s.charAt(i));
            if (integer == -2) {
                continue;
            } else if (integer == -1) {
                if (!valide) {
                    throw new ParseException("no digit", 0);
                } else {
                    break;
                }
            } else {
                valide = true;
                if (debut) {
                    if (integer == 0) {
                        continue;
                    } else {
                        debut = false;
                    }
                }
                degre = degre*10 + integer;
            }
        }
        if (nextIndex < length) {
            for(int i = nextIndex; i < length; i++) {
                if (toInt(s.charAt(i)) < 0) {
                    nextIndex = i +1;
                }
                else break;
            }
        }
        if (nextIndex < length) {
            debut = true;
            for(int i = nextIndex; i < length; i++) {
                nextIndex = i +1;
                int integer = toInt(s.charAt(i));
                if (integer == -2) {
                    continue;
                } else if (integer == -1) {
                    break;
                } else {
                     if (debut) {
                        if (integer == 0) {
                            continue;
                        } else {
                            debut = false;
                        }
                    }
                    minute = minute*10 + integer;
                }
            }
        }
        if (nextIndex < length) {
            for(int i = nextIndex; i < length; i++) {
                if (toInt(s.charAt(i)) < 0) {
                    nextIndex = i +1;
                }
                else break;
            }
        }
        if (nextIndex < length) {
            debut = true;
            for(int i = nextIndex; i < length; i++) {
                nextIndex = i +1;
                int integer = toInt(s.charAt(i));
                if (integer == -2) {
                    continue;
                } else if (integer == -1) {
                    break;
                } else {
                     if (debut) {
                        if (integer == 0) {
                            continue;
                        } else {
                            debut = false;
                        }
                    }
                    seconde = seconde*10 + integer;
                }
            }
        }
        int modulo = seconde / 60;
        seconde = seconde - (60 * modulo);
        minute = minute + modulo;
        modulo = minute / 60;
        minute = minute - (60 * modulo);
        degre = degre + modulo;
        degre = degre % 360;
        if ((degre > 180) || ((degre == 180) & ((!positif) || ((minute == 0) && (seconde == 0)))))  {
            minute = 60 -minute;
            seconde = 60 - seconde;
            degre = 360 -180;
            positif = !positif;
        }
        DegreSexagesimal sexa = new DegreSexagesimal();
        sexa.setDegre(degre);
        sexa.setMinute(minute);
        sexa.setSeconde(seconde);
        sexa.setPositif(positif);
        return sexa;
    }
    
    /*public static class ParseResult implements GeoCommon {
        private short direction;
        private DegreSexagesimal degreSexagesimal;
     
        ParseResult() {
     
        }
     
     
    }*/
    
    private static int toInt(char carac) {
        if (carac == ' ') return -2;
        if ((carac < '0') || (carac > '9')) {
            return -1;
        }
        return ((int) carac) - 48;
    }
    
}
