/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.primitives;

/**
 *
 * @author  Vincent Calame
 */
public final class Range implements Comparable {
    public static int MAXIMUM = 0x7FFFFFFF;
    public static int MINIMUM = 0xFFFFFFFF;
    int min,max;
    
    /**
     */
    public Range(int min, int max) {
        if (max < min) {
            this.max = min;
            this.min = max;
        }
        else {
            this.min = min;
            this.max = max;
        }
    }
    
    public int min() {
        return min;
    }

    public int max() {
        return max;
    }
    
    public int length() {
        return max - min + 1;
    }
    
    public Range headRange(int newMax) {
        if (newMax < min) return null;
        if (newMax >= max) return this;
        return new Range(min,newMax);
        
        
    }
    
    public Range headRange(Range Range) {
        return headRange(Range.max -1);
    }
    
    public Range tailRange(int newMin) {
        if (newMin > max) return null;
        if (newMin <= min) return this;
        return new Range(newMin,max);
    }
    
    public Range tailRange(Range Range) {
        return tailRange(Range.min + 1);
    }
    
    public boolean intersects(Range Range) {
        if (this.max < Range.min) return false;
        if (this.min > Range.max) return false;
        return true;
    }
    
    public boolean isNeighbour(Range range) {
        if (this.max == (range.min-1)) return true;
        if (this.min == (range.max+1)) return true;
        return false;
    }
    
    public Range intersection(Range range) {
        if (!intersects(range)) return null;
        return new Range(Math.max(this.min,range.min), Math.min(this.max,range.max));
    }
    

    public Range union(Range range) {
        return new Range(Math.min(min,range.min),Math.max(max,range.max));
    }
    
    public boolean contains(Range Range) {
        return ((this.min <= Range.min) && (this.max >= Range.max));
    }
    
    public boolean contains(int value) {
        return ((value >= min) && (value <= max));
    }
    
    public int hashCode() {
        return max;
    }
    
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if ((obj == null) || (!(obj instanceof Range))) return false;
        Range Range = (Range) obj;
        return (Range.max == this.max) && (Range.min == this.min);
    }
    
    public int compareTo(Object obj) {
       Range other = (Range) obj;
        if (this.min < other.min) return -1;
        if (this.min > other.min) return 1;
        if (this.max < other.max) return -1;
        if (this.max > other.max) return 1;
        return 0;
    }
    
}
