/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.primitives;

import java.text.ParseException;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class RangeUtils {
    
    private RangeUtils() {
    }
    
    public static boolean isPositiveRangeList(RangeList rangeList) {
        return (rangeList.getGlobalMin() >= 0);
    }
    
    /**
     *
     *@throws IllegalArgumentException si RangeList.getGlobalMin() < 0
     */
    public static String positiveRangetoString(RangeList rangeList) {
        if (rangeList.getGlobalMin() < 0) throw new IllegalArgumentException("not a positive range");
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < rangeList.size();i++) {
            if (i > 0) buf.append(';');
            append(buf, rangeList.getRange(i));
        }
        return buf.toString();
    }
    
    private static void append(StringBuffer buf,Range range) {
        int min = range.min();
        int max = range.max();
        if (min == max) {
            buf.append(min);
            return;
        }
        if (min != 0) buf.append(min);
        buf.append('-');
        if (max != Range.MAXIMUM) {
            buf.append(max);
        }
    }
    
    /**
     * Retourne <em>null</em> si c'est vide. sinon contient au moins un élément.
     *
     */
    public static RangeList positiveRangeParse(String s) {
        if ((s == null) || (s.length() == 0)) 	return  null;
        RangeList rangeList = new RangeList();
        StringTokenizer st = new StringTokenizer(s,";");
        while (st.hasMoreTokens()) {
            String token = st.nextToken().trim();
            Range range = parse(token);
            if (range != null) {
                rangeList.addRange(range);
            }
        }
        if (rangeList.size() == 0) return null;
        return rangeList;
    }
    
    private static Range parse(String token) {
        token = token.trim();
        if (token.length() == 0) return null;
        int idx = token.indexOf('-');
        try {
            if (idx == -1) {
                int value = Integer.parseInt(token);
                return new Range(value,value);
            }
            if (idx == 0) {
                int max = Integer.parseInt(token.substring(1).trim());
                return new Range(0,max);
            }
            if (idx == token.length()-1) {
                int min = Integer.parseInt(token.substring(0,token.length()-1).trim());
                return new Range(min,Range.MAXIMUM);
            }
            int min = Integer.parseInt(token.substring(0,idx).trim());
            int max = Integer.parseInt(token.substring(idx+1,token.length()).trim());
            return new Range(min,max);
        } catch(NumberFormatException nfe) {
            nfe.printStackTrace();
            return null;
        }
    }
    
    
    
}
