/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.sql;

import java.io.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class CreateTableWriter {
    public static final int PRIMARY_KEY = 1;
    private Writer writer;
    private boolean firstField = true;
    private ArrayList primaryKeyList = new ArrayList();
    
    public CreateTableWriter(Writer writer) {
        this.writer = writer;
    }
    
    public void startTable(String tableName) throws IOException{
        firstField = true;
        primaryKeyList.clear();
        writer.write("CREATE TABLE ");
        writer.write(tableName);
        writer.write(" (");
    }
    
    public void addIntegerField(String fieldName, int mask) throws IOException {
        testFirstField();
        writer.write(fieldName);
        writer.write(" INTEGER");
        if ((mask & PRIMARY_KEY) != 0) {
            primaryKeyList.add(fieldName);
        }
    }
    
    public void addDateField(String fieldName, int mask) throws IOException {
        testFirstField();
        writer.write(fieldName);
        writer.write(" DATE");
        if ((mask & PRIMARY_KEY) != 0) {
            primaryKeyList.add(fieldName);
        }
    }
    
    public void addDoubleField(String fieldName, int mask) throws IOException {
        testFirstField();
        writer.write(fieldName);
        writer.write(" DOUBLE");
        if ((mask & PRIMARY_KEY) != 0) {
            primaryKeyList.add(fieldName);
        }
    }
    
    public void addTextField(String fieldName, int mask) throws IOException {
        testFirstField();
        writer.write(fieldName);
        writer.write(" VARCHAR(255)");
        if ((mask & PRIMARY_KEY) != 0) {
            primaryKeyList.add(fieldName);
        }
    }
    
    public void endTable() throws IOException {
        int size = primaryKeyList.size();
        if (size > 0) {
            writer.write(", PRIMARY KEY (");
            for (int i = 0; i < size;i++) {
                if (i > 0) writer.write(", ");
                writer.write((String) primaryKeyList.get(i));
            }
            writer.write(")");
        }
         writer.write(")");
    }
    
    private void testFirstField() throws IOException {
        if (firstField) firstField = false;
        else writer.write(", ");
    }
    
}
