/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.sql;

import java.io.*;
import java.util.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.primitives.*;

/**
 *
 * @author Vincent Calame
 */
public class InsertWriter {
    private Writer writer;
    private boolean firstField = true;
    private int defaultInteger = 0;
    private String defaultString = "";
    
    public InsertWriter(Writer writer) {
        this.writer = writer;
    }
    
    public void setDefaultInteger(int defaultInteger) {
        this.defaultInteger = defaultInteger;
    }
    
    public void setDefaultString(String defaultString) {
        this.defaultString = defaultString;
    }
    
    public void startInsert(String tableName) throws IOException{
        writer.write("INSERT INTO ");
        writer.write(tableName);
        writer.write(" VALUES(");
        firstField = true;
    }
    
    public void endInsert() throws IOException {
        writer.write(")");
    }
    
    public void endLine() throws IOException {
        writer.write(";\n");
    }
    
    public void endInsertLine() throws IOException {
        endInsert();
        endLine();
    }
    
    public void addInteger(long lg) throws IOException {
        testFirstField();
        writer.write(String.valueOf(lg));
    }
    
    public void addDouble(double d) throws IOException {
        testFirstField();
        writer.write(String.valueOf(d));
    }
    
    public void addDecimal(Decimal decimal) throws IOException {
        testFirstField();
        writer.write(decimal.toString());
    }
    
    public void addLang(int langInteger) throws IOException {
        testFirstField();
        if (langInteger == LangInteger.UNDETERMINED) {
            writer.write("NULL");
        }
        else {
            writer.write('\'');
            LangInteger.writeString(writer,langInteger);
            writer.write('\'');
        }
    }
    
    public void addText(String text) throws IOException {
        testFirstField();
        if (text == null) writer.write("NULL");
        else {
            writer.write('\'');
            int length = text.length();
            for(int i = 0; i < length;i++) {
                char carac = text.charAt(i);
                if (carac == '\'') {
                    writer.write("''");
                }
                else {
                    writer.write(carac);
                }
            }
            writer.write('\'');
        }
    }
    
    public void addNull() throws IOException{
        testFirstField();
        writer.write("NULL");
    }
    
    public void addNull(int length) throws IOException{
        for(int i = 0; i < length;i++) {
            addNull();
        }
    }
    
    public void addDefaultInteger(int length) throws IOException {
        for(int i = 0; i < length;i++) {
            addInteger(defaultInteger);
        }
    }
    
    public void addDefaultString(int length) throws IOException {
        for(int i = 0; i < length;i++) {
            addText(defaultString);
        }
    }
    
    private void testFirstField() throws IOException {
        if (firstField) firstField = false;
        else writer.write(",");
    }
}
