/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.sxio;

import java.io.*;
import java.awt.Point;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.primitives.*;

/**
 *
 * @author  Vincent Calame
 */
public class SXUtils {
    
    /** Creates a new instance of SXUtils */
    private SXUtils() {
    }
    
    public static void writeArc(Writer writer,int x, int y, int width, int height, int startAngle,int arcAngle, String layerName, String styleName) throws IOException {
        writer.write("<draw:ellipse draw:kind='arc' draw:style-name='");
        writer.write(styleName);
        writer.write('\'');
        writer.write(" draw:layer='");
        writer.write(layerName);
        writer.write('\'');
        writer.write(" svg:width='");
        writer.write(String.valueOf(width));
        writer.write("pt'");
        writer.write(" svg:height='");
        writer.write(String.valueOf(height));
        writer.write("pt'");
        writer.write(" svg:x='");
        writer.write(String.valueOf(x));
        writer.write("pt'");
        writer.write(" svg:y='");
        writer.write(String.valueOf(y));
        writer.write("pt'");
        writer.write(" draw:start-angle='");
        writer.write(String.valueOf(startAngle));
        writer.write('\'');
        writer.write(" draw:end-angle='");
        writer.write(String.valueOf(arcAngle+startAngle));
        writer.write('\'');
        writer.write("/>");
    }
    
    public static void writeLine(Writer writer,Point A, Point B, String layerName, String styleName) throws IOException {
        writer.write("<draw:line draw:style-name='");
        writer.write(styleName);
        writer.write('\'');
        writer.write(" draw:layer='");
        writer.write(layerName);
        writer.write('\'');
        writer.write(" svg:x1='");
        writer.write(String.valueOf(A.x));
        writer.write("pt'");
        writer.write(" svg:y1='");
        writer.write(String.valueOf(A.y));
        writer.write("pt'");
        writer.write(" svg:x2='");
        writer.write(String.valueOf(B.x));
        writer.write("pt'");
        writer.write(" svg:y2='");
        writer.write(String.valueOf(B.y));
        writer.write("pt'");
        writer.write("/>");
    }
    
    public static void writeEmptyCell(Writer writer) throws IOException {
        startCell(writer);
        endCell(writer);
    }
    
    public static void writeIntCell(Writer writer,int value) throws IOException {
        writer.write("<table:table-cell table:value-type=\"float\" table:value=\"");
        writer.write(String.valueOf(value));
        writer.write("\">");
        writeP(writer,String.valueOf(value));
        endCell(writer);
    }
    
    public static void writeBooleanCell(Writer writer,boolean value) throws IOException {
        int val = (value)?1:0;
        writeIntCell(writer,val);
    }
    
    public static void writeStringCell(Writer writer, String s) throws IOException {
        startCell(writer);
        writeP(writer,s);
        endCell(writer);
    }
    
    public static void startCell(Writer writer) throws IOException {
        writer.write("<table:table-cell>");
    }
    
    public static void endCell(Writer writer) throws IOException {
        writer.write("</table:table-cell>");
    }
    
    public static void startP(Writer writer) throws IOException {
        writer.write("<text:p>");
    }
    
    public static void endP(Writer writer) throws IOException {
        writer.write("</text:p>");
    }
    
    public static void writeP(Writer writer, String s) throws IOException {
        writer.write("<text:p>");
        if ((s!= null)) {
            XMLUtils.escapeXML(s,writer);
        }
        writer.write("</text:p>");
    }
    
    public static void writeDate(Writer writer,int dateInteger)throws IOException {
        writeDate(writer,IntDate.toISOString(dateInteger));
    }
    
    public static void writeDate(Writer writer,String isoString)throws IOException {
        writer.write("<table:table-cell table:value-type=\"date\" table:style-name=\"c0\" table:date-value=\"");
        writer.write( isoString);
        writer.write("\">");
        writer.write( isoString);
        writer.write("</table:table-cell>");
    }
    
    public static void writeSpace(Writer writer,int number) throws IOException {
        writer.write("<text:s text:c=\"");
        writer.write(String.valueOf(number));
        writer.write("\"/>");
    }
    
    public static void startGroup(Writer writer) {
        
    }
    

    
}
