/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.sxio;

import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import org.xml.sax.SAXException;

/**
 *
 * @author  Vincent Calame
 */
public class TransformSXContentSource implements SXContentSource {
    String formatName;
    Transformer transformer;
    String encoding;
    Source source;
    
    /** Creates a new instance of TransformSXContentSource */
    public TransformSXContentSource(String formatName, Transformer transformer,Source source) {
        this.formatName = formatName;
        this.transformer = transformer;
        this.source = source;
        this.encoding = transformer.getOutputProperty("encoding");
        transformer.setOutputProperty("omit-xml-declaration","yes");
    }
    
    public String getEncoding() {
        return encoding;
    }
    
    public String getFormatName() {
        return formatName;
    }
    
    public boolean needDeclaration() {
        return true;
    }
    
    public boolean needRootElement() {
        return false;
    }
    
    public void writeContent(java.io.OutputStream outputStream) throws SXIOException {
        try {
            transformer.transform(source, new StreamResult(outputStream));
        }
        catch(TransformerException te) {
            throw new SXIOException(te);
        }
    }
    
}
