/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.templates;

import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class TemplateTransformer {
    static private final short TEXT = 0;
    static private final short VARIABLE_BEGIN = 1;
    static private final short VARIABLE_COURSE = 2;
    private TemplateSource templateSource;
    
    public TemplateTransformer(TemplateSource templateSource) {
        this.templateSource = templateSource;
    }
    
    public void transform(Reader reader,Writer writer) throws IOException {
        int character;
        ReadState readState = new ReadState(writer);
        while((character = reader.read()) != -1) {
            readState.read((char) character);
        }
        readState.end();
    }
    
    public String transform(String s) {
        StringReader reader = new StringReader(s);
        try {
            return transform(reader);
        } catch(IOException ioe) {
            // n'a jamais lieu !
        }
        return "";
    }
    
    public String transform(Reader reader) throws IOException{
        StringWriter writer = new StringWriter();
        transform(reader,writer);
        return writer.toString();
    }
    
    public String transform(InputStream inputStream, String encoding) throws IOException {
        BufferedReader bufreader = new BufferedReader(new InputStreamReader(inputStream,encoding));
        return transform(bufreader);
    }
    
    class ReadState {
        Writer writer;
        short state = TEXT;
        StringBuffer variableBuffer = new StringBuffer();
        
        ReadState(Writer writer) {
            this.writer = writer;
        }
        
        public void read(char c) throws IOException {
            switch(state) {
                case TEXT :
                    if (c == '$') state = VARIABLE_BEGIN;
                    else writer.write(c);
                    break;
                case VARIABLE_BEGIN :
                    if (c == '$') {
                        state = TEXT;
                        writer.write('$');
                    } else if (testVariableChar(c)) {
                        variableBuffer = new StringBuffer();
                        state = VARIABLE_COURSE;
                        variableBuffer.append(c);
                    } else {
                        state = TEXT;
                        writer.write('$');
                        writer.write(c);
                    }
                    break;
                case VARIABLE_COURSE :
                    if (testVariableChar(c)) {
                        variableBuffer.append(c);
                    }
                    else if (c == '$') {
                        replace();
                    } else {
                        
                        if (c == '$') state = VARIABLE_BEGIN;
                        else writer.write(c);
                    }
                    break;
            }
        }
        
        void flush() throws IOException {
            if (state != TEXT) {
                writer.write('$');
                if (variableBuffer != null) writer.write(variableBuffer.toString());
            }
            state = TEXT;
            variableBuffer = null;
        }
        
        void replace() throws IOException {
            String key = variableBuffer.toString();
            String text = templateSource.getText(key);
            if (text != null) writer.write(text);
            else {
                writer.write('$');
                writer.write(key);
                writer.write('$');
            }
            variableBuffer = null;
            state = TEXT;
        }
        
        void end() throws IOException {
            flush();
        }
        
        /*
         * Ne sont acceptés que les majuscules ASCII et '_'.
         */
        boolean testVariableChar(char c) {
            if (c == '_') return true;
            if (c < 'A') return false;
            if ((c < 'a') && (c > 'Z')) return false;
            if (c > 'z') return false;
            return true;
        }
    }
    
}
