/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text;

import java.text.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class DateFormatBundle {
    private static HashMap cacheMap = new HashMap();
    public final static String S1_KEy="s1";
    public final static String S2_KEy="s2";
    public final static String T1_KEy="t1";
    public final static String T2_KEy="t2";
    public final static String T3_KEy="t3";
    public final static String T4_KEy="t4";
    private DateFormat datejour;
    private DateFormat datemois;
    private DateFormat dateannee;
    private DateFormat parserdate;
    private ResourceBundle datationBundle;
    
    private DateFormatBundle(Locale locale) {
        parserdate = DateFormat.getDateInstance(DateFormat.SHORT,locale);
        datejour = DateFormat.getDateInstance(DateFormat.LONG,locale);
        datemois = new SimpleDateFormat("MMMM yyyy",locale);
        dateannee = new SimpleDateFormat("yyyy",locale);
        datationBundle = ResourceBundle.getBundle("net.mapeadores.util.text.resources.DateResourceBundle",locale);
    }
    
    public static DateFormatBundle getDateFormatBundle(Locale locale) {
        DateFormatBundle dateFormatBundle = (DateFormatBundle) cacheMap.get(locale);
        if (dateFormatBundle == null) {
            synchronized(cacheMap) {
                dateFormatBundle = new DateFormatBundle(locale);
                cacheMap.put(locale, dateFormatBundle);
            }
        }
        return dateFormatBundle;
    }
    
    public DateFormat getFormatAnnee() {
        return dateannee;
    }
    
    public DateFormat getFormatJour() {
        return datejour;
    }
    
    public DateFormat getFormatMois() {
        return datemois;
    }
    
    public DateFormat getParseFormat() {
        return parserdate;
    }
    
    public String getLocalizedString(String key) {
        return datationBundle.getString(key);
    }
    
}
