/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text;

import net.mapeadores.util.langinteger.*;
import java.io.Serializable;

/**
 * Implémentation de l'interface Libelle qui suit les spécifications Java Beans.
 *
 * @author Vincent Calame
 */
public final class LibelleBean implements Serializable {
    private boolean vo = true;
    private String lang="und";
    private String libelleString;
    
    public LibelleBean() {
    }
    
    public LibelleBean(Libelle libelle) {
        setLangInteger(libelle.getLangInteger());
        setVo(libelle.isVo());
        setLibelleString(libelle.getLibelleString());
    }
    
    public LibelleBean(int langInteger,String libelleString, boolean vo) {
        this.lang = LangInteger.toString(langInteger);
        this.libelleString = libelleString;
        this.vo = vo;
    }

    public String getLang() {
        return lang;
    }
    
    public void setLang(String lang) {
        if (lang == null) throw new IllegalArgumentException("lang cannot be null");
        this.lang = lang;
    }
    
    public void setLangInteger(int langInteger) {
        this.lang = LangInteger.toString(langInteger);
    }

    public String getLibelleString() {
        return libelleString;
    }
    
    public void setLibelleString(String libelleString) {
        this.libelleString = libelleString;
    }

    public boolean isVo() {
        return vo;
    }
    
    public void setVo(boolean vo) {
        this.vo = vo;
    }
    
    public  Libelle toLibelle() {
        return new InternalLibelle(this);
    }
    
    private static class InternalLibelle implements Libelle {
        private boolean vo;
        private int langInteger;
        private String libelleString;
        
        private InternalLibelle(LibelleBean libelleBean) {
            this.libelleString = libelleBean.libelleString;
            this.vo = libelleBean.vo;
            try {
                langInteger = LangInteger.parse(libelleBean.lang);
            }
            catch(LangIntegerException lie) {
                langInteger = LangInteger.UNDETERMINED;
            }
        }
        
        public String getLibelleString() {
            return libelleString;
        }
        
        public boolean isVo() {
            return vo;
        }
        
        public int getLangInteger() {
            return langInteger;
        }
    }
    
}
