/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text;

import java.util.*;
import net.mapeadores.util.langinteger.LangInteger;

/**
 * Classe permettant le stockage de listes de libellés pour des clés quelconques.
 * Cette classe est utile pour conserver des données de localisation (souvent initialisées une fois pour toute).
 *Cette classe ne propose d'ailleurs que des fonctions d'ajout.
 *Cela permet d'avoir des bibliothèques de libellés.
 *
 * @author Vincent Calame
 */
public class LibelleListMap {
    private HashMap libelleListHashMap = new HashMap();
    
    
    public List getLibelleList(Object key) {
        return (List) libelleListHashMap.get(key);
    }
    
    public Libelle getLibelle(Object key,int langInteger) {
        LibList libList = (LibList) libelleListHashMap.get(key);
        if (libList == null) return null;
        return libList.getLibelle(langInteger);
    }
    
    public void setLibelle(Object key,int langInteger,String libelle) {
        if (libelle == null) throw new IllegalArgumentException("libelle argument is null");
        LibList libList = (LibList) libelleListHashMap.get(key);
        if (libList == null) {
            libList = new LibList(langInteger,libelle);
            libelleListHashMap.put(key,libList);
        }
        else {
            libList.add(langInteger,libelle);
        }
   }
    
    private class UnmodifiableLib implements Libelle {
        int langInteger;
        String libelle;
        
        UnmodifiableLib(int langInteger,String libelle) {
            this.langInteger = langInteger;
            this.libelle = libelle;
        }
        
        
        public int getLangInteger() {
            return langInteger;
        }
        
        public String getLibelleString() {
            return libelle;
        }
        
        public boolean isVo() {
            return true;
        }
        
    }
    
    private class LibList extends AbstractList implements RandomAccess {
        UnmodifiableLib[] tab;
        
        LibList(int langInteger,String libelle) {
            tab = new UnmodifiableLib[1];
            tab[0] = new UnmodifiableLib(langInteger,libelle);
        }
        
        void add(int langInteger,String lib) {
            int length = tab.length;
            for (int i = 0; i < length;i++) {
                if (tab[i].langInteger == langInteger) {
                    tab[i].libelle = lib;
                    return;
                }
            }
            UnmodifiableLib[] nvtab = new UnmodifiableLib[length+1];
            System.arraycopy(tab,0,nvtab, 0,length);
            nvtab[length] = new UnmodifiableLib(langInteger,lib);
            tab = nvtab;
        }
        
        public Object get(int index) {
            return tab[index];
        }
        
        public int size() {
            return tab.length;
        }
        
        public Libelle getLibelle(int langInteger) {
            int length = tab.length;
            for (int i = 0; i < length;i++) {
                if (tab[i].langInteger == langInteger) {
                    return tab[i];
                }
            }
            return null;
        }
    }
}
