/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text;

import java.util.*;
import java.text.*;

/**
 *
 * @author Vincent Calame
 */
public class Pattern {
    public final static NonePatternValueSet DEFAULT_PATTERNVALUESET = new NonePatternValueSet();
    public final static DefaultPatternArgumentFactory DEFAULT_PATTERNARGUMENTFACTORY = new DefaultPatternArgumentFactory();
    private String patternString;
    /*HashSet argumentSet = new HashSet();
    Set unmodifiableSet = Collections.unmodifiableSet(argumentSet);*/
    PatternPart[] patternPartArray;
    int bufferSize = 0;
    int argumentCount = 0;
    
    private Pattern() {
        
    }
    
    public Pattern(String patternString) throws ParseException{
        this(patternString,null);
    }
    
    public Pattern(String patternString, PatternArgumentFactory patternArgumentFactory) throws ParseException{
        this.patternString = patternString;
        if (patternArgumentFactory == null) patternArgumentFactory = DEFAULT_PATTERNARGUMENTFACTORY;
        List list = buildPattern(patternString,patternArgumentFactory);
        int size = list.size();
        patternPartArray = new PatternPart[size];
        for(int i = 0; i < size;i++) {
            patternPartArray[i] = (PatternPart) list.get(i);
        }
        if (bufferSize < 8) bufferSize = 16;
        else bufferSize = bufferSize + (10 * argumentCount);
    }
    
    public static Pattern getNoArgumentPattern(String prefix,String rawString) {
        String text = prefix + rawString;
        Pattern pattern = new Pattern();
        PatternPart[] pp = new PatternPart[1];
        pp[0] = new PatternPart(text);
        pattern.patternPartArray = pp;
        pattern.patternString = rawString;
        pattern.bufferSize = text.length();
        return pattern;
    }
    
    public int getArgumentCount() {
        return argumentCount;
    }
    
    /*public boolean hasExtension() {
        PatternPart patternPart = patternPartArray[patternPartArray.length - 1];
        if (patternPart.isArgumentPart()) return false;
        String text = patternPart.getText();
        int idx = text.lastIndexOf('.');
        if (idx == -1) return false;
        if (idx == (text.length() -1)) return false;
        return true;
    }*/
    
    public String format(PatternValueSet patternValueSet) {
        StringBuffer buf = new StringBuffer(bufferSize);
        for (int i = 0; i < patternPartArray.length;i++) {
            PatternPart patternPart = patternPartArray[i];
            if (patternPart.isArgumentPart()) {
                buf.append(patternValueSet.getValue(patternPart.getPatternArgument()));
            } else {
                buf.append(patternPart.getText());
            }
        }
        return buf.toString();
    }
    
    private List buildPattern(String strg_pattern, PatternArgumentFactory patternArgumentFactory) throws ParseException {
        boolean intoBrace = false;
        ArrayList partsList = new ArrayList();
        StringBuffer buf = new StringBuffer();
        int formatNumber = 0;
        boolean inQuote = false;
        int braceStack = 0;
        for (int i = 0; i < strg_pattern.length(); ++i) {
            char ch = strg_pattern.charAt(i);
            if (inQuote) {
                if (ch == '\'') {
                    if (i + 1 < strg_pattern.length() && strg_pattern.charAt(i+1) == '\'') {
                        buf.append(ch);
                        ++i;
                    } else {
                        inQuote = false;
                    }
                } else {
                    buf.append(ch);
                }
            } else if (!intoBrace) {
                if (ch == '\'') {
                    if (i + 1 < strg_pattern.length() && strg_pattern.charAt(i+1) == '\'') {
                        buf.append(ch);
                        ++i;
                    } else {
                        inQuote = true;
                    }
                } else if (ch == '{') {
                    intoBrace = true;
                    buf = flushTextPatternPartBuffer(partsList,buf);
                } else if (ch == '}') {
                    throw new ParseException("Unmatched braces in the pattern at position",i);
                } else {
                    buf.append(ch);
                }
            } else {
                switch (ch) {
                    case '{':
                        throw new ParseException("Unmatched braces in the pattern at position",i);
                    case '}':
                        intoBrace = false;
                        buf = flushArgumentPatternPartBuffer(partsList,buf,patternArgumentFactory);
                        break;
                    default:
                        buf.append(ch);
                        break;
                }
            }
        }
        if (intoBrace) {
            throw new ParseException("Unmatched braces in the pattern (brace not closed).", strg_pattern.length()-1);
        }
        flushTextPatternPartBuffer(partsList,buf);
        return partsList;
    }
    
    
    private StringBuffer flushArgumentPatternPartBuffer(List list,StringBuffer buf, PatternArgumentFactory patternArgumentFactory) {
        String name = buf.toString().trim();
        String mode = null;
        int idx = name.indexOf('!');
        if (idx != -1) {
            mode = name.substring(idx +1).trim();
            if (mode.length() == 0) mode = null;
            name = name.substring(0,idx).trim();
        }
        if (name.length() > 0) {
            PatternArgument patternArgument = patternArgumentFactory.getPatternArgument(name,mode);
            if (patternArgument != null) {
                list.add(newArgumentPatternPart(patternArgument));
            } else {
                list.add(new PatternPart(toString(name,mode)));
            }
        } else {
            list.add(new PatternPart(toString(name,mode)));
        }
        return new StringBuffer();
    }
    
    private StringBuffer flushTextPatternPartBuffer(List list,StringBuffer buf) {
        String strg = buf.toString();
        bufferSize = bufferSize + strg.length();
        if (strg.length() > 0) list.add(new PatternPart(strg));
        return new StringBuffer();
    }
    
    /**
     * Retourne la chaîne qui a servit à construire l'instance de Pattern.
     */
    public String toString() {
        return patternString;
    }
    
    public static String toString(String name, String mode) {
        StringBuffer buf = new StringBuffer();
        buf.append('{');
        buf.append(name);
        if (mode != null) {
            buf.append('!');
            buf.append(mode);
        }
        buf.append('}');
        return buf.toString();
    }
    
    public static String toString(PatternArgument patternArgument) {
        return toString(patternArgument.getName(),patternArgument.getMode());
    }
    
    private PatternPart newArgumentPatternPart(PatternArgument patternArgument) {
        argumentCount ++;
        return new PatternPart(patternArgument);
    }
    
    static class PatternPart {
        PatternArgument patternArgument;
        String text;
        
        PatternPart(String text) {
            this.text = text;
        }
        
        PatternPart(PatternArgument patternArgument) {
            this.patternArgument = patternArgument;
        }
        
        public boolean isArgumentPart() {
            return (patternArgument != null);
        }
        
        public PatternArgument getPatternArgument() {
            return patternArgument;
        }
        
        public String getText() {
            return text;
        }
        
    }
    
    
    private static class NonePatternValueSet implements PatternValueSet {
        
        public String getValue(PatternArgument patternArgument) {
            return Pattern.toString(patternArgument);
        }
        
    }
    
    private static class DefaultPatternArgumentFactory implements PatternArgumentFactory {
        
        public PatternArgument getPatternArgument(String name, String mode) {
            return new DefaultPatternArgument(name,mode);
        }
        
    }
    
    private static class DefaultPatternArgument implements PatternArgument {
        private String name;
        private String mode;
        
        DefaultPatternArgument(String name,String mode) {
            this.name = name;
            this.mode = mode;
        }
        
        public String getName() {
            return name;
        }
        
        public String getMode() {
            return mode;
        }
    }
}
