/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text;

/**
 *
 * @author Vincent Calame
 */
public class SearchSequenceUtils {
    
    private SearchSequenceUtils() {
    }
    
    public static String toSQLCondition(String fieldName, SearchSequence searchSequence, short combinaisonType) {
        int  tokenCount = searchSequence.getTokenCount();
        if (tokenCount == 0) return "";
        StringBuffer buf = new StringBuffer();
        buf.append('(');
        for (int i = 0; i < tokenCount;i++) {
            if (i > 0)  {
                if (combinaisonType == TextConstants.AND) {
                    buf.append(" AND ");
                }
                else if (combinaisonType == TextConstants.OR) {
                    buf.append(" OR ");
                }
            }
            SearchSequence.Token token = searchSequence.getToken(i);
            if (token.isMultiToken()) {
                SearchSequence.SimpleToken[] array = ((SearchSequence.MultiToken) token).getSubtokenArray();
                for(int j = 0; j < array.length;j++) {
                    if (j > 0)  {
                        if (combinaisonType == TextConstants.AND) {
                            buf.append(" AND ");
                        }
                        else if (combinaisonType == TextConstants.OR) {
                            buf.append(" OR ");
                        }
                    }
                    SearchSequence.SimpleToken simpleToken = array[j];
                    appendCondition(buf,fieldName,simpleToken.toString(false),simpleToken.getSearchType());
                }
            }
            else {
                SearchSequence.SimpleToken simpleToken = (SearchSequence.SimpleToken) token;
                appendCondition(buf,fieldName,simpleToken.toString(false),simpleToken.getSearchType());
            }
        }
        buf.append(')');
        return buf.toString();
    }
    
    private static void appendCondition(StringBuffer buf,String fieldName,String token,int type) {
        buf.append('(');
        buf.append(fieldName);
        switch(type) {
            case TextConstants.MATCHES :
                buf.append(" LIKE '");
                appendString(buf,token);
                buf.append("'");
                break;
            case TextConstants.CONTAINS :
                buf.append(" LIKE '%");
                appendString(buf,token);
                buf.append("%'");
                break;
            case TextConstants.STARTSWITH :
                buf.append(" LIKE '");
                appendString(buf,token);
                buf.append("%'");
                break;
            case TextConstants.ENDSWITH :
                buf.append(" LIKE '%");
                appendString(buf,token);
                buf.append("'");
                break;
        }
        buf.append(')');
    }
    
    private static void appendString(StringBuffer buf,String token) {
        int length = token.length();
        for (int i = 0; i < length;i++) {
            char carac = token.charAt(i);
            if (carac == '\'') buf.append("''");
            else buf.append(carac);
        }
    }
    
    /*public String toConditionSQL() {
        StringBuffer buf = new StringBuffer();
        switch(typeCondition) {
            case EST_VIDE :
                buf.append(" IS NULL");
                break;
            case EST_NON_VIDE :
                buf.append(" IS NOT NULL");
                break;
            case COMMENCE_PAR :
                buf.append(" LIKE '");
                preparation(buf);
                buf.append("%'");
                break;
            case FINIT_PAR :
                buf.append(" LIKE '%");
                preparation(buf);
                buf.append("'");
                break;
            case CONTIENT:
                buf.append(" LIKE '%");
                preparation(buf);
                buf.append("%'");
                break;
            case CORRESPOND_A:
     
                break;
            case NE_COMMENCE_PAS_PAR:
                buf.append(" NOT LIKE '%");
                preparation(buf);
                buf.append("%'");
                break;
            case NE_FINIT_PAS_PAR:
                buf.append(" NOT LIKE '%");
                preparation(buf);
                buf.append("'");
                break;
            case NE_CONTIENT_PAS :
                buf.append(" NOT LIKE '%");
                preparation(buf);
                buf.append("%'");
                break;
            case NE_CORRESPOND_PAS_A:
                buf.append(" NOT LIKE '");
                preparation(buf);
                buf.append("'");
                break;
        }
        return buf.toString();
    }*/
    
    
}
