/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008-2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.alphabet;

import net.mapeadores.util.collections.*;
import net.mapeadores.util.langinteger.*;
import java.io.*;
import net.mapeadores.util.exceptions.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class Alphabet {
    static final IntHashMap articleArrayMap = new IntHashMap();
    private String sourceString;
    private String articleString;
    private String alphabetString;
    private boolean withEspace;
    
    public Alphabet() {
    }
    
    private Alphabet(String sourceString) {
        setSourceString(sourceString);
        setAlphabetString(sourceString);
        
    }
    
    public static Alphabet newInstance(String sourceString, int langInteger) {
        Article[] array = getArray(langInteger);
        if (array == null) return new Alphabet(sourceString);
        Alphabet alphabet = new Alphabet();
        alphabet.setSourceString(sourceString);
        String articleString = null;
        boolean withEspace = false;
        String alphabetString = sourceString;
        for(int i = 0; i < array.length; i++) {
            Article currentArticle = array[i];
            if (sourceString.startsWith(currentArticle.getArticleString())) {
                int length = currentArticle.getArticleLength();
                if (sourceString.length() > length) {
                    articleString = currentArticle.getTrimedArticleString();
                    withEspace = currentArticle.isWithEspace();
                    alphabetString = sourceString.substring(length);
                }
                break;
            }
        }
        alphabet.setAlphabetString(alphabetString);
        alphabet.setArticleString(articleString);
        alphabet.setWithEspace(withEspace);
        return alphabet;
    }
    
    
    
    private static Article[] getArray(int langInteger) {
        Object obj =  articleArrayMap.get(langInteger);
        if (obj == null) {
            obj = initArray(langInteger);
            if (obj == null) {
                int langPart = LangInteger.getLangPart(langInteger);
                if (langPart != langInteger) {
                    obj = getArray(langPart);
                }
            }
            if (obj == null) obj = Boolean.FALSE;
            articleArrayMap.put(langInteger, obj);
        }
        if (obj instanceof Boolean) return  null;
        else return (Article[]) obj;
    }
    
    /*
     * Les articles doivent commencer par les plus longs (voir la boucle de newInstance)
     */
    private static Article[] initArray(int langInteger) {
        InputStream inputStream = Alphabet.class.getResourceAsStream("resources/article_" + LangInteger.toString(langInteger) + ".txt");
        if (inputStream == null) return null;
        try {
            List list = new ArrayList();
            BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream,"UTF-8"));
            String ligne = null;
            while((ligne = reader.readLine()) != null) {
                ligne = ligne.trim();
                if ((ligne.length() > 0) && (ligne.charAt(0) != '#')) {
                    ligne = ligne.replace('_',' ');
                    list.add(ligne);
                }
            }
            reader.close();
            int size = list.size();
            Article[] result = new Article[size];
            for(int i = 0; i < size; i++) {
                result[i] = new Article((String) list.get(i));
            }
            return result;
        }
        catch(IOException ioe) {
            throw new InternalResourceException(ioe);
        }
    }

    public boolean isWithArticle() {
        return (sourceString != getAlphabetString());
    }
    
    public String getSourceString() {
        return sourceString;
    }

    public void setSourceString(String sourceString) {
        this.sourceString = sourceString;
    }

    public String getArticleString() {
        return articleString;
    }

    public void setArticleString(String articleString) {
        this.articleString = articleString;
    }

    public String getAlphabetString() {
        return alphabetString;
    }

    public void setAlphabetString(String alphabetString) {
        this.alphabetString = alphabetString;
    }
    
    public boolean isWithEspace() {
        return withEspace;
    }

    public void setWithEspace(boolean withEspace) {
        this.withEspace = withEspace;
    }
    
    private static class Article {
        private String articleString;
        private boolean withEspace;
        private String trimedArticleString;
        
        Article(String articleString) {
            this.articleString = articleString;
            if (articleString.charAt(articleString.length() - 1) == ' ') {
                trimedArticleString = articleString.substring(0,articleString.length() - 1);
                withEspace = true;
            }
            else {
                withEspace = false;
                trimedArticleString = articleString;
            }
        }

        public String getArticleString() {
            return articleString;
        }

        public boolean isWithEspace() {
            return withEspace;
        }

        public String getTrimedArticleString() {
            return trimedArticleString;
        }
        
        public int getArticleLength() {
            return articleString.length();
        }
        
    }

}
