/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.collation.group;

import java.util.*;
import java.text.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.collation.CollationUnit;

/**
 *
 * @author Vincent Calame
 */
public class CollationGroupBuilder {
    private RuleBasedCollator collator;
    private Locale locale;
    private List groupList = new ArrayList();
    private Map groupMap = new HashMap();
    
    public CollationGroupBuilder() {
        this.locale = Locale.getDefault();
        init();
    }
    
    public CollationGroupBuilder(Locale locale) {
        this.locale = locale;
        init();
    }
    
    public CollationGroupBuilder(int langInteger) {
        this.locale = LangInteger.getLocale(langInteger);
        init();
    }
    
    private void init() {
        collator = (RuleBasedCollator) Collator.getInstance(locale);
        collator.setStrength(Collator.PRIMARY);
        for(char c = '0'; c <= '9';c++) {
            addInitiale(c);
        }
        groupList.add(new DefaultCollationGroup(CollationGroup.NOLETTER_INITIAL));
        for(char c = 'A'; c <= 'Z';c++) {
            addInitiale(c);
        }
        groupList.add(new DefaultCollationGroup(CollationGroup.NOEUROPEAN_INITIAL));
    }
    
    private void addInitiale(char c) {
        DefaultCollationGroup collationGroup = new DefaultCollationGroup(c);
        char[] array = {c};
        String collatedKey = CollationUnit.collate(new String(array),collator);
        if (collatedKey.length() > 1) throw new IllegalStateException("should not occur : initiale = " + c);
        groupMap.put(new Character(collatedKey.charAt(0)),collationGroup);
        groupList.add(collationGroup);
    }
    
    public void addObjectWrapper(ObjectWrapper objectWrapper) {
        String string = objectWrapper.getString();
        if (string.length() == 0) {
            ((DefaultCollationGroup) groupList.get(10)).addObject(CollationUnit.collate(objectWrapper.getSourceString(),collator),objectWrapper);
        }
        else {
            String collatedKey = CollationUnit.collate(objectWrapper.getString(),collator);
            Character initial = new Character(collatedKey.charAt(0));
            DefaultCollationGroup collationGroup = (DefaultCollationGroup) groupMap.get(initial);
            if (collationGroup == null) {
                collationGroup = (DefaultCollationGroup) groupList.get(37);
            }
            collationGroup.addObject(collatedKey,objectWrapper);
        }
    }
    
    /**
     * implémente RandomAccess
     */
    public List getCollationGroupList() {
        List result = new ArrayList();
        int size = groupList.size();
        for (int i = 0; i < groupList.size();i++) {
            DefaultCollationGroup collationGroup = (DefaultCollationGroup) groupList.get(i);
            if (!collationGroup.isEmpty()) {
                result.add(collationGroup);
            }
        }
        return result;
    }
}
