/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.collation.group;

import java.util.*;

/**
 *
 * @author Vincent Calame
 */
class DefaultCollationGroup implements CollationGroup {
    private char initial;
    private TreeMap sortedMap =new TreeMap();
    
    DefaultCollationGroup(char initial) {
        this.initial = initial;
    }
    
    public char getInitial() {
        return initial;
    }

    
    public List getSortedObjectWrapperList() {
        ArrayList result = new ArrayList();
        for(Iterator it= sortedMap.values().iterator();it.hasNext();) {
            Object obj = it.next();
            if (obj instanceof List) {
                result.addAll((List)obj);
            }
            else {
                result.add(obj);
            }
        }
        return result;
    }
    
    boolean isEmpty() {
        return sortedMap.isEmpty();
    }
    
    void addObject(String collationKey, ObjectWrapper objectWrapper) {
        Object obj = sortedMap.get(collationKey);
        if (obj == null) {
            sortedMap.put(collationKey,objectWrapper);
        }
        else if (obj instanceof List) {
            addInList(((List) obj),objectWrapper);
        }
        else {
            ArrayList arrayList = new ArrayList();
            arrayList.add(obj);
            addInList(arrayList,objectWrapper);
            sortedMap.put(collationKey,arrayList);
        }
    }
    
    
    private void addInList(List list, ObjectWrapper objectWrapper) {
        int size = list.size();
        String sourceString = objectWrapper.getSourceString();
        boolean done = false;
        for(int i = 0; i < size; i++) {
            ObjectWrapper obj2 = (ObjectWrapper) list.get(i);
            if (obj2.getSourceString().compareTo(sourceString) > 0) {
                list.add(i,objectWrapper);
                done = true;
                break;
            }
        }
        if (!done) list.add(objectWrapper);
    }
    
    
}
