/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.collation.map;

import java.util.*;
import net.mapeadores.util.text.collation.*;
import java.text.*;

/**
 *
 * @author Vincent Calame
 */
public class UnmodifiableCollatedKeyMap implements CollatedKeyMap {
    private CollatedKeyMap collatedKeyMap;
    
    public UnmodifiableCollatedKeyMap(CollatedKeyMap collatedKeyMap) {
        this.collatedKeyMap = collatedKeyMap;
    }
    
    public void clear() {
        throw new UnsupportedOperationException();
    }

    /**
     * Crèe un objet CollationUnit avec comme source s et en se basant sur le Collator de l'instance de CollatedKeyHashMap
     * Équivalent de new CollationUnit(s,getCollator().
     */
    public CollationUnit collateString(String s) {
        return collatedKeyMap.collateString(s);
    }

    /**
     * Retourne l'instance de Collator utilisée pour la création des « collatedKey » et la recherche.
     */
    public Collator getCollator() {
        return collatedKeyMap.getCollator();
    }

    /**
     * Retourne l'objet Locale qui a servi à la construction de l'objet.
     */
    public Locale getLocale() {
        return collatedKeyMap.getLocale();
    }

    /**
     * Effectue une recherche sur les clés de l'instance de CollatedKeyHashMap et retourne le résultat sous la forme d'une instance de ResultEntryList.
     * Cette méthode vérifie la présence de searchText dans les clés suivant trois conditions : TextConstants.CONTAINS, TextConstants.ENDSWITH, TextConstants.STARTSWITH
     * searchText subit évidemment une « collation » pour que les conditions répondent aux règles du Collator.
     */
   public  List search(String searchText, int type) {
        return collatedKeyMap.search(searchText, type);
    }

    /**
     * Retourne la valeur stockée à la clé « s ». La chaîne s subit d'abord une « collation ».
     * Cette méthode est équivalente à getValueByCollatedKey(toCollateKey(key))
     */
    public Object getValue(String key) {
        return collatedKeyMap.getValue(key);
    }

    /**
     * Retourne la valeur stockée à la clé déjà « collatée », collatedKey.
     */
    public Object getValueByCollatedKey(String collatedKey) {
        return collatedKeyMap.getValueByCollatedKey(collatedKey);
    }


    public void putValue(String key, Object value) {
        throw new UnsupportedOperationException();
    }

    public void putValueByCollatedKey(String collatedKey, Object value){
        throw new UnsupportedOperationException();
    }


    public Object removeValue(String key) {
        throw new UnsupportedOperationException();
    }

    public Object removeValueByCollatedKey(String collatedKey) {
        throw new UnsupportedOperationException();
    }

    /**
     * Retourne le nombre de valeurs stockées dans l'instance de CollatedKeyHashMap.
     */
    public int size() {
        return collatedKeyMap.size();
    }

    /**
     * Retourne la forme « collatée » d'une clé.
     */
    public String toCollatedKey(String key) {
        return collatedKeyMap.toCollatedKey(key);
    }
    
    public Collection values() {
         return Collections.unmodifiableCollection(collatedKeyMap.values());
     }
    
}
