/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.insertpattern.api;

import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class InsertionBuffer implements InsertionHandler, InsertionUnitType {
    private IfStack ifStack = new IfStack();
    private List insertionUnitList = new ArrayList();
    
    public InsertionBuffer() {
    }
    
    public void appendString(String s) {
        insertionUnitList.add(new InsertionUnit(insertionUnitList.size(),s));
    }

    public void doInsertion(Instruction[] instructionArray) {
        insertionUnitList.add(new InsertionUnit(insertionUnitList.size(),instructionArray));
    }
    
    public void startIf(String clause, boolean unique) {
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(),START_IF);
        insertionUnit.setUnique(true);
        insertionUnit.setClause(clause);
        insertionUnitList.add(insertionUnit);
        ifStack.increase(insertionUnit);
    }
    
    public void startElseIf(String clause) {
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(),START_ELSEIF);
        insertionUnit.setClause(clause);
        insertionUnitList.add(insertionUnit);
        ifStack.checkNotUnique();
    }
    
    public void startElse() {
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(),START_ELSE);
        insertionUnitList.add(insertionUnit);
        ifStack.checkNotUnique();
    }
    
    public void endIf(boolean unique, boolean onElse) {
        boolean isUnique = ifStack.decrease();
        InsertionUnit insertionUnit = new InsertionUnit(insertionUnitList.size(),END_IF);
        insertionUnit.setUnique(isUnique);
        insertionUnit.setOnElse(onElse);
        insertionUnitList.add(insertionUnit);
    }
    
    public InsertionUnit getLastInsertionUnit() {
        return (InsertionUnit) insertionUnitList.get(insertionUnitList.size() - 1);
    }
    
    public List getInsertionUnitList() {
        return insertionUnitList;
    }
    
    private class IfStack {
        Stack stack = new Stack();
        
        boolean decrease() {
            InsertionUnit insertionUnit = (InsertionUnit) stack.pop();
            return insertionUnit.isUnique();
         }
        
        void increase(InsertionUnit insertionUnit) {
            stack.add(insertionUnit);
        }
        
        void checkNotUnique() {
            InsertionUnit insertionUnit = (InsertionUnit) stack.peek();
            insertionUnit.setUnique(false);
        }
    }
}
