/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.insertpattern.impl;

import net.mapeadores.util.text.insertpattern.api.*;
import net.mapeadores.util.text.StringUtils;

import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class CheckHandler implements ErrorHandler, InsertionHandler {
    final static String INC_INSTRUCTION = "\n   ";
    final static String INC_ARGUMENT = "\n      ";
    final static String INC_KEY = "            ";
    final static String INC_VALUE = "            ";
    private Writer writer;
    private int ifCount =0;
    private String ifInc = "";
    final static String INC_IF = "   ";
    
    public CheckHandler(OutputStream outputStream) throws IOException {
        writer = new OutputStreamWriter(outputStream,"UTF-8");
    }

    public void characterError(char carac,String key,int row, int col) {
        try {
            writer.write("\n%%%%% ParseError : ");
            writer.write(key);
            writer.write(" ");
            writer.write(StringUtils.charToString(carac));
            writer.write(" at ");
            writer.write(String.valueOf(row));
            writer.write(":");
            writer.write(String.valueOf(col));
            writer.write(" %%%%%\n");
        }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }
    }
    
    public void warning(String key, int row, int col) {
         try {
            writer.write("\n%%%%% Warning : ");
            writer.write(key);
            writer.write(" at ");
            writer.write(String.valueOf(row));
            writer.write(":");
            writer.write(String.valueOf(col));
            writer.write(" %%%%%\n");
        }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }
     }
    
    public void appendString(String litteralString) {
        try {
            //writer.write("\n------ LiteralString:\n");
            writer.write(litteralString);
        }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }
    }
    
    public void close() throws IOException{
        writer.close();
    }
    
    public void flush() throws IOException{
        writer.flush();
    }
    
    public void doInsertion(Instruction[] instructionArray) {
        try {
            int length = instructionArray.length;
                writer.write("\n------ Insertion :\n");
                for(int i = 0; i < length;i++) {
                    Instruction arg = instructionArray[i];
                    writer.write(INC_INSTRUCTION);
                    writer.write("---- Instruction ");
                    writer.write(String.valueOf(i+1));
                    writer.write(" ----\n");
                    int ln = arg.getLength();
                    for(int j =  0; j < ln;j++) {
                        writer.write(INC_ARGUMENT);
                        writer.write("--  argument ");
                        writer.write(String.valueOf(j+1));
                        writer.write("  --\n");
                        Argument argEn = arg.getArgument(j);
                        writer.write(INC_KEY);
                        writer.write("key = ");
                        writer.write(argEn.getKey());
                        writer.write("\n");
                        String value = argEn.getValue();
                        if (value != null) {
                            writer.write(INC_VALUE);
                            writer.write("value = ");
                            writer.write(value);
                            writer.write("\n");
                        }
                    }
                }
            writer.write("\n------\n");
        }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }
    }
    
    public void startIf(String clause, boolean unique) {
        increaseInc();
        try {
            writer.write("\n\n");
            writer.write(ifInc);
            writer.write("***** IF");
            if (unique) writer.write("F");
            writer.write("  ");
            writer.write(clause);
            writer.write("  :\n");
         }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }
            
    }

    private void increaseInc() {
        StringBuffer buf = new StringBuffer();
        for(int i = 0; i < ifCount;i++) {
            buf.append(INC_IF);
        }
        ifInc = buf.toString();
        ifCount++;
    }
    
    private void decreaseInc() {
        ifCount--;
        StringBuffer buf = new StringBuffer();
        for(int i = 0; i < ifCount;i++) {
            buf.append(INC_IF);
        }
        ifInc = buf.toString();
    }
    
    public void startElseIf(String clause) {
        try {
             writer.write("\n\n");
             writer.write(ifInc);
            writer.write("***** ELSEIF  ");
            writer.write(clause);
            writer.write("  :\n");
         }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }

    }

    public void startElse() {
        try {
            writer.write("\n\n");
             writer.write(ifInc);
            writer.write("***** ELSE  :\n");
         }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }
    }

     public void endIf(boolean unique, boolean onElse) {
        decreaseInc();
        try {
            writer.write("\n\n");
             writer.write(ifInc);
            writer.write("***** FI  *****\n");
         }
        catch(IOException ioe) {
            ioe.printStackTrace();
        }
     }
     
     
    
    
}
