/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.lexie;

import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieParser {
    LexieParserListener listener;
    int wordStart = -1;
    int bufLength = 0;
    char[] bufArray;
    
    private LexieParser(LexieParserListener listener) {
        this.listener = listener;
    }
    
    public static void parse(String chaine,LexieParserListener listener) {
        if (chaine == null) return;
        LexieParser lexieParser = new LexieParser(listener);
        lexieParser.parse(chaine);
     }
    
     private void parse(String chaine) {
        int taille = chaine.length();
        bufArray = new char[taille];
        for (int i = 0; i < taille; i++) {
                char carac = chaine.charAt(i);
                if (Character.isLetterOrDigit(carac)) {
                    if (StringUtils.isIdeogram(carac)) {
                        flushWord();
                        wordStart = i;
                        bufLength = 1;
                        bufArray[0] = carac;
                        flushWord();
                    }
                    else {
                        bufArray[bufLength] = carac;
                        bufLength++;
                        if (wordStart == -1) wordStart = i;
                    }
                } else {
                    flushWord();
                    listener.checkSpecialChar(carac);
                }
            }
            flushWord();
    }
     
     private void flushWord() {
         if (wordStart > -1) {
             String word = new String(bufArray,0,bufLength);
             listener.flushLexie(word,wordStart);
             bufLength = 0;
             wordStart = -1;
         }
     }
}
