/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.lexie.xml;

import net.mapeadores.util.xml.*;
import java.io.*;
import net.mapeadores.util.text.*;
import net.mapeadores.util.text.lexie.*;
import net.mapeadores.util.text.collation.map.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class LexieMapXMLWriter extends DefaultXMLWriter{
    
    public LexieMapXMLWriter(XMLDestination xmlDestination,int indentLength) {
        super();
        setXmlDestination(xmlDestination);
        setIndentLength(indentLength);
    }
    
    public void appendLexieMap(LexieMap lexieMap) throws IOException {
        openTag("lexiemap");
        openTag("lexies");
        for (Iterator it = lexieMap.getCollatedKeyMap().values().iterator(); it.hasNext();) {
            TextLexieUnit lexieUnit = (TextLexieUnit) it.next();
            appendLexieUnit(lexieUnit);
        }
        closeTag("lexies");
        openTag("sources");
        for(Iterator it = lexieMap.getParsedLexieSourceMap().values().iterator(); it.hasNext();) {
            ParsedLexieSource parsedLexieSource = (ParsedLexieSource) it.next();
            appendParsedLexieSource(parsedLexieSource);
        }
        closeTag("sources");
        closeTag("lexiemap");
    }
    
    public void appendLexieUnit(TextLexieUnit lexieUnit) throws IOException {
        startOpenTag("lexie");
        addAttribute("lx-id",lexieUnit.getLexieId());
        addAttribute("value",lexieUnit.getCanonicalLexie());
        endOpenTag();
        List occurenceList = lexieUnit.getOccurenceList();
        for(Iterator it = occurenceList.iterator();it.hasNext();) {
            Occurence occurence = (Occurence) it.next();
            ParsedLexieSource parsedLexieSource = occurence.getParsedLexieSource();
            startOpenTag("occurence");
            addAttribute("src-id",parsedLexieSource.getLexieSource().getSourceId());
            addAttribute("pos",String.valueOf(occurence.getOccurencePosition()));
            closeEmptyTag();
        }
        closeTag("lexie");
    }
    
    public void appendParsedLexieSource(ParsedLexieSource parsedLexieSource) throws IOException {
        LexieSource lexieSource = parsedLexieSource.getLexieSource();
        startOpenTag("source");
        addAttribute("src-id",lexieSource.getSourceId());
        endOpenTag();
        openTag("text");
        appendText(lexieSource.getSourceText(),parsedLexieSource.getOccurenceList());
        closeTag("text",false,true);
        closeTag("source");
    }
    
    private void appendText(String text, List occurenceList) throws IOException {
        int length = text.length();
        int size = occurenceList.size();
        int startLexieIndex = -1;
        int endLexieIndex = -1;
        Occurence currentOccurence = null;
        int currentOccurenceIndex = -1;
        if (size > 0) {
            currentOccurenceIndex = 0;
            currentOccurence = (Occurence) occurenceList.get(0);
            SubstringPosition substringPosition = currentOccurence.getSubstringPosition();
            startLexieIndex = substringPosition.getBeginIndex();
            endLexieIndex = substringPosition.getEndIndex();
        }
        for(int i = 0; i < length; i++) {
            if (i == startLexieIndex) {
                startOpenTag("lx",false);
                addAttribute("lx-id", currentOccurence.getLexieUnit().getLexieId());
                addAttribute("pos", String.valueOf(currentOccurence.getOccurencePosition()));
                endOpenTag(false);
            }
            escape(text.charAt(i));
            if (i == endLexieIndex) {
                closeTag("lx",false,false);
                currentOccurenceIndex ++;
                if (currentOccurenceIndex < size) {
                    currentOccurence = (Occurence) occurenceList.get(currentOccurenceIndex);
                    SubstringPosition substringPosition = currentOccurence.getSubstringPosition();
                    startLexieIndex = substringPosition.getBeginIndex();
                    endLexieIndex = substringPosition.getEndIndex();
                }
                else  {
                    currentOccurence = null;
                    startLexieIndex = -1;
                    endLexieIndex = -1;
                }
            }
            
        }
        
    }
    
     public static String getLexieMapXML(LexieMap lexieMap,boolean withTabs) {
        StringBuffer stringBuffer = new StringBuffer(512);
        try {
            XMLDestination destination = XMLDestinationFactory.newInstance(stringBuffer);
            LexieMapXMLWriter lexieMapXMLWriter = new LexieMapXMLWriter(destination,(withTabs)?0:-999);
            lexieMapXMLWriter.appendXMLDeclaration();
            lexieMapXMLWriter.appendLexieMap(lexieMap);
        } catch(IOException ioe) {
            
        }
        return stringBuffer.toString();
    }
    
}
