/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.tableparser;

import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class CSVParameters implements TableParser.Parameters {
    private char delimiter = ',';
    private char quote = '"';
    private boolean excelCSV = false;
    
    public CSVParameters() {
    }
    
    public char getDelimiter() {
        return delimiter;
    }
    
    public void setDelimiter(char delimiter) {
        switch(delimiter) {
            case ',' :
            case ';' :
            case '\t' :
            case '|' :
                this.delimiter = delimiter;
                break;
            default :
                throw new IllegalArgumentException("Wrong delimiter value = " + delimiter);
        }
    }
    
    public char getQuote() {
        return quote;
    }
    
    public void setQuote(char quote) {
        switch(quote) {
            case '\'' :
            case '"' :
                this.quote = quote;
                break;
            default :
                throw new IllegalArgumentException("Wrong quote value = " + quote);
        }
    }
    
    public boolean isExcelCSV() {
        return excelCSV;
    }
    
    public void setExcelCSV(boolean excelCSV) {
        this.excelCSV = excelCSV;
    }
    
    public static char getDefaultDelimiter() {
        return ',';
    }
    
    public static char getDefaultQuote() {
        return '"';
    }
    
    public static char[] getAvailableDelimiterArray() {
        char[] caracs = {',',';','\t','|'};
        return caracs;
    }
    
    public static char[] getAvailableQuoteArray() {
        char[] caracs = {'\'','\"'};
        return caracs;
    }
    
    public static String[] getAvalaibleDelimiterArrayToString() {
        return StringUtils.toStringArray(getAvailableDelimiterArray());
    }
    
    public static String[] getAvalaibleQuoteArrayToString() {
        return StringUtils.toStringArray(getAvailableQuoteArray());
    }
    
}
