/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.tableparser;

import net.mapeadores.util.text.*;

/**
 *
 * @author Vincent Calame
 */
public class MysqlParameters implements TableParser.Parameters {
    public final static String ALLOWED_FIELDS_TERMINATEDBY = ",;\t|";
    public final static String ALLOWED_LINES_TERMINATEDBY = "\n§\u00B6$";
    private char linesTerminatedBy = '\n';
    private char escapedBy = '\\';
    private char fieldsTerminatedBy = '\t';
    
    public MysqlParameters() {
    }
    
    public char getEscapedBy() {
        return escapedBy;
    }
    
    public char getFieldsTerminatedBy() {
        return fieldsTerminatedBy;
    }
    
    public char getLinesTerminatedBy() {
        return linesTerminatedBy;
    }
    
    public void setFieldsTerminatedBy(char fieldsTerminatedBy) {
        if (ALLOWED_FIELDS_TERMINATEDBY.indexOf(fieldsTerminatedBy) == -1) throw new IllegalArgumentException("Wrong fieldsTerminatedBy = " + fieldsTerminatedBy);
        this.fieldsTerminatedBy = fieldsTerminatedBy;
    }
    
    public void setLinesTerminatedBy(char linesTerminatedBy) {
        if (ALLOWED_LINES_TERMINATEDBY.indexOf(linesTerminatedBy) == -1) throw new IllegalArgumentException("Wrong linesTerminatedBy = " + linesTerminatedBy);
        this.linesTerminatedBy = linesTerminatedBy;
    }
    
    public static char getDefaultFieldsTerminatedBy() {
        return '\t';
    }
    
    public static char getDefaultLinesTerminatedBy() {
        return '\n';
    }
    
    public static char[] getAvailableFieldsTerminatedByArray() {
        return ALLOWED_FIELDS_TERMINATEDBY.toCharArray();
    }
    
    public static char[] getAvailableLinesTerminatedByArray() {
        return ALLOWED_LINES_TERMINATEDBY.toCharArray();
    }
    
    public static String[] getAvalaibleFieldsTerminatedByArrayToString() {
        return StringUtils.toStringArray(ALLOWED_FIELDS_TERMINATEDBY);
    }
    
    public static String[] getAvalaibleLinesTerminatedByArrayToString() {
        return StringUtils.toStringArray(ALLOWED_LINES_TERMINATEDBY);
    }
    
    
}
