/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.tableparser;

import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class MysqlParser {
    
    private MysqlParser() {
    }
    
    public static String[][] parse(String s, MysqlParameters mysqlParameters) {
        char linesTerminatedBy = mysqlParameters.getLinesTerminatedBy();
        char escapedBy = mysqlParameters.getEscapedBy();
        char fieldsTerminatedBy = mysqlParameters.getFieldsTerminatedBy();
        int length = s.length();
        LineBuffer lineBuffer = new LineBuffer();
        for (int i = 0; i < length;i++) {
            char carac = s.charAt(i);
            if (carac == escapedBy) {
                if (i < (length - 1)) {
                    char next = s.charAt(i+1);
                    if (next == 'N') {
                        
                    } else if (next == '0') {
                        lineBuffer.append((char) 0);
                    } else {
                        lineBuffer.append(next);
                    }
                    if (next == '\r') {
                        if (i < (length-2)) {
                            if (s.charAt(i+2) == '\n') {
                                lineBuffer.append('\n');
                                i++;
                            }
                        }
                    }
                    i++;
               } else lineBuffer.append(escapedBy);
            }
            else if (carac == fieldsTerminatedBy) {
                lineBuffer.endField();
            }
            else if (carac == '\r') {
                if (linesTerminatedBy == '\n') {
                    if (i < (length - 1)) {
                        if (s.charAt(i+1) == '\n') i++;
                    }
                    lineBuffer.endLine();
                }
                else {
                    lineBuffer.append(carac);
                }
            } else if (carac == linesTerminatedBy) {
                lineBuffer.endLine();
            } else {
                lineBuffer.append(carac);
            }
        }
        lineBuffer.endLine();
        return lineBuffer.flush();
    }
    
    static class LineBuffer {
        List lineList = new ArrayList();
        List fieldList = new ArrayList();
        StringBuffer fieldBuffer;
        
        LineBuffer() {
            fieldBuffer = new StringBuffer();
        }
        
        void append(char carac) {
            fieldBuffer.append(carac);
        }
        
        void endField() {
            fieldList.add(fieldBuffer.toString());
            fieldBuffer = new StringBuffer();
        }
        
        void endLine() {
            endField();
            int size = fieldList.size();
            String[] line = new String[size];
            for (int i = 0; i < size;i++) {
                line[i] = (String) fieldList.get(i);
            }
            lineList.add(line);
            fieldList.clear();
        }
        
        String[][] flush() {
            int size = lineList.size();
            String[][] result = new String[size][];
            for (int i = 0; i< size;i++) {
                result[i] = (String[]) lineList.get(i);
            }
            lineList.clear();
            return result;
        }
    }
    
}
