/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.text.tableparser;

import net.mapeadores.util.text.*;
import com.Ostermiller.util.*;
import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class TableParser {
    
    private TableParser() {
    }
    
    public static String[][] parse(String s, Parameters parameters) {
        if (parameters instanceof OOCopyParameters) {
            return parseOOCopy(s,(OOCopyParameters) parameters);
        }
        if (parameters instanceof CSVParameters) {
            return parseCSV(s,(CSVParameters) parameters);
        }
        if (parameters instanceof MysqlParameters) {
            return MysqlParser.parse(s,(MysqlParameters) parameters);
        }
        throw new IllegalArgumentException("unknown TableParser.Parameters implementations = " + parameters.getClass().getName());
    }
    
    private static String[][] parseOOCopy(String s,OOCopyParameters parameters) {
        String[] lines = StringUtils.getLineTokens(s, StringUtils.NOTCLEAN);
        int length = lines.length;
        String[][] result = new String[length][];
        for (int i = 0; i < length;i++) {
            String line = lines[i];
            String[] tokens = StringUtils.getTokens(line, '\t', StringUtils.NOTCLEAN);
            result[i] = tokens;
        }
        return result;
    }
    
    private static String[][] parseCSV(String s,CSVParameters parameters) {
        CSVParse csvParse = null;
        Reader stringReader = new StringReader(s);
        if (parameters.isExcelCSV()) {
            csvParse = new ExcelCSVParser(stringReader);
        }
        else {
            csvParse = new CSVParser(stringReader);
        }
        csvParse.changeDelimiter(parameters.getDelimiter());
        csvParse.changeQuote(parameters.getQuote());
        try {
            return csvParse.getAllValues();
        }
        catch(IOException ioe) {
            throw new IllegalStateException("should not occur",ioe);
        }
    }
    
    
    public static interface Parameters {
        
    }
}
