/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.xml;

import java.io.*;
import net.mapeadores.util.langinteger.*;
import org.w3c.dom.*;
import java.awt.Color;
import java.util.*;
import net.mapeadores.util.text.*;

/**
 *
 * @author  Vincent Calame
 */
public class TagUtils {
    
    private TagUtils() {
        
    }
    
    /**
     * @throw LangIntegerException si la valeur de l'attribut xml:lang est faux ou égal à LangInteger.NO_LINGUISTIC_CONTENT ("zxx")
     */
    static public Libelle getLibelle(Element element) throws LangIntegerException{
        int langInteger = LangInteger.UNDETERMINED;
        String langIntegerString = element.getAttribute("xml:lang");
        if (langIntegerString.length() > 0) {
            langInteger = LangInteger.parse(langIntegerString);
            if (langInteger == LangInteger.NO_LINGUISTIC_CONTENT) throw new LangIntegerException();
        }
        String libelle = XMLUtils.getData(element);
        String vo = element.getAttribute("vo");
        boolean isvo = true;
        if ((vo.equals("0")) || (vo.equals("false"))) isvo = false;
        return new XMLLib(langInteger,libelle,isvo);
    }
    
    static public Color getColor(Element element) {
        try {
            return Color.decode(XMLUtils.getData(element));
        } catch(NumberFormatException nfe) {
            return  null;
        }
    }
    
    /**
     *langFilter peut être null.
     */
    static public void writeLibelleList(Writer writer,List libelleList,int nbredetabulations, LangFilter langFilter) throws IOException {
        for (Iterator it = libelleList.iterator();it.hasNext();) {
            Libelle lib = (Libelle) it.next();
            if ((langFilter == null) || (langFilter.accept(lib.getLangInteger()))) {
                XMLUtils.writeTabs(writer, nbredetabulations);
                writeLibelle(writer, lib.getLibelleString(), lib.getLangInteger(), lib.isVo());
            }
        }
    }
    
    static public void writeLibelleList(Writer writer,LibelleHolder libelleHolder,int nbredetabulations, LangFilter langFilter) throws IOException {
        List libelleList = libelleHolder.getLibelleList();
        for (Iterator it = libelleList.iterator();it.hasNext();) {
            Libelle lib = (Libelle) it.next();
            if ((langFilter == null) || (langFilter.accept(lib.getLangInteger()))) {
                XMLUtils.writeTabs(writer, nbredetabulations);
                writeLibelle(writer, lib.getLibelleString(), lib.getLangInteger(), lib.isVo());
            }
        }
    }
    
    static public void writeLibelle(Writer writer,String libelleString,int langInteger,boolean isvo) throws IOException {
        if (langInteger == LangInteger.NO_LINGUISTIC_CONTENT) return;
        writer.write("<lib");
        writeXmlLangAttribute(writer,langInteger);
        if (!isvo) writer.write(" vo='0'");
        writer.write(">");
        XMLUtils.escapeXML(libelleString,writer);
        writer.write("</lib>");
    }
    
    static public void writeXmlLangAttribute(Writer writer,int langInteger) throws IOException {
        if (langInteger != LangInteger.UNDETERMINED) {
            writer.write(" xml:lang=\"");
            LangInteger.writeString(writer,langInteger);
            writer.write("\"");
        }
    }
    
    static class XMLLib implements Libelle {
        int langInteger;
        String libelle;
        boolean libellevo;
        
        XMLLib(int langInteger,String libelle, boolean libellevo) {
            this.langInteger = langInteger;
            this.libelle = libelle;
            this.libellevo = libellevo;
        }
        
        public int getLangInteger() {
            return langInteger;
        }
        
        public String getLibelleString() {
            return libelle;
        }
        
        public boolean isVo() {
            return libellevo;
        }
    }

}
