/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.xml;

import java.awt.Color;
import java.io.*;
import net.mapeadores.util.langinteger.*;
import net.mapeadores.util.text.*;
import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public class UtilXMLWriter extends DefaultXMLWriter {
    
    public UtilXMLWriter() {
        
    }
    
    public UtilXMLWriter(XMLDestination xmlDestination,int indentLength) {
        super();
        setXmlDestination(xmlDestination);
        setIndentLength(indentLength);
    }
    
    public void addColor(Color color) throws IOException {
        openTag("color");
        append(XMLUtils.toHexFormat(color));
        closeTag("color");
    }
    
    public void addLibElement(int langInteger, String lib) throws IOException {
        addLibElement(langInteger,lib,true,false);
    }
    
    
    public void addLibElement(int langInteger, String lib, boolean isVo, boolean withDefaultVo) throws IOException {
        if (langInteger == LangInteger.NO_LINGUISTIC_CONTENT) return;
        startOpenTag("lib");
        if (!isVo) {
            addAttribute("vo", "0");
        } else if (withDefaultVo) {
            addAttribute("vo", "1");
        }
        if (langInteger != LangInteger.UNDETERMINED) {
            addAttribute("xml:lang", LangInteger.toString(langInteger));
        }
        endOpenTag(false);
        escape(lib);
        closeTag("lib",false,false);
    }
    
    public void addXmlLangAttribute(int langInteger) throws IOException {
        if (langInteger != LangInteger.UNDETERMINED) {
            addAttribute("xml:lang",LangInteger.toString(langInteger));
        }
    }
    
    public void addLibelleList(LibelleHolder libelleHolder, boolean withVoDefault)  throws IOException {
        addLibelleList(libelleHolder, null, withVoDefault);
    }
    
    public void addLibelleList(LibelleHolder libelleHolder,LangFilter langFilter, boolean withVoDefault)  throws IOException {
        List libelleList = libelleHolder.getLibelleList();
        for (Iterator it = libelleList.iterator();it.hasNext();) {
            Libelle lib = (Libelle) it.next();
            if ((langFilter == null) || (langFilter.accept(lib.getLangInteger()))) {
                addLibElement(lib.getLangInteger(), lib.getLibelleString(),lib.isVo(),withVoDefault);
            }
        }
    }
}
