/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2007 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.xml;

import java.io.*;

/**
 *
 * @author Vincent Calame
 */
public class XMLDestinationFactory {
    
    private XMLDestinationFactory() {
    }
    
    public static XMLDestination newInstance(Writer writer) {
        return new WriterXMLDestination(writer);
    }
    
    public static XMLDestination newInstance(StringBuffer buf) {
        return new StringBufferXMLDestination(buf);
    }
    
    static class WriterXMLDestination implements XMLDestination {
        Writer writer;
        
        WriterXMLDestination(Writer writer) {
            this.writer = writer;
        }
        
        public void append(CharSequence charSequence) throws IOException {
            int length = charSequence.length();
            for(int i = 0; i < length; i++) {
                writer.write(charSequence.charAt(i));
            }
        }
    
        public void append(char c)  throws IOException {
            writer.write(c);
        }
        
    }
    
    static class StringBufferXMLDestination implements XMLDestination {
        StringBuffer buf;
        
        StringBufferXMLDestination(StringBuffer buf) {
            this.buf = buf;
        }
        
        public void append(CharSequence charSequence) {
            if (charSequence instanceof StringBuffer) {
                buf.append((StringBuffer)charSequence);
            }
            else buf.append(charSequence.toString());
        }
    
        public void append(char c) {
            buf.append(c);
        }
        
    }
    
}
