/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2005-2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.mapeadores.util.xml;

import java.util.*;
import java.io.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.awt.Color;
import net.mapeadores.util.text.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import java.io.*;
import net.mapeadores.util.exceptions.*;

/**
 * Contients des méthodes statiques
 */

public class XMLUtils {
    
    static public String toString(Element element, boolean elementInclude) {
        StringBuffer stringBuffer = new StringBuffer();
        DefaultXMLWriter xmlWriter = new DefaultXMLWriter();
        xmlWriter.setXmlDestination(XMLDestinationFactory.newInstance(stringBuffer));
        xmlWriter.setIndentLength(-999);
        try {
            addElement(xmlWriter, element, elementInclude);
        }
        catch(IOException ioe) {
            
        }
        return stringBuffer.toString();
    }
    
    static public void addElement(XMLWriter xmlWriter,Element element, boolean elementInclude) throws IOException {
        String tagName = element.getTagName();
        if (elementInclude) {
            xmlWriter.startOpenTag(tagName);
            if (element.hasAttributes()) {
                NamedNodeMap map = element.getAttributes();
                int count = map.getLength();
                for (int i = 0; i < count;i++) {
                    Attr attribute = (Attr) map.item(i);
                    xmlWriter.addAttribute(attribute.getName(),attribute.getValue());
                }
            }
        }
        NodeList children = element.getChildNodes();
         if (elementInclude) {
            if (children.getLength() == 0) {
                xmlWriter.closeEmptyTag();
                return;
            }
            else {
                xmlWriter.endOpenTag();
            }
         }
        int count = children.getLength();
        for (int i = 0; i <count;i++) {
            Node node = children.item(i);
            if (node instanceof Text) {
                xmlWriter.append(((Text) node).getData());
            }
            else if (node instanceof Element) {
                addElement(xmlWriter, (Element)node,true);
            }
        }
        if (elementInclude) {
            xmlWriter.closeTag(tagName);
        }
    }
    
    static public void writeDomTree(Document doc,OutputStream outputStream) throws TransformerException {
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);
            StreamResult result = new StreamResult(outputStream);
            transformer.transform(source, result);
    }
    
    static public void appendTabs(StringBuffer buf, int nbre) {
        if (nbre > - 1) {
            buf.append('\n');
            for (int i = 0; i < nbre;i++) {
                buf.append('\t');
            }
        }
    }
    
    static public void writeTabs(Writer writer, int nbre) throws IOException {
        if (nbre > - 1) {
            writer.write('\n');
            for (int i = 0; i < nbre;i++) {
                writer.write('\t');
            }
        }
    }
    
    static public void printTabs(PrintWriter printWriter, int nbre) {
        if (nbre > - 1) {
            printWriter.print('\n');
            for (int i = 0; i < nbre;i++) {
                printWriter.print('\t');
            }
        }
    }
    
    /**
     *
     */
    static public  void escapeXML(String s,StringBuffer buf) {
        if ((s== null) || (s.length() == 0)) return;
        char[] caracs = s.toCharArray();
        for (int i = 0; i < caracs.length;i++) {
            if (caracs[i] < 32) buf.append(' ');
            else {
                switch(caracs[i]) {
                    case '&' :
                        buf.append("&amp;");
                        break;
                    case '"' :
                        buf.append("&quot;");
                        break;
                    case '\'' :
                        buf.append("&apos;");
                        break;
                    case '<' :
                        buf.append("&lt;");
                        break;
                    case '>' :
                        buf.append("&gt;");
                        break;
                    case '\u00A0' :
                        buf.append("&#x00A0;");
                        break;
                    default :
                        buf.append(caracs[i]);
                }
            }
        }
    }
    
    static public  void escapeXML(String s,Writer writer) throws IOException {
        if ((s== null) || (s.length() == 0)) return;
        char[] caracs = s.toCharArray();
        for (int i = 0; i < caracs.length;i++) {
            if (caracs[i] < 32) writer.write(' ');
            else {
                switch(caracs[i]) {
                    case '&' :
                        writer.write("&amp;");
                        break;
                    case '"' :
                        writer.write("&quot;");
                        break;
                    case '\'' :
                        writer.write("&apos;");
                        break;
                    case '<' :
                        writer.write("&lt;");
                        break;
                    case '>' :
                        writer.write("&gt;");
                        break;
                    case '\u00A0' :
                         writer.write("&#x00A0;");
                        break;
                    default :
                        writer.write(caracs[i]);
                }
            }
        }
    }
    
     static public  void escapeTechXML(String s,Writer writer) throws IOException {
        if ((s== null) || (s.length() == 0)) return;
        char[] caracs = s.toCharArray();
        for (int i = 0; i < caracs.length;i++) {
            if (caracs[i] < 32) writer.write(' ');
            else {
                switch(caracs[i]) {
                    case '&' :
                        writer.write("&amp;");
                        break;
                    case '"' :
                        writer.write("&quot;");
                        break;
                    case '<' :
                        writer.write("&lt;");
                        break;
                    case '>' :
                        writer.write("&gt;");
                        break;
                    case '\u00A0' :
                         writer.write("&#x00A0;");
                        break;
                    default :
                        writer.write(caracs[i]);
                }
            }
        }
    }
    
    static public  void escapeXML(String s,PrintWriter pw) {
        if ((s== null) || (s.length() == 0)) return;
        char[] caracs = s.toCharArray();
        for (int i = 0; i < caracs.length;i++) {
            escapeXML(caracs[i],pw);
        }
    }
    
    static public  void escapeXML(char carac,PrintWriter pw) {
            if (carac < 32) pw.print(' ');
            else {
                switch(carac) {
                    case '&' :
                        pw.print("&amp;");
                        break;
                    case '"' :
                        pw.print("&quot;");
                        break;
                    case '\'' :
                        pw.print("&apos;");
                        break;
                    case '<' :
                        pw.print("&lt;");
                        break;
                    case '>' :
                        pw.print("&gt;");
                        break;
                    case '\u00A0' :
                        pw.print("&#x00A0;");
                        break;
                    default :
                        pw.print(carac);
                }
            }
    }
    
    public static String cleanString(String s) {
        return StringUtils.cleanString(s,true);
    }
    
    
    public static String cleanString(String s, boolean trim) {
        return StringUtils.cleanString(s,trim);
    }
    
    /**
     */
    public static String getData(Element el) {
        return getData(el,true);
    }
    
        /*
         */
    public static String getData(Element el, boolean trim) {
        Node nd = el.getFirstChild();
        if ((nd != null) &&  ((nd.getNodeType() == Node.TEXT_NODE) || (nd.getNodeType() == Node.CDATA_SECTION_NODE))) {
            String result = ((Text) nd).getData();
            if (result == null) return "";
            else return cleanString(result,trim);
        }
        else return "";
    }
    
    public static Element getFirstElement(Element el) {
        NodeList list = el.getChildNodes();
        for (int i = 0; i < list.getLength(); i ++) {
            Node nd = list.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE ) {
                    return (Element) nd;
            }
        }
        return null;
    }
    
    /**
     * Retourne l'expression de la couleur en argument sous la forme de la chaîne utlisée en HTML #00FF00.
     **/
    public static String toHexFormat(Color color) {
        StringBuffer buf = new StringBuffer();
        buf.append('#');
        appendHexString(color.getRed(),buf);
        appendHexString(color.getGreen(),buf);
        appendHexString(color.getBlue(),buf);
        return buf.toString();
    }
    
    /*
     * Méthode privée utilisée par toHexFormat pour traduire un entier en un hexadécimal de deux lettres.
     */
    private static void appendHexString(int codecouleur,StringBuffer buf) {
        String hex = Integer.toHexString(codecouleur);
        if (hex.length() == 1) buf.append('0');
        buf.append(hex);
    }
    
    public static void writeAttribute(Writer writer,String attributename,String value) throws IOException{
        if ((value != null) && (value.length() > 0)) {
            writer.write(' ');
            writer.write(attributename);
            writer.write("=\"");
            escapeXML(value,writer);
            writer.write('\"');
        }
    }
    
    public static void printAttribute(PrintWriter pw,String attributename,String value) {
        if ((value != null) && (value.length() > 0)) {
            pw.print(' ');
            pw.print(attributename);
            pw.print("=\"");
            escapeXML(value,pw);
            pw.print('\"');
        }
    }
    
    public static void writeTag(Writer writer,String tagname,String value,int nbredetabulations) throws IOException{
        if ((value != null) && (value.length() > 0)) {
            writeTabs(writer,nbredetabulations);
            writer.write("<");
            writer.write(tagname);
            writer.write(">");
            escapeXML(value,writer);
            writer.write("</");
            writer.write(tagname);
            writer.write(">");
        }
    }
    
    public static void printTag(PrintWriter pw,String tagname,String value,int nbredetabulations) {
        if ((value != null) && (value.length() > 0)) {
            printTabs(pw,nbredetabulations);
            pw.print("<");
            pw.print(tagname);
            pw.print(">");
            escapeXML(value,pw);
            pw.print("</");
            pw.print(tagname);
            pw.print(">");
        }
    }
    
    public static void writeXMLDeclaration(Writer writer) throws IOException {
        writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    }
    
    public static void printXMLDeclaration(PrintWriter pw) {
        pw.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    }
    
    public static void printXMLStylesheet(PrintWriter pw, String type, String href) {
        pw.print("<?xml-stylesheet type=\"");
        pw.print(type);
        pw.print("\" href=\"");
        pw.print(href);
        pw.print("\"?>");
    }
    
    
    
    public static Document getDocument(String xml) throws XMLException {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            InputSource inputSource = new InputSource(new StringReader(xml));
            Document doc = db.parse(inputSource);
            return doc;
        }
        catch(ParserConfigurationException pce) {
            throw new XMLException(pce);
        }
        catch(SAXParseException spe) {
            throw new XMLException(spe);
        }
        catch(SAXException se) {
            throw new XMLException(se);
        }
        catch(IOException ioe) {
            throw new XMLException(ioe);
        }
    }
    
    public static Object testXml(File xslFile) {
        try {
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docbuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = docbuilder.parse(xslFile);
            return Boolean.TRUE;
        } catch(Exception e) {
            String message = e.getMessage();
            if (e instanceof SAXParseException) {
                SAXParseException sax = (SAXParseException) e;
                message = message + " at " + sax.getLineNumber() + "," + sax.getColumnNumber();
            }
            return message;
        }
    }
}
