/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2009 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.xml;

import java.io.*;
import net.mapeadores.util.xml.ns.NameSpace;

/**
 *
 * @author Vincent Calame
 */
public class XMLWriterPart implements XMLWriter {
    XMLWriter xmlWriter;
    
    public XMLWriterPart(XMLWriter xmlWriter) {
        this.xmlWriter = xmlWriter;
    }
    
    public XMLDestination getXMLDestination() throws IOException {
        return xmlWriter.getXMLDestination();
    }
    
    public boolean isAposEscaped() {
        return xmlWriter.isAposEscaped();
    }
    
    public int getCurrentIndentValue() {
        return xmlWriter.getCurrentIndentValue();
    }
    
    public void appendIndent() throws IOException {
        xmlWriter.appendIndent();
    }
    
    public void escape(CharSequence charSequence) throws IOException {
        xmlWriter.escape(charSequence);
    }
    
    public void escape(char carac) throws IOException {
        xmlWriter.escape(carac);
    }
    
    public void openTag(String name) throws IOException {
        xmlWriter.openTag(name);
    }
    
    public void openTag(String name, boolean indentBefore, boolean indentIncrease) throws IOException {
        xmlWriter.openTag(name, indentBefore, indentIncrease);
    }
    
    public void startOpenTag(String name) throws IOException {
        xmlWriter.startOpenTag(name);
    }
    
    public void startOpenTag(String name, boolean indentBefore) throws IOException {
        xmlWriter.startOpenTag(name, indentBefore);
    }
    
    public void closeTag(String name) throws IOException {
        xmlWriter.closeTag(name);
    }
    
    public void closeTag(String name, boolean indentBefore, boolean indentDecrease) throws IOException {
        xmlWriter.closeTag(name, indentBefore, indentDecrease);
    }
    
    public void endOpenTag() throws IOException {
        xmlWriter.endOpenTag();
    }
    
    public void endOpenTag(boolean increaseIndent) throws IOException {
        xmlWriter.endOpenTag(increaseIndent);
    }
    
    public void addAttribute(String name, String value) throws IOException {
        xmlWriter.addAttribute(name, value);
    }
    
    public void addAttribute(String name, int value) throws IOException {
        xmlWriter.addAttribute(name, value);
    }
    
    public void addAttribute(String attributeName,char c) throws IOException {
        xmlWriter.addAttribute(attributeName, c);
    }
    
    public void closeEmptyTag() throws IOException {
        xmlWriter.closeEmptyTag();
    }
    
    public void addNameSpaceAttribute(NameSpace nameSpace) throws IOException {
        xmlWriter.addNameSpaceAttribute(nameSpace);
    }
    
    public void addNameSpaceAttribute(String prefix, String nameSpace) throws IOException {
        xmlWriter.addNameSpaceAttribute(prefix,nameSpace);
    }
    
    public void addSimpleElement(String tagname,String value) throws IOException {
        xmlWriter.addSimpleElement(tagname, value);
    }
    
    public void addSimpleElement(String tagname, String attribute, String attributeValue, String value) throws IOException {
        xmlWriter.addSimpleElement(tagname, attribute, attributeValue, value);
    }
    
    public void addEmptyElement(String tagname) throws IOException {
        xmlWriter.addEmptyElement(tagname);
    }
    
    public void appendXMLDeclaration() throws IOException {
        xmlWriter.appendXMLDeclaration();
    }
    
    public void appendStylesheet(String href, String type) throws IOException {
        xmlWriter.appendStylesheet(href, type);
    }
   
    public void appendDocType(String doctype) throws IOException {
        xmlWriter.appendDocType(doctype);
    }
    
    public void append(char c) throws IOException {
        xmlWriter.append(c);
    }
    
    public void append(CharSequence charSequence) throws IOException {
        xmlWriter.append(charSequence);
    }

}
