/* Mapeadores Libraries - Open source libraries dedicated to collective intelligence tools
 *
 * Copyright (C) 2006 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.xml;

import java.text.DecimalFormatSymbols;
import java.io.*;

/**
 * Classe contenant des méthodes utilitaires pour l'écriture de feuilles de styles XSLT
 *
 * @author Vincent Calame
 */
public class XSLTUtils {
    
    private XSLTUtils() {
    }
    
    public static void appendImport(StringBuffer buf,String href) {
        buf.append("<xsl:import href=\"");
        buf.append(href);
        buf.append("\"/>");
    }

    
    public static void writeValueVariable(Writer writer, String name, String value) throws IOException{
        writer.write("<xsl:variable name=\"");
        writer.write(name);
        writer.write("\" select=\"'");
        XMLUtils.escapeXML(value,writer);
        writer.write("'\"/>");
    }
    
    public static void writeNodesetVariable(Writer writer, String name, String nodeset) throws IOException{
        writer.write("<xsl:variable name=\"");
        writer.write(name);
        writer.write("\" select=\"");
        writer.write(nodeset);
        writer.write("\"/>");
    }
    
    public static void appendHtmlStrictOutput(StringBuffer buf) {
        buf.append("<xsl:output method='html' encoding='UTF-8' doctype-public='-//W3C//DTD HTML 4.01//EN' doctype-system='http://www.w3.org/TR/html4/strict.dtd'/>");
    }
    
    public static void appendText(StringBuffer buf, String text, boolean disableOutputEscaping) {
        buf.append("<xsl:text");
        if (disableOutputEscaping) buf.append(" disable-output-escaping=\"yes\"");
        buf.append(">");
        XMLUtils.escapeXML(text,buf);
        buf.append("</xsl:text>");
    }
    
    public static void startXSLT(StringBuffer buf, boolean linebreak) {
        buf.append("<?xml version='1.0' encoding='UTF-8'?>");
        if (linebreak) buf.append("\n");
        buf.append("<xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>");
    }
    
    
    public static void endXSLT(StringBuffer buf) {
        buf.append("</xsl:stylesheet>");
    }
    
    public static void endXSLT(Writer writer) throws IOException {
        writer.write("</xsl:stylesheet>");
    }
}
