/* @@@@@APPLICATION@@@@@
 *
 * Copyright (C) 2008 Vincent Calame - Exemole <vincent.calame@exemole.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package net.mapeadores.util.xml.xmlpull;

import java.util.*;

/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractElementHandler implements XmlPullElementHandler {
    XmlPullElementHandler subElementHandler;
    
    public AbstractElementHandler() {
    }
    
   
    public void processStartElement(String tagname, Map attributeMap) {
            if (subElementHandler != null) {
                subElementHandler.processStartElement(tagname, attributeMap);
            }
            else {
                subElementHandler = startSubElement(tagname, attributeMap);
                if (subElementHandler == null) {
                    subElementHandler = new NullHandler();
                }
            }
        }

        
        public boolean processEndElement(String tagname) {
            if (subElementHandler != null) {
                boolean endSubElement = subElementHandler.processEndElement(tagname);
                if (endSubElement) {
                    if (subElementHandler instanceof SimpleElementHandler) {
                        SimpleElementHandler simple =  (SimpleElementHandler) subElementHandler;
                        endSimpleSubElement(simple.getType(), simple.getText());
                    }
                    else endSubElement(subElementHandler);
                    subElementHandler = null;
                }
                return false;
            }
            else {
                return true;
            }
        }
        
        public void processText(String text) {
            if (subElementHandler != null) subElementHandler.processText(text);
            else {
                appendText(text);
            }
        }
        
        public abstract XmlPullElementHandler startSubElement(String tagname, Map attributeMap);
        
        public abstract void endSubElement(XmlPullElementHandler subElementHandler);
        
        public abstract void endSimpleSubElement(int type, String text);
        
        public abstract void appendText(String text);
        
        public static String getAttributeString(Map attributeMap, String name) {
            if (attributeMap == null) return null;
            String result = (String) attributeMap.get(name);
            if (result == null) return null;
            if (result.length() == 0) return null;
            return result;
        }
    
}
